/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.ItemListProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemProxy");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");


(function(){

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
dojo.declare("com.ibm.team.workitem.web.cache.internal.ItemListProxy", [com.ibm.team.workitem.web.cache.internal.ItemProxy], {
    
        constructor: function(){
		
        },
        addItems: function(parms /* items, attributeId */){
			var newItems = parms.items;
			if(!newItems || !newItems.length) return;
			var isChanged = false;
            for (var i = 0; i < newItems.length; i++) {
                isChanged = this._addItem(newItems[i], parms.attributeId) || isChanged;
            }
			if(isChanged) this._markChanged(parms.attributeId);
        },
        deleteItems: function(parms /* items, attributeId */){
			var items = parms.items;
			if(!items || !items.length) return;
			var isChanged = false;
            for (var i = 0; i < items.length; i++) {
	            isChanged = this._deleteItem(items[i], parms.attributeId) || isChanged;
            }
			if(isChanged) this._markChanged(parms.attributeId);
		},
         restoreItems: function(parms){
			var items = parms.items;
			if(!items || !items.length) return;
			var isChanged = false;
            for (var i = 0; i < items.length; i++) {
	            isChanged = this._restoreItem(items[i], parms.attributeId) || isChanged;
            }
			if(isChanged) this._markChanged(parms.attributeId);
		},
        getItems: function(parms){ 
		    var items =this._getItems(parms.attributeId);
			if(!parms.skipDeleted) return items;
			var newItems = [];
			for(var i=0;i<items.length;i++) {
				if(!items[i]._isDeleted) {
					newItems.push(items[i]);
				}
			} 
            return newItems;
        },
		includesItem: function(parms /* item, attributeId */){
			var item = parms.item
			var itemId = item.itemId || item.id;
			var items = this._getItems(parms.attributeId);
			var len = items.length;
            for (var i = 0; i < len; i++) {
				var s = items[i];
                if (itemId == s.itemId || itemId == s.id ) {
                    return !s._isDeleted;
				}
            }
            return false;
		},
        /**
         * Returns an array of the itemIds of the current subscribers.
         * Include the creator (but not the owner)
         */
        getCurrentItems: function(parms /* includeCreator, attributeId */){
            var items = [];
            
			// Special case for subscribers
            if (parms.includeCreator && this.object && this.object.attributes && this.object.attributes.creator && this.object.attributes.creator.itemId) {
                items.push({
					itemId: this.object.attributes.creator.itemId
				});
            }
			var currentItems = this._getItems(parms.attributeId);
            for (var i = 0; i < currentItems.length; i++) {
				var itemId = currentItems[i].itemId || currentItems[i].id;
                if (itemId && !currentItems[i]._isDeleted) 
                    items.push({
						itemId: itemId  
					});
            }
            
            return items;
        },
		_markChanged: function(attributeId){
			/* The approvals are updated in place, so just notify senders */
			this.setValue({
				attributeId: attributeId,
				caller: this,
				path: ["attributes" ,attributeId, "items"],
				value: 	this._getItems(attributeId)
			});
		},
		_getItems: function(attributeId) {
			return this.getValue({ attributeId: attributeId, path: ["attributes" ,attributeId, "items"], defaultValue: [] }) ;
		},
		_getItemsFormElements: function(attributeValueArray) {
			// conver the object array to an array of string of item id's
			var itemIds = [];
			for(var i=0;i<attributeValueArray.length;i++) {
				if(!attributeValueArray[i]._isDeleted) {
					itemIds.push(attributeValueArray[i].itemId || attributeValueArray[i].id )
				}
			}
			return itemIds;
		},
        _addItem: function(item, attributeId){
            var found = false;
			var isChanged = false;
			var items = this._getItems(attributeId);
			var itemId = item.itemId || item.id;
            for (var i = 0; i < items.length; i++) {
                if (items[i].itemId == itemId || items[i].id == itemId ) {
                    found = true;
					if (items[i]._isDeleted) {
						items[i]._isDeleted = false;
						isChanged = true;
					}
                    break;
                }
            }
            if (!found) {
                item._isNew = true;
                items.push(item);
				isChanged = true;
            }

			return isChanged;
        },
         _deleteItem: function(item, attributeId){
		 	
			var isChanged = false;
			var items = this._getItems(attributeId);
            for (var i = 0; i < items.length; i++) {
				var itemId = item.itemId || item.id;
                if (items[i].itemId == itemId || items[i].id == itemId) {
					if (items[i]._isNew) {
						items.splice(i, 1);
						isChanged = true;
					}
					else {
						if(!items[i]._isDeleted) {
							items[i]._isDeleted = true;
							isChanged = true;
						}
					}
                    break;
                }
            }
			return isChanged;
        },
         _restoreItem: function(item, attributeId){
			var isChanged = false;
			var items = this._getItems(attributeId);
            for (var i = 0; i < items.length; i++) {
				var itemId = item.itemId || item.id;
                if (items[i].itemId == itemId || items[i].id == itemId) {
					if (items[i]._isDeleted) {
						items[i]._isDeleted = false;
						isChanged = true;
					}
                    break;
                }
            }
			return isChanged;
        }
    });
    
})();
