/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient");

dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

(function() {

var Util = com.ibm.team.dashboard.web.util.internal.Util;

/*
 * Constants
 */
var SERVICE_ID = "com.ibm.team.dashboard.viewlets.service.internal.feed.IFeedService";

/*
 * Static class for calling feed service operations.
 */
com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient = {
	
	/*
	 * Retrieves a FeedResponseDTO from the feed at the given url.
	 */
	getFeed: function(/* string */ url, /* string*/ user, /* string */ password, /* string */ id, /* string */ ref, /* number */ startIndex, /* number */ count, /* boolean */ entries, /* boolean */ details, /* boolean */ feedInfo, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var params = {url: url};
		if (user) {
			params.user = user;
		}
		if (password) {
			params.password = password;
		}
		if (id) {
			// id is used to fetch one specific entry by id
			params.id = id;
		}
		else {
			if (startIndex && startIndex > 0) {
				params.startIndex = startIndex;
			}
			if (count) {
				params.count = count;
			}
		}
		if (ref) {
			/*
			 * Read marks are stored as item indexes relative to a reference point
			 * (the first item for that feed at that time). Because of paging, we
			 * may lose our reference on a different page, so we pass in the old ref
			 * here and get its new index in the response (refIndex).
			 */
			params.ref = ref;
		}
		if (entries) {
			params.entries = true;
		}
		if (details) {
			params.details = true;
		}
		if (feedInfo) {
			params.feedInfo = true;
		}
		var bindArgs = {
			url: Util.getServiceURL(SERVICE_ID, "feed", params),
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(response, ioArgs){
				if (response instanceof Error) {
					srh.doErrorCallback({
						status: ioArgs.xhr.status,
						message: ioArgs.xhr.statusText
					});
				}
				else {
					srh.doCallback(response);
				}
			}
		};
		var xhr = dojo.xhrGet(bindArgs).ioArgs.xhr;
		if (tracker) {
			tracker.register(xhr, srh);
		}
	},
	
	getJazzFeed: function(/* string[] */ filters, /* string */ processItemId, /* number */processItemScope, /* boolean */ showActive, /* string */ tagsKey, /* string */ tags, /* boolean */ scopeSubteams, /* string */ id, /* string */ ref, /* number */ startIndex, /* number */ count, /* boolean */ entries, /* boolean */ details, /* boolean */ feedInfo, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var params = {
			filters: dojo.toJson(filters),
			processItemId: processItemId,
			processItemScope: processItemScope
		};
		if (showActive) {
			params.showActive = showActive;
		}
		if (tagsKey && tags) {
			params.tagsKey = tagsKey;
			params.tags = tags;
		}
		if (scopeSubteams) {
			params.scopeSubteams = scopeSubteams;
		}
		if (id) {
			// id is used to fetch one specific entry by id
			params.id = id;
		}
		else {
			if (startIndex && startIndex > 0) {
				params.startIndex = startIndex;
			}
			if (count) {
				params.count = count;
			}
		}
		if (ref) {
			/*
			 * Read marks are stored as item indexes relative to a reference point
			 * (the first item for that feed at that time). Because of paging, we
			 * may lose our reference on a different page, so we pass in the old ref
			 * here and get its new index in the response (refIndex).
			 */
			params.ref = ref;
		}
		if (entries) {
			params.entries = true;
		}
		if (details) {
			params.details = true;
		}
		if (feedInfo) {
			params.feedInfo = true;
		}
		var bindArgs = {
			url: Util.getServiceURL(SERVICE_ID, "jazzFeed", params),
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(response, ioArgs){
				if (response instanceof Error) {
					srh.doErrorCallback({
						status: ioArgs.xhr.status,
						message: ioArgs.xhr.statusText
					});
				}
				else {
					srh.doCallback(response);
				}
			}
		};
		var xhr = dojo.xhrGet(bindArgs).ioArgs.xhr;
		if (tracker) {
			tracker.register(xhr, srh);
		}
	},
	
	getEventGroups: function(/* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var args = {
			url: Util.getServiceURL(SERVICE_ID, "eventGroups"),
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(response, ioArgs){
				if (response instanceof Error) {
					srh.doErrorCallback({
						status: ioArgs.xhr.status,
						message: ioArgs.xhr.statusText
					});
				}
				else {
					srh.doCallback(response);
				}
			}
		};
		var xhr = dojo.xhrGet(args).ioArgs.xhr;
		if (tracker) {
			tracker.register(xhr, srh);
		}
	}
};

})();
