/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.process.web.client.projectarea");

(function() { // module container

var session					= com.ibm.team.repository.web.client.session;
var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

com.ibm.team.reports.web.client.internal.ReportsClient = {

	ALL: 0,
	SHARED: 1 << 0,
	PRIVATE: 1 << 2,
	EXCLUDE_MICRO: 1 << 4,
	
	SERVICE_URI: "com.ibm.team.reports.common.internal.service.IReportRestService",
	
	/**
	 * Fetch whether the current user can deploy queries.
	 */
	getCanDeployQuery: function(responseHandler) {
		this._invokeReportsService(responseHandler, "getCanDeployQuery"); 
	},
	
	/**
	 * Fetch whether the current user can manage folders.
	 */
	getCanManageFolder: function(responseHandler) {
		this._invokeReportsService(responseHandler, "getCanManageFolder"); 
	},
	
	/**
	 * Fetch whether the current user can see reports UI.
	 */
	 getCanSeeReports: function(responseHandler) {
		this._invokeReportsService(responseHandler, "getCanSeeReports"); 
	},
	
	/**
	 * Fetch all visible root folder descriptors. The client does not need to supply arguments.
	 * This method will supply the currently-authenticated user and current projectArea.
	 */
	getSystemRootFolderDescriptors: function(responseHandler) {
		this._invokeReportsService(responseHandler, "getSystemRootFolderDescriptors");
	},
	
	/**
	 * Fetch query descriptors. 
	 * This method will supply the projectAreaUUID.
	 */
	getQueryDescriptors: function(responseHandler, args) {
		if (args.projectAreaUUID == undefined) {
			var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			args.projectAreaUUID = currentProjectArea.itemId;
		}
		this._invokeReportsService(responseHandler, "getQueryDescriptors", args);
	},

	/**
	 * Fetch report descriptors. 
	 * This method will supply the projectAreaUUID.
	 */
	getReportDescriptors: function(responseHandler, args) {
		if (args.projectAreaUUID == undefined) {
			var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			args.projectAreaUUID = currentProjectArea.itemId;
		}
		this._invokeReportsService(responseHandler, "getReportDescriptors", args);
	},
	
	getReportDescriptor: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "getReportDescriptor", args);
	},

	/**
	 * Fetch folders. 
	 * This method will supply the projectAreaUUID.
	 */
	getFolderDescriptors: function(responseHandler, args) {
		if (args.projectAreaUUID == undefined) {
			var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			args.projectAreaUUID = currentProjectArea.itemId;
		}
		this._invokeReportsService(responseHandler, "getFolderDescriptors", args);
	},
	
	
	/**
	 * Fetch parameters
	 */
	getParameters: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "getParameters", args);
	},
	
	/**
	 * Fetch parameter values
	 */
	getParameterValues: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "getParameterValues", args);
	},
	
	/**
	 * Fetch parameter group values
	 */
	getParameterGroupValues: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "getParameterGroupValues", args);
	},
	
	/**
	 * Get the parameters section for the given query.
	 */
	getQueryParametersSection: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "getQueryParametersSection", args);
	},

	/**
	 * Get the rendered report for the given query.
	 */
	getRenderedQuery: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "getRenderedQuery", args);
	},

	/**
	 * Get the rendered report for the given query.
	 */
	postRenderQuery: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postRenderQuery", args);
	},
	
	/**
	 * Get the rendered report for the given report.
	 */
	getRenderedReport: function(responseHandler, args) {
		if (args.projectAreaUUID == undefined) {
			var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			args.projectAreaUUID = currentProjectArea.itemId;
		}
		this._invokeReportsService(responseHandler, "getRenderedReport", args);
	},
	
	/**
	 * Get the rendered report for the given report.
	 */
	postRenderReport: function(responseHandler, args) {
		if (args.projectAreaUUID == undefined) {
			var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			args.projectAreaUUID = currentProjectArea.itemId;
		}
		this._invokeReportsService(responseHandler, "postRenderReport", args);
	},
	
	/**
	 * Get whether the given report exists
	 */
	getDoesReportDescriptorExist: function(responseHandler, args) {
		if (args.projectAreaUUID == undefined) {
			var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			args.projectAreaUUID = currentProjectArea.itemId;
		}
		this._invokeReportsService(responseHandler, "getDoesReportDescriptorExist", args);
	},
	
	/**
	 * Get the default query for the authenticated contributor
	 */
	getDefaultQueryDescriptor: function(responseHandler) {
		var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		var args = {
			contributorUUID: session.getAuthenticatedContributor().itemId,
			projectAreaUUID: currentProjectArea.itemId
		};
		this._invokeReportsService(responseHandler, "getDefaultQueryDescriptor", args);
	},
	
	/**
	 * Get the default report for the project area
	 */
	getDefaultReportDescriptor: function(responseHandler) {
		var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		var args = {
			projectAreaUUID: currentProjectArea.itemId
		};
		this._invokeReportsService(responseHandler, "getDefaultReportDescriptor", args);
	},
	
	/**
	 * Get the query descriptor for the queryUUID provided
	 */
	getQueryDescriptor: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "getQueryDescriptor", args);
	},
	
	/**
	 * Delete the given folder.
	 */
	postDeleteFolderDescriptor: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postDeleteFolderDescriptor", args);
	},

	/**
	 * Delete the given query.
	 */
	postDeleteQueryDescriptor: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postDeleteQueryDescriptor", args);
	},

	/**
	 * Rename the given folder.
	 */
	postRenameFolderDescriptor: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postRenameFolderDescriptor", args);
	},

	/**
	 * Rename the given query.
	 */
	postRenameQueryDescriptor: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postRenameQueryDescriptor", args);
	},

	/**
	 * Save a folder with the given arguments. This could be creating a new
	 * one or saving an existing one. 
	 */
	postSaveFolderDescriptor: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postSaveFolderDescriptor", args);
	},
	
	/**
	 * Save a query with the given arguments. This could be creating a new
	 * one or saving an existing one. 
	 */
	postSaveQueryDescriptor: function(responseHandler, args) {
		if (args.projectAreaUUID == undefined) {
			var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			args.projectAreaUUID = currentProjectArea.itemId;
		}
		this._invokeReportsService(responseHandler, "postSaveQueryDescriptor", args);
	},
	
	/**
	 * Set the given query as the default query for its team area (or user)
	 */
	postSetDefaultQueryDescriptor: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postSetDefaultQueryDescriptor", args);
	},
	
	/**
	 * Get whether the current user is a data warehouse administrator
	 */
	getIsDWAdmin: function(responseHandler) {
		this._invokeReportsService(responseHandler, "getIsDWAdmin");
	},

	/**
	 * Get the list of snapshot names
	 */
	getAllSnapshotNames: function(responseHandler) {
		this._invokeReportsService(responseHandler, "getAllSnapshotNames");
	},

	/**
	 * Get the snapshot descriptor
	 */
	getSnapshotDescriptor: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "getSnapshotDescriptor", args);
	},

	/**
	 * Data Warehouse Administration functions.
	 */
	postUpdateAllSnapshotData: function(responseHandler) {
		this._invokeReportsService(responseHandler, "postUpdateAllSnapshotData");
	},
	
	postUpdateSnapshotData: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postUpdateSnapshotData", args);
	},
	
	postExportAllSnapshotData: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postExportAllSnapshotData", args);
	},
	
	postExportSnapshotData: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postExportSnapshotData", args);
	},
	
	postImportSnapshotData: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postImportSnapshotData", args);
	},

	postCreateAllSnapshotSchemas: function(responseHandler) {
		this._invokeReportsService(responseHandler, "postCreateAllSnapshotSchemas");
	},
	
	postCreateSnapshotSchema: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postCreateSnapshotSchema", args);
	},
		
	postUpdateParameterValues: function(responseHandler) {
		this._invokeReportsService(responseHandler, "postUpdateParameterValues");
	},

	postSaveQueryParameters: function(responseHandler, args) {
		this._invokeReportsService(responseHandler, "postSaveQueryParameters", args);
	},
	
	/**
	 * Invokes the requested method on the Reports REST service.
	 */
	_invokeReportsService: function(responseHandler, method, args) {
		// args is optional, so test whether or not it was sent
		var serviceRequest;
		if (arguments.length == 2) {
			serviceRequest = new ServiceRequest(
					this.SERVICE_URI,
					method);
		} else {
			serviceRequest = new ServiceRequest(
					this.SERVICE_URI,
					method,
					args);
		}
		TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	}
};

})();