/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * @author hyassine
 */
dojo.provide("com.ibm.team.scm.web.client.util");

dojo.require("dojox.string.Builder");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("dojo.date");
dojo.require("dojox.date.posix");
dojo.require("dojo.i18n");
dojo.require("net.jazz.ajax.ui.internal.PageRegistry");

(function() {

var checkNotNull			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative		= net.jazz.ajax.toolbox.lang.checkNonNegative;
var PageRegistry = net.jazz.ajax.ui.internal.PageRegistry;

com.ibm.team.scm.web.client.util= {

	_basicPrefix: "#action=com.ibm.team.scm.",
	
	//Action URI prefixes.
	_browseStreamURIPrefix: "#action=com.ibm.team.scm.browseStream&id=",
	_browseWorkspaceURIPrefix: "#action=com.ibm.team.scm.browseWorkspace&id=",
	_browseElementURIPrefix: "#action=com.ibm.team.scm.browseElement&workspaceItemId=",
	_viewComponentHistoryURIPrefix: "#action=com.ibm.team.scm.viewComponentHistory&workspaceItemId=",
	_viewItemHistoryURIPrefix: "#action=com.ibm.team.scm.viewItemHistory&workspaceItemId=",
	_viewChangeSetURIPrefix: "#action=com.ibm.team.scm.viewChangeSet&workspaceItemId=",
	_viewChangeSetItemURIPrefix: "#action=com.ibm.team.scm.viewChangeSetItem&workspaceItemId=",
	_viewSnapshotsURIPrefix: "#action=com.ibm.team.scm.viewSnapshots&workspaceItemId=",
	_showTeamChangesURIPrefix: "#action=com.ibm.team.scm.showTeamChanges",
	_contributorProfilePrefix: "#action=com.ibm.team.repository.viewUserProfile&itemId=",
	_workItemLocationPattern: /^itemName\/.*WorkItem\/(.*)$/,
	_getFileDownloadURIPrefix: "../../service/com.ibm.team.filesystem.service.internal.rest.IFilesystemContentService/",
	_workItemPrefix: "#action=com.ibm.team.workitem.viewWorkItem&id=",
	_defaultServicePrefix: "../../resource/",
	
	//Icons locations.
	_streamIconSrc: "../com.ibm.team.scm.web/ui/internal/view/browse/icons/stream_obj.gif",
	_workspaceIconSrc: "../com.ibm.team.scm.web/ui/internal/view/browse/icons/workspace_obj.gif",
	_componentIconSrc: "../com.ibm.team.scm.web/ui/internal/view/browse/icons/compont_obj.gif",
	_folderIconSrc: "../com.ibm.team.scm.web/ui/internal/view/browse/icons/fldr_obj.gif",
	_fileIconSrc: "../com.ibm.team.scm.web/ui/internal/view/browse/icons/file_obj.gif",
	_historyIconSrc: "../com.ibm.team.scm.web/ui/internal/view/browse/icons/history_view.gif",
	_snapshotIconSrc: "../com.ibm.team.scm.web/ui/internal/view/browse/icons/snapshot_obj.gif",
	
	///////////////////////////////////////////////////////////////////////////////////
	// URI Retrieval
	///////////////////////////////////////////////////////////////////////////////////
	
	//Return the URI for file content view/download.
	getFileDownloadURI: function(workspaceId, componentId, fileName, fileItemId, stateId) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		checkNotNull(fileName, "fileName");
		checkNotNull(fileItemId, "fileItemId");
		checkNotNull(stateId, "stateId");
		
		return this._getFileDownloadURIPrefix + encodeURIComponent(workspaceId) + "/" + encodeURIComponent(componentId) + "/" + encodeURIComponent(fileName) + "?" + "itemId=" + encodeURIComponent(fileItemId) + "&" + "stateId=" + encodeURIComponent(stateId);
	},
	
	//Return the URI for viewing a stream.
	getBrowseStreamURI: function(id) {
		checkNotNull(id, "id");

		return this._browseStreamURIPrefix + encodeURIComponent(id);
	},
	
	//Return the URI for viewing a workspace.
	getBrowseWorkspaceURI: function(id) {
		checkNotNull(id, "id");

		return this._browseWorkspaceURIPrefix + encodeURIComponent(id);
	},
	
	//Return the URI for viewing active change sets within a team.
	getShowTeamChangesURI: function() {
		return this._showTeamChangesURIPrefix;
	},
	
	//Return the URI for viewing a component, folder, or file.
	getBrowseElementURI: function(workspaceId, componentId, path) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		checkNotNull(path, "path");
		
		return this._browseElementURIPrefix + encodeURIComponent(workspaceId) + "&componentItemId=" + encodeURIComponent(componentId) + "&path=" + encodeURIComponent(path);
	},
	
	//Return the URI for viewing a component's history.
	getComponentHistoryURI: function(workspaceId, componentId, n) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		
		//If a paging size (n) is provided.
		if (n) {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&n=" + encodeURIComponent(n);
		}
		else {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId);
		}
	},
	
	//Return the URI for viewing older change sets within the history view.
	getComponentHistoryURIWithFirst: function(workspaceId, componentId, first, n) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		
		//If a paging size (n) is provided.
		if (n) {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&first=" + encodeURIComponent(first) +
			"&n=" + encodeURIComponent(n);
		}
		else {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&first=" + encodeURIComponent(first);
		}	
	},
	
	//Return the URI for viewing newer (more recent) change sets within the history view.
	getComponentHistoryURIWithLast: function(workspaceId, componentId, last, n) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		
		//If a paging size (n) is provided.
		if (n) {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&last=" + encodeURIComponent(last) +
			"&n=" + encodeURIComponent(n);
		}
		else {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&last=" + encodeURIComponent(last);
		}
	},
	
	//Return the URI for viewing a folder or file's history.
	getItemHistoryURI: function(workspaceId, componentId, oid) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		checkNotNull(oid, "oid");
		
		return this._viewItemHistoryURIPrefix + encodeURIComponent(workspaceId) + 
		"&componentItemId=" + encodeURIComponent(componentId) +
		"&oid=" + encodeURIComponent(oid);
	},
	
	//Return the URI for viewing a particular change set.
	getChangeSetURI: function(workspaceId, changeSetId, fromHistory) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(changeSetId, "changeSetId");
		checkNotNull(fromHistory, "fromHistory");
		
		return this._viewChangeSetURIPrefix + encodeURIComponent(workspaceId) + "&changeSetItemId=" + encodeURIComponent(changeSetId) + "&fromHistory=" + encodeURIComponent(fromHistory);
	},
	
	//Return the URI for viewing a particular change set with an anchor to a specific diff within the change set (diffId).
	getChangeSetURIWithLocation: function(workspaceId, changeSetId, fromHistory, diffId) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(changeSetId, "changeSetId");
		checkNotNull(fromHistory, "fromHistory");
		checkNotNull(diffId, "diffId");
		
		return this._viewChangeSetURIPrefix + encodeURIComponent(workspaceId) + "&changeSetItemId=" + encodeURIComponent(changeSetId) + "&fromHistory=" + encodeURIComponent(fromHistory) + "&diffId=" + encodeURIComponent(diffId);
	},
	
	//Return the URI for viewing an item within a change set (before and after states of files and folders).
	getViewChangeSetItemURI: function(workspaceId, componentId, itemType, itemId, stateId) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		checkNotNull(itemType, "itemType");
		checkNotNull(itemId, "itemId");

		if(stateId) {
			return this._viewChangeSetItemURIPrefix + encodeURIComponent(workspaceId) + "&componentItemId=" 
						+ encodeURIComponent(componentId) + "&itemType=" + encodeURIComponent(itemType)
						+ "&itemId=" + encodeURIComponent(itemId) + "&stateId=" + encodeURIComponent(stateId);
		}
		else {
			return this._viewFileByItemIdPrefix + encodeURIComponent(workspaceId) + "&componentItemId=" 
						+ encodeURIComponent(componentId) + "&itemType=" + encodeURIComponent(itemType)
						+ "&itemId=" + encodeURIComponent(itemId);
		}
	},
	
	//Return the URI for viewing snapshots of a workspace/stream 
	getViewSnapshotsURI: function(workspaceId) {
		checkNotNull(workspaceId, "workspaceId");
		
		return this._viewSnapshotsURIPrefix + encodeURIComponent(workspaceId);
	},
	
	//Create and return a DOM element for a contributor.
	createContributorElement: function(/* ContributorDTO */ contributor) {
		if (!contributor) {
			return document.createTextNode("<unknown>");
		}
		var anchor = document.createElement("a");
		anchor.href = this.getContributorURI(contributor);
		var text = document.createTextNode(contributor.name);
		anchor.appendChild(text);
		return anchor;
	},
	
	//Return the URI to use for a contributor.
	getContributorURI: function(/* ContributorDTO */ contributor) {
		return this._contributorProfilePrefix + contributor.itemId;
	},
	
	//Create and return a DOM element for a reason (e.g. a work item).
	createReasonElement: function(/* ReasonDTO */ reason) {
		var anchor= document.createElement("a");
		anchor.href = this.getReasonURI(reason);
		//anchor.title= reason.label;
		var text= document.createTextNode(this.restoreTags(reason.label));
		anchor.appendChild(text);
		return anchor;
	},
		restoreTags: function(value) {
		var rawText = value;
		rawText= rawText.replace(/\&gt;/gi,">");
		rawText= rawText.replace(/\&lt;/gi,"<");
		rawText= rawText.replace(/\&amp;/gi,"&");
		rawText= rawText.replace(/\&tilde;/gi,"~");
		rawText= rawText.replace(/\&quot;/gi,"\"");
		rawText= rawText.replace(/\&circ;/gi,"^");
		rawText= rawText.replace(/\&nbsp;/gi," ");
		rawText= rawText.replace(/<br\/>/gi,"\n");
		rawText= rawText.replace(/\&#09;/g, "\t");
		return rawText
	},
	//Return the URI to use for a reason (e.g. a work item).
	getReasonURI: function(/* ReasonDTO */ reason) {
		if (this.workItemsInstalled()) {
		    var match = this._workItemLocationPattern.exec(reason.location);
		    if (match) {
		    	return this._workItemPrefix + encodeURIComponent(match[1]);
		    }
		}
		return this._defaultServicePrefix + encodeURIComponent(reason.location);
	},
	
	createFlowTargetElement: function(stream) {
		var anchor = document.createElement("a");
		anchor.href = this.getBrowseStreamURI(stream.itemId);
		var img = document.createElement("img");
		img.src = this.getIconSrc("stream");
		img.className = "headerIcon";
		anchor.appendChild(img);
		var text = document.createTextNode(stream.name);
		anchor.appendChild(text);
		return anchor;
	},
	
	dashboardInstalled: function() {
		if (this._dashboardInstalled == undefined) {
			this._dashboardInstalled = PageRegistry.findPageWithId("com.ibm.team.dashboard") != null;
		}
		return this._dashboardInstalled;
	},
	
	workItemsInstalled: function() {
		if (this._workItemsInstalled == undefined) {
			this._workItemsInstalled = PageRegistry.findPageWithId("com.ibm.team.workitem") != null;
		}
		return this._workItemsInstalled;
	},	
	
	///////////////////////////////////////////////////////////////////////////////////
	// Other Utility Services
	///////////////////////////////////////////////////////////////////////////////////
	
	//Return the icon location path depending on the specified iconName parameter.
	getIconSrc: function(iconName){
		if(iconName == "stream") {
			return this._streamIconSrc;
		}
		else if(iconName == "workspace") {
			return this._workspaceIconSrc;
		}
		else if(iconName == "component") {
			return this._componentIconSrc;
		}
		else if(iconName == "folder") {
			return this._folderIconSrc;
		}
		else if(iconName == "file") {
			return this._fileIconSrc;
		}
		else if(iconName == "history") {
			return this._historyIconSrc;
		}
		else if(iconName == "snapshot") {
			return this._snapshotIconSrc;
		}
	},
	
	//Return a formatted date string given a date object.
	_getDateString: function(timestamp) {
		if(timestamp.length == 28) {
			timestamp = timestamp.substring(0,19) + "-" + timestamp.substring(24, 26) + ":" + timestamp.substring(26,28);
		}
		var dateObject = dojo.date.stamp.fromISOString(timestamp); 
	    var locale = dojo.i18n.normalizeLocale();
	    
		if (locale == "en-us") {
			var dateStr = dojox.date.posix.strftime(dateObject, "%b %e, %Y");
			dateStr += " ";
			var hour = dojox.date.posix.strftime(dateObject, "%I");
			dateStr += hour.charAt(0) !== "0" ? hour : hour.substring(1);
			dateStr += ":" + dojox.date.posix.strftime(dateObject, "%M") + " ";
			dateStr += dojox.date.posix.strftime(dateObject, "%p").toUpperCase();
			return dateStr;
		}
		else {
			return dojo.date.locale.format(dateObject, {formatLength: 'medium' });
		}
	},
	
	//Return an array containing the list of items constituting the specified path.
	parsePathIntoArray: function(path) {
		var subElements = [];
		var count = 1;
		var i = 0;
		var seperatorIndex = 0;

		while(count<path.length) {
			seperatorIndex = path.indexOf("/", count);
			//If a '/' seperator is found, extract the item prior to the seperator.
			if(seperatorIndex!=-1) {
				subElements[i] = path.slice(count, seperatorIndex);
				i++;
				count = seperatorIndex + 1;
			}
			//If no more '/' seperators remain, extract the last item and end the parsing process.
			else {
				subElements[i] = path.slice(count);
				count = path.length;
			}
		}
		
		return subElements;
	},
	
	//Parse a diff and return an array of the set of changes within the diff.
	//Each entry in the returned array contains a change beginning with @@ -#,# +#,# @@
	parseDiff: function(diff) {
		var changes = [];
		var count = 1;
		var i = 0;
		var regexIndex;
		
		count = diff.indexOf("@@", 0);
		while(count<diff.length && count!=-1) {
			regexIndex = diff.indexOf("@@", count+2);
			if(diff.indexOf("@@", regexIndex + 1)!=-1) {
				regexIndex = diff.indexOf("@@", regexIndex + 1);
				changes[i] = diff.slice(count, regexIndex);
				count = regexIndex;
				i++;
			}
			else {
				changes[i] = diff.slice(count, diff.length);
				count = diff.length;
			}
		}
		
		return changes;
	}	
};
})();
