/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
dojo.provide("com.ibm.rational.cm.web.ui.data.GridModel");

// 65684: dojo 1.2 migration
//dojo["require"]("dojox.grid._data.model");
dojo.require("dojox.grid.Grid");

dojo.require("com.ibm.rational.cm.web._base");

dojo.declare(
	"com.ibm.rational.cm.web.ui.data.GridModel",
	dojox.grid.data.DojoData,
{
	query: null, // override dojox.grid.data.DojoData
	clientSort: true,
	_executing: false,
	_fetchRequest: null,
	
	// RATLC01261833 : CQBeta7.1.2r: Falcon: Sort method for Current Page sorting should use the case sensitivity setting for the DB in use.
	// override comparator to use case-insensitive sort (default is case-sensitive)
	generateComparator: function(inCompare, inField, inTrueForAscend, inSubCompare){
		dojo["require"]("com.ibm.rational.cm.web.ui.view.common.sort");
		inCompare = com.ibm.rational.cm.web.caseInsensitiveSort;
		return this.inherited(arguments);
	},
		
	// use colData instead of first record
	_setupFields: function(colData){
		// abort if we already have setup fields
		if(this.fields._nameMaps){
			this.fields._nameMaps = null; // don't return so that field map will be updated
		}
		
		// set up field/index mappings
		var m = {};
		var fields = dojo.map(colData,
			function(item, idx){
				// after 2009C, item.field is field path and item.name is display name (RATLC01371849)
				// prior to 2009C, there is only item.field which is display name
				// always use item.field for both cases
				var fieldName = item.field;
				m[fieldName] = idx;
				m[idx+".idx"] = fieldName;
				// it says: name == display name, key = property name
				// in dojox.grid.data.DojoData but actually the same value is set for both name & key
				// and they doesn't seem to be distinguished...
				return { name:fieldName, key:fieldName };
			},
			this
		);

		this.fields._nameMaps = m;
		this.fields.set(fields);
		this.notify("FieldsChange");
	},
	
	// The dialog that created this grid has been cancelled.  Terminate the fetch request that is in progress.
	// Cancelling the fetch causes onError logic of the request to execute, so necessary cleanup completes.
	cancelFetch: function() {
		if (this.isExecuting()) {
			this._fetchRequest.abort();
		}
	},
	
	// override dojox.grid.data.DojoData so that
	// - use colData instead of first record
	// - row data to allow boolean false value
	// - performance improvement to calculates position/size after all rows are added
	processRows: function(items, store){
		if(!items){ return; }

		this._setupFields(this.store.colData);  // modified line
		
		var replaceAll = (store.start == 0 && (this.data.length <= items.length));

		dojo.forEach(items, function(item, idx){
			var row = {};
			row.__dojo_data_item = item;
			dojo.forEach(this.fields.values, function(a){
				row[a.name] = this.store.getValue(item, a.name);  // modified line so that false won't be converted to "" (needed for remotely mastered)
			}, this);
			this._rowIdentities[this.store.getIdentity(item)] = store.start+idx;
			if(replaceAll){
				this.data[store.start+idx] = row;
			}else{
				this.setRow(row, store.start+idx);
			}
		}, this);
		if(replaceAll){
			this.allChange();
		}
	},
	
	// override dojox.grid.data.DojoData
	// third argument is an optional parameter that would be passed as 'content' when calling XHR
	requestRows: function(inRowIndex, inCount, /* Object */ content){
		// somehow requestRows gets called twice in FF3 (because of grid#model#requestPage) and causing the following error
		// RATLC01363302 : two resultset limit dialog poped up in Firefox 3
		// RATLC01261434 : Load initial result set record causes query results to close immediately
		if(this._executing){ return; }
		this._executing = true;
										
		this.onBeforeFetch();
		var row  = inRowIndex || 0;
		var self = this;
		var request = {
			start: row,
			count: this.rowsPerPage,
			query: this.query,
			sort: this.sortFields,
			abort: function() {self.store.cancelRequest();},
			queryOptions: this.queryOptions,
			onBegin: dojo.hitch(this, "beginReturn"),
			onComplete: function(items, requestObject){
				self._executing = false;
				self._fetchRequest = null;
				self.processRows(items, requestObject) // add to deferred?
				self.onComplete(items, requestObject);
				self.onReturn(requestObject);
			},
			onError: function(errorData, requestObject){
				self._executing = false;
				self._fetchRequest = null;
				self.onError(errorData, requestObject);
				self.onReturn(requestObject);
			}
		}
		if(content){
			request.content = content;
		}
		// Save the request object in case the request must be aborted.     
		// See http://dojotoolkit.org/forum/dojo-core-dojo-0-9/dojo-core-support/how-cancel-store-fetch and	
		// http://trac.dojotoolkit.org/ticket/8315
		this._fetchRequest = this.store.fetch(request);	
	},
	
	isExecuting: function(){
		return this._executing;
	},
	
	// a stub function that can be connected
	onBeforeFetch: function(){},

	// a stub function that can be connected
	onComplete: function(items, requestObject){},
	
	onError: function(errorData, requestObject){
		console.debug("GridModel error: "+errorData);
	},
	
	// a stub function that can be connected
	// called for both success/error cases
	onReturn: function(requestObject){}
});