/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.BuildRequest");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.build.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.build.web.util.BuildUtil");
dojo.require("com.ibm.team.build.web.ui.internal.utils.WorkspaceSelectionDialog");
dojo.require("com.ibm.team.build.web.ui.internal.utils.UIHelper");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ResultUtil");
dojo.require("com.ibm.team.build.web.client.internal.BuildClient");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.util.html");

dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function(){

var BuildUtil = com.ibm.team.build.web.util.BuildUtil;
var UIHelper = com.ibm.team.build.web.ui.internal.utils.UIHelper;
var ResultUtil = com.ibm.team.build.web.ui.internal.utils.ResultUtil;
var BuildClient = com.ibm.team.build.web.client.internal.BuildClient;
var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ScmClient = com.ibm.team.scm.web.client.internal.ScmClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");
// placeholders that will be populated on the first postCreate
var buttonWidth;
var submitWidth;
var browseWidth;
var tableHeadingWidth;

dojo.declare("com.ibm.team.build.web.ui.internal.BuildRequest",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.build.web", "ui/internal/templates/BuildRequest.html"),
	buttonAddImageUrl:dojo.moduleUrl("com.ibm.team.build.web", "graphics/common/icons/add-button.gif"),
	widgetsInTemplate: true,
	
	constructor: function(args) {
		this.messages = messages;
	},
	
	postCreate: function() {
		this.metanav.populateBreadcrumbs();
		this._initButtonWidths();
		this.submitBottom.style.width = buttonWidth;
		this.browseWorkspace.style.width = browseWidth;
		this.submit = document.createElement("input");
		this.submit.type = "button";
		this.submit.className = "button";
		this.submit.value = messages.submitLabel;
		this.submit.style.width = submitWidth;
		this.submit.disabled = true;
		this.metanav.toolbar.addWidget({domNode: this.submit});
		if (this.definitionId) {
			var callback = dojo.hitch(this,this._handleBuildDefCallback);
			BuildClient.getDefinition({definitionItemId: this.definitionId, errback: callback, callback: callback, refreshTime: new Date().getTime() });
		} else if (this.resultId) {
			var callback = dojo.hitch(this,this._handleBuildResultCallback);
			BuildClient.getResultPresentation({resultItemId: this.resultId, errback: callback, callback: callback});
		}
		new jazz.ui.HoverPopup({around: this.personalControl, html: messages.personalBuildMessage, persist: false, openDelay: 1000});
		this.connect(this.submit,"onclick",dojo.hitch(this,this.sendRequest));
		this.connect(this.submitBottom,"onclick",dojo.hitch(this,this.sendRequest));
		this.connect(this.browseWorkspace,"onclick",dojo.hitch(this,this._browseWorkspace));
		this.connect(this.personalCheckbox,"onclick",dojo.hitch(this,this._onCheckboxChange));
		this.connect(this.toggleDescriptions,"onclick",dojo.hitch(this,this._toggleDescriptions));
	},
	
	_initButtonWidths: function() {
		if (!buttonWidth) {
			var testNode = document.createElement("div");
			testNode.style.position = "relative";
			document.body.appendChild(testNode);
			tableHeadingWidth = UIHelper.convertToEms(testNode, UIHelper.findMaxStringDisplayWidth([messages.nameHeading, messages.descriptionHeading],testNode)) + "em";
			buttonWidth = UIHelper.convertToEms(testNode, UIHelper.findMaxStringDisplayWidth([messages.addLabel, messages.removeLabel, messages.submitLabel],testNode, 61) + 12) + "em";
			submitWidth = UIHelper.convertToEms(testNode, UIHelper.findMaxStringDisplayWidth([messages.submitLabel],testNode, 61) + 12) + "em";
			browseWidth = UIHelper.convertToEms(testNode, UIHelper.findMaxStringDisplayWidth([messages.browseLabel],testNode, 61) + 12) + "em";
			document.body.removeChild(testNode);
		}
	},
	
	_handleBuildResultCallback: function(response) {
		if (!this.domNode)
			return;
		if (response instanceof Error) {
			ErrorUtil.handleErrorResponse(response, messages.resultNotFound);
		}
		else {
			this.result = response;
			this.definition = response.buildDefinition;
			this._populate();
		}
		PlatformUI.getWorkbench().hideMessageArea();
	},
	
	_handleBuildDefCallback: function(response) {
		if (!this.domNode)
			return;
		if (response instanceof Error) {
			ErrorUtil.handleErrorResponse(response, messages.definitionNotFound);
		}
		else {
			this.definition = response;
			this._populate();
		}
		PlatformUI.getWorkbench().hideMessageArea();
	},
	
	_populate: function() {
		this.metanav.populateBreadcrumbs(this.definition, this.result);
		var heading = dojo.string.substitute(messages.requestPageTitle, [this.definition.id]);
		this.metanav.setTitle(jazz.util.html.escape(heading));
		PlatformUI.getWorkbench().setTitle(dojo.string.substitute(messages.buildsTitle,[heading]));
		for (var i = 0; i < this.definition.configurationElements.length; i++) {
			if (this.definition.configurationElements[i].elementId == "com.ibm.team.build.jazzscm") {
				this.hasScmConfiguration = true;
				break;
			}
		}
		if (this.result)
			this.propertiesHeading.innerHTML = dojo.string.substitute(messages.propertiesHeadingCopied,[jazz.util.html.escape(ResultUtil.getLabel(this.result))]);
		this.defProperties = this._mapProperties(this.definition.properties);
		this.baseProperties = this.result && this.result.buildRequests ? this._mapProperties(this._getBuildRequest().buildDefinitionInstance.properties) : this.defProperties;
		this._populateProperties();
		if (this.hasScmConfiguration && this.result && this.result.buildResult.personalBuild) {
			this.personalCheckbox.checked = true;
			this.personalCheckbox.defaultChecked = true;
			this.personalWorkspace = "loading";
			this.workspaceField.value = messages.loading;
			dojo.addClass(this.workspaceField, "loading");
			this._loadWorkspace(this.baseProperties["team.scm.workspaceUUID"].value);
			this._onCheckboxChange();
		} else {
			this._validate();
		}
		if (!this.hasScmConfiguration)
			dojo.addClass(this.optionsSection,"hidden");
		dojo.removeClass(this.content, "hidden");
	},
	
	_loadWorkspace: function(workspaceId) {
		var serviceObject = {
			self: this,
			success: function(response) {
				this.self.personalWorkspace = response.workspace;
				this.self.workspaceField.value = response.workspace.name;
				dojo.removeClass(this.self.workspaceField, "loading");
				this.self._validate();
			},
			failure: function(error) {
				console.error(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		ScmClient.getWorkspace(srh, {itemId: workspaceId});
	},
	
	_getBuildRequest: function() {
		if (!this.result || !this.result.buildRequests)
			return null;
		for (var i = 0; i < this.result.buildRequests.length; i++) {
			if (this.result.buildRequests[i].buildAction.action == "com.ibm.team.build.action.requestBuild")
				return this.result.buildRequests[i];
		}
	},
	
	_populateProperties: function(properties) {
		this.properties = [];
		this.requiredProperties = [];
		var sortedbaseProperties = this._ascendingProperties(this.baseProperties);
		for (var i = 0; i < sortedbaseProperties.length; i++) {
			if (sortedbaseProperties[i].genericEditAllowed != false && sortedbaseProperties[i].name != "scheduledBuild") {
				var widget = this.addProperty(sortedbaseProperties[i]);
				if (sortedbaseProperties[i].required) {
					this.requiredProperties.push(widget);
				}
			}
		}
		if (this.properties.length == 0) {
			this.addProperty();
		}
		if (this.properties.length == 1) {
			this.properties[0].remove.disabled = this.properties[0].isEmpty();
			this._validate();
		}
		this._hiddenAllAddImagesExceptLastOne();
	},
	
	_mapProperties: function(array) {
		var result = {};
		for (var i = 0; i < array.length; i++) {
			result[array[i].name] = array[i];
		}
		return result;
	},
	
	removeProperty: function(widget) {
		if (this.properties.length == 1) {
			var currentPropertyWidget = this.properties[0];
			if (!currentPropertyWidget.isEmpty()) {
				//if the bottom property was not empty, clear it
				currentPropertyWidget.name.value = "";
				currentPropertyWidget.value.value = ""; 
				currentPropertyWidget.description.value = "";
				currentPropertyWidget.property.name = "";
				currentPropertyWidget.property.value = "";
				currentPropertyWidget.property.description = "";
			}
			// disable the remove button
			currentPropertyWidget.remove.disabled = true;
			this._validate();
			return;
		}
		for (var i = 0; i < this.properties.length; i++) {
			if (this.properties[i] == widget) {
				var node = widget.domNode.parentNode;
				if (node) {
					node.removeChild(widget.domNode);
				}
				this.properties.splice(i,1);
				widget.destroy();
				this._validate();
				
				//If remove the bottom property, set the current last 'add' button visible
				var lastWidget = this.properties[this.properties.length-1];
				dojo.removeClass(lastWidget.addRowButton,"hidden");
				dojo.disconnect(this.addButtonConnect);
				this.addButtonConnect = dojo.connect(lastWidget.addRowButton,"onclick",dojo.hitch(this,this._handleAddRow,lastWidget.addRowButton));
				this.showAddButton(lastWidget.addRowButton);
			}
		}
	},
	
	//handle add row from addRowButton.
	_handleAddRow: function(addButton) {
		var widget = this.addProperty();
		widget.name.focus();
		dojo.addClass(addButton,"hidden");
		dojo.disconnect(this.addButtonConnect);
		this.addButtonConnect = dojo.connect(widget.addRowButton,"onclick",dojo.hitch(this,this._handleAddRow,widget.addRowButton));
		this.showAddButton(widget.addRowButton);
		if(this.properties[0].remove.disabled == true){
			this.properties[0].remove.disabled = false;
		}
		this._validate();
	},
	
	showAddButton: function(addButton){
		addButton.style.filter = "";
		dojo.removeClass(addButton,"rightAddButtonDisable");
		dojo.addClass(addButton,"rightAddButton");
	},
	
	//hide all add row button, keep it after the last property.
	_hiddenAllAddImagesExceptLastOne: function(){
		for(var i = 0 ; i < this.properties.length ; i++){
			if(i == this.properties.length - 1){
				this.addButtonConnect = dojo.connect(this.properties[i].addRowButton,"onclick",dojo.hitch(this,this._handleAddRow,this.properties[i].addRowButton));	
			}else{
				dojo.addClass(this.properties[i].addRowButton,"hidden");	
			}
		}
	},
	
	addProperty: function(property) {
		var widget = new com.ibm.team.build.web.ui.internal.BuildRequestProperty({parent: this, property: property});
		this.properties.push(widget);
		this.propertiesBody.insertBefore(widget.domNode, this.addRow);
		return widget;
	},
	
	sendRequest: function() {
		PlatformUI.getWorkbench().showMessage(messages.sendingRequest);
		this.submit.disabled = true;
		this.submitBottom.disabled = true;
		
		var addRowButton = this.properties[this.properties.length - 1].addRowButton;
		addRowButton.style.filter = "alpha(opacity=30)";
		dojo.removeClass(addRowButton,"rightAddButton");
		dojo.addClass(addRowButton,"rightAddButtonDisable");
		
		addRowButton.disabled = true;
		this._submitRequestPending = true;
		BuildClient.requestBuild({
			postData: dojo.toJson(this._createRequestObject()),
			callback: dojo.hitch(this, this._handleSubmitCallback),
			errback: dojo.hitch(this, this._handleErrorCallback)
		});
	},
	
	_handleErrorCallback: function(response){
		ErrorUtil.handleErrorResponseAlert(response, messages.serviceNotFound, messages.requestFailed);
		if (this.domNode) {
			this._submitRequestPending = false;
			this._validate();
			PlatformUI.getWorkbench().hideMessageArea();
		}
	},
	
	_createRequestObject: function() {
		var result = {definition: this.definition.id, allowDuplicateRequests: true};
		var found = {};
		var modified = [];
		for (var i = 0; i < this.properties.length; i++) {
			var current = this.properties[i].getProperty();
			if (!current)
				continue;
			found[current.name] = true;
			if (!this.defProperties[current.name] || (this.defProperties[current.name].value != current.value && this.defProperties[current.name].value && current.value)) {
				modified.push(current);
			}
		}
		if (this.hasScmConfiguration && this.personalCheckbox.checked) {
			result.personalBuild = true;
			var workspace = this.baseProperties["team.scm.workspaceUUID"];
			workspace.value = this.personalWorkspace.itemId;
			modified.push(workspace);
		}
		var deleted = [];
		for (var i in this.defProperties) {
			if (!found[i] && this.defProperties[i].genericEditAllowed != false)
				deleted.push({name: i});
		}
		result.newOrModifiedProperties = modified;
		result.deletedProperties = deleted;
		return result;
	},
	
	_handleSubmitCallback: function() {
		BuildClient.flushResults({definitionItemId: this.definition.itemId});
		BuildClient.flushQueue({projectAreaItemId: getCurrentProjectArea().itemId});
		// only forward to the build queue if the user has not already browsed away
		if (this.domNode)
			location.href = BuildUtil.getDefinitionHref(this.definition.itemId);
	},
	
	_browseWorkspace: function() {
		new com.ibm.team.build.web.ui.internal.utils.WorkspaceSelectionDialog({
			onOk: dojo.hitch(this,this._onBrowseWorkspaceOk),
			selection: this.personalWorkspace ? this.personalWorkspace.itemId : null
		});
	},
	
	_onBrowseWorkspaceOk: function(selection) {
		this.personalWorkspace = selection[0];
		this.workspaceField.value = this.personalWorkspace.label;
		this._validate();
	},
	
	_onCheckboxChange: function(event) {
		if (this.personalCheckbox.checked) {
			dojo.removeClass(this.workspaceWrapper,"hidden");
		} else {
			dojo.addClass(this.workspaceWrapper,"hidden");
		}
		this._validate();
	},
	
	_validate: function() {
		// abort validation if there is a submit request in progress or if the widget has been destroyed (user navigated)
		if (this._submitRequestPending || !this.domNode)
			return;
		var errorMsg = "";
		var valid = !this.personalCheckbox.checked || (this.personalWorkspace && this.personalWorkspace != "loading") || !this.hasScmConfiguration;
		if(!valid){
			errorMsg = messages.noWorkSpace;
		}
		
		if (this.requiredProperties) {
			for (var i = 0; i < this.requiredProperties.length && valid; i++) {
				if (!this.requiredProperties[i].getValue()){
					valid = false;
					errorMsg = messages.noValueInRequiredProperties;
				}
			}
		}
		
		// check properties to make sure there is no duplicate or blank property name.
		if (this.hasProperties()) {
			for (i = 0; i < this.properties.length && valid; i++) {
				if (this.properties[i].getName() === ""){
					valid = false;
					errorMsg = messages.propertyHasBlankName;
				}
				for (var j = i + 1; j < this.properties.length && valid; j++){
					if (this.properties[i].getName() == this.properties[j].getName()){
						valid = false;
						errorMsg = messages.propertyExists;
					}
				}
			}
		}
		this.submit.disabled = !valid;
		this.submitBottom.disabled = !valid;
		
		var addRowButton = this.properties[this.properties.length - 1].addRowButton;
		this.showAddButton(addRowButton);
		addRowButton.disabled = !valid;
		// Show error message
		if(valid){
			this.metanav.clearMessage();
		} else{
			this.metanav.displayErrorMessage(errorMsg);
		} 
	}, 
	
	_toggleDescriptions: function(event) {
		dojo.stopEvent(event);
		if (dojo.hasClass(this.propertiesBody, "hidePropertiesDescriptions")) {
			dojo.removeClass(this.propertiesBody, "hidePropertiesDescriptions");
			this.toggleDescriptions.innerHTML = messages.hideDescriptions;
		} else {
			dojo.addClass(this.propertiesBody, "hidePropertiesDescriptions");
			this.toggleDescriptions.innerHTML = messages.showDescriptions;
		}
	},
	
	_ascendingProperties: function(properties){
		var sortedProperites = [];
		for (var i in properties) {
			sortedProperites.push(properties[i]);
		}
		return sortedProperites.sort(function(a,b) {
			return (a.name || "").localeCompare(b.name || "");
		});
	},
	
	// set the remove button available if name or value is being modified.
	// this function is to determine the status of remove button.
	inputOnChange: function(widget){
		widget.remove.disabled = !this.hasProperties();	
		this._validate();
	},
	
	hasProperties: function() {
		return this.properties && (this.properties.length > 1 || (this.properties.length == 1 && !this.properties[0].isEmpty()));
	}
	
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildRequestProperty",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.build.web", "ui/internal/templates/BuildRequestProperty.html"),
	addImageUrl:dojo.moduleUrl("com.ibm.team.build.web", "graphics/common/icons/add-button.gif"),
	constructor: function(args) {
		this.messages = messages;
	},
	
	postCreate: function() {
		this.remove.style.width = buttonWidth;
		this.topLabel.style.width = tableHeadingWidth;
		this.bottomLabel.style.width = tableHeadingWidth;
		if (!this.property)
			this.property = {};
		if (this.property.required) {
			this.valueHeading.innerHTML = messages.valueHeadingRequired;
			this.name.readOnly = true;
			new jazz.ui.HoverPopup({around: this.name, html: messages.requiredHoverName, persist: false});
			this.description.readOnly = true;
			new jazz.ui.HoverPopup({around: this.description, html: messages.requiredHoverDescription, persist: false});
			this.remove.disabled = true;
		}
		if (this.property.name)
			this.name.value = this.property.name;
		if (this.property.value)
			this.value.value = this.property.value;
		if (this.property.description)
			this.description.value = this.property.description;
		this.connect(this.name,"onkeyup",dojo.hitch(this.parent,this.parent.inputOnChange,this));
		this.connect(this.value,"onkeyup",dojo.hitch(this.parent,this.parent.inputOnChange,this));
		this.connect(this.description,"onkeyup",dojo.hitch(this.parent,this.parent.inputOnChange,this));
		this.connect(this.remove,"onclick",dojo.hitch(this.parent,this.parent.removeProperty,this));
	},
	
	getProperty: function() {
		if (!this.name.value)
			return null;
		var result = {name: this.name.value, value: this.value.value || ""};
		if (this.description.value)
			result.description = this.description.value;
		if (this.property.kind)
			result.kind = this.property.kind;
		return result;
	},
	
	getValue: function() {
		return this.value.value;
	},
	
	getName: function() {
		return this.name.value;
	},
	
	isEmpty: function() {
		return this.name.value.length == 0 
			&& this.value.value.length == 0
			&& this.description.value.length == 0;
	}	

});

})();
