/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.HeadlinesViewlet");

/*
 * Imports 
 */
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.HeadlinesClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "HeadlinesMessages");

(function() {
	
/*
 * Aliases 
 */
var HeadlinesClient = com.ibm.team.dashboard.viewlets.web.client.internal.HeadlinesClient;
var ResizeNotifier = jazz.util.ResizeNotifier;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

var SERVICE_IMG_ID = "com.ibm.team.dashboard.common.internal.service.IDashboardImageService";

var messages;

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.HeadlinesViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/HeadlinesViewlet.html"),
	
	_CLASSNAME: "com.ibm.team.dashboard.viewlets.web.ui.internal.HeadlinesViewlet",
	
	_STATE_PAGES: "pages",
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "HeadlinesMessages");
		this.noContent = messages.noContent;
		this.labelTitle = messages.labelTitle;
		this.labelTitleColor = messages.labelTitleColor;
		this.labelBody = messages.labelBody;
		this.labelImage = messages.labelImage;
		this.labelImageURL = messages.labelImageURL;
		this.labelImagePos = messages.labelImagePos;
		this.left = messages.left;
		this.right = messages.right;
		this.shiftLeft = messages.shiftLeft;
		this.shiftRight = messages.shiftRight;
	},

	init: function() {
		this._editContentsLink = I18nUtil.addMessage(this._editModeMessage, messages.noContentEdit)[0];
		
		// Instance Variables //--------------------------------------------------->
		this._colors = ["blue","gray","gold","green","teal","purple","orange"];
		this._editor = null;
		this._predefImagesArray = null;
		
		this._currentPageIndex = 0;
		
		// Load the display HTML
		this._loadDisplayArea();
		
		this.connect(this._editContentsLink,"onclick", this._doEdit);
		this.connect(this._shiftLeft, "onclick", this._doShiftLeft);
		this.connect(this._shiftRight, "onclick", this._doShiftRight);
		this.connect(this._predefImages, "onchange", this._doChangeImage);
		this.connect(this._imageURL,"onchange",this._doChangeImageCustom);
		
		// Resize events
		this._dashboardResizeEvent = ResizeNotifier.register({context: this, callback: this._updateHeight});
	},	
	
	_loadDisplayArea: function() {
		// Add content to display area
		this._updateDisplayAreaContent();
		
		// Update navigation links
		this._rebuildCounter();
		
		// Hide the editor area
		this._editorArea.style.display = "none";
		
		if (this._isEmpty()) {
			// Show the message area
			this._messageArea.style.display = "block";
			if (this._site.isDashboardEditing()) {
				this._editModeMessage.style.display = "inline";
			} else {
				this._editModeMessage.style.display = "none";
			}
			this._displayArea.style.display = "none";
			this._navArea.style.display = "none";
		} else {
			// Show the display area
			this._displayArea.style.display = "block";
			this._messageArea.style.display = "none";
			this._navArea.style.display = "block";
			this._updateNavigation();
		}
	},
	
	_updateDisplayAreaContent: function() {
		var pages = this.getViewletState().get(this._STATE_PAGES);
		if (!pages)
			return;
			
		if (!this._pageObjs) {
			this._pageObjs = new Array();
			var init = true;
		}
		for (var i = 0; i < pages.length; i++) {
			if (!init) {
				var pageObj = this._pageObjs[i];
				if (pageObj.dirty) {
					var oldNode = pageObj.node;
					var newNode = this._createPageNode(pages[i]);
					pageObj.dirty = false;
					this._displayArea.replaceChild(newNode, oldNode);
					pageObj.node = newNode;
				}
			} else {
				var pageDiv = this._createPageNode(pages[i]);
				this._displayArea.appendChild(pageDiv);
				this._pageObjs.push({node: pageDiv, dirty: false});
				var pageObj = this._pageObjs[i];
			}
			if (i == this._currentPageIndex) {
				pageObj.node.style.visibility = "visible";
			} else {
				pageObj.node.style.visibility = "hidden";
			}
		}
	},
	_createPageNode: function(page) {
		var node = document.createElement("div");
		node.className = "headline-page";
		var el = document.createElement("div");
		el.className="display-title";
		if (typeof page.titleColor != "undefined") {
			dojo.addClass(node,this._colors[page.titleColor]);
		}
		el.innerHTML = page.title;
		node.appendChild(el);
		el = document.createElement("div");
		el.className="display-text";
		if (page.image) {
			var cname = "display-image-right";
			if (page.imagePosition != undefined) {
				cname = page.imagePosition;
			}
			var img = document.createElement("img");
			if (page.image.indexOf("://") != -1) {
				img.src = page.image;
			}
			else {
				img.src = Util.getServiceURL(SERVICE_IMG_ID, null, {path: page.image});
			}
			img.className = cname;
			el.appendChild(img);
			var htmlContainer = document.createElement("div");
			htmlContainer.innerHTML = page.text;
			el.appendChild(htmlContainer);
			this.connect(img,"onload",this._updateHeight);
		} else {
			var htmlContainer = document.createElement("div");
			htmlContainer.innerHTML = page.text;
			el.appendChild(htmlContainer);
		}
		node.appendChild(el);
		return node;
	},
		
	_rebuildCounter: function() {
		this._removeAllChildren(this._navCounter);
		var pages = this.getViewletState().get(this._STATE_PAGES);
		if (pages==undefined)
			return;
		for (var i=0; i<pages.length; i++) {
			var link = document.createElement("a");
			var obj = {
				viewlet: this,
				index: -(i+2),
				func: function(event) {
					this.viewlet._doCounterNav(event, this.index);
				}
			};
			this.connect(link, "onclick", dojo.hitch(obj, obj.func));
			link.innerHTML=i+1;
			link.href = "#";
			this._navCounter.appendChild(link);
		}
		this._updateCounter();
	},
	_updateNavigation: function(leftNode, rightNode) {
		var pages = this.getViewletState().get(this._STATE_PAGES);
		if (!pages || pages.length < 2) {
			this._navArea.style.display = "none";
			this._shiftArea.style.display = "none";
		} else {
			this._navArea.style.display = "block";
			this._shiftArea.style.display = "block";		
		}
	},

	_doEdit: function(event) {
		dojo.stopEvent(event);
		this._cancelAnimation();
		this.getSite().editContents();
		this._scheduleAnimation();
	},
	
	_cancelAnimation: function() {
		if (this._timeoutEvent)
			clearTimeout(this._timeoutEvent);
		if (this._animQueued)
			clearTimeout(this._animQueued);
		this._timeoutEvent = null;			
		this._animQueued = null;
		dojo.removeClass(this._playPause,"pause");
		dojo.addClass(this._playPause,"play");
		this._playPause.title = messages.play;
	},
	_toggleAnimation: function(e){
		dojo.stopEvent(e);
		if (dojo.hasClass(this._playPause,"pause")){
			this._cancelAnimation();
		} else {
			this._scheduleAnimation();
		}
	},
	_doShiftLeft: function(event) {
		dojo.stopEvent(event);
		this._doShift(false);

	},
	
	_doShiftRight: function(event) {
		dojo.stopEvent(event);
		this._doShift(true);
	},
	
	_doShift: function(right) {
		if ((right && this._currentPageIndex == this._pageObjs.length - 1) || (!right && this._currentPageIndex == 0)) {
			return;
		}
		var pages = this.getViewletState().get(this._STATE_PAGES);
		var page = pages[this._currentPageIndex];
		var pageObj= this._pageObjs[this._currentPageIndex];
		this._pageObjs.splice(this._currentPageIndex, 1);
		pages.splice(this._currentPageIndex, 1);
		if (right) 
			this._currentPageIndex++;
		else
			this._currentPageIndex--;
		// Insert at a new position
		pages.splice(this._currentPageIndex, 0, page);
		this._pageObjs.splice(this._currentPageIndex, 0, pageObj);
		this._doNav(0);
	},

	_doNav: function(next) {
		var prevIndex = this._currentPageIndex;
		if (next!=0 && this.isEditing()) {
			this._updateCurrentPage();
		}
		if (next == 1) 
			this._currentPageIndex++;
		else 
			if (next == -1) 
				this._currentPageIndex--;
			else 
				if (next < -1) {
					// random access to a page
					this._currentPageIndex = -(next + 2);
				}
		if (this.isEditing()) {
			this._updateEditorAreaContent();
			this._updateNavigation(this._shiftLeft, this._shiftRight);	
		}
		else {
			this._updateDisplayAreaContent();
		}
		this._updateCounter();
		this._scheduleAnimation();
	},

	_updateHeight: function() {
		if (!this._pageObjs) {
			return;
		}
		var maxHeight = 0;
		for(var i = 0; i < this._pageObjs.length; i++) {
			var nodeHeight = this._pageObjs[i].node.clientHeight;
			if (nodeHeight > maxHeight) maxHeight = nodeHeight;
		}
		if (maxHeight == 0) {
			if (!this._heightLoops) this._heightLoops = 0;
			this._heightLoops++;
			if (this._heightLoops < 20)
				setTimeout(dojo.hitch(this,this._updateHeight),100);  //delay until viewlet is added to dom
		} else {
			this._displayArea.style.height = maxHeight + "px";	
			this._heightLoops = 0;
		}
	},

	_scheduleAnimation: function() {
		if (this.getPreference("slideshow")==false)
			return;
		if (this._getPageNumber()<2)
			return;
		this._cancelAnimation();
		if (!this.isEditing()) {
			var interval = this.getPreference("interval");
			if (interval <= 0 || interval > 30)
				interval = 3;
			dojo.addClass(this._playPause,"pause");
			dojo.removeClass(this._playPause,"play");
			this._playPause.title = messages.pause;
			this._timeoutEvent = setTimeout(dojo.hitch(this, this._loop), interval*1000);
		}
	},
	
	_getPageNumber: function() {
		var pages = this.getViewletState().get(this._STATE_PAGES);
		if (pages==undefined)
			return 0;
		return pages.length;
	},
	
	_loop: function () {
		//Check things happened between the animation was scheduled and now
		if (this.isEditing()||this.getPreference("slideshow")==false)
			return;
		this._timeoutEvent = undefined;
		this._doNextInLoop(true);
	},
	
	_doNextInLoop: function(forward) {
		var pages = this.getViewletState().get(this._STATE_PAGES);
		if (typeof pages == "undefined")
			return;
		var nextIndex;
		if (forward) {
			if (this._currentPageIndex == pages.length - 1) {
				// return
				nextIndex = 0;
			}
			else 
				nextIndex = this._currentPageIndex + 1;
		}
		else {
			// back
			if (this._currentPageIndex == 0) {
				// return
				nextIndex = pages.length-1;
			}
			else 
				nextIndex = this._currentPageIndex - 1;			
		}
		this._doNav(-(nextIndex+2));
		if (nextIndex == 0) this._cancelAnimation();		
	},
	
	_doCounterNav: function(event, index) {
		dojo.stopEvent(event);
		this._doNav(index);
	},
	
	_updateCounter: function() {
		var links = this._navCounter.childNodes;
		for (var i = 0; i < links.length; i++) {
			links[i].className = (i == this._currentPageIndex) ? "current" : "";
		}
	},
	
	_doAddPage: function(event) {
		if (event)
			dojo.stopEvent(event);
		var pages = this.getViewletState().get(this._STATE_PAGES);
		var newIndex = pages.length;
		var page = this._createPage();
		pages.push(page);
		var node = this._createPageNode(page);
		this._displayArea.appendChild(node);
		this._pageObjs.push({node: node, dirty: false});
		this._currentPageIndex = newIndex;
		this._doNav(0); 
		this._rebuildCounter();
	},
	
	_doRemovePage: function(event) {
		if(event)
			dojo.stopEvent(event);
		if (this._pageObjs.length == 1) {
			var makeNewPage = true;
		}
		if (!confirm(dojo.string.substitute(messages.confirmDelete, [this._currentPageIndex + 1]))) {
			return;
		}
		
		var pages = this.getViewletState().get(this._STATE_PAGES);
		if (this._pageObjs && this._pageObjs[this._currentPageIndex])
			this._displayArea.removeChild(this._pageObjs[this._currentPageIndex].node);
			this._pageObjs.splice(this._currentPageIndex, 1);
		if (pages && pages[this._currentPageIndex])
			pages.splice(this._currentPageIndex, 1);
		if (pages.length == 0) {
			this._doAddPage();
		} else {
			if (this._currentPageIndex > pages.length - 1)
				this._currentPageIndex = pages.length - 1;
			this._doNav(0);
			this._rebuildCounter();	
		}
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return boolean
	 * @overrides super
 	 */		
	validateEditedContent: function() {
		// Get the title and edited contents
		var title = new String(this._title.value);
		var content = new String(this._textArea.value);		
		// Check for insecure content
		if (jazz.util.html.hasScript(title) || jazz.util.html.hasScript(content)) {
			alert(messages.errorScript);
			return false;
		}
		return true;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 * @overrides super
 	 */		
	preSaveState: function() {
		// Save button was pressed in edit mode
		// Update the display HTML using the contents of the editor
		this._updateCurrentPage();
		this._saved = true;
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 * @overrides super
 	 */		
	update: function() {
		// Determine whether we are entering or leaving edit mode
		this._site.getActionArea().innerHTML = "";
		if (this.isEditing()) {
			this._ensureStateAvailable();
			// Load the editor area
			this._loadEditorArea();
			this._addActions();
		}
		else {
			// Leave
			// Load the display HTML
			if (!this._saved) {
				while (this._pageObjs && this._pageObjs.length > 0) {
					var pageObj = this._pageObjs.pop();
					this._displayArea.removeChild(pageObj.node);
				}
				this._pageObjs = null;
				this._currentPageIndex = 0;
			}
			this._loadDisplayArea();
			this._scheduleAnimation();
			this._saved = false;
			this._updateHeight();
		}
	},
	
	_ensureStateAvailable: function() {
		var pages = this.getViewletState().get(this._STATE_PAGES);
		if (!pages) {
			pages = new Array();
			pages[0] = this._createPage();
			this.getViewletState().set(this._STATE_PAGES, pages);
			if (!this._pageObjs) {
				this._pageObjs = new Array();
				var pageDiv = this._createPageNode(pages[0]);
				this._displayArea.appendChild(pageDiv);
				this._pageObjs.push({node: pageDiv, dirty: false});
			}
		}
	},
	
	_createPage: function() {
		var page = new Object();
		page.title = messages.defaultTitle;
		page.text = messages.defaultBody;
		page.titleColor = 0;
		return page;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_loadEditorArea: function() {
		// Add content to editor area
		this._loadPredefinedImages();
		this._updateEditorAreaContent();

		// Show the editor area
		this._editorArea.style.display = "";
		// Hide the display area
		this._displayArea.style.display = "none";
		// Hide the message area
		this._messageArea.style.display = "none";
		// Show the nav area
		this._navArea.style.display = "block";
		
		this._updateNavigation(this._shiftLeft, this._shiftRight);
	},
	
	_addActions: function() {
		var actionArea = this.getSite().getActionArea();
		var doc = actionArea.ownerDocument;
		
		var addPageWrapper = doc.createElement("div");
		addPageWrapper.className = "action add";
		var addPageLink = doc.createElement("a");
		addPageLink.innerHTML = messages.addPage;
		addPageLink.href = "#";
		addPageWrapper.appendChild(addPageLink);
		actionArea.appendChild(addPageWrapper);
		this.connect(addPageLink, "onclick", this._doAddPage);
		
		var deletePageWrapper = doc.createElement("div");
		deletePageWrapper.className = "action delete";
		var deletePageLink = doc.createElement("a");
		deletePageLink.innerHTML = messages.deletePage;
		deletePageLink.href = "#";
		deletePageWrapper.appendChild(deletePageLink);
		actionArea.appendChild(deletePageWrapper);
		this.connect(deletePageLink, "onclick", this._doRemovePage);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */          
	_createEditor: function() {
		// TODO: MP: Implement
	},		
	_selectColor: function(event) {
		if (!event.target) return;
		var className = event.target.className;
		switch(className) {
			case "blue-option": 
				var color = this.BLUE;
				break;
			case "gray-option":
				var color = this.GRAY;
				break;
			case "gold-option":
				var color = this.GOLD;
				break;
			case "green-option":
				var color = this.GREEN;
				break;
			case "teal-option":
				var color = this.TEAL;
				break;
			case "purple-option":
				var color = this.PURPLE;
				break;
			case "orange-option":
				var color = this.ORANGE;
				break;
		}
		if (typeof color == "undefined") return;
		dojo.addClass(event.target,"selected");
		if (this._editorSelectedColor) {
			dojo.removeClass(this._editorSelectedColor,"selected");	
		}
		this._editorSelectedColor = event.target;
		this._titleColor = color;
	},
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_updateCurrentPage: function() {
		var title = this._title.value;
		var text = this._textArea.value;
		var page = this.getViewletState().get(this._STATE_PAGES)[this._currentPageIndex];
		page.title = title;
		page.text = text;
		page.titleColor = this._titleColor;
		
		var index = this._predefImages.selectedIndex;
		if (index > 0 && this._imageURL.value.length == 0) {
			var url = this._predefImagesArray[index-1].url;
			page.image = url;
			page.imageId = this._predefImages.options[this._predefImages.selectedIndex].value;
		} else {
			page.imageId = null;
			if (this._imageURL.value.length>0)
				page.image = this._imageURL.value;
			else
				page.image = undefined;
		}
		if (this._imagePosition.selectedIndex >= 0) {
			page.imagePosition = this._imagePosition.options[this._imagePosition.selectedIndex].value;
		} else 
			page.imagePosition = undefined;
		if (this._pageObjs)
			this._pageObjs[this._currentPageIndex].dirty = true;
	},


	setDashboardEditing: function(editing) {
		if (editing) {
			this._editModeMessage.style.display = "inline";
		} else {
			this._editModeMessage.style.display = "none";
		}
	},
	
	_isEmpty: function() {
		var pages = this.getViewletState().get(this._STATE_PAGES);
		return (pages==undefined || pages.length==0);
	},
	_updateEditorAreaContent: function() {
		var pages = this.getViewletState().get(this._STATE_PAGES);
		var page = pages[this._currentPageIndex];
		this._title.value = page.title;
		this._textArea.value = page.text;
		if (this._editorSelectedColor)
			dojo.removeClass(this._editorSelectedColor,"selected");	
		dojo.addClass(this._colorPicker.childNodes[page.titleColor],"selected");
		this._editorSelectedColor = this._colorPicker.childNodes[page.titleColor];
		this._titleColor = page.titleColor;
		this._selectPredefinedImage(page);
		this._selectImagePosition(page.imagePosition);
	},
	
	_selectPredefinedImage: function(page) {
		if (page.imageId != undefined && page.imageId != null) {
			// predefined image - select in the combo
			var options = this._predefImages.options;
			for (var i = 0; i < options.length; i++) {
				options[i].selected = (options[i].value == page.imageId);
			}
			this._imageURL.value = "";
		}
		else {
			this._predefImages.selectedIndex = 0;
			this._imageURL.value = page.image!=undefined?page.image:"";
		}
		if (page.image) {
			var url = Util.getServiceURL(SERVICE_IMG_ID, null, {path: page.image});
			this._updateImagePreview(url);
		}
	},
	
	_selectImagePosition: function(position) {
		var options = this._imagePosition.options;
		for (var i=0; i<options.length; i++)
			options.selected = false;
		if (position) {
			for (var i=0; i<options.length; i++)
				options[i].selected = (options[i].value == position);			
		}
	},
	
	_updateImagePreview: function(src) {
		if (src!=undefined && src!=null) {
			this._imagePreview.style.visibility="visible";
			this._imagePreview.src = src;
		}
		else {
			this._imagePreview.style.visibility="hidden";
		}
	},
	
	_loadPredefinedImages: function() {
		this._removeAllChildren(this._predefImages);
		var noimage = document.createElement("option");
		noimage.innerHTML = ""; 
		noimage.value="none";
		this._predefImages.appendChild(noimage);
		var serviceObj = {
			viewlet: this,
			success: function(images) {
				var combo = this.viewlet._predefImages;	
				this.viewlet._predefImagesArray= images.images;			
				for (var i=0; i<images.images.length; i++) {
					var id = images.images[i].id;
					var name = images.images[i].name;
					combo.options[combo.options.length] = new Option(name, id);
				}
				var pages = this.viewlet.getViewletState().get(this.viewlet._STATE_PAGES);
				var page = pages[this.viewlet._currentPageIndex];				
				this.viewlet._selectPredefinedImage(page);
			},
			failure: function(errorObj) {
				this.viewlet.getSite().error(errorObj);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		HeadlinesClient.getHeadlineImages(srh, this.getServiceTracker());		
	},

	_doChangeImage: function(event) {
		if (this._predefImagesArray == null) return;
		var index = this._predefImages.selectedIndex;
		var src = null;
		if (index > 0){
			var url = this._predefImagesArray[index-1].url;
			src = Util.getServiceURL(SERVICE_IMG_ID, null, {path: url});
			this._imageURL.value = "";
		}
		this._updateImagePreview(src);
	},
	
	_doChangeImageCustom: function (event) {
		var src = null;
		if (this._imageURL.value.length > 0) {
			src = this._imageURL.value;
			this._predefImages.selectedIndex = 0;
		}
		this._updateImagePreview(src);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Node node
 	 * @return <none>
 	 */	
	_removeAllChildren: function(node) {
		// Remove all child nodes if any are defined
		while (node.childNodes.length > 0) {
			node.removeChild(node.firstChild);
		}
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @return <none>
 	 * @overrides super
 	 */	
	refresh: function() {
		this.getSite().doneRefresh();
		this._updateHeight();
		this._scheduleAnimation();
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param <none>
 	 * @return String
 	 */		
	toString: function() {
		return this._CLASSNAME;
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		if (newSettings.preferences.slideshow != oldSettings.preferences.slideshow) {
			this.update();
		}
	},
	
	setActive: function(active) {
		if (active) {
			// start the animation
			this._animQueued = setTimeout(dojo.hitch(this,this._scheduleAnimation), 1000);
		}
		else {
			// stop the animation
			this._cancelAnimation();
		}
	},
	destroy: function() {
		ResizeNotifier.unregister(this._dashboardResizeEvent);
		this.inherited(arguments);
	},
	
	BLUE: 0,
	GRAY: 1,
	GOLD: 2,
	GREEN: 3,
	TEAL: 4,
	PURPLE: 5,
	ORANGE: 6
});


})();

