/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedViewlet");

dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet");
dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.ui.SwapListDialog");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "JazzFeedMessages");

(function() {

/*
 * Aliases
 */
var FeedClient = com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient;
var FeedViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "JazzFeedMessages");

// 81030: Migrate event log preferences to new group ids
var MIGRATION_MAP = {
	"news": "com.ibm.team.feed.ui.internal.itemview.NewsFilter",
	"dashboard": "com.ibm.team.dashboard.eventGroup",
	"build": "com.ibm.team.build.internal.ui.dashboard.BuildResultChangeEventFilter",
	"changes": "com.ibm.team.feed.ui.internal.itemview.ChangesFilter",
	"stream": "com.ibm.team.feed.ui.internal.itemview.StreamsFilter",
	"process": "com.ibm.team.process.filter",
	"reports": "com.ibm.team.feed.ui.internal.itemview.ReportsFilter",
	"workItems": "com.ibm.team.feed.ui.internal.itemview.workitemFilter"
};

var migrateFilters = /* string[] */ function(/* string[] */ filters) {
	for (var i = 0; i < filters.length; i++) {
		var newFilter = MIGRATION_MAP[filters[i]];
		if (newFilter) {
			filters[i] = newFilter;
		}
	}
	return filters;
};

/*
 * A feed viewlet specific to Jazz feeds.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedViewlet",
	FeedViewlet, {
	
	constructor: function() {
		this.setColumnCount(2);
	},
	
	init: function() {
		this._updateTitle();
		this.inherited(arguments);
	},
	
	fetch: function (/* ServiceResponseHandler */ srh, /* number */ startIndex, /* number */ count, /* boolean */ entries, /* boolean */ details) {
		delete this._errorMsgNode;
		// handle Auto(none) scope when no team areas
		if (!this.getScopeItem()) {
			this._errorMsgNode = this._createNoScopeMsg();
			this._entryCount = 0;
			this._updateSecondaryTitle();
			srh.doCallback();
			return;
		}
		var serviceObj = {
			_srh: srh,
			_viewlet: this,
			_entries: entries,
			_details: details,
			success: function(/* FeedResponseDTO */ response) {
				this._viewlet.handleFetch(response, this._entries, this._details, this._srh);
			},
			error: function(/* Error */ error) {
				this._srh.doErrorCallback(error);
			}
		};
		var srh2 = new ServiceResponseHandler(serviceObj, "success", "error");
		var filters = this.getFilters();
		if (filters.length == 0)
			srh2.doCallback({entries:[],entryCount:0,itemCount:0,ref:null,refIndex:0});
		else
			FeedClient.getJazzFeed(this.getFilters(), this.getScopeItem().itemId, this.getScope(), this.getShowActiveBuilds(), this.getTagsKey(), this.getTags(), this.getScopeSubteams(), null, this.getReferenceItemId(), startIndex, count, entries, details, !this.getFeedInfo(), srh2, this.getServiceTracker());
	},
	
	getFilters: function() {
		return migrateFilters(eval(this.getPreference("filters")));
	},
	
	getNoItemsMessage: function() {
		return messages.noEntries;
	},
	
	getShowActiveBuilds: function() {
		return true;
	},

	getTagsKey: function() {
		return null;
	},

	getTags: function() {
		return "";
	},

	fetchTooltip: function(/* EntryDTO */ entry, /* ServiceResponseHandler */ srh) {
		FeedClient.getJazzFeed(this.getFilters(), this.getScopeItem().itemId, this.getScope(), this.getShowActiveBuilds(), this.getTagsKey(), this.getTags(), this.getScopeSubteams(), entry.items[0].linkURL, null, null, null, true, true, false, srh, this.getServiceTracker());
	},
	
	getPreferenceProvider: function(id) {
		if (this._provider == null)
			this._provider = new com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedFilterPreferenceProvider();
		return this._provider;
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var scopeChanged = false;
		var updateTitle = false;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			scopeChanged = true;
		}
		if (updateTitle || newSettings.userTitle != oldSettings.userTitle)
			this._updateTitle();
		if (scopeChanged || newSettings.scope != DashboardConstants.SCOPE_CONTRIBUTOR && newSettings.scopeSubteams != oldSettings.scopeSubteams
			|| oldSettings.preferences.filters != newSettings.preferences.filters) {
			this.reset(true);
		}
		else if (oldSettings.preferences.items != newSettings.preferences.items) {
			this.reset(false);
		}
		else {
			this.update();
			this.updateTotalPageFooter();
		}
	},
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var links = I18nUtil.addMessage(editSpan, messages.notScopedEdit);
			this.connect(links[0], "onclick", this._handleToggleSettings);
			msgSpan.appendChild(editSpan);
		}
		return msgSpan;
	},

	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},

	_updateTitle: function() {
		this.getSite().setTitle((this.getScopeItem() && this.getScopeItem().name) ? dojo.string.substitute(messages.titleScoped, [this.getScopeItem().name]) : messages.title);
	}
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedFilterPreferenceProvider", com.ibm.team.dashboard.web.ui.CustomPreferenceProvider, {
	
	_groups: null,
	_groupsRequested: false,
	
	constructor: function() {
		this._serviceTracker = new ServiceTracker();
	},
	
	getLabel: function(value){
		if (this._groups) {
			var filters = migrateFilters(eval(value));
			var groupsById = {};
			for (var i=0;i<this._groups.length;i++) {
				var group = this._groups[i];
				groupsById[group.id] = group;
			}
			var result = [];
			for (var i=0;i<filters.length;i++) {
				var group = groupsById[filters[i]];
				if (group) {
					result.push(group.name);
				}
			}
			if (result.length == 0)
				result.push(messages.noneSelected);
			return "<input type='text' readonly='readonly' value='" + result.join(", ") + "'/>";
		}
		if (!this._groupsRequested) {
			this._getGroups();
		}
		// not ready yet
		return null;
	},
	getLoadingLabel: function() {
		return messages.loading;
	},
	createChooser: function(onOk, settings, viewlet){
		var iconMap = {};
		var dialog = new jazz.ui.SwapListDialog({
			iconMap: iconMap,
			onOK: function(dialog) {
				var selected = dialog.getSelectedEntries();
				var segments = [];
				for (var i = 0; i < selected.length; i++)
					segments.push("\"" + selected[i].id + "\"");
				var result = "[" + segments.join(",") + "]";
				onOk(result, "filters");
			}
		});
		dialog.show();
		dialog.showLoading(true);
		
		var serviceObj = {
			_dialog: dialog,
			_iconMap: iconMap,
			_self: this,
			_srh: srh,
			success: function(/* object[] */ groups) {
				// triage groups into selected / unselected				
				var filters = migrateFilters(eval(settings.preferences.filters));
				var filterSet = this._self._createSet(filters);
				var selected = [];
				var unselected = [];
				for (var i=0;i<groups.length;i++) {
					var group = groups[i];
					var bucket = filterSet[group.id] ? selected : unselected;
					bucket.push(group);
				}
				// populate dialog
				var entries = this._self._createEntries(groups);
				this._self._populateIconMap(this._dialog.iconMap, groups);
				this._dialog.setSelectedEntries(this._self._createEntries(selected));
				this._dialog.setUnselectedEntries(this._self._createEntries(unselected));
				this._dialog.showLoading(false);
			},
			failure: function(/* Error */ error) {
				this._dialog.iconMap.error = dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "graphics/feeds/icons/error.gif").uri;
				this._dialog.setSelectedEntries([]);
				this._dialog.setUnselectedEntries([{label: messages.errorFetchingGroups, id: "error", value: "error", type: "error"}]);
				this._dialog.showLoading(false);
				console.debug(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		this._getGroups(srh);
	},
	_createEntries: function(/* object[] */ groups) {
		var entries = [];
		for (var i=0;i<groups.length;i++) {
			entries.push(this._createEntry(groups[i]));
		}
		return entries;
	},
	_createEntry: function(/* object */ group) {
		return { id: group.id, label: group.name, value: group.id, type: group.id };
	},
	_createSet: function(/* string[] */ array) {
		var set = {};
		for (var i=0;i<array.length;i++) {
			set[array[i]] = true;
		}
		return set;
	},
	_getGroups: function(/* ServiceResponseHandler */ srh) {
		if (this._groups ) {
			if (srh) {
				srh.doCallback(this._groups);
			}
		}
		else {
			var serviceObj = {
				_self: this,
				_srh: srh,
				success: function(/* object */ groups) {
					this._self._groups = groups;
					if (this._srh) {
						this._srh.doCallback(groups);
					}
				},
				failure: function(/* Error */ error) {
					if (this._srh) {
						this._srh.doErrorCallback(error);
					}
				}
			};
			var srh2 = new ServiceResponseHandler(serviceObj, "success", "failure");
			FeedClient.getEventGroups(srh2, this._serviceTracker);
			this._groupsRequested = true;
		}
	},
	_populateIconMap: function(/* object */ iconMap, /* object[] */ groups) {
		for (var i=0;i<groups.length;i++) {
			var group = groups[i];
			iconMap[group.id] = group.iconUrl;
		}
	}
});

})();
