/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.StatisticsViewlet");

/*
 * Imports 
 */
dojo.require("jazz.util.ResizeNotifier");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.ParameterNameChooserDialog");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.URLUtility");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.UUIDUtility");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.MetagroupsPreferenceProvider");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("jazz.app.proxy");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.QueryClient2");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");

dojo.require("dojo.i18n"); 
dojo.require("dojo.string");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");

dojo.require("jazz.ui.TagCloud");

dojo.requireLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "StatisticsMessages");

(function() {

/*
 * Observables to listen for viewlet log-in, to allow viewlets with C/ALM
 * queries to the same server to load as soon as one logs in.
 */
var globalLoginObservables = {};
	
/*
 * Aliases 
 */
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient2 = com.ibm.team.workitem.viewlets.web.ui.internal.QueryClient2;
var WorkItemQueryPreferenceProvider = com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider;
var URLUtility = com.ibm.team.workitem.viewlets.web.ui.internal.utils.URLUtility;
var UUIDUtility = com.ibm.team.workitem.viewlets.web.ui.internal.utils.UUIDUtility;
var ParameterNameChooserDialog = com.ibm.team.workitem.viewlets.web.ui.internal.utils.ParameterNameChooserDialog;
var MetagroupsPreferenceProvider = com.ibm.team.workitem.viewlets.web.ui.internal.utils.MetagroupsPreferenceProvider;
var ResizeNotifier = jazz.util.ResizeNotifier;
var WorkItemUtil = com.ibm.team.workitem.web.client.util;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var WorkItemAttributes = com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var WorkItemClient = com.ibm.team.workitem.web.client.internal.WorkItemClient;
var TagCloud = jazz.ui.TagCloud;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;

/*
 * Class 
 */
dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.StatisticsViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "ui/internal/templates/StatisticsViewlet.html"),

	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * @return StatisticsViewlet
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "StatisticsMessages");
		// Temporary
		// See Work Item: 94180
		this.workItemsMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

	},
	
	init: function() {

		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		
		this._queryPreferenceProvider = new WorkItemQueryPreferenceProvider(
				{ prefId: this._static._PREFERENCE_ID_QUERY,
				queryCategoryPrefId: this._static._PREFERENCE_ID_CATEGORY,
				suppressCrossRepoQueries: false });
		this._metagroupsPreferenceProvider = null;
		this._cachedChartParameters = null;
		this._currentQueryItemID = null;
		this._lastClientWidth = this._static._DEFAULT_CLIENT_WIDTH;
		this._lastClientHeight = this._static._DEFAULT_CLIENT_HEIGHT;
		this._excessParameterStartIndex = -1;
		
		// Window resize listener related variables
		this._windowResizeHandler = null;
		// Tables
		this._tableLabelElements = null;
		
		// Registries
		this._internalValueRegistry = new Array();
		this._iconURLRegistry = new Array();
		
		// Report node reference
		this._reportNode = null;
		
		// Tag Cloud
		this._tagCloud = null;

		// Work item attributes DTO
		this._attributesDTO = {};
		// DTO polling timeout value
		this._timeoutDTOCount = 0;
		
		// Animation
		this._doOpeningAnimation = true;
		
		// Pattern Groups Model
		this._patternGroups = null;
		// Pattern Results Model
		this._patternResults = null;
		
		// Drill-in related variables
		this._titleHyperlinkURL = null;
		this._serverRoot = null;
		this._imageServerRoot = null;
		this._serverURL = null;
		this._projectAreaName = null;
		
		// Local vs. Remote viewlet flag
		if (dojo.getObject("com.ibm.team.process.web.projectArea")) {
			this._local = true;
		} else {
			this._local = false;
		}

		// C/ALM query related variables
		this._skipOAuth = true;
		this._loginTracker = null;
		this._loginCallback = null;
		this._workItemQueryResultsHandler = null;

	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		// TODO: MP: BUG: Static variables are initialized on every 
		// instantiation

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

		this._static._CLASSNAME = "com.ibm.team.workitem.viewlets.web.ui.internal.StatisticsViewlet";

		this._static._PREFERENCE_ID_QUERY = "query";
		this._static._PREFERENCE_ID_PRESENTATION = "presentation";
		this._static._PREFERENCE_ID_PARAMETER = "parameter";
		this._static._PREFERENCE_ID_CATEGORY = "queryCategory";
		this._static._PREFERENCE_ID_METAGROUPS = "metagroups";
		this._static._PREFERENCE_ID_OTHERGROUP = "othergroup";
		
		this._static._PREFERENCE_VALUE_PIE = "pie";
		this._static._PREFERENCE_VALUE_COLUMN = "bar";
		this._static._PREFERENCE_VALUE_BAR = "h-bar";
		this._static._PREFERENCE_VALUE_TABLE = "table";
		this._static._PREFERENCE_VALUE_LIST = "list";
		
		this._static._PREFERENCE_VALUE_UNDEFINED = "-1";

		this._static._ICONS_URI = dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "graphics/statistics/icons/");

		this._static._PARAMETER_UNDEFINED = this.messages.msgUndefined;
		this._static._PARAMETER_OTHER = this.messages.msgOther;
		
		this._static._PARAMETER_ID_METAGROUP = "mg";

		// TODO: MP: Temporary workaround for 1.0.1.
		// 0: Unassigned
		// 1: Low
		// 2: Medium
		// 3: High
		this._static._COLOUR_RGB_PRIORITY = ["R240G240B240", "R196G220B245", "R250G234B187", "R255G191B191"];
		this._static._COLOUR_REGEX_PRIORITY = ["unassign", "low", "medium", "high"];
		
		// TODO: MP: Temporary workaround for 1.0.1.
		// 0: Unclassified
		// 1: Minor
		// 2: Normal
		// 3: Major
		// 4: Critical
		// 5: Blocker
		this._static._COLOUR_RGB_SEVERITY = ["R240G240B240", "R255G255B222", "R229G218B184", "R255G204B204", "R223G127B127", "R179G107B107"];
		this._static._COLOUR_REGEX_SEVERITY = ["unassign", "minor", "normal", "major", "critical", "blocker"];

		// TODO: MP: Temporary workaround for 1.0.1.
		// 0: Defect
		// 1: Enhancement
		// 2: Task
		// 3: RFS
		// 4: Retrospective
		// 5: Plan Item
		// 6: Track Build Item
		// 7: Story
		this._static._COLOUR_RGB_TYPE = ["R206G206B242", "R229G247B210", "R189G245B188", "R204G163B163", "R255G191B191", "R217G195B173", "R207G250B250", "R229G229B161"];
		this._static._COLOUR_REGEX_TYPE = ["bug", "enhanc", "task", "rfs", "risk", "planitem", "build", "story"];

		// TODO: MP: Make this a set of random colours to choose from to prevent duplicate colours
		this._static._COLOUR_RGB_UNDEFINED = "R216G227B230"; // Light Blue	  

		// Table Icons
		this._static._IMAGE_DEFAULT = "icon-default.gif";
		this._static._IMAGE_CATEGORY = "icon-category.gif";
		this._static._IMAGE_TAGS = "icon-tags.gif";
		this._static._IMAGE_OTHER = "icon-other.gif";
		this._static._IMAGE_METAGROUP = "icon-metagroup.gif";

		this._static._DEFAULT_CLIENT_WIDTH = 325; // pixels
		this._static._DEFAULT_CLIENT_HEIGHT = 162; // pixels (assuming pie chart ratio)
		
		this._static._LEGEND_ITEM_HEIGHT = 21;  // pixels
		
		this._static._CHART_SCALE_RATIO = 0.95;
		this._static._CHART_MINIMUM_WIDTH = 120;  // Same as charting engine
		this._static._CHART_MAXIMUM_WIDTH = 2800; // Flash movie clip height, width limit is 2880 pixels	
		this._static._CHART_MINIMUM_HEIGHT = 100; 
		this._static._CHART_MAXIMUM_HEIGHT = 2800; // Flash movie clip height, width limit is 2880 pixels	
		this._static._CHART_HEIGHT_SCALE_RATIO_PIE_CHART = 2;
		this._static._CHART_HEIGHT_SCALE_RATIO_COLUMN_CHART = 1.5;	
		this._static._CHART_HEIGHT_SCALE_RATIO_BAR_CHART = 1.5;	
		this._static._CHART_BAR_HEIGHT_FACTOR = 26; // Pixels	
		this._static._CHART_BAR_HEIGHT_BASE = 25; // Pixels	
		
		// Table
		this._static._MAX_TABLE_DISPLAY_ITEMS = 11;			// Includes "Other"
		this._static._MAX_TABLE_DISPLAY_ITEMS_ZOOMED = 201;	// Includes "Other"
		
		// Palette types
		this._static._PALETTE_TYPE_FIXED = 1;
		this._static._PALETTE_TYPE_RANDOM = 2;
		// Colours
		this._static._RGB_COLOUR_GREY = "R229G229B229";

		// JavaScript Array to Flash var delimiter
		this._static._JS_ARRAY_DELIMITER = "|";

		// Jazz Flash Chart SWF file path
		this._static._SWF_PATH = dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "ui/internal/flash/JFChart.swf");
		// Jazz Flash Chart onclick handler function name
		this._static._ONCLICK_HANDLER_NAME = "com.ibm.team.workitem.viewlets.web.ui.internal.StatisticsViewlet.dispatchEventOnClickFlashChart";		
		
		// DTO in progress status
		this._static._DTO_IN_PROGRESS = "progress";
		// DTO reception failure
		this._static._DTO_FAILURE = "error";
		// DTO polling timeout limit
		this._static._DTO_TIMEOUT_LIMIT = 40;  // 10 s (polling interval 250 ms)
		
		// C/ALM query login property 
		this._static._CALM_QUERY_LOGIN = "login";
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_turnOffAnimationFlashVar: function() {
		// Get flashvars DOM object	
		var flashVarsObject = dojo.byId(this._getFlashVarsDomID());
		if (!flashVarsObject) {
			return;
		}
		// Turn off the opening animation directly in the DOM Object parameter
		// Required to prevent bug:
		// 87991: Work Item Statistics Viewlet:  Flash object reloads unexpectedly on FF	
		var oldValue = flashVarsObject.value;
		var newValue = oldValue.replace("_fanimation=true", "_fanimation=false");
		flashVarsObject.value = newValue;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Number index
 	 * @return void
 	 */	
	_setExcessParameterStartIndex: function(index) {
		this._excessParameterStartIndex = index;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getExcessParameterStartIndex: function() {
		return this._excessParameterStartIndex;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array - Element values
 	 */	
	_getTableLabelElements: function() {
		return this._tableLabelElements;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_resetTableLabelElements: function() {
		this._tableLabelElements = new Array();
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element element
 	 * @return void
 	 */	
	_addTableLabelElement: function(element) {
		this._tableLabelElements.push(element);
	},		
	
	/*
	 * Private Method
	 * 
	 * @param Number ratio
	 * @return Object
	 * 			. Number width
	 * 			. Number height
	 */	
	_getPreferredChartDimensions: function(ratio) {
		// Create the container
		var container = new Object();
		// Get the current width of the viewlet
		var width = this._getClientWidth();
		// Constrain width to acceptable boundaries
		var preferredWidth = this._getPreferredWidth(width);

		// Calculate height
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		var chartParameters = this._getCachedChartParameters();
		
		var height;
		var proportionalHeight = Math.round(preferredWidth / ratio);
		if (chartParameters && 
			(presentation === this._static._PREFERENCE_VALUE_BAR)) {
			// Adjust height proportionally to the number of parameter groups
			height = (chartParameters.parameterNames.length * this._static._CHART_BAR_HEIGHT_FACTOR) +
					 this._static._CHART_BAR_HEIGHT_BASE;
			var displayAllGroups = this.getPreference(this._static._PREFERENCE_ID_OTHERGROUP);	

			if ((this.isZoomed() === false) &&
				(!displayAllGroups) && 
				(height > proportionalHeight)) {
				// Restrict height only in the following case:
				// * Unzoomed state
				// * Other grouping on
				// * Preferred height exceeds proportional height
				height = proportionalHeight;
			}
		} else {
			// Adjust height proportionally to width
			height = proportionalHeight;
		}
		
		// Calculate preferred height
		var preferredHeight = this._getPreferredHeight(height);
		
		// Set the width
		container.width = preferredWidth;
		// Set the height
		container.height = preferredHeight;

		return container;
	},

	/*
 	 * @param Number width
 	 * 
 	 * @return Number
 	 */	
	_getPreferredWidth: function(width) {
		var preferredWidth = width;
		if (width < this._static._CHART_MINIMUM_WIDTH) {
			preferredWidth = this._static._CHART_MINIMUM_WIDTH;
		} else if (width > this._static._CHART_MAXIMUM_WIDTH) {
			preferredWidth = this._static._CHART_MAXIMUM_WIDTH;
		}
		return preferredWidth;
	},	

	/*
 	 * @param Number height
 	 * 
 	 * @return Number
 	 */	
	_getPreferredHeight: function(height) {
		var preferredHeight = height;
		if (height < this._static._CHART_MINIMUM_HEIGHT) {
			preferredHeight = this._static._CHART_MINIMUM_HEIGHT;
		} else if (height > this._static._CHART_MAXIMUM_HEIGHT) {
			preferredHeight = this._static._CHART_MAXIMUM_HEIGHT;
		}
		return preferredHeight;	
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Number height
 	 * @return Number
 	 */	
	_getMaxLegendItems: function(height) {
		// Note:  This calculation is only valid if the legend item font size
		// specified on the server to BIRT is 7.0 pt
		var maxLegendItems = Math.round(height / this._static._LEGEND_ITEM_HEIGHT);
		return maxLegendItems;
	},		
	
	/*
 	 * Note:  A direct reference to the report node is now used instead of
 	 * retrieving the node by ID
 	 * 
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getReportDomID: function() {
		return "report:" + this.id;	
	},		

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getFlashVarsDomID: function() {
		return "vars:" + this.id;	
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getMaxTableLabelWidth: function() {
		// All units are in pixels
		var viewletAreaWidth = this._getClientWidth();
		// Matches property in "label-entry" class
		var columnWidthPercentage = 0.7; // 70%
		var labelColumnWidth = Math.round(viewletAreaWidth * columnWidthPercentage);
		var margins = 20;
		var imageWidth = 16;
		var maxWidth = labelColumnWidth - margins - imageWidth;
		
		return maxWidth;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_updateTableLabelElementWidths: function() {
		// Get the elements
		var elements = this._getTableLabelElements();
		// Ensure we have elements
		if (elements == null) {
			return;
		}		
		// Get the maximum table label length
		var maxWidth = this._getMaxTableLabelWidth();

		// Update their lengths
		for (var i = 0; i < elements.length; i++) {
			var element = elements[i];
			element.style.width = maxWidth + "px";
		}		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_disconnectWindowResizeHandler: function() {
		// Disconnect the previous handler (if any)
		if (this._windowResizeHandler) {
			ResizeNotifier.unregister(this._windowResizeHandler);
			this._windowResizeHandler = null;
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_registerTableResizeHandler: function() {
		// Clear previous elements (if any)
		this._resetTableLabelElements();
		// Disconnect the previous handler (if any)
		this._disconnectWindowResizeHandler();
		// Connect a new resize handler
		this._windowResizeHandler = ResizeNotifier.register({context: this, callback: this._updateTableLabelElementWidths}); 
	},	

	/*
 	 * @param Object chartParameters
 	 * 
 	 * @return void
 	 */		
	_createTable: function(chartParameters) {
		// Register the table resize handler
		this._registerTableResizeHandler();
		
		// Regroup data to fit the viewlet area
		var regroupedChartParams = this._regroupDataToFitViewletArea(
			chartParameters, 
			this._getMaxTableItems());
		
		// Get the attach point
		var node = this._contentArea;
		// Create the table
		var table = document.createElement("table");
		table.setAttribute("id", this._getReportDomID());
		table.setAttribute("cellspacing", "1");
		// Create the table body container
		var tbody = document.createElement("tbody");
		// Create the table body
 		for (var i = 0; i < regroupedChartParams.parameterCount; i++) {
			// Get the name
			var name = regroupedChartParams.parameterNames[i];
			// Get the value
			var value = regroupedChartParams.parameterValues[i];
			// Create the table row
			this._createTableRow(tbody, name, value, regroupedChartParams.parameterID);
		}
		this._removeLastTableRowBorder(tbody);

		// Create the table header
		var thead = this._createTableHeader(regroupedChartParams);
 		// Assemble table contents
 		table.appendChild(thead);
 		table.appendChild(tbody);
		
		this._addReportNode(node, table);

		// Update the table label widths
		this._updateTableLabelElementWidths();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element tbody
 	 * @return void
 	 */	
	_removeLastTableRowBorder: function(tbody) {
		// Remove the border from the last table row
		var lastRow = tbody.lastChild;
		// Column #1
		if (this._isObjectDefined(lastRow.firstChild, false, null)) {
			dojo.removeClass(lastRow.firstChild, "bottom-line");
		}
		// Column #2
		if (this._isObjectDefined(lastRow.lastChild, false, null)) {
			dojo.removeClass(lastRow.lastChild, "bottom-line");
		}	
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * @return Element
 	 */		
	_createTableHeader: function(chartParameters) {
		// Create the table head container
		var thead = document.createElement("thead");
		
		// Row #1
		var row1 = document.createElement("tr");
		// Table head 1
		var row1_th1 = document.createElement("th");
		dojo.addClass(row1_th1, "header-topline");
		var line1 = document.createElement("div");
		dojo.addClass(line1, "topline");
		row1_th1.appendChild(line1);
		// Table head 2
		var row1_th2 = document.createElement("th");	
		dojo.addClass(row1_th2, "header-topline");
		var line2 = document.createElement("div");
		dojo.addClass(line2, "topline");
		row1_th2.appendChild(line2);
		// Assemble contents
		row1.appendChild(row1_th1);
		row1.appendChild(row1_th2);		
		
		// Row #2
 		// Create the table header
		var row2 = document.createElement("tr");
		// Table head 1
		var row2_th1 = document.createElement("th");
		dojo.addClass(row2_th1, "parameter-header");
		// Table head 2
		var row2_th2 = document.createElement("th");	
		dojo.addClass(row2_th2, "count-header");
		// Create headings
		var header1 = document.createTextNode(chartParameters.displayName);
		var header2 = document.createTextNode(this.messages.msgCount);
		// Assemble table header contents
		row2_th1.appendChild(header1);
		row2_th2.appendChild(header2);
		row2.appendChild(row2_th1);
		row2.appendChild(row2_th2);
		
		thead.appendChild(row1);
		thead.appendChild(row2);
		
		return thead;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element tbody
 	 * @param String name
 	 * @param String value
 	 * @param String parameterID
 	 * @return void
 	 */		
	_createTableRow: function(tbody, name, value, parameterID) {
 		// Row
		var tr = document.createElement("tr");
		// Column 1
		var td1 = document.createElement("td");
		dojo.addClass(td1, "label-entry");
		dojo.addClass(td1, "bottom-line");
		// Column 2
		var td2 = document.createElement("td");
		dojo.addClass(td2, "count-entry");
		dojo.addClass(td2, "bottom-line");
		// Icon
		var img = document.createElement("img");
		var iconURL = this._getIconURL(parameterID, name);
		img.src = iconURL;
		img.width = "16";
		img.height = "16";
		img.alt = this.messages.msgIcon;
		// Name
		var link = this._createTableClickThroughLink(name, value);
		var span1 = document.createElement("span");
		dojo.addClass(span1, "label");
		this._addTableLabelElement(span1);
		span1.appendChild(link);
		// Value
		var data2 = document.createTextNode(value);
		var span2 = document.createElement("span");
		span2.appendChild(data2);		
		// Assemble contents
		td1.appendChild(img);
		td1.appendChild(span1);
		td2.appendChild(span2);
		tr.appendChild(td1);
		tr.appendChild(td2);
		tbody.appendChild(tr);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String name
 	 * @param String value
 	 * @return Element
 	 */	
	_createTableClickThroughLink: function(name, value) {
		// Name
		var data1 = document.createTextNode(name);		
		// Create click through link on name
		var link = document.createElement("a");
		link.setAttribute("title", value);
		link.appendChild(data1);
		// Create a reference to this object to use in the service object
		var self = this;		
		// Handle click events
		this.connect(link, "onclick", 
			function() {
				var label = name;
				var queryItemID = self._getQueryItemID();
				// Get chart parameters
				var chartParameters = self._getCachedChartParameters();
				// Validate chart parameters
				if (chartParameters == null) {
					// This should never happen
					self._alertErrorMessage(
						self.messages.msgDrillChartParamsUndef, null);
					return;
				}
				// Get the parameter ID
				var parameterID = chartParameters.parameterID;
				// Get the filtered work item query results
				// TODO: MP: Show error based on return status?
				self._getWorkItemQueryFilteredResults(label, queryItemID, parameterID);
			}
		);
		
		return link;				
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * 
 	 * @return void
 	 */		
	_createTagCloud: function(chartParameters) {

		// Clean-up previous cloud
		if (this._tagCloud) {
			this._tagCloud.destroy();
		}

		// Create tags
		var tags = [];
		for (var index = 0; index < chartParameters.parameterNames.length; index++) {
			tags[index] = {
				name: chartParameters.parameterNames[index],
				value: chartParameters.parameterValues[index]
			};
		}

		// Create cloud
		this._tagCloud = new TagCloud({
			tags: tags, 
			onTagClick: dojo.hitch(this, this._handleEventTagClick)
		});		

		// Embed in container
		var container = document.createElement("div");
		container.id = this._getReportDomID();
		container.appendChild(this._tagCloud.domNode);

		this._addReportNode(this._contentArea, container);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param TagCloud tagCloud
 	 * @param String tagName
 	 * 
 	 * @return void
 	 */		
	_handleEventTagClick: function(tagCloud, tagName) {

		var chartParameters = this._getCachedChartParameters();
		// Validate chart parameters
		if (!chartParameters) {
			// This should never happen
			this._alertErrorMessage(this.messages.msgDrillChartParamsUndef, null);
			return;
		}
		// Get the filtered work item query results
		this._getWorkItemQueryFilteredResults(tagName, 
											  this._getQueryItemID(), 
											  chartParameters.parameterID);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Element targetParent
 	 * @param Element newReportNode
 	 * @return void
 	 */		
	_addReportNode: function(targetParent, newReportNode) {
		// Get the old report node (if any)
		var oldReportNode = this._reportNode;		
		
		if (!oldReportNode) {
			// To be safe, remove all children to prevent duplicate report nodes
			this._removeAllChildren(targetParent);
			// Add new report node
			targetParent.appendChild(newReportNode);
		} else {
			// Replace old report node with new report node
			targetParent.replaceChild(newReportNode, oldReportNode);
		}

		// Set new report node
		this._reportNode = newReportNode;
		// Unset the loading message and show the content area
		this._displayLoadingMessage(null);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getCurrentWidth: function() {
		if (this._isObjectDefined(this.domNode, false, "domNode") == false) {
			return 0;
		} else if (this._hasProperty(this.domNode, "clientWidth", false, "domNode") == false) {
			return 0;
		}
		
		return this.domNode.clientWidth;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getCurrentHeight: function() {
		if (this._isObjectDefined(this.domNode, false, "domNode") == false) {
			return 0;
		} else if (this._hasProperty(this.domNode, "clientHeight", false, "domNode") == false) {
			return 0;
		}

		return this.domNode.clientHeight;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getClientWidth: function() {
		// Calculate client width
		// Use content area width (target)
		var clientWidth = Math.round(this.getSite().getWidth() *
			this._static._CHART_SCALE_RATIO);
		// Validate the width
		if (clientWidth <= 0) {
			// Use the current width
			clientWidth = Math.round(this._getCurrentWidth() *
				this._static._CHART_SCALE_RATIO);
		}
		// Validate the width
		if (clientWidth <= 0) {
			// Use the last valid width
			clientWidth = this._lastClientWidth;
		} else {
			// Set as the last valid width
			this._lastClientWidth = clientWidth;
		}

		return clientWidth;
	},	

	/*
 	 * DEAD CODE (at the moment)
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getClientHeight: function() {
		// Calculate client height
		// Use content area height (target)
		var clientHeight = Math.round(this.getSite().getHeight() *
			this._static._CHART_SCALE_RATIO);
		// Validate the height
		if (clientHeight <= 0) {
			// Use the current height
			clientHeight = Math.round(this._getCurrentHeight() *
				this._static._CHART_SCALE_RATIO);
		}
		// Validate the height
		if (clientHeight <= 0) {
			// Use the last valid height
			clientHeight = this._lastClientHeight;
		} else {
			// Set as the last valid height
			this._lastClientHeight = clientHeight;
		}
		
		return clientHeight;
	},	

	/*
 	 * @param String parameterName
 	 * 
 	 * @return String
 	 */	
	_getIconURL: function(parameterID, parameterName) {
		// TODO: MP: Add all hardcoded images to the registry on initial filtering
		if (parameterName in this._iconURLRegistry) {
			// Severity, Priority, Resolution, Status, Type, Owned By, Created By, etc.
			return this._iconURLRegistry[parameterName];
		} else if (parameterName === this._static._PARAMETER_OTHER) {
			// Other
			return this._static._ICONS_URI + this._static._IMAGE_OTHER;
		} else if (parameterID === WorkItemAttributes.EXT_CATEGORY) {
			// Category
			return this._static._ICONS_URI + this._static._IMAGE_CATEGORY;
		} else if (parameterID === WorkItemAttributes.EXT_TAGS) {
			// Tags
			return this._static._ICONS_URI + this._static._IMAGE_TAGS;
		}
		// Default
		return this._static._ICONS_URI + this._static._IMAGE_DEFAULT;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showMessageArea: function() {
		// Hide the content area
		this._updateVisibility(this._contentArea, false);
		// Show the message area		
		this._updateVisibility(this._messageArea, true);		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showContentArea: function() {
		// Hide the message area
		this._updateVisibility(this._messageArea, false);	
		// Show the content area
		this._updateVisibility(this._contentArea, true);
	},	

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */		
	_getChartID: function() {
		return "chart:" + this.id;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount
 	 * 					. Array parameterIcons : [Number] = String
 	 * 					. Array parameterInternalValues : [Number] = String
 	 * 					. Array parameterNames : [Number] = String
 	 * 					. Array parameterValues : [Number] = Number
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 * 					. Array parameterColours : [Number] = String
 	 * @param String presentation
 	 *  
 	 * @return void
 	 */	
	_createFlashChart: function(chartParameters, presentation) {

		// Calculate the preferred chart width and height
		var dimensions = this._getPreferredChartDimensions(this._getChartHeightScaleRatio());
		// Get the maximum number of legend items that can fit in the 
		// vertical area of the viewlet
		var maxLegendItems = this._getMaxLegendItems(dimensions.height);
		// Regroup data to fit the viewlet area
		var regroupedChartParams = this._regroupDataToFitViewletArea(chartParameters, maxLegendItems);
		// Get previous flash chart object
		var flashChartObject = this._getFlashChartObject();

		// Create in-memory model for Flash
 	 	// Object 
 	  	//		. Array parameterNames : [Number] = String				- LENGTH = MAX ITEMS
 	 	//		. Array parameterValues : [Number] = Number				- LENGTH = MAX ITEMS
 	 	//		. Array parameterColours : [Number] = String			- LENGTH = MAX ITEMS (IF FIXED)
 	 	//		. String presentation
 	 	//		. String onClickHandler
		var flashChartParameters = new Object();
		flashChartParameters.parameterNames = this._getFlashChartParameterNames(regroupedChartParams.parameterNames);
		flashChartParameters.parameterValues = regroupedChartParams.parameterValues;
		flashChartParameters.parameterColours = regroupedChartParams.parameterColours;
		flashChartParameters.presentation = this._getFlashChartPresentation(presentation);
		flashChartParameters.onClickHandler = this._static._ONCLICK_HANDLER_NAME;
		flashChartParameters.chartID = this.id;

		
		if (djConfig.isDebug) {
			// Debug mode
			flashChartParameters.isDebug = true;
		}		
		if (this._doOpeningAnimation && !flashChartObject) {
			// Show opening animation only when chart is first loaded
			flashChartParameters.animation = true;
		}
		
		var flashvars = this._generateFlashVars(flashChartParameters);
		
		if (flashChartObject) {
			// Refresh the existing Flash chart object
			this._refreshFlashChartObject(flashChartObject, flashvars);
		} else {
			// Generate the Flash chart object 
			this._addFlashChartObject(flashvars);
		}
		
		// Handle window resize events
		this._registerFlashChartResizeHandler();
		// Update report size
		this._updateFlashChartDimensions();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Array parameterNames
 	 * 
 	 * @return String
 	 */
	_getFlashChartParameterNames: function(parameterNames) {
		
		// Escape parameter names with the following special characters:
		//
		// Double-quote: "     - Prevent broken flashvars attribute value syntax (HTML DOM)
		// Ampersand:    &     - Prevent broken flashvars parameter name parsing (Flash)
		// Equal Sign:   =     - Prevent broken flashvars parameter value parsing (Flash)
		// Pipe:         |     - Prevent broken parameter value array parsing (JFChart)
		// Backslash:    \     - Allow literal escape characters to be rendered (JFChart)
		
		// i.e. \
		var patternEscape = /\\/g;
		// i.e. \\
		var escapedEscape = "\\\\";

		// i.e. <delimiter>
		var patternDelimiter = new RegExp("\\" + this._static._JS_ARRAY_DELIMITER, "g");
		// i.e. \<delimiter>
		var escapedDelimiter = "\\" + this._static._JS_ARRAY_DELIMITER;
		
		
		var escapedParameterNames = new Array(); 
		
		for (var index = 0; index < parameterNames.length; index++) {
			// Escape all escape characters
			// \ -> \\
			var pNormalized_1 = parameterNames[index].replace(patternEscape, escapedEscape);
			// Escape all delimiter characters
			// <delimiter> -> \<delimiter>
			var pNormalized_2 = pNormalized_1.replace(patternDelimiter, escapedDelimiter);
			// Encode the parameter name
			var pEncoded = encodeURIComponent(pNormalized_2);
			
			escapedParameterNames.push(pEncoded);
		}
				
		return escapedParameterNames;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String presentation
 	 * 
 	 * @return String
 	 */
	_getFlashChartPresentation: function(presentation) {
		// Normalize presentation into proper values for JFChart
		if (presentation === this._static._PREFERENCE_VALUE_BAR) {
			return "bar";
		} else if (presentation === this._static._PREFERENCE_VALUE_COLUMN) {
			return "column";
		} else {
			return presentation;
		}
	},

	/*
 	 * @param String flashvars
 	 * 
 	 * @return void
 	 */
	_addFlashChartObject: function(flashvars) {
		// Create Flash chart object
		var flashObject = this._createFlashChartObject(flashvars);
		// Add Flash report 
		this._addReportNode(this._contentArea, flashObject);
		
		// Turn off opening animation		
		if (this._doOpeningAnimation) {
			setTimeout(dojo.hitch(this, function() {
				this._turnOffAnimationFlashVar();
			}), 1000);
		}	
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return Function
 	 */
	_getFlashChartObject: function() {
		// Validate parent
		if (!this._reportNode) {
			return null;
		}
		// Search children
		var nodes = this._reportNode.childNodes;
		var chartID = this._getChartID();

		for (var index = 0; index < nodes.length; index++) {
			var child = nodes[index];
			if (child.id === chartID) {
				return child;
			}
		}
		
		return null;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Function flashChartObject
 	 * @param String flashvars
 	 * 
 	 * @return void
 	 */	
	_refreshFlashChartObject: function(flashChartObject, flashvars) {
		// Call refresh method directly on Flash chart object: JS->AS
		try {
			flashChartObject.refresh(flashvars);
		} catch (errorObject) {
			// Refresh failed. Regenerate the flash object
			this._addFlashChartObject(flashvars);
			// Add warning
			this._logWarning("Refresh operation failed ("+ this._getChartID() + ")", errorObject);
			return;
		}
		
		// Update flashvars param element used by flash object on initial
		// creation.
		// Required to ensure latest chart data is used on flash object recreate 
		// (e.g. drag and drop)
		this._updateFlashvars(flashChartObject, flashvars);
		// Unset the loading message and show the content area
		this._displayLoadingMessage(null);		
	},

	/*
 	 * @param Function flashChartObject
 	 * @param String flashvars
 	 * 
 	 * @return void
 	 */	
	_updateFlashvars: function(flashChartObject, flashvars) {

		if (!flashChartObject) {
			// TODO: MP: Error message?
			return;
		} else if (flashChartObject.hasChildNodes() === false) {
			// TODO: MP: Error message?
			return;
		}
		
		// Get flashvars param element			
		var params = flashChartObject.childNodes;

		for (var index = 0; index < params.length; index++) {
			var parameter = params[index];
			if (parameter &&
				(parameter.name === "flashvars")) {
				// Update param element value
				parameter.value = flashvars;
				break;
			}
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_removeFlashChartObject: function() {

		var flashChartObject = this._getFlashChartObject();		
		if (!flashChartObject) {
			return;
		}

		this._disconnectWindowResizeHandler();
		
		// Remove the flash chart object and leave behind it's report 
		// node container.
		// Special case required for zoom out operations.
		// Required to prevent the viewlet container from zooming in all the way
		// to an empty zero height content area and then visually popping back  
		// out once the flash chart object is readded.
		this._reportNode.removeChild(flashChartObject);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String flashvars
 	 * 
 	 * @return Element
 	 */	
	_createFlashChartObject: function(flashvars) {
		// Common Data
		var dataSource = this._getFlashDataSource();
		var chartID = this._getChartID(); 
		// Common Attributes
		var attributes = {
			id: chartID,
			name: chartID,
			height: "100%",
			width: "100%",
			align: "middle",
			type: "application/x-shockwave-flash",
			data: dataSource
		};		
		// Common Parameters
		var params = {
			quality: "high",
			allowScriptAccess: "always",
			wmode: "transparent"
		};
		// Common Data
		var paramName;
		var attrName;

		// Container
		var container = document.createElement("div");
		dojo.addClass(container, "image");
		container.id = this._getReportDomID();

		// Body
		if (dojo.isIE) {
			// IE Workaround: Embedding an object using the DOM API fails to load
			// the Flash object via ActiveX
			// Reason: Cannot set class ID attribute at runtime
			
			// Object (Open)
			var html = '<object';
			
			// IE Specific Attributes
			attributes.classid = "clsid:d27cdb6e-ae6d-11cf-96b8-444553540000";
			attributes.codebase = "https://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=9,0,0,0";
			
			// Attributes
			for (attrName in attributes) {			
				html = html + ' ' + attrName + '="' + attributes[attrName] + '"';
			}				
			html = html + '>';

			// Flashvars Parameter
			// Need to specify an ID in order to update flashvars later on 
			html = html +		
						'<param' +
							' id="' + this._getFlashVarsDomID() + '"' +
							' name="flashvars"' +
							' value="' + flashvars + '"' +
						'/>';	

			// IE Specific Parameters
			// Note: This param value is duplicated with the data attribute.
			// This is required in order to call ActionScript methods from
			// JavaScript on IE.
			params.movie = dataSource;
					
			// Parameters
			for (paramName in params) {			
				html = html +		
							'<param' +
								' name="' + paramName + '"' +
								' value="' + params[paramName] + '"' +
							'/>';			
			}		
			
			// Object (Close)
			html = html + '</object>';
			
			// Assemble object
			container.innerHTML = html;

		} else {
			// Object
			var elementObject = document.createElement("object");
			
			// Attributes
			for (attrName in attributes) {			
				elementObject[attrName] = attributes[attrName];
			}
			
			// Flashvars Parameter
			// Need to specify an ID in order to update flashvars later on 
			var flashvarsParam = document.createElement("param");
			flashvarsParam.id = this._getFlashVarsDomID();
			flashvarsParam.name = "flashvars";
			flashvarsParam.value = flashvars;
			elementObject.appendChild(flashvarsParam);
			
			// Parameters
			for (paramName in params) {
				var elementParam = document.createElement("param");
				elementParam.name = paramName;
				elementParam.value = params[paramName];

				elementObject.appendChild(elementParam);
			}
			
			// Assemble object
			container.appendChild(elementObject);
		}

		return container;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getFlashDataSource: function() {
		
		if (dojo.isIE) {
			// Note: The "id" attribute on the SWF file serves no functional 
			// purpose; but needs to be added to make the Flash source URI unique 
			// for IE.
			// If the "id" attribute is not specified, JavaScript to ActionScript 
			// method calls fail on IE (because the URI for all embedded Flash 
			// chart objects are identical). 
			// Specifically, only the first Flash chart object created is augmented  
			// to include the proper JavaScript binding to the ActionScript "refresh" 
			// method.
			// Without the binding, the following error message occurs:
			// TypeError: Object doesn't support this property or method
			// Unfortunately, this work-around prevents the browser from 
			// caching a single SWF file and using it for all viewlet instances.
			var date = new Date();
			return this._static._SWF_PATH +
				   "?id=" +
				   date.getTime();
		} 
		// All other browsers
		return this._static._SWF_PATH;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return Number
 	 */	
	_getChartHeightScaleRatio: function() {
		
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		var chartHeightScaleRatio = 1;

		if (presentation === this._static._PREFERENCE_VALUE_PIE) {
			chartHeightScaleRatio = this._static._CHART_HEIGHT_SCALE_RATIO_PIE_CHART;
		} else if (presentation === this._static._PREFERENCE_VALUE_COLUMN) {
			chartHeightScaleRatio = this._static._CHART_HEIGHT_SCALE_RATIO_COLUMN_CHART;
		} else if (presentation === this._static._PREFERENCE_VALUE_BAR) {
			chartHeightScaleRatio = this._static._CHART_HEIGHT_SCALE_RATIO_BAR_CHART;
		}
		
		return chartHeightScaleRatio;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 					. Array parameterNames : [Number] = String				- LENGTH = MAX ITEMS
 	 * 					. Array parameterValues : [Number] = Number				- LENGTH = MAX ITEMS
 	 * 					. Array parameterColours : [Number] = String			- LENGTH = MAX ITEMS (IF FIXED)
 	 *					. String presentation
 	 *					. Number width
 	 *					. Number height
 	 *  
 	 * @return String
 	 */	
	_generateFlashVars: function(container) {
		
		var result = "";
		
		for (var propertyName in container) {
			
			var flashPropertyValue;
			var flashContainerValue = container[propertyName];

			if (flashContainerValue instanceof Array) {
				// Array
				flashPropertyValue = flashContainerValue.join(this._static._JS_ARRAY_DELIMITER);
			} else {
				// String
				// Number
				// Boolean
				flashPropertyValue = flashContainerValue;
			}
			
			var flashPropertyName = "_f" + propertyName;
			
			if (result.length > 0) {
				result = result + "&" + flashPropertyName + "=" + flashPropertyValue;
			} else {
				result = flashPropertyName + "=" + flashPropertyValue;
			}
		}		
		
		return result;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_registerFlashChartResizeHandler: function() {
		// Disconnect the previous handler (if any)
		this._disconnectWindowResizeHandler();
		// Connect a new resize handler
		this._windowResizeHandler = ResizeNotifier.register({context: this, callback: this._updateFlashChartDimensions});
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_updateFlashChartDimensions: function() {
		// Calculate the preferred chart width and height
		var dimensions = this._getPreferredChartDimensions(this._getChartHeightScaleRatio());
		// Set new dimensions
		this._reportNode.style.width = dimensions.width + "px";
		this._reportNode.style.height = dimensions.height + "px";
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 
 	 * @return void
 	 */	
	_dumpObject: function(container) {
		
		console.log("=== DUMP OBJECT START ===");
		
		for (var propertyName in container) {
			console.log(" ");
			console.log("Property Name: " + propertyName);
			
			var propertyValue = container[propertyName];
			
			if (propertyValue instanceof Array) {
				// Array
				console.log("Property Type: Array");
				console.log("Property Values: Length = " + propertyValue.length);
				this._dumpArray(propertyValue);
			} else if (typeof(propertyValue) === "string") {
				// String
				console.log("Property Type: String");
				console.log("Property Value: " + propertyValue);
			} else if (typeof(propertyValue) === "number") {
				// Number
				console.log("Property Type: Number");
				console.log("Property Value: " + propertyValue);
			} else if (typeof(propertyValue) === "object") {
				// Object
				console.log("Property Type: Object");
				this._dumpObject(propertyValue);
			} else if (typeof(propertyValue) === "function") {
				// Function
				console.log("Property Type: Function");
				console.log("Property Value: " + propertyValue);
			} else {
				console.log("Property Type: ?");
				console.log("Property Value: " + propertyValue);
			}			
		}

		console.log("=== DUMP OBJECT END ===");

	},

	/*
 	 * Private Method
 	 * 
 	 * @param Array array - PREREQ: Scalar indexed entries
 	 * 
 	 * @return void
 	 */	
	_dumpArray: function(array) {
		for (var index = 0; index < array.length; index++) {
			console.log("[" + index + "]: " + array[index]);			
		}				
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 
 	 * @return Object
 	 */	
	_cloneObject: function(container) {
		var newContainer = new Object();
		
		for (var propertyName in container) {
			var propertyValue = container[propertyName];
			
			if (propertyValue instanceof Array) {
				// Array
				newContainer[propertyName] = this._cloneArray(propertyValue);
			} else if (typeof(propertyValue) === "string") {
				// String
				newContainer[propertyName] = propertyValue;
			} else if (typeof(propertyValue) === "number") {
				// Number
				newContainer[propertyName] = propertyValue;
			} else if (typeof(propertyValue) === "object") {
				// Object
				newContainer[propertyName] = this._cloneObject(propertyValue);
			} else if (typeof(propertyValue) === "function") {
				// Function
				newContainer[propertyName] = propertyValue;
			} else {
				newContainer[propertyName] = propertyValue;
			}			
		}
		
		return newContainer;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Array oldArray - PREREQ: Scalar indexed entries
 	 * 
 	 * @return Array
 	 */	
	_cloneArray: function(oldArray) {
		var newArray = new Array();
		
		for (var index = 0; index < oldArray.length; index++) {
			newArray[index] = oldArray[index];
		}				
		
		return newArray;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount
 	 * 					. Array parameterIcons : [Number] = String
 	 * 					. Array parameterInternalValues : [Number] = String
 	 * 					. Array parameterNames : [Number] = String
 	 * 					. Array parameterValues : [Number] = Number
 	 * 					. Array parameterColours : [Number] = String
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 * @param Number maxItems
 	 * 
 	 * @return Object 
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount 								- MAX ITEMS
 	 * 					. Array parameterIcons : [Number] = String				- LENGTH = MAX ITEMS
 	 * 					. Array parameterInternalValues : [Number] = String		- LENGTH = MAX ITEMS
 	 * 					. Array parameterNames : [Number] = String				- LENGTH = MAX ITEMS
 	 * 					. Array parameterValues : [Number] = Number				- LENGTH = MAX ITEMS
 	 * 					. Array parameterColours : [Number] = String			- LENGTH = MAX ITEMS (IF FIXED)
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 */
	_regroupDataToFitViewletArea: function(chartParameters, maxItems) {
		// PREREQ:  Parameter names, values and colours are already sorted
		// in descending order by parameter value.
		var displayAllGroups = this.getPreference(this._static._PREFERENCE_ID_OTHERGROUP);

		// Determine if a regrouping is necessary
		if (displayAllGroups) {
			// Overflow grouping into the 'Other' bucket is disabled
			return chartParameters;
		} else if (maxItems <= 0) {
			// Bad input
			return chartParameters;
		} else if (chartParameters.parameterCount <= maxItems) {
			// Already fits.  No regrouping necessary
			return chartParameters;
		}

		// Container for regrouped parameters
		var result = this._cloneObject(chartParameters);

		// Get the start index to regroup items into the Other category
		// 0 <= replaceStartIndex <= (parameterCount - 1)
		var replaceStartIndex = maxItems - 1;
		// Store the index to perform a drill-through on the Other category
		// if requested
		this._setExcessParameterStartIndex(replaceStartIndex);
		// Tally count for the Other category
		var otherCategoryTally = 0;
		for (var i = replaceStartIndex; i < chartParameters.parameterCount; i++) {
			otherCategoryTally = otherCategoryTally + 
								 chartParameters.parameterValues[i];
		}
		
		// Parameter Names
		// Extract the parameter names that fit in the viewlet area
		var newParameterNames = chartParameters.parameterNames.slice(0, maxItems);
		// Replace the last parameter name with the Other category name
		newParameterNames[replaceStartIndex] = this._static._PARAMETER_OTHER;
		result.parameterNames = newParameterNames;
		
		// Parameter Values
		// Extract the parameter values that fit in the viewlet area
		var newParameterValues = chartParameters.parameterValues.slice(0, maxItems);
		// Replace the last parameter value with the Other category tally
		newParameterValues[replaceStartIndex] = otherCategoryTally;
		result.parameterValues = newParameterValues;
		
		// Parameter Colours
		if (chartParameters.paletteType == this._static._PALETTE_TYPE_FIXED) {
			// Extract the parameter colours that fit the viewlet area
			var newParameterColours = chartParameters.parameterColours.slice(0, maxItems);
			// Replace the last parameter colour with the Other category colour
			newParameterColours[replaceStartIndex] = this._static._RGB_COLOUR_GREY;			
			result.parameterColours = newParameterColours;
		}
		
		// Parameter Icons
		if (chartParameters.parameterIcons.length >= maxItems) {
			// Extract the parameter icons that fit in the viewlet area
			var newParameterIcons = chartParameters.parameterIcons.slice(0, maxItems);
			// Replace the last parameter value with the Other category tally
			newParameterIcons[replaceStartIndex] = this._static._ICONS_URI + this._static._IMAGE_OTHER;
			result.parameterIcons = newParameterIcons;
		}

		// Parameter Internal Values
		// Extract the parameter names that fit in the viewlet area
		var newParameterInternal = chartParameters.parameterInternalValues.slice(0, maxItems);
		// Replace the last parameter name with the Other category name
		newParameterInternal[replaceStartIndex] = this._static._PARAMETER_OTHER;
		result.parameterInternalValues = newParameterInternal;
	
		// Parameter Count	
		result.parameterCount = maxItems;

		// Max Value
		if (otherCategoryTally > result.maxValue) {
			result.maxValue = otherCategoryTally;
		}

		// Min Value
		// Recalculate
		var minValue = Number.MAX_VALUE;
		for (var i = 0; i <= replaceStartIndex; i++) {
			var currentValue = result.parameterValues[i];
			if (currentValue === result.minValue) {
				// Previous min value still holds
				minValue = result.minValue;
				break;
			} else if (currentValue < minValue) {
				minValue = currentValue;
			}
		}
		result.minValue = minValue;

		// Debug
		// console.log("=== BEFORE REGROUP ===");
		// this._dumpObject(chartParameters);
		// console.log("=== AFTER REGROUP ===");
		// this._dumpObject(result);

		return result;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_getWorkItemQuery: function() {
		// Set loading message
		this._displayLoadingMessage(this.messages.msgLoading);

		var self = this;

		var serviceObject = {
			success: function(queryDTO) {
				if (!self._isDestroyed) {
					self._processQueryDTO(queryDTO);
				}
			},
			failure: function(errorObject) {
				if (!self._isDestroyed) {
					if (errorObject && errorObject.status == 403) {
						self._displayErrorMessage(self.messages.msgWorkItemQueryPermissionDenied);
					}
					else {
						self._displayErrorMessage(self.messages.msgWorkItemQueryFailure, errorObject);
					}
				}
			}
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		this._callRESTGetQuery(handler);
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_getWorkItemQueryResults: function() {
		// Set loading message
		this._displayLoadingMessage(this.messages.msgLoading);
		
		var self = this;
		
		var serviceObject = {
			success: function(queryResultsDTO) {
				if (!self._isDestroyed) {
					self._processQueryResultsDTO(queryResultsDTO);
				}
			},
			failure: function(errorObject) {
				if (!self._isDestroyed) {
					self._processQueryResultsError(errorObject);
				}
			}
		};

		this._workItemQueryResultsHandler = new ServiceResponseHandler(serviceObject, "success", "failure");

		this._callRESTGetFilteredQueryResults(this._workItemQueryResultsHandler);
	},	

	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */
	_processQueryResultsError: function(errorObject) {
		// Check the header to determine if an OAuth login is required for a
		// C/ALM query
		var oauthHeader;
		if (this._workItemQueryResultsHandler && 
			this._workItemQueryResultsHandler.xhr) {
			oauthHeader = this._workItemQueryResultsHandler.xhr.getResponseHeader("X-jazz-web-oauth-url");
		}
		if (oauthHeader) {
			var oauthURL = new dojo._Url(oauthHeader);
			this._displayMessageLogin(oauthURL);
		} else {
			// Notify viewlets of successful login for C/ALM query
			this._fireEventCALMQueryLogin();
			// Display error
			this._displayErrorMessage(this.messages.msgWorkItemQueryFailure, errorObject);
		}	
	},

	/*
 	 * @param String projectAreaItemId
 	 * 
 	 * @return void
 	 */
	_getWorkItemAttributes: function(projectAreaItemId) {

		// Indicate REST service call in progress
		this._attributesDTO[projectAreaItemId] = this._static._DTO_IN_PROGRESS;

		var context = this;
		var serviceObject = {
			success: function(attributesDTO) {
				if (!context._isDestroyed) {
					context._attributesDTO[projectAreaItemId] = attributesDTO;
				}
			},
			failure: function(errorObject) {
				if (!context._isDestroyed) {
					context._attributesDTO[projectAreaItemId] = context._static._DTO_FAILURE;
					context._displayErrorMessage(context.messages.msgWorkItemAttributesFailure, errorObject);
				}
			}
		};
		
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._callRESTGetAttributes(handler, projectAreaItemId);
	},	

	/*
 	 * @param String projectAreaItemId
 	 * 
 	 * @return void
 	 */
	_getProjectAreaName: function(projectAreaItemId) {

		if (!projectAreaItemId) {
			return;
		}

		var context = this;
		var serviceObject = {
			success: function(dto) {
				if (!context._isDestroyed) {
					context._projectAreaName = dto.name;
				}
			},
			failure: function(errorObject) {
				if (!context._isDestroyed) {
					context._logError(errorObject);					
					context._logError(new Error("Failed to retrieve the project area name for " + projectAreaItemId));					
				}
			}
		};
		
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._callRESTGetProjectArea(handler, projectAreaItemId);
	},	

	/*
 	 * @param String queryItemID
 	 * @param String parameterID
 	 * 
 	 * @return void
 	 */
	_createOtherPNameChooser: function(queryItemID, parameterID) {
		
		var chartParameters = this._getCachedChartParameters();
		if (!chartParameters) {
			// This should never happen
			this._alertErrorMessage(this.messages.msgDrillChartParamsUndef, null);
			return;
		}
		
		var index = this._getExcessParameterStartIndex();
		// Create the dialog
		var dialog = new ParameterNameChooserDialog({
			parameterNames: chartParameters.parameterNames.slice(index),
			parameterValues: chartParameters.parameterValues.slice(index),
			queryItemID: queryItemID,
			parameterID: parameterID,
			onOK: dojo.hitch(this, this._handleEventParamNameDialogExit),
			iconProvider: dojo.hitch(this, this._getIconURL)
		}); 		
		
		dialog.show();			
	},

	/*
 	 * @param String queryItemID
 	 * @param String parameterID
 	 * @param Array results
 	 * 
 	 * @return void
 	 */
	_createMetaGroupPNameChooser: function(queryItemID, parameterID, results) {
		
		// Adapt parameter names and values
		var names = [];
		var values = [];
		for (var index = 0; index < results.length; index++) {
			var entry = results[index];
			names.push(entry.name);
			values.push(entry.value);
		}
		
		// Create the dialog
		var dialog = new ParameterNameChooserDialog({
			parameterNames: names,
			parameterValues: values,
			queryItemID: queryItemID,
			parameterID: parameterID,
			onOK: dojo.hitch(this, this._handleEventParamNameDialogExit),
			iconProvider: dojo.hitch(this, this._getIconURL)
		}); 		
		
		dialog.show();
	},

	/*
 	 * @param ParameterNameChooserDialog dialog
 	 * 
 	 * @return void
 	 */
	_handleEventParamNameDialogExit: function(dialog) {

		var selection = dialog.getSelection();
		if (selection.name) {
			this._getWorkItemQueryFilteredResults(
				selection.name, 
				dialog.queryItemID, 
				dialog.parameterID);
		}
	},

	/*
 	 * @param String label
 	 * @param String queryItemID
 	 * @param String parameterID
 	 * 
 	 * @return Boolean
 	 */
	_getWorkItemQueryFilteredResults: function(label, queryItemID, parameterID) {
		// NOTE: Count parameter is not used
		// Handle undefined parameters
		if (label === this._static._PARAMETER_UNDEFINED) {
			alert(this.messages.msgNoDrillInUndefParams);
			return true;
		}
		// Handle the Other bucket
		if (label === this._static._PARAMETER_OTHER) {
			// Create the parameter name chooser dialog
			this._createOtherPNameChooser(queryItemID, parameterID);
			return true;
		}
		// Handle the meta-group bucket
		if (this._patternResults && 
			(label in this._patternResults)) {
			this._createMetaGroupPNameChooser(queryItemID, parameterID, this._patternResults[label].results);
			return true;
		}

		// Get the internal filter value
		var internalFilterValue = this._internalValueRegistry[label];
		// Ensure we have a value
		if (internalFilterValue === this._static._PARAMETER_UNDEFINED) {
			this._alertErrorMessage(
				dojo.string.substitute(this.messages.msgNoFilterValueForLabel, [label]), 
				null);
			return false;			
		}

		// Get the URL of the filtered query
		var url = WorkItemUtil.getRunSavedQueryUri(
					queryItemID, 
					null, 
					null, 
					null,
					null,
					/* isUnique */ true, 
					/* preserve filter */ false, 
					WorkItemAttributes.getInternalId(parameterID), 
					internalFilterValue);
		// Validate the URL
		if (URLUtility.isValidURL(url) === false) {
			// Inform user of invalid URL
			this._alertErrorMessage(
				dojo.string.substitute(this.messages.msgInvalidURLDetected, [url]), 
				null);
			return false;
		}
		// Check to see if we can leave the Dashboard
		if (!this.getSite().navigateAwayEvent(false, true)) {
			// Abort drill-in
			// e.g. User attempted to navigate away from an unsaved Dashboard
			// and clicked "Cancel" to stay on the Dashboard.
			return true;
		}

		// Go to the work item query results page
		if (this._local) {
			// Local viewlet
			location.href = url;
			return true;
		}
		
		// Go to the work item query results page
		if (dojo.getObject("window.parent.location.href")) {
			// Remote viewlet
			var serverURL = this._getServerURL();
			if (!serverURL) {
				// TODO: MP: Add proper alert error message (NLS String)
				this._logError(new Error("Work item drill-in failure encountered. Unable to generate server URL."));
				return false;
			}
			// The url is an action that starts with a hash.
			// e.g. #action=com.ibm.team.workitem.runSavedQuery&id=_piwGgGv7Ed6ENJFATPcSng&filterAttribute=internalSeverity&filterValue=severity.literal.l6&tq=1250797037000
			// Prefix it with the server url to make it absolute.
			window.parent.location.href = serverURL + url;
			return true;
		}
		
		// TODO: MP: Add proper alert error message (NLS String)
		this._logError(new Error("Work item drill-in failure encountered. Remote viewlet property is undefined: 'window.parent.location.href'"));
		
		return false;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */
	_getServerURL: function() {
		// e.g.
		// https://localhost:9443/jazz/web/projects/Rational%20Team%20Concert?debug=true
		
		if (this._serverURL) {
			return this._serverURL;
		} else if (!this._projectAreaName) {
			return "";
		}
		
		var url = this._getServerRoot() + 
					"/web/projects/" +
					encodeURIComponent(this._projectAreaName);
		
		if (djConfig.isDebug) {
			url = url + "?debug=true";
		}

		this._serverURL = url;
		
		return this._serverURL;
	},

	/*
 	 * @param ServiceResponseHandler handler
 	 * 
 	 * @return void
 	 */	
	_callRESTGetQuery: function(handler) {
		// Get the query preference
		var query = this.getPreference(this._static._PREFERENCE_ID_QUERY);
		// Create the arguments
		var arguments = new Object();
		arguments.includeCount = false;
		// Determine whether we have a query item ID or a query string ID
		if (UUIDUtility.isValidUUID(query)) {
			// Query item ID
			// e.g. _H9EB5esEEdycivs5PH0c-g
			// (from work item query chooser)
			arguments.itemId = query;
		} else {
			// Query string ID
			// e.g. com.ibm.team.workitem.queries.predefined.openCreated
			// (from process or extension point templates)
			arguments.id = query;
		}
		// Project area item ID
		var queryCategory = this.getPreference(this._static._PREFERENCE_ID_CATEGORY);
		arguments.projectAreaItemId = this._getProjectAreaItemID(queryCategory);
		
		// Execute the query
		// Use QueryClient2 for C/ALM queries. 
		// Temporary workaround.
		// See Work Item: 85357
		QueryClient2.getQuery2(handler, arguments);
	},	

	/*
 	 * @param ServiceResponseHandler handler
 	 * 
 	 * @return void
 	 */	
	_callRESTGetFilteredQueryResults: function(handler) {
		var targetParameter = this.getPreference(this._static._PREFERENCE_ID_PARAMETER);
		var queryCategory = this.getPreference(this._static._PREFERENCE_ID_CATEGORY);
		
		var arguments = {
			queryItemId: this._getQueryItemID(),
			attributeId: targetParameter,
			includeCount: true,
			projectAreaItemId: this._getProjectAreaItemID(queryCategory),
			skipOAuth: this._skipOAuth
		};
		
		// Use QueryClient2 for C/ALM queries. 
		// Temporary workaround.
		// See Work Item: 85357
		QueryClient2.getHistogram(handler, arguments);
	},	

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String projectAreaItemId
 	 * 
 	 * @return void
 	 */	
	_callRESTGetAttributes: function(handler, projectAreaItemId) {
		var args = {
			projectAreaItemId: projectAreaItemId,
			filter: WorkItemClient.FILTER_HISTOGRAM
		};
		WorkItemClient.getAttributes(handler, args);
	},	

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String projectAreaItemId
 	 * 
 	 * @return void
 	 */	
	_callRESTGetProjectArea: function(handler, projectAreaItemId) {
		var args = {
			uuid: projectAreaItemId
		};
		ProcessClient.getProjectAreaByUUID(args, handler);
	},	

	/*
 	 * @param _Url oauthURL
 	 * 
 	 * @return void
 	 */	
	_displayMessageLogin: function(oauthURL) {
		// Show login icon and title
		this._setViewletHeaderLogin();
			
		// Listen for logins for this origin
		this._addLoginTracker(oauthURL);
			
		// Container
		var containerDiv = document.createElement("div");
		containerDiv.id = this._getReportDomID();
		dojo.addClass(containerDiv, "login");

		// Message
		var loginMessage = document.createElement("div");
		if (oauthURL &&
			oauthURL.host) {
			loginMessage.innerHTML = dojo.string.substitute(
				this.workItemsMessages.viewletLogin, 
				[oauthURL.host]);
		} else {
			loginMessage.innerHTML = dojo.string.substitute(
				this.workItemsMessages.viewletLogin, 
				[this.messages.msgUndefined]);
		}
		
		// Link
		var link = document.createElement("a");
		link.href = "javascript:void(0);";
		link.innerHTML = this.workItemsMessages.viewletLogin2;
		this.connect(link, "onclick", dojo.hitch(this, this._handleEventLoginClicked));
		link.style.display = "block";
		dojo.addClass(link, "link");
		
		// Assemble
		containerDiv.appendChild(loginMessage);
		containerDiv.appendChild(link);

		// Site
		this._addReportNode(this._contentArea, containerDiv);
			
	},

	/*
 	 * @param String text
 	 * @param String fallbackText
 	 * 
 	 * @return void
 	 */	
	_displayMessageSelectPreference: function(text, fallbackText) {
		// Container
		var div = document.createElement("div");
		div.id = this._getReportDomID();
		dojo.addClass(div, "status");

		// Action message		
		this._addActionOpenSettings(
			div, 
			text,
			fallbackText);
		
		// Site
		this._addReportNode(this._contentArea, div);
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_displayMessageSelectQuery: function() {
		this._displayMessageSelectPreference(
			this.messages.msgNoWorkItemQueryDefAction, 
			this.messages.msgNoWorkItemQueryDef);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_displayMessageNoResults: function() {
		// Get the attach point
		var node = this._contentArea;
		// Create the container
		var div = document.createElement("div");
		div.setAttribute("id", this._getReportDomID());
		dojo.addClass(div, "status");
		// Create text node
		var message = document.createTextNode(this.messages.msgNoWorkItemsFound);
		// Assemble content
		div.appendChild(message);
		
		// Add the div to the node
		this._addReportNode(node, div);
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_displayMessageNonExistingQuery: function() {
		this._displayMessageSelectPreference(
			this.messages.msgWorkItemQueryNotExistAction, 
			this.messages.msgWorkItemQueryNotExist);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param String text
 	 * @param String fallbackText
 	 * 
 	 * @return void
 	 */	
	_addActionOpenSettings: function(parent, text, fallbackText) {

		if (this.getSite().isDashboardEditing()) {
			// The user has permission to edit the Dashboard
			// Add action
			var links = I18nUtil.addMessage(parent, text);
			var link = links[0];
			link.href = "javascript:void(0);";
			this.connect(link, "onclick", this._handleEventPrefLinkClicked);
			
		} else if (this._isStringDef(fallbackText)) {
			// The user does NOT have permission to edit the Dashboard
			// Do NOT add action.  Use fallback text
			
			var fallbackMessage = document.createTextNode(fallbackText);
			parent.appendChild(fallbackMessage);
		}
		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String parameterName
 	 * 
 	 * @return void
 	 */	
	_displayMessageNoParameters: function(parameterName) {
		// Get the attach point
		var node = this._contentArea;
		// Create the container
		var div = document.createElement("div");
		div.setAttribute("id", this._getReportDomID());
		dojo.addClass(div, "status");
		// Create text node
		var message = document.createTextNode(
			dojo.string.substitute(this.messages.msgNoWorkItemsFoundGrouping, [parameterName]));
		// Assemble content
		div.appendChild(message);
		
		// Add the div to the node
		this._addReportNode(node, div);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return <none>
 	 */	
	_displayLoadingMessage: function(text) {
		// Determine whether we are loading or not
		var loading;
		if (this._isStringDefined(text, false, null)) {
			loading = true;
			// Get the old report node (if any)
			var oldReportNode = this._reportNode;
			// Show message area only if there is no previous content	
			if (this._isObjectDefined(oldReportNode, false, "oldReportNode") == false) {
				// No previous report
				this._showMessageArea();
			} else if (dojo.hasClass(oldReportNode, "status")) {
				// "Click here to configure this report" message
				this._showMessageArea();
			}
		} else {
			loading = false;
			this.getSite().doneRefresh();
			// Show content area
			this._showContentArea();
		}
		// Update the loading message		
		this._updateLoadingMessage(text, this._messageArea, loading);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param Error errorObject
 	 * @return <none>
 	 */	
	_displayErrorMessage: function(text, errorObject) {
		// Clear the loading status
		this.getSite().setLoading(false);
		// Inform framework refresh is done
		this.getSite().doneRefresh();
		// Log error to console
		if (errorObject) {
			this._logError(errorObject);
		}
		// Update error message in viewlet
		this._updateErrorMessage(text, errorObject, this._messageArea);
		// Show message area
		this._showMessageArea();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param Error errorObject
 	 * @return <none>
 	 */	
	_alertErrorMessage: function(message, errorObject) {
		// Qualify message
		var text = dojo.string.substitute(this.messages.msgError, [message]);
		// Check if error object is defined
		if (errorObject == null) {
			// Display alert
			alert(text);
			return;	
		}
		// Log error to console
		this._logError(errorObject);
		// Get the error message
		var errorMessage = errorObject.message;
		// Add the error message to the alert text
		if ((typeof(errorMessage) == "string") &&
			(errorMessage.length != 0)) {
			text = text + 
					": " +
					errorMessage;
		} 			
		// Display alert
		alert(text);
	},	

	/*
 	 * @param _Url oauthURL
 	 * 
 	 * @return void
 	 */	
	_addLoginTracker: function(oauthURL) {
		
		var url = oauthURL.scheme + "://" + oauthURL.host + (oauthURL.port ? ":" + oauthURL.port : "");
		
		this._loginTracker = globalLoginObservables[url];
		if (!this._loginTracker) {
			this._loginTracker = globalLoginObservables[url] = new Observable();
		}
		
		this._loginCallback = dojo.hitch(this, function() {
			// Workaround
			// See Work Item 85382
			setTimeout(dojo.hitch(this, function() {
				if (!this.isDestroyed()) {
					this._removeLoginTracker();
					this._handleEventLoginClicked(null);
				}
			}), 0);
		});
		this._loginTracker.addListener(this._loginCallback, this._static._CALM_QUERY_LOGIN);
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_removeLoginTracker: function() {
		if (this._loginCallback && 
			this._loginTracker) {
				
			this._loginTracker.removeListener(this._loginCallback, this._static._CALM_QUERY_LOGIN);
			
			delete this._loginCallback;
			delete this._loginTracker;
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_fireEventCALMQueryLogin: function() {
		// Notify viewlets waiting to login to load
		var tracker = this._loginTracker;
		if (tracker) {
			this._removeLoginTracker();
			tracker.propertyChanged(this._static._CALM_QUERY_LOGIN);
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_setViewletHeaderDefault: function() {
		// Reset icon
		this.getSite().setIcon(dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "graphics/statistics/icons/statistics.gif"));
		// Reset title
		this.getSite().setSecondaryTitle("");
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_setViewletHeaderLogin: function() {
		// Show login icon
		this.getSite().setIcon(dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/login.gif"));
		// Show login title
		this.getSite().setSecondaryTitle(this.workItemsMessages.viewletNotLoggedIn);
	},
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventLoginClicked: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		// Reset icon and title
		this._setViewletHeaderDefault();
		// Enable OAuth
		this._skipOAuth = false;
		// Refresh the viewlet
		this.refresh();
		
		return true;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return <none>
 	 */	
	_handleEventPrefLinkClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Load the settings area
		this.getSite().toggleSettings();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * 						. Object attribute   					(NOT USED)
 	 * 									. String attributeTypeId	(NOT USED)
 	 * 									. String id					(NOT USED)
 	 * 									. String label
 	 * 						. Array counts : [Number] = Object
 	 * 									. Number count
 	 * 									. Object value
 	 * 											. String iconUrl
 	 * 											. String id
 	 * 											. String label
 	 * 						. String queryItemId					(NOT USED)
 	 * 						. Number totalCount
 	 * 
 	 * @return void
 	 */		
	_processQueryResultsDTO: function(queryResultsDTO) {
		// Debug
		// console.debug(queryResultsDTO);
		// this._displayErrorMessage("ABORT", new Error("Test"));
		// return;		
		
		// Notify viewlets configured with a C/ALM query that a successful OAuth 
		// login occurred.
		this._fireEventCALMQueryLogin();
		
		// Validate filtered query results DTO
		try {
			this._validateFilteredQueryResultsDTO(queryResultsDTO);
		} catch(errorObject) {
			this._displayErrorMessage(this.messages.msgWorkItemDTOParseFailed, errorObject);
			return;
		}
		var total = queryResultsDTO.totalCount;
		// Set the secondary viewlet title
		this._updateSecondaryTitle(total, queryResultsDTO.attribute.label);		
		// Check to see if there are any query results
		if (total === 0) {
			this._displayMessageNoResults();
			return;
		}
		
		// Check to see if any of the results found are grouped by the selected
		// parameter
		if (this._hasParameterResults(queryResultsDTO) === false) {
			this._displayMessageNoParameters(queryResultsDTO.attribute.label);
			return;
		}

		// Generate the chart parameters
		var chartParameters = null;
		try {
			chartParameters = this._generateChartParameters(queryResultsDTO);
		} catch(errorObject) {
			this._displayErrorMessage(this.messages.msgChartParamGenFailure, errorObject);
			return;
		}
		
		// Debug
		//this._debugChartParameters(chartParameters);
		
		// Cache the chart parameters for zoom-in and zoom-out operations
		this._setCachedChartParameters(chartParameters);
		// Display the report
		this._displayReport(chartParameters);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount
 	 * 					. Array parameterIcons : [Number] = String
 	 * 					. Array parameterInternalValues : [Number] = String
 	 * 					. Array parameterNames : [Number] = String
 	 * 					. Array parameterValues : [Number] = Number
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 * 					. Array parameterColours : [Number] = String
 	 * 
 	 * @return void
 	 */		
	_displayReport: function(chartParameters) {
		// Get the presentation preference
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		// Check preference value
		if ((presentation === this._static._PREFERENCE_VALUE_BAR) ||
			(presentation === this._static._PREFERENCE_VALUE_PIE) ||
			(presentation === this._static._PREFERENCE_VALUE_COLUMN)) {
			// Create the chart
			this._createFlashChart(chartParameters, presentation);
		} else if (presentation === this._static._PREFERENCE_VALUE_TABLE) {
			// Create the table
			this._createTable(chartParameters);
		} else if (presentation === this._static._PREFERENCE_VALUE_LIST) {
			this._createTagCloud(chartParameters);
		} else {
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnknownPresentationVal, [presentation]));
			this._displayErrorMessage(this.messages.msgUnknownPresentation, errorObject);
			return;
		}			
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * @return void
 	 */	
	_sortParameters: function(container) {
		// Get the presentation preference
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		// Get the target parameter
		var targetParameter = this.getPreference(this._static._PREFERENCE_ID_PARAMETER);

		// Sorting Note:
		//
		// The following DTO arrays ALWAYS need to be sorted:
		//
		// * Presentation Names: parameterNames
		// * Presentation Values: parameterValues
		// * Drill-in Internal Values:  parameterInternalValues
		// * Presentation Icons:  parameterIcons
		// 
		// The following DTO arrays NEVER need to be sorted:
		//
		// * Presentation Colours:  parameterColours
		//   => They are added after sorting
		
		// Sort according to selected preferences
		if ((targetParameter === WorkItemAttributes.EXT_SEVERITY) ||
			(targetParameter === WorkItemAttributes.EXT_PRIORITY) ||
			(targetParameter === WorkItemAttributes.EXT_STATE) ||
			(targetParameter === WorkItemAttributes.EXT_RESOLUTION)) {
			// Sort the names and values by descending internal value
			// The internal values for these grouping parameters are defined
			// as numeric IDs in the process specification and are organized
			// via desired sorting order
			this._parameterInternalValuesShellSort(container);
		} else if (presentation == this._static._PREFERENCE_VALUE_LIST) {
			// Sort the names and values by ascending name
			this._parameterNamesShellSort(container);			
		} else {
			// Sort the names and values by descending value
			this._parameterValuesShellSort(container);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 				. Number parameterCount
 	 * 				. Array parameterNames
 	 * 				. Array parameterValues
 	 * 				. Array parameterInternalValues
 	 * 				. Array parameterIcons
 	 * @return void
 	 */	
	_parameterInternalValuesShellSort: function(container) {
		// Validate data
		if (container.parameterCount <= 0) {
			// Nothing to do
			return;
		}
		// Custom sort required to sort names and values simultaneously
		var names = container.parameterNames;
		var values = container.parameterValues;
		var internalValues = container.parameterInternalValues;
		var icons = container.parameterIcons;
		// Shell sort O(n^2)
		for (var increment = internalValues.length / 2; increment > 0; increment = (increment == 2 ? 1 : Math.round(increment / 2.2))) {
			for (var i = increment; i < internalValues.length; i++) {
				// Descending sort
				for (var j = i; j >= increment && internalValues[j - increment] < internalValues[j]; j -= increment) {
					// Values
					var temp1 = values[j];
					values[j] = values[j - increment];
					values[j - increment] = temp1;
					// Names
					var temp2 = names[j];
					names[j] = names[j - increment];
					names[j - increment] = temp2;
					// Internal Values
					var temp3 = internalValues[j];
					internalValues[j] = internalValues[j - increment];
					internalValues[j - increment] = temp3;
					// Icons
					var temp4 = icons[j];
					icons[j] = icons[j - increment];
					icons[j - increment] = temp4;					
				}
			}
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 				. Number parameterCount
 	 * 				. Array parameterNames
 	 * 				. Array parameterValues
 	 * 				. Array parameterIcons
 	 * @return void
 	 */	
	_parameterNamesShellSort: function(container) {
		// Validate data
		if (container.parameterCount <= 0) {
			// Nothing to do
			return;
		}
		// Custom sort required to sort names and values simultaneously
		var names = container.parameterNames;
		var values = container.parameterValues;
		var internalValues = container.parameterInternalValues;
		var icons = container.parameterIcons;
		// Shell sort O(n^2)
		for (var increment = names.length / 2; increment > 0; increment = (increment == 2 ? 1 : Math.round(increment / 2.2))) {
			for (var i = increment; i < names.length; i++) {
				// Ascending sort
				for (var j = i; j >= increment && names[j - increment] > names[j]; j -= increment) {
					// Values
					var temp1 = values[j];
					values[j] = values[j - increment];
					values[j - increment] = temp1;
					// Names
					var temp2 = names[j];
					names[j] = names[j - increment];
					names[j - increment] = temp2;
					// Internal Values
					var temp3 = internalValues[j];
					internalValues[j] = internalValues[j - increment];
					internalValues[j - increment] = temp3;
					// Icons
					var temp4 = icons[j];
					icons[j] = icons[j - increment];
					icons[j - increment] = temp4;					
				}
			}
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 				. Number parameterCount
 	 * 				. Array parameterNames
 	 * 				. Array parameterValues
 	 * 				. Array parameterIcons
 	 * @return void
 	 */	
	_parameterValuesShellSort: function(container) {
		// Validate data
		if (container.parameterCount <= 0) {
			// Nothing to do
			return;
		}
		// Custom sort required to sort names and values simultaneously
		var names = container.parameterNames;
		var values = container.parameterValues;
		var internalValues = container.parameterInternalValues;
		var icons = container.parameterIcons;
		// Shell sort O(n^2)
		for (var increment = values.length / 2; increment > 0; increment = (increment == 2 ? 1 : Math.round(increment / 2.2))) {
			for (var i = increment; i < values.length; i++) {
				// Descending sort
				for (var j = i; j >= increment && values[j - increment] < values[j]; j -= increment) {
					// Values
					var temp1 = values[j];
					values[j] = values[j - increment];
					values[j - increment] = temp1;
					// Names
					var temp2 = names[j];
					names[j] = names[j - increment];
					names[j - increment] = temp2;
					// Internal Values
					var temp3 = internalValues[j];
					internalValues[j] = internalValues[j - increment];
					internalValues[j - increment] = temp3;
					// Icons
					var temp4 = icons[j];
					icons[j] = icons[j - increment];
					icons[j - increment] = temp4;					
				}
			}
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 
 	 * @return void
 	 */	
	_addColoursToChartParameters: function(container) {
		var parameter = container.parameterID;
		var values = container.parameterIcons;

		if (parameter === WorkItemAttributes.EXT_SEVERITY) {
			// Severity
			container.paletteType = this._static._PALETTE_TYPE_FIXED;
			container.parameterColours = this._generateParameterColours(
											values,  
											this._static._COLOUR_REGEX_SEVERITY,
											this._static._COLOUR_RGB_SEVERITY);

		} else if (parameter === WorkItemAttributes.EXT_PRIORITY) {
			// Priority
			container.paletteType = this._static._PALETTE_TYPE_FIXED;
			container.parameterColours = this._generateParameterColours(
											values, 
											this._static._COLOUR_REGEX_PRIORITY,
											this._static._COLOUR_RGB_PRIORITY);

		} else if (parameter === WorkItemAttributes.EXT_TYPE) {
			// Type
			container.paletteType = this._static._PALETTE_TYPE_FIXED;
			container.parameterColours = this._generateParameterColours(
											values,
											this._static._COLOUR_REGEX_TYPE,
											this._static._COLOUR_RGB_TYPE);

		} else {
			// Rest
			container.paletteType = this._static._PALETTE_TYPE_RANDOM;
			container.parameterColours = new Array();
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Array values : String
 	 * @param Array patterns : String
 	 * @param Array colours : String
 	 * 
 	 * @return Array
 	 */	
	_generateParameterColours: function(values, patterns, colours) {

		var parameterColours = new Array();
			
		for (var index = 0; index < values.length; index++) {
			var value = this._getIconLocalFilename(values[index]);
			var colour = this._getAssociatedColour(value, patterns, colours);
			parameterColours.push(colour);
		}
			
		return parameterColours;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String iconPath
 	 * 
 	 * @return String
 	 */	
	_getIconLocalFilename: function(iconPath) {
		var elements = iconPath.split(/\//);
		if (elements.length === 0) {
			return "";
		}
		var lastIndex = elements.length - 1;
		return elements[lastIndex];
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @param Array patterns : String
 	 * @param Array colours : String
 	 * 
 	 * @return String
 	 */	
	_getAssociatedColour: function(value, patterns, colours) {
		
		for (var index = 0; index < patterns.length; index++) {
			var pattern = new RegExp(patterns[index]);
			var found = value.match(pattern);
			if (found) {
				return colours[index];
			}
		}
		
		return this._static._COLOUR_RGB_UNDEFINED;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String parameterName
 	 * @return String
 	 */	
	_normalizeParameterName: function(parameterName) {
		// BIRT renders the name as a label enclosed in single quotes
		// This breaks the image map click-through for names that include
		// single quotes:  Curtis D'Entremont
		// Replace single quotes
		var pattern = /'/g;
		var result = parameterName.replace(pattern, "`");
		return result;
	},

	/*
 	 * @param Object queryDTO
 	 * 
 	 * @return void
 	 */		
	_processQueryDTO: function(queryDTO) {
		// Check if the query exists
		if (this._isObjectDef(queryDTO) === false) {
			this._displayMessageNonExistingQuery();
			return;
		}
		
		// Debug
		// console.debug(queryDTO);
		try {
			this._validateQueryDTO(queryDTO);
		} catch (errorObject) {
			this._displayErrorMessage(this.messages.msgWorkItemDTOProblem, errorObject);
			return;
		}
		// Initialize query preference provider
		this._initQueryPreferenceProvider(queryDTO);
		// Update the title every time we fetch the query in case the name has changed
		// Do not persist it to avoid dirtying dashboard
		this.getSite().setTitle(queryDTO.name, false);
		// Store the work item query item ID
		this._setQueryItemID(queryDTO.itemId);
		
		// Execute the query
		this._getWorkItemQueryResults();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryDTO
 	 * 					. String itemId
 	 * 					. String name
 	 * @return void
 	 */		
	_initQueryPreferenceProvider: function(queryDTO) {
		// Get the query preference
		var query = this.getPreference(this._static._PREFERENCE_ID_QUERY);
		// Check query ID type:  Item or String
		if (UUIDUtility.isValidUUID(query)) {
			// Query item ID
			// Nothing to do
			return;
		}
		// Query string ID
		// Initialize defaults
		this._queryPreferenceProvider.initDefault(queryDTO);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Number queryTotal
 	 * @param String parameterName
 	 * 
 	 * @return void
 	 */	
	_updateSecondaryTitle: function(queryTotal, parameterName) {
		// Create the secondary title
		var secondaryTitle = "(" +
					queryTotal +
					") " +
					parameterName;
		// Set the secondary title
		this.getSite().setSecondaryTitle(secondaryTitle);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String queryItemID
 	 * @param boolean persist
 	 * @return void
 	 */	
	_updatePrimaryTitle: function(queryItemID, persist) {
		// Get the new query name
		var queryName = this._queryPreferenceProvider.getTextLabel(queryItemID);
		// Set the primary title
		if (this._isStringDefined(queryName, false, "queryName")) {
			this.getSite().setTitle(queryName, persist);
		}	
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryDTO
 	 * 
 	 * @return void
 	 * @throws Error
 	 */	
	_validateQueryDTO: function(queryDTO) {
		// Name property
		this._hasProperty(queryDTO, "name", true, "queryDTO");
		this._isStringDefined(queryDTO.name, true, "name");
		// ItemId property
		this._hasProperty(queryDTO, "itemId", true, "queryDTO");
		this._isStringDefined(queryDTO.itemId, true, "itemId");
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * 						. Object attribute   					(NOT USED)
 	 * 									. String attributeTypeId	(NOT USED)
 	 * 									. String id					(NOT USED)
 	 * 									. String label				(NOT USED)
 	 * 						. Array counts : [Number] = Object
 	 * 									. Number count
 	 * 									. Object value
 	 * 											. String iconUrl
 	 * 											. String id
 	 * 											. String label
 	 * 						. String queryItemId					(NOT USED)
 	 * 						. Number totalCount
 	 * 
 	 * @return void
 	 */	
	_validateFilteredQueryResultsDTO: function(queryResultsDTO) {
		// DTO
		this._isObjectDefined(queryResultsDTO, true, "queryResultsDTO");

		// Attribute label
		if (!queryResultsDTO.attribute ||
			!queryResultsDTO.attribute.label) {
			throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotAString, ["queryResultsDTO.attribute.label"]));
		}		
		
		// DTO . Total Count
		this._hasProperty(queryResultsDTO, "totalCount", true, "queryResultsDTO");
		this._isNumberDefined(queryResultsDTO.totalCount, true, "totalCount");		
		
		if (queryResultsDTO.totalCount === 0) {
			// When the Total Count is 0, the Counts property does not exist
			// Abort validation
			return;
		} else if (("counts" in queryResultsDTO) === false) {
			// Abort validation
			return;
		}
		
		// DTO . Counts
		this._isArray(queryResultsDTO.counts, true, "counts");
		
		for (var index = 0; index < queryResultsDTO.counts.length; index++) {
			var result = queryResultsDTO.counts[index];
			var displayPath = "queryResultsDTO.counts[" + index + "]";	
			
			// DTO . Counts[x]
			this._isObjectDefined(result, true, displayPath);
			
			// DTO . Counts[x] . Count
			this._hasProperty(result, "count", true, displayPath);
			this._isNumberDefined(result.count, true, displayPath + ".count");		
			
			// DTO . Counts[x] . Value
			this._hasProperty(result, "value", true, displayPath);
			this._isObjectDefined(result.value, true, displayPath + ".value");
			
			// Icon URL not specified for certain grouping parameters
			// DTO . Counts[x] . Value . Icon URL
			//this._hasProperty(result.value, "iconUrl", true, displayPath + ".value");
			//this._isStringDefined(result.value.iconUrl, true, displayPath + ".value.iconUrl");
			
			// DTO . Counts[x] . Value . ID
			this._hasProperty(result.value, "id", true, displayPath + ".value");
			// ID can be an empty string for the tags grouping parameter
			//this._isStringDefined(result.value.id, true, displayPath + ".value.id");
			
			// DTO . Counts[x] . Value . Label
			// Label can be an empty string for the tags grouping parameter
			this._hasProperty(result.value, "label", true, displayPath + ".value");
			//this._isStringDefined(result.value.label, true, displayPath + ".value.label");
		}
		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String queryItemID
 	 * @return <none>
 	 */	
	_setQueryItemID: function(queryItemID) {
		this._currentQueryItemID = queryItemID;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getQueryItemID: function() {
		return this._currentQueryItemID;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * @return <none>
 	 */	
	_setCachedChartParameters: function(chartParameters) {
		this._cachedChartParameters = chartParameters;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_clearCachedChartParameters: function() {
		this._cachedChartParameters = null;
		this._internalValueRegistry = new Array();
		this._iconURLRegistry = new Array();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return Object
 	 */	
	_getCachedChartParameters: function() {
		return this._cachedChartParameters;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String label
 	 * 
 	 * @return Boolean
 	 */	
	handleEventOnClickFlashChart: function(label) {
		// Get the filtered work item query results
		return this._getWorkItemQueryFilteredResults(label, 
													 this._getQueryItemID(), 
													 this._getCachedChartParameters().parameterID);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * 						. Object attribute   					(NOT USED)
 	 * 									. String attributeTypeId	(NOT USED)
 	 * 									. String id					(NOT USED)
 	 * 									. String label				(NOT USED)
 	 * 						. Array counts : [Number] = Object
 	 * 									. Number count
 	 * 									. Object value
 	 * 											. String iconUrl
 	 * 											. String id
 	 * 											. String label
 	 * 						. String queryItemId					(NOT USED)
 	 * 						. Number totalCount
 	 * 
 	 * @return Object
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount
 	 * 					. Array parameterIcons : [Number] = String
 	 * 					. Array parameterInternalValues : [Number] = String
 	 * 					. Array parameterNames : [Number] = String
 	 * 					. Array parameterValues : [Number] = Number
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 * 					. Array parameterColours : [Number] = String
 	 * @throws Error
 	 */	
	_generateChartParameters: function(queryResultsDTO) {
				
		// DTO to chart parameters mappings:
		//
		// . Array counts : [Number] = Object   				counts.length => . Number parameterCount
		// 				. Number count							=> . Number parameterValues[index]
		// 				. Object value
		// 						. String iconUrl				=> . String parameterIcons[index]
		// 						. String id						=> . String parameterInternalValues[index]
		// 						. String label					=> . String parameterNames[index]
		// . Number totalCount 									=> . Number total
		
		var parameterID = this.getPreference(this._static._PREFERENCE_ID_PARAMETER);
		var parameterValues = new Array();
		var parameterNames = new Array();
		var parameterIcons = new Array();
		var parameterInternalValues = new Array();
		var maxValue = Number.MIN_VALUE;
		var minValue = Number.MAX_VALUE;
		var parameterCount = 0;
		var patternGroups = this._getPatternGroups();
		var inMetaGroup = false;
		
		this._patternResults = {};	
	
		for (var index = 0; index < queryResultsDTO.counts.length; index++) {
			
			// Parameter Value ------------------------------------------------

			var count = queryResultsDTO.counts[index].count;

			// Parameter Name -------------------------------------------------

			var value = queryResultsDTO.counts[index].value;
			var label;
			if (this._isStringDefined(value.label, false, null)) {
				label = this._normalizeParameterName(value.label);
			} else if (parameterID === WorkItemAttributes.EXT_TAGS) {
				// Skip. Do not create a bucket for work items that have no tags.
				// Represented by an empty label ("") and empty id ("").
				continue;
			} else {
				label = this._static._PARAMETER_UNDEFINED;
			}			
			
			inMetaGroup = false;
			if (patternGroups) {
				for (var pCount = 0; pCount < patternGroups.length; pCount++) {
					var pattern = patternGroups[pCount];
					var re = pattern.value;
					var pName = pattern.name;
					if (re.test(label)) {
						// Pattern match
						if (pName in this._patternResults) {
							// Update existing meta-group
							this._patternResults[pName].total += count;
						} else {
							// Create new meta-group
							this._patternResults[pName] = {
								total: count,
								results: []
							};
						}
						// Add match to meta-group
						this._patternResults[pName].results.push({
							name: label,
							value: count
						});
						inMetaGroup = true;
						// Do not allow the entry to show up in more than one
						// meta-group
						break;
						// TODO: MP: Add option to allow duplicates?
					}
				}
			}
			
			// Icon URL -------------------------------------------------------
			
			var iconUrl;
			if (value.iconUrl) {
				if (this._local) {
					// Local viewlet
					iconUrl = value.iconUrl;
				} else {
					// Remote viewlet
					iconUrl = this._getImageServerRoot() + value.iconUrl;
				}
				// Register Icon URL
				this._iconURLRegistry[label] = iconUrl;
			} else {
				iconUrl = this._static._PARAMETER_UNDEFINED;
			}
			
			// Parameter Internal Value ---------------------------------------
			
			var id;
			if (typeof(value.id) === "string") {
				id = value.id;
			} else {
				id = this._static._PARAMETER_UNDEFINED;
			}
			// Register Parameter Internal Value by Parameter Name
			this._internalValueRegistry[label] = id;
			
			// Model Properties -----------------------------------------------

			if (inMetaGroup) {
				continue;
			}

			parameterNames.push(label);
			parameterValues.push(count);
			parameterIcons.push(iconUrl);
			parameterInternalValues.push(id);

			// Statistics -----------------------------------------------------
			
			// Track max value
			if (count > maxValue) {
				maxValue = count;
			}
			// Track min value
			if (count < minValue) {
				minValue = count;
			}

			// Track count
			parameterCount++;
		}
		
		// Add meta-groups
		for (var ptrnName in this._patternResults) {
			var pResultCount = this._patternResults[ptrnName].total;
			
			// TODO: MP: This icon could overwrite an icon from an existing group
			// if the labels match
			var metagroupIconURL = this._static._ICONS_URI + this._static._IMAGE_METAGROUP;
			this._iconURLRegistry[ptrnName] = metagroupIconURL;
			
			parameterNames.push(ptrnName);
			parameterValues.push(pResultCount);
			parameterIcons.push(metagroupIconURL);
			parameterInternalValues.push(this._static._PARAMETER_ID_METAGROUP);
			
			// Track max value
			if (pResultCount > maxValue) {
				maxValue = pResultCount;
			}
			// Track min value
			if (pResultCount < minValue) {
				minValue = pResultCount;
			}
			// Track count
			parameterCount++;			
		}
		
		var chartParameters = new Object();
		// Max Value
		chartParameters.maxValue = maxValue;
		// Min Value
		chartParameters.minValue = minValue;
		// Parameter Count
		chartParameters.parameterCount = parameterCount;
		// Parameter Icons
		chartParameters.parameterIcons = parameterIcons;
		// Parameter Internal Values
		chartParameters.parameterInternalValues = parameterInternalValues;
		// Parameter Names
		chartParameters.parameterNames = parameterNames;
		// Parameter Values		
		chartParameters.parameterValues = parameterValues;
		// Total
		// Note: The total does not reflect the sum of parameter values when 
		// the grouping parameter is tags because work items that have no tags
		// are not represented by a bucket.
		chartParameters.total = queryResultsDTO.totalCount;
		// Parameter ID
		chartParameters.parameterID = parameterID;
		// Display Name
		chartParameters.displayName = queryResultsDTO.attribute.label; 
		
		// Sort the parameters
		this._sortParameters(chartParameters);
		// Add colours
		this._addColoursToChartParameters(chartParameters);
		
		return chartParameters;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * 						. Object attribute   					(NOT USED)
 	 * 									. String attributeTypeId	(NOT USED)
 	 * 									. String id					(NOT USED)
 	 * 									. String label				(NOT USED)
 	 * 						. Array counts : [Number] = Object
 	 * 									. Number count
 	 * 									. Object value
 	 * 											. String iconUrl
 	 * 											. String id
 	 * 											. String label
 	 * 						. String queryItemId					(NOT USED)
 	 * 						. Number totalCount
 	 * 
 	 * @return boolean
 	 */		
	_hasParameterResults: function(queryResultsDTO) {
		
		if (("counts" in queryResultsDTO) === false) {
			// e.g. Story Points
			return false;
		} else if ((queryResultsDTO.counts.length === 1) &&
			(this._isStringDefined(queryResultsDTO.counts[0].value.id, false, null) === false)) {
			// e.g. Found In, Planned For, Tags
			return false;
		}
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getPatternGroups: function() {
		
		if (this._patternGroups) {
			return this._patternGroups;
		}
		// Format:
		// [PatternName 1]~~[Pattern Value 1]@@[PatternName 2]~~[Pattern Value 2]@@[PatternName N]~~[Pattern Value N]
		
		var metagroups = this.getPreference(this._static._PREFERENCE_ID_METAGROUPS);
		// TEST
		//metagroups = "Jazz User~~TestJazzUser@@Jazz Admin~~TestJazzAdmin@@Jazz Guest~~TestJazzGuest";
		
		if (!metagroups) {
			return null;
		}
		
		// TODO: MP: Reuse parsing code from metagroups dialog
		
		var patternSegments = metagroups.split("@@");
		
		this._patternGroups = [];
		
		for (var index = 0; index < patternSegments.length; index++) {
			var pattern = patternSegments[index];
			if (!pattern) {
				// TODO: MP: Error message?
				continue;
			}
			
			var patternPair = pattern.split("~~", 2);
			if (patternPair.length !== 2) {
				continue;
			}
			
			var patternName = patternPair[0];
			var patternValue = patternPair[1];
			
			this._patternGroups.push({
				name: patternName,
				value: new RegExp(patternValue, "i")
			});
		}

		if (this._patternGroups.length === 0) {
			return null;
		}

		return this._patternGroups;
	},

	/*
 	 * Private Method
 	 * 
 	 * @return Object chartParameters
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount
 	 * 					. Array parameterIcons : [Number] = String
 	 * 					. Array parameterInternalValues : [Number] = String
 	 * 					. Array parameterNames : [Number] = String
 	 * 					. Array parameterValues : [Number] = Number
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 * 					. Array parameterColours : [Number] = String

 	 * 
 	 * @return void
 	 */		
	_debugChartParameters: function(chartParameters) {
		console.log("= START ==========================");	
		console.log("Display Name: " + chartParameters.displayName);
		console.log("Parameter ID: " + chartParameters.parameterID);
		console.log("Total: " + chartParameters.total);
		console.log("Parameter Count: " + chartParameters.parameterCount);
		console.log("Max Value: " + chartParameters.maxValue);
		console.log("Min Value: " + chartParameters.minValue);
		console.log("Palette Type: " + chartParameters.paletteType);
		
		for (var index = 0; index < chartParameters.parameterCount; index++) {
			console.log("---------------------");
			console.log("Index: " + index);
			console.log("---------------------");
			console.log("Parameter Name: " + chartParameters.parameterNames[index]);
			console.log("Parameter Value: " + chartParameters.parameterValues[index]);
			console.log("Parameter Internal Value: " + chartParameters.parameterInternalValues[index]);
			console.log("Parameter Icon: " + chartParameters.parameterIcons[index]);
			console.log("Parameter Colour: " + chartParameters.parameterColours[index]);
		}
		
		console.log("= END   ==========================");		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getPreferenceFailureList: function() {
		var result = {
			options: [{
				label: dojo.string.substitute(this.messages.msgError, [this.messages.msgPreferenceFailure]),
				value: ""
			}]
		};
		return result;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getPreferenceSelectList: function() {
		var result = {
			options: [{
				label: this.messages.msgSelectQuery,
				value: ""
			}]
		};
		return result;
	},	

	/*
 	 * @param String projectAreaItemId
 	 * 
 	 * @return Object
 	 */		
	_getPreferenceParameterList: function(projectAreaItemId) {
	
		// Format	
	 	// Array attributesDTO [Number] = Object
	 	//									. String id					(NOT USED)
	 	// 									. String label
	 	// 									. String externalId
	 	// 									. String attributeTypeId	(NOT USED)
		
		// TEST
		//console.debug(this._attributesDTO[projectAreaItemId]);
		
		var result = new Object();
		result.options = new Array();

		for (var index = 0; index < this._attributesDTO[projectAreaItemId].length; index++) {
			var attribute = this._attributesDTO[projectAreaItemId][index];
			var entry = new Object();
			entry.label = attribute.label;
			entry.value = attribute.externalId;

			result.options.push(entry);
		}

		// Sort preferences
		result.options.sort(this._sortPreferencesArray);
		
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object a
 	 * @param Object b
 	 * 
 	 * @return Object
 	 */		
	_sortPreferencesArray: function(a, b) {
		if (a.label < b.label) {
			// Less than
			return -1;
		} else if (a.label > b.label) {
			// Greater than
			return 1;
		}
		// Equal
		return 0;
	},	

	// Viewlet Methods -------------------------------------------------------->

	/*
 	 * @param String preferenceId
 	 * @param Object currentSettings
 	 * 
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceOptions: function(preferenceId, currentSettings) {
		
		if (preferenceId !== this._static._PREFERENCE_ID_PARAMETER) {
			// Wrong preference
			this._logError(new Error("Unexpected preference ID encountered"));
			return this._getPreferenceFailureList();
		}
		
		var projectAreaItemID = this._getProjectAreaItemID(currentSettings.preferences[this._static._PREFERENCE_ID_CATEGORY]);
		if (!projectAreaItemID) {
			return this._getPreferenceSelectList();
		}
		
		this._timeoutDTOCount++;

		if (this._isArrayDef(this._attributesDTO[projectAreaItemID])) {
			// Get Parameters
			this._timeoutDTOCount = 0;
			return this._getPreferenceParameterList(projectAreaItemID);
		} else if (this._timeoutDTOCount >= this._static._DTO_TIMEOUT_LIMIT) {
			// Time-out
			this._logError(new Error("Preference loading timed-out"));
			this._timeoutDTOCount = 0;
			this._attributesDTO[projectAreaItemID] = null;
			return this._getPreferenceFailureList();
		} else if (this._attributesDTO[projectAreaItemID] === this._static._DTO_IN_PROGRESS) {
			// Polling
			// The DTO has not been recieved from the REST service call yet.
			// Return null to start an interval-based polling of this method
			// until the DTO is recieved.
			return null;
		} else if (this._attributesDTO[projectAreaItemID] === this._static._DTO_FAILURE) {
			// REST service call error
			this._logError(new Error("Preference loading failed"));
			this._timeoutDTOCount = 0;
			this._attributesDTO[projectAreaItemID] = null;
			return this._getPreferenceFailureList();
		}
		// Execute REST service call
		this._getWorkItemAttributes(projectAreaItemID);
		// Return null to start an interval-based polling of this method
		// until the DTO is recieved.
		return null;
	},

	/*
 	 * @param String queryCategory
 	 * 
 	 * @return String
 	 */		
	_getProjectAreaItemID: function(queryCategory) {
		if (!queryCategory) {
			// Required for primed viewlets
			// Viewlets cannot be primed with remote or cross-project queries
			// Get current project area
			if (this._local) {
				// Local viewlet
				return com.ibm.team.process.web.projectArea.itemId;
			}
			// Remote viewlet
			return null;
		} else if (queryCategory.indexOf("|") > 0) {
			// Required for team-scoped queries selected from the team category
			// (from a Team Dashboard) in the chooser.
			// Format:  <project area item id>|<team area item id>
			var itemIds = queryCategory.split("|");
			return itemIds[0];
		}
		return queryCategory;
	},

	/*
 	 * Recieves notification when the setting areas is opened or closed.
 	 * 
 	 * @param boolean opened
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	settingsAreaToggled: function(opened) {
		// NO-OP		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return boolean
 	 * @overrides super
 	 */		
	hasZoomableContent: function() {
		if (this.getSite().isLoading()) {
			return false;
		}
		return true; 
	},	
	
	/*
 	 * @param String preferenceId
 	 * 
 	 * @return Array
 	 * @overrides super
 	 */		
	settingChanging: function(preferenceId) {
		
		if (preferenceId === this._static._PREFERENCE_ID_CATEGORY) {
			// Query category changed
			// Update grouping parameter preference list
			return [this._static._PREFERENCE_ID_PARAMETER];
		}
		return [];
	},
	
	/*
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	settingsChanged: function(oldSettings, newSettings) {
		// Relevant preferences
		var query = this._static._PREFERENCE_ID_QUERY;
		var presentation = this._static._PREFERENCE_ID_PRESENTATION;
		var parameter = this._static._PREFERENCE_ID_PARAMETER;
		var metagroups = this._static._PREFERENCE_ID_METAGROUPS;
		var othergroup = this._static._PREFERENCE_ID_OTHERGROUP;	
		var queryCategory = this._static._PREFERENCE_ID_CATEGORY;

		var doRefresh = false;
		var doUpdate = false;

		// Query
		if (oldSettings.preferences[query] != newSettings.preferences[query]) {	
			// Refresh required
			doRefresh = true;
			this._titleHyperlinkURL = null;
		}
		// Presentation
		if (oldSettings.preferences[presentation] != newSettings.preferences[presentation]) {
			// Update required
			doUpdate = true;
		}
		// Parameter
		if (oldSettings.preferences[parameter] != newSettings.preferences[parameter]) {
			// Refresh required
			doRefresh = true;
		}
		// Meta-Groups
		if (oldSettings.preferences[metagroups] !== newSettings.preferences[metagroups]) {
			// Refresh required
			doRefresh = true;
			this._patternGroups = null;
		}
		// Other Group
		if (oldSettings.preferences[othergroup] !== newSettings.preferences[othergroup]) {
			// Refresh required
			doRefresh = true;
		}
		// Query Category
		if (oldSettings.preferences[queryCategory] !== newSettings.preferences[queryCategory]) {
			// Refresh required
			doRefresh = true;
			this._projectAreaName = null;
			this._serverURL = null;			
		}
		
		if (doRefresh) {
			// Do refresh
			this.getSite().refresh();
		} else if (doUpdate) {
			// Do update
			this._updatePresentation();
		}
		// Update the primary title on every setting change in case the query name changes
		if (UUIDUtility.isValidUUID(newSettings.preferences[query]))
			this._updatePrimaryTitle(newSettings.preferences[query], true);	
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_updatePresentation: function() {
		// Disconnect the previous handler (if any)
		this._disconnectWindowResizeHandler();		
		// Get the cached chart parameters from the last REST service call
		var chartParameters = this._getCachedChartParameters();
		// Nothing to update if the viewlet has not been configured yet
		if (!chartParameters) {
			return;
		}
		// Display the report using cached parameters
		this._displayReport(chartParameters);
	},	
	
	/*
 	 * @param String preferenceId
 	 * 
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceProvider: function(preferenceId) {
		
		if (preferenceId === this._static._PREFERENCE_ID_QUERY) {
			return this._queryPreferenceProvider;
		} else if (preferenceId === this._static._PREFERENCE_ID_METAGROUPS) {
			return this._getMetagroupsPreferenceProvider();
		}
		return null;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getMetagroupsPreferenceProvider: function() {
		
		if (!this._metagroupsPreferenceProvider) {
			this._metagroupsPreferenceProvider = new MetagroupsPreferenceProvider();
		}
		return this._metagroupsPreferenceProvider;
	},

	/*
 	 * Called to update viewlet content (e.g. Settings changed)
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
		// Disconnect the previous handler (if any)
		this._disconnectWindowResizeHandler();		
		// Clear the cached chart parameters
		this._clearCachedChartParameters();

		// Verify that all required preferences are defined
		// Query
		var query = this.getPreference(this._static._PREFERENCE_ID_QUERY);
		if (!query ||
			(query === this._static._PREFERENCE_VALUE_UNDEFINED)) {
			this._displayMessageSelectQuery();
			return;
		} 
		// Parameter
		var parameter = this.getPreference(this._static._PREFERENCE_ID_PARAMETER);		
		if (!parameter) {
			this._displayErrorMessage(this.messages.msgMissingParameter);			
			return;
		}
		// Query Category
		var projectAreaItemID = this._getProjectAreaItemID(this.getPreference(this._static._PREFERENCE_ID_CATEGORY));		
		if (!projectAreaItemID) {
			this._displayErrorMessage(this.messages.msgMissingProjectArea);			
			return;
		}
		// Note: Presentation preference has a default value
		
		if (!this._local && !this._projectAreaName) {
			// Remote viewlet
			// Non-blocking request. Project area name used for content area
			// drill-in. Request will finish long before the presentation 
			// content is generated.
			this._getProjectAreaName(projectAreaItemID);
		}
		
		this._getWorkItemQuery();
	},

	/*
 	 * Called to update UI (e.g. Zoom)
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	update: function() {

		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		if (presentation === this._static._PREFERENCE_VALUE_TABLE) {
			// Get chart parameters
			var chartParameters = this._getCachedChartParameters();
			if (!chartParameters) {
				// Nothing to update if the viewlet has not been configured yet
				return;
			} else if (chartParameters.parameterCount <= this._static._MAX_TABLE_DISPLAY_ITEMS) {
				// No need to re-render.  No "Other" grouping exists for
				// expansion
				// Absolute table label widths needed to be updated to
				// prevent field overlapping 
				this._updateTableLabelElementWidths();		
				return;
			}
			
			this._createTable(chartParameters);
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return Number
 	 */	
	_getMaxTableItems: function() {
		if (this.isZoomed()) {
			return this._static._MAX_TABLE_DISPLAY_ITEMS_ZOOMED;
		}
		return this._static._MAX_TABLE_DISPLAY_ITEMS;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_removeFlashContent: function() {
		
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		
		if ((presentation === this._static._PREFERENCE_VALUE_PIE) ||
			(presentation === this._static._PREFERENCE_VALUE_COLUMN) ||
			(presentation === this._static._PREFERENCE_VALUE_BAR)) {
			// Remove flash object
			this._removeFlashChartObject();	
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_addFlashContent: function() {

		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);

		if ((presentation === this._static._PREFERENCE_VALUE_PIE) ||
			(presentation === this._static._PREFERENCE_VALUE_BAR) ||
			(presentation === this._static._PREFERENCE_VALUE_COLUMN)) {
			// Get the cached chart parameters from the last REST service call
			var chartParameters = this._getCachedChartParameters();
			// Nothing to update if the viewlet has not been configured yet
			if (!chartParameters) {
				return;
			}
			// Create flash object
			this._doOpeningAnimation = false;
			this._createFlashChart(chartParameters, presentation);
		}		
	},

	/*
 	 * Called just before a viewlet zoom operation is invoked
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	preZoom: function() {
		this._removeFlashContent();
	},

	/*
 	 * Called after a viewlet zoom operation is invoked
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	postZoom: function() {
		this._addFlashContent();	
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 * @overrides super
 	 */	
	getTitleHyperlinkURL: function() {
		
		if (this._titleHyperlinkURL) {
			return this._titleHyperlinkURL;
		}

		var queryItemID = this._getQueryItemID();
		
		if (this._local) {
			// Local viewlet
			if (!queryItemID) {
				// No work item query selected
				// Show Work Items welcome page
				this._titleHyperlinkURL = WorkItemUtil.getWelcomePageUri();
			} else {
				// Work item query selected
				// Show work item query results
				this._titleHyperlinkURL = WorkItemUtil.getRunSavedQueryUri(queryItemID);
			}
		} else {
			// Remote viewlet
			if (!queryItemID) {
				// No work item query selected
				// The project area is determined by the work item query.
				// There is no applicable current project area to use to build
				// a URL to the Work Items welcome page. 
				// No drill-in
				this._titleHyperlinkURL = null;
			} else {
				// Work item query selected
				// Show work item query results
				this._titleHyperlinkURL = this._getServerRoot() + "/resource/itemOid/com.ibm.team.workitem.query.QueryDescriptor/" + queryItemID;
			}
		}

		return this._titleHyperlinkURL;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getServerRoot: function() {
		
		if (typeof(this._serverRoot) === "string") {
			return this._serverRoot;
		}
		
		// e.g. Data server root:
		// Local:   undefined
		// Remote:  https://127.0.0.1:9444/jazz
		var root = dojo.getObject("net.jazz.ajax._dataServerRoot");
		if (!root) {
			// e.g. Context root:
			// Local:   /jazz
			// Remote:  https://127.0.0.1:9444/jazz 
			root = dojo.getObject("net.jazz.ajax._contextRoot");
		}
		if (!root) {
			root = "";
		}
		this._serverRoot = root;
		
		return this._serverRoot;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getImageServerRoot: function() {
		
		if (typeof(this._imageServerRoot) === "string") {
			return this._imageServerRoot;
		}		
		
		var serverRoot = this._getServerRoot();
		if (!serverRoot) {
			this._imageServerRoot = "";
			return this._imageServerRoot;
		}
		
		var root;
		var url = new dojo._Url(serverRoot);
		// Remove the path segments (e.g. /jazz)
		if (url.scheme && 
			url.host) {
			// e.g. URL
			// Remote: https://127.0.0.1:9444/jazz 			
			root = url.scheme + 
					"://" + 
					url.host;
			if (url.port) {
				root = root + ":" + url.port;
			}
			// Create a proxy url in order to embed images from another 
			// domain in the current domain.
			// Absolute links on their own work; however, security related
			// issues (e.g. firewall, invalid certificates, etc.) prevent the 
			// images from resolving in test environments.
			root = jazz.app.proxy.encodeProxiedURI(root);
		} else {
			// e.g. URL 
			// Local: /jazz
			root = "";
		}

		// e.g. Image server root
		// Local:   ""
		// Remote:  https://127.0.0.1:9444
		this._imageServerRoot = root;
		
		return this._imageServerRoot;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * 
 	 * @return boolean
 	 */	
	_isObjectDef: function(variable) {
	    if (typeof(variable) !== "object") {
			return false;
		} else if (variable === null) {
			return false;
		}
		return true;
	},	

	/*
 	 * @param Array variable
 	 * 
 	 * @return boolean
 	 */	
	_isArrayDef: function(variable) {
		if ((variable instanceof Array) === false) {
			return false;
		} else if (variable.length === 0) {
			return false;
		}
		return true;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * 
 	 * @return boolean
 	 */	
	_isStringDef: function(text) {
	    if (typeof(text) != 'string') {
			return false;
		} else if (text.length === 0) {
			return false;
		}		
		return true;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isObjectDefined: function(variable, throwError, variableName) {
		// Check if the variable exists (Any scope)
	    if (typeof(variable) != 'object') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgNotValidObjectType, [variableName]));
			}
			return false;
		}
		// Check if the variable is not null (with casting)
		if (variable == null) {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNull, [variableName]));
			}
			return false;
		}
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Number number
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isNumberDefined: function(number, throwError, variableName) {
		// Check if the number is a Number
	    if (typeof(number) != 'number') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotANumber, [variableName]));
			}
			return false;
		}
		
		return true;
	},			
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isStringDefined: function(text, throwError, variableName) {
		// Check if the text is null (with casting)
		if (text == null) {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNull, [variableName]));
			}
			return false;
		}		
		// Check if the text is a String
	    if (typeof(text) != 'string') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotAString, [variableName]));
			}
			return false;
		}
		// Check if the text has content
		if (text.length == 0) {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsZeroLength, [variableName]));
			}
			return false;
		}		
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Array variable
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 */	
	_isArray: function(variable, throwError, variableName) {
		// Check to see if the variable is an array
		if (variable instanceof Array) {
			return true;
		}
		if (throwError) {
			throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotAnArray, [variableName]));
		}
		return false;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @param String property
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 */	
	_hasProperty: function(variable, property, throwError, variableName) {
		// Check to see if the property is in the variable
		if (property in variable) {
			return true;
		}
		if (throwError) {
			throw new Error(dojo.string.substitute(this.messages.msgPropertyDoesNotExist, [variableName, property]));
		}
		return false;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Error
 	 * 
 	 * @return void
 	 */
	_logError: function(errorObject) {
		
		var prefix = "[ERROR]: ";
		var classID = "";
		var lineNumber = "";
		var message = "";

		// Class name
		if (this.id) {
			classID = this.id + ": ";
		} 

		if (errorObject) {
			// Line number
			if (errorObject.lineNumber) {
				lineNumber = "(line " + errorObject.lineNumber + "): ";
			} 
			// Error message
			if (errorObject.message) {
				message = errorObject.message;
			} else {
				message = "Message unavailable";
			}
		} else {
			// Error message
			message = "Undefined exception";			
		}
		
		var errorMessage = prefix +
							classID +
							lineNumber +
							message;
							
		console.error(errorMessage);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String warningText
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */
	_logWarning: function(warningText, errorObject) {
		var errorObjectText = "";
		if (errorObject && errorObject.message) {
			errorObjectText = ": " + errorObject.message;
		}
		
		var warningMessage = "[WARNING]: " +
							this._static._CLASSNAME + 
							": " +
							warningText +
							errorObjectText;
							
		console.warn(warningMessage);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String message
 	 * @param Error errorObject
 	 * @param Element node
 	 * @return <none>
 	 */	
	_updateErrorMessage: function(message, errorObject, node) {
		var text;
		if (errorObject) {
			text = dojo.string.substitute(this.messages.msgError, [message]);
			var errorMessage = errorObject.message;
			// Add the error message
			if ((typeof(errorMessage) == "string") &&
				(errorMessage.length != 0)) {
				text = text + 
						": " +
						errorMessage;
			} 
		}
		else {
			text = message;
		}
		// Update the message
		this._updateMessage(text, node, "#990000");	
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param Element node
 	 * @param boolean loading
 	 * @return <none>
 	 */	
	_updateLoadingMessage: function(text, node, loading) {
		// Set the loading viewlet indicator
		this.getSite().setLoading(loading);
		// Update the message
		this._updateMessage(text, node, "#BBBBBB");	
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param Element node
 	 * @param String color
 	 * 
 	 * @return void
 	 */	
	_updateMessage: function(text, node, color) {
		// Update text colour
		node.style.color = color;
		// Get the text node
		var textNode = node.firstChild;
		
		if ((textNode == null) &&
			(this._isStringDefined(text, false, null) == false)) {
			// Nothing to do
		} else if (this._isStringDefined(text, false, null) == false) {
			// Remove the old text node
			node.removeChild(textNode);
		} else if (textNode == null) {
			// Create a new text node
			textNode = document.createTextNode(text);
			node.appendChild(textNode);
		} else {
			// Modify the existing text node
			textNode.replaceData(0, textNode.length, text); 
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Node node
 	 * @return void
 	 */	
	_removeAllChildren: function(node) {
		// Remove all child nodes if any are defined
		while (node.firstChild != null) {
			node.removeChild(node.firstChild);
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param boolean show
 	 * @return <none>
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			// Show
			if (node.style.display != "block") {
				node.style.display = "block";	
			}
		} else {
			// Hide
			if (node.style.display != "none") {	
				node.style.display = "none";	
			}
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 * @overrides super
 	 */		
	destroy: function(){
		
		if (this._tagCloud) {
			this._tagCloud.destroy();
			this._tagCloud = null;
		}
		
		this._disconnectWindowResizeHandler();
		this.inherited(arguments);
		this._isDestroyed = true;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param <none>
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}

});

/*
 * Static Methods
 */
dojo.mixin(com.ibm.team.workitem.viewlets.web.ui.internal.StatisticsViewlet, {

	/*
 	 * Public Method
 	 * 
 	 * @param String chartID
 	 * @param String label
 	 * 
 	 * @return Boolean
 	 */	
	dispatchEventOnClickFlashChart: function(chartID, label) {
		
		if (!chartID) {
			return false;
		} else if (!label) {
			return false;
		}
		
		var widget = dijit.byId(chartID);
		
		if (!widget) {
			return false;
		}
		
		return widget.handleEventOnClickFlashChart(label);
	}
	
});

})();

