/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.TabCharts");

dojo.require("dijit._Templated");

dojo.require("dojo.i18n");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportDescriptorSelectionDialog");

dojo.require("com.ibm.team.apt.web.ui.contribution.ContributedTab");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {

var ReportsClient	= com.ibm.team.reports.web.client.internal.ReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var SelectReportDialog = com.ibm.team.reports.web.ui.internal.ReportDescriptorSelectionDialog;

dojo.declare("com.ibm.team.reports.web.ui.internal.TabCharts",
			 [com.ibm.team.apt.web.ui.contribution.ContributedTab, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_renderedReportDiv: null,

	constructor: function() {
		this._notYetShown= true;
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
	},

	// ---- api ------------------------------------------------------------------------------------------------------------	

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/TabCharts.html"),

	// ---- ContributedTab.js implementation -------------------------------------------------------------------------------

	onShowTab: function() {
		if (this._notYetShown) {
			var responseHandler = new ServiceResponseHandler(
				this,
				"_defaultReportCallback",
				"_remoteError");		
			ReportsClient.getDefaultReportDescriptor(responseHandler);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_defaultReportCallback: function(result) {
		if (result == null) {
			var responseHandler = new ServiceResponseHandler(
				this,
				"_existenceCallback",
				"_remoteError");		
			var clientArgs = {
				reportId: "apt.WorkItems"
			};
			ReportsClient.getDoesReportDescriptorExist(responseHandler, clientArgs);
		} else {
			this._displayReport(result.reportId);
		}
	},
	
	_existenceCallback: function(result) {
		if (result == "true") {
			this._displayReport("apt.WorkItems");
		} else {
			this._displayReport("workitems.OpenWorkItemsByType");
		}
	},
	
	_displayReport: function(id) {
		this._renderedReportDiv.innerHTML = "<span class='loading'>" + this.messages.loading + "</span>";	
		var iterations = this.getPlanIncludedIterations();
		var processAreas = this.getPlanIncludedTeamMemberAreas();
		
		var iterationParam = "";
		for (var i = 0; i < iterations.length; i++) {
			if (i > 0) {
				iterationParam += ",";
			}
			iterationParam += "'" + iterations[i].label + "'";
		}
		var processAreasParam = "";
		for (var i = 0; i < processAreas.length; i++) {
			if (i > 0) {
				processAreasParam + ",";
			}
			processAreasParam += "'" + processAreas[i].path + "'";
		}
		var clientArgs = {
			reportUUID: id,
			urlPrefix: "../../",
			webui: true,
			parameterName: ["Interval", "TeamAreaName"],
			parameterValue: [iterationParam, processAreasParam]
		};
		var responseHandler = new ServiceResponseHandler(
			this,
			"_displayRenderedReport",
			"_remoteError");		
	
		ReportsClient.postRenderReport(responseHandler, clientArgs);
	},
	
	_displayRenderedReport: function(result) {
		if (dojo.isIE) {
			// Insert innerHTML directly. That, coupled with defer tags
			// on the scripts, causes new Javascript to be executed properly.
			this._renderedReportDiv.innerHTML = result;
		} else {
			// Create a new node and appendChild so javascript gets executed.
			var newNode = document.createElement('div');
			newNode.innerHTML = result;
			dojox.data.dom.removeChildren(this._renderedReportDiv);
			this._renderedReportDiv.appendChild(newNode);
		} 
		this._selectButton.style.display = "inline";
	},
	
	_handleShowSelect: function() {
		var args = {
			parameters: ["ProjectAreaName", "TeamAreaName", "Interval"],
			flags: ReportsClient.EXCLUDE_MICRO
		};
		var responseHandler = new ServiceResponseHandler(
			this,
			"_displaySelectDialog",
			"_remoteError");				
		ReportsClient.getReportDescriptors(responseHandler, args);
	},
	
	_displaySelectDialog: function(reports) {
		var self = this;
		this.reportListWidget = new SelectReportDialog({reportTemplates: reports});
		this.connect(this.reportListWidget, "onOk", dojo.hitch(this, function(reportTemplate) {
			self._renderedReportDiv.innerHTML = "";
			self._selectButton.style.display = "none";
			self._displayReport(reportTemplate.reportId);
		}));
	},
	
	_remoteError: function(error) {
		alert(error.message);
		this._renderedReportDiv.innerHTML = "<span class='loading'>" + error.message + "</span>";	
		this._selectButton.style.display = "inline";
	}
});

})();

