/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.viewlets.web.ui.internal.TrendReportViewlet");

/*
 * Imports 
 */
dojo.require("dojo.i18n");
dojo.require("dojo.string");  
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.app.proxy");

dojo.require("com.ibm.team.reports.viewlets.web.ui.internal.utils.ReportsPreferenceProvider");

dojo.requireLocalization("com.ibm.team.reports.viewlets.web.ui.internal", "TrendReportMessages");

(function() {

/*
 * Aliases 
 */
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ReportsPreferenceProvider = com.ibm.team.reports.viewlets.web.ui.internal.utils.ReportsPreferenceProvider;
var HoverPopup = jazz.ui.HoverPopup;

/*
 * Class 
 */
dojo.declare("com.ibm.team.reports.viewlets.web.ui.internal.TrendReportViewlet",
	Viewlet, {

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "ui/internal/templates/TrendReportViewlet.html"),
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 

	/*
 	 * @param Object args
 	 * 
 	 * @return TrendReportViewlet
 	 */
	constructor: function(args) {
		
		// Messages //--------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.viewlets.web.ui.internal", "TrendReportMessages");

		// Icons // ----------------------------------------------------------->

		this._chevronDownSrc = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/icons/chevron_dn.gif");
		this._chevronUpSrc = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/icons/chevron_up.gif");
		
	},
	
	// Viewlet Methods -------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	init: function() {

		// Static Variables //------------------------------------------------->

		this._initializeStaticCommons();

		// Instance Variables //----------------------------------------------->

		// Local vs. Remote viewlet flag
		if (dojo.getObject("com.ibm.team.process.web.projectArea")) {
			this._local = true;
		} else {
			this._local = false;
		}
		
		this._messageNoResults = this.getPreference(this._static._PREFERENCE_ID_NO_RESULTS_MESSAGE);
		this._reportUUID = this.getPreference(this._static._PREFERENCE_ID_REPORT);
		
		this._savedQueryUUID = this.getPreference(this._static._PREFERENCE_ID_QUERY_UUID);
		this._transientQueryUUID = null;

		// TODO: MP: POST 2.0: Figure out better way to share code
		// Preference provider uses:
		// * _logError
		// * _displayErrorMessage
		// * _reportUUID
		// * _PREFERENCE_ID_QUERY_UUID
		// * _isValidUUID
		// * getSite
		// * getPreference
		// * getServiceTracker
		// * _alertErrorMessage
		// * _projectAreaWhitelist
		// * _getImageServerRoot
		// * _getDrillInServerRoot
		this._preferenceProvider = new ReportsPreferenceProvider({
			context: this,
			projectAreaUUID: this._getProjectAreaUUID()
		});
		
		this._parameterHoverPopups = [];
		this._parametersAreaOpen = false;

		this._projectAreaWhitelist = {
			stale: true,
			entries: {}
		};

		// Drill-in related variables
		this._titleHyperlinkURL = null; 
		this._serverRoot = null;
		this._serverURL = null;
		this._parentReportUUID = this._getParentReportUUID();
		this._canViewDashboard = this.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA);

		// Setup //------------------------------------------------------------>

		this._initParametersArea();
	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function(){
		
		if (this._isValidUUID(this._transientQueryUUID)) {
			com.ibm.team.reports.viewlets.web.ui.internal.TrendReportViewlet.deleteTransientReportQueryUUID(this._transientQueryUUID, this.id);
			delete this._transientQueryUUID;
		}
		
		this._deleteParameterHoverPopups();
		
		this._preferenceProvider.destroy();
		
		this.inherited(arguments);
		this._isDestroyed = true;
	},

	/*
 	 * @param Element node
 	 * @param boolean show
 	 * 
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			node.style.display = "block";	
		} else {
			node.style.display = "none";	
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initParametersArea: function() {
		this.connect(this._chevronDownControl, "onclick", this._handleEventChevDownLinkClicked);
		this.connect(this._chevronUpControl, "onclick", this._handleEventChevUpLinkClicked);
	},

	/*
 	 * @param Deferred deferred
 	 * 
 	 * @return void
 	 */	
	_getProjectAreaWhitelist: function(deferred) {
		
		if (this._projectAreaWhitelist.stale === false) {
			deferred.callback({success: true});
			return;
		}
		
		var self = this;
		var serviceObject = {
			success: function(list) {
				self._processProjectAreaWhitelist(list);
				deferred.callback({success: true});
			},
			failure: function(errorObject) {
				self._displayErrorMessage(self.messages.msgWhitelistError, errorObject);
				self._logError(errorObject);
				self.getSite().doneRefresh();
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");		
		
		this._callRESTGetContainingProjectAreaItemIds(handler);
	},

	/*
 	 * @param Array list
 	 * 
 	 * @return void
 	 */	
	_processProjectAreaWhitelist: function(list) {
		
		this._projectAreaWhitelist.stale = false;
		this._projectAreaWhitelist.entries = {};
		
		if (((list instanceof Array) === false) ||
			(list.length === 0)) {
			this._logWarning(new Error("Report is not deployed to any project area: " + this._reportUUID));
			return;
		}
		// Index list		
		for (var index = 0; index < list.length; index++) {
			var projectAreaUUID = list[index]; 
			this._projectAreaWhitelist.entries[projectAreaUUID] = true;
		}
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventChevDownLinkClicked: function(event) {
		dojo.stopEvent(event);
		
		this._updateVisibility(this._chevronDownControl, false);
		this._updateVisibility(this._chevronUpControl, true);
		this._updateVisibility(this._parameterTable, true);
		
		this._parametersAreaOpen = true;
		
		this._addParameters();
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventChevUpLinkClicked: function(event) {
		dojo.stopEvent(event);
		
		this._updateVisibility(this._chevronDownControl, true);
		this._updateVisibility(this._chevronUpControl, false);
		this._updateVisibility(this._parameterTable, false);
		
		this._parametersAreaOpen = false;
		
		return true;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	update: function() {
		// Called to update UI (e.g. Zoom)
		this.getSite().refresh();
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 * @overrides super
 	 */	
	getTitleHyperlinkURL: function() {
		
		// Check license
		if (!this._canViewDashboard) {
			// No Reports Web UI vertical
			return null;
		} else if (this._titleHyperlinkURL) {
			// Note: Changes to the project area or report query render this link stale
			return this._titleHyperlinkURL;
		}
		
		var queryUUID = this._getReportQueryUUID();

		if (this._local) {
			// Local viewlet
			if (this._isValidUUID(queryUUID)) {
				// Report query selected
				// Show report query results
				// e.g. 
				// #action=com.ibm.team.reports.viewReport
				// &queryUUID=_jCf14DWSEd6DNqVVoYmf3w
				// &reportUUID="workitems.OpenWorkItemsByType"
				// &projectAreaUUID=_VYHd8EVTEd6_vsgEcFTCrA
				this._titleHyperlinkURL = this._static._URI_VIEWREPORT_ACTION + 
											this._static._URI_COMP_QUERY + 
											queryUUID +
											this._static._URI_COMP_REPORT +
											this._parentReportUUID + 
											this._static._URI_COMP_PROJECT +
											this._getProjectAreaUUID();
			} else {
				// No report query selected
				// Show Reports welcome page
				this._titleHyperlinkURL = this._static._URI_VIEWPAGE_ACTION;
			}
		} else {
			// Remote viewlet
			if (this._isValidUUID(queryUUID)) {
				// Report query selected
				// Show report query results
				// e.g. 
				// https://localhost:9443/jazz/web/projects/Rational%20Team%20Concert?debug=true
				// #action=com.ibm.team.reports.viewReport
				// &queryUUID=_jCf14DWSEd6DNqVVoYmf3w
				// &reportUUID="workitems.OpenWorkItemsByType"
				// &projectAreaUUID=_VYHd8EVTEd6_vsgEcFTCrA
				this._titleHyperlinkURL = this._getServerURL() +
											this._static._URI_VIEWREPORT_ACTION + 
											this._static._URI_COMP_QUERY + 
											queryUUID +
											this._static._URI_COMP_REPORT +
											this._parentReportUUID + 
											this._static._URI_COMP_PROJECT +
											this._getProjectAreaUUID();
			} else {
				// No report query selected
				// Electing not to build a URL to the Reports welcome page if
				// the project area is specified because it is too much of an
				// edge case.
				// No drill-in
				this._titleHyperlinkURL = null;
			}
		}
		
		return this._titleHyperlinkURL;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return boolean
 	 * @overrides super
 	 */		
	hasZoomableContent: function() {
		if (this.getSite().isLoading()) {
			return false;
		}
		return true; 
	},

	/*
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	settingsChanged: function(oldSettings, newSettings) {
		// Viewlet settings area: "Save" button pressed

		var newQueryUUID = newSettings.preferences[this._static._PREFERENCE_ID_QUERY_UUID];
		var oldQueryUUID = oldSettings.preferences[this._static._PREFERENCE_ID_QUERY_UUID];
		
		// Query
		var queryChanged;
		if (newQueryUUID === oldQueryUUID) {
			queryChanged = false;
		} else {
			queryChanged = true;
			this._titleHyperlinkURL = null;
		}

		// Project Area
		var scopeChanged;
		if (oldSettings.scopeItemId === newSettings.scopeItemId) {
			scopeChanged = false;
		} else {
			scopeChanged = true;
			this._serverURL = null;
			this._titleHyperlinkURL = null;
		}
		
		if (this._isValidUUID(newQueryUUID) === false) {
			this._logWarning(new Error(this.messages.msgInvalidQueryUUID));
		} else if (this._isReportDeployed() === false) {
			this._displayActionReportNotDeployed();
		} else if (queryChanged) {
			// Report query was created / updated
			// Save the report query in permanent storage
			this._resaveQuery(newQueryUUID, dojo.hitch(this, function(queryUUID) {
				this._setSavedReportQueryUUID(queryUUID);
				this._updatePrimaryTitle(queryUUID, true);
				this.getSite().refresh();			
			}));
		} else if (scopeChanged) {
			// Project area was selected / updated
			// Report query exists but has not changed
			// Save the report query in the new project area
			this._resaveQuery(newQueryUUID, dojo.hitch(this, function(queryUUID) {
				this.getSite().refresh();			
			}));
		}		
		
	},

	/*
 	 * @param Object canceledSettings
 	 * @param Object restoredSettings
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	settingsCanceled: function(canceledSettings, restoredSettings) {
		// Viewlet settings area:  "Cancel" button pressed
		
		var newQueryUUID = canceledSettings.preferences[this._static._PREFERENCE_ID_QUERY_UUID];
		var oldQueryUUID = restoredSettings.preferences[this._static._PREFERENCE_ID_QUERY_UUID];

		if (newQueryUUID !== oldQueryUUID) {
			// Delete the canceled query
			this._deleteReportQueryUUID(newQueryUUID, true);
		}
	},

	/*
 	 * @param String queryUUID
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_resaveQuery: function(queryUUID, callback) {

		var projectAreaUUID = this._getProjectAreaUUID();
		if (!projectAreaUUID) {
			this._logError(new Error("Project area is not defined"));
			this._displayErrorMessage(this.messages.msgQuerySaveFailed);
			return;
		}			

		var self = this;
		var serviceObject = {
			success: function(queryUUID2){
				if (self._isValidUUID(queryUUID2) === false) {
					self._logError(new Error(self.messages.msgInvalidQueryUUID));
					self._displayErrorMessage(self.messages.msgQuerySaveFailed, errorObject);
				} else if (queryUUID !== queryUUID2) {
					self._logError(new Error(self.messages.msgQueryMismatchError));
					self._displayErrorMessage(self.messages.msgQuerySaveFailed, errorObject);
				} else {
					callback(queryUUID2);
				}
			},
			failure: function(errorObject){
				self._logError(errorObject);
				self._displayErrorMessage(self.messages.msgQuerySaveFailed, errorObject);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		this._callRESTPostSaveQueryDescriptor(
			handler, 
			this._reportUUID,
			projectAreaUUID,
			queryUUID);
	},		

	/*
 	 * @param String queryUUID
 	 * @param Boolean transientQuery
 	 * 
 	 * @return void
 	 */	
	_deleteReportQueryUUID: function(queryUUID, transientQuery) {

		var self = this;
		var serviceObject = {
			success: function() {
				if (transientQuery) {
					self._logInfo(new Error("Transient query deleted: " + queryUUID));
				} else {
					self._logInfo(new Error("Saved query deleted: " + queryUUID));
				}
			},
			failure: function(errorObject) {
				if (transientQuery) {
					self._logError(new Error("Failed to delete transient query: " + queryUUID));
				} else {
					self._logError(new Error("Failed to delete saved query:" + queryUUID));
				}
				self._logError(errorObject);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		this._callRESTPostDeleteQueryDescriptor(
			handler, 
			queryUUID,
			transientQuery);
	},	

	/*
 	 * @param String queryUUID
 	 * @param Boolean persist
 	 * 
 	 * @return void
 	 */	
	_updatePrimaryTitle: function(queryUUID, persist) {

		var queryLabel = this._preferenceProvider.getCachedQueryLabel(queryUUID);
		if (queryLabel) {
			this.getSite().setTitle(queryLabel, persist);
		}	
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticCommons: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

		this._static._MAX_VISIBLE_PARAMETERS = 3;

		this._static._PREFERENCE_ID_REPORT = "report";	
		this._static._PREFERENCE_ID_QUERY_UUID = "reportQueryUUID";	
		this._static._PREFERENCE_ID_REPORT_PARENT = "reportParent";	
		this._static._PREFERENCE_ID_NO_RESULTS_MESSAGE = "noResultsMessage";	

		this._static._REPORT_REST_SERVICE_URI = "com.ibm.team.reports.common.internal.service.IReportRestService";

		this._static._PARAMETER_NAME_TEAM = "TeamAreaName";
		this._static._PARAMETER_NAME_CATEGORY = "Category";
		this._static._PARAMETER_NAME_DURATION = "WhichBuilds";
		this._static._PARAMETER_NAME_ITERATION = "Interval";

		// TODO: MP: POST 2.0: Use extension point post 2.0
		// Work Item Micro Reports
		this._static._WORK_ITEMS_OPEN_VS_CLOSED = "workitems.MicroOpenVsClosedWorkItems";
		this._static._WORK_ITEMS_OPEN_BY_TYPE = "workitems.MicroOpenWorkItemsByType";
		this._static._WORK_ITEMS_OPEN_BY_PRIORITY = "workitems.MicroOpenWorkItemsByPriority";
		this._static._WORK_ITEMS_BLOCKING = "workitems.MicroBlockingWorkItems";
		this._static._WORK_ITEMS_NEW_BY_SEVERITY = "workitems.MicroNewWorkItemsBySeverity";
		this._static._WORK_ITEMS_CLOSED_BY_PRIORITY = "workitems.MicroClosedWorkItemsByPriority";
		this._static._WORK_ITEMS_BURNDOWN = "workitems.MicroBurndown";
		this._static._WORK_ITEMS_BURNUP = "workitems.MicroBurnup";
		this._static._WORK_ITEMS_RELEASE_BURNDOWN = "workitems.MicroReleaseBurndown";
		this._static._WORK_ITEMS_STORY_POINTS = "workitems.MicroStoryPoints";
		this._static._WORK_ITEMS_STORY_POINTS_REMAINING = "workitems.MicroStoryPointsRemaining";
		this._static._WORK_ITEMS_ESTIMATED_VS_ACTUAL_WORK = "workitems.MicroEstimatedVsActualWork";
		this._static._WORK_ITEMS_TEAM_VELOCITY = "workitems.MicroTeamVelocity";
		this._static._WORK_ITEMS_CLOSED_DAILY = "workitems.MicroClosedWorkItemsDaily";
		this._static._WORK_ITEMS_COMPARISON = "workitems.MicroWorkItemComparison";
		this._static._WORK_ITEMS_DEFERRED = "workitems.MicroDeferredWorkItems";

		// TODO: MP: POST 2.0: Use extension point post 2.0
		// Build Micro Reports
		this._static._REPORT_BUILD_HEALTH = "build.MicroBuildHealth";
		this._static._REPORT_BUILD_DURATION = "build.MicroBuildDuration";
		this._static._REPORT_JUNIT_TESTS = "build.MicroJUnitTests";
		this._static._REPORT_JUNIT_FAILURES = "build.MicroJUnitFailures";
		this._static._REPORT_COMPILATION_WARNINGS = "build.MicroCompilationWarnings";
		this._static._REPORT_FREQUENT_TEST_FAILURES = "build.MicroFrequentTestFailures";
		this._static._REPORT_FREQUENT_TEST_REGRESSIONS = "build.MicroFrequentTestRegressions";

		// TODO: MP: POST 2.0: Use extension point post 2.0
		// Source Control Micro Reports
		this._static._REPORT_FILE_CHANGES = "scm.MicroFileChangeActivity";
		this._static._REPORT_CHANGE_SETS = "scm.MicroChangeSetActivity";
		
		// TODO: MP: POST 2.0: Use extension point post 2.0
		// Mapping of micro reports to normal reports
		this._static._microReportIDMap = new Array();
		// Work Items
		this._static._microReportIDMap[this._static._WORK_ITEMS_OPEN_VS_CLOSED] = "workitems.OpenVsClosedWorkItems";
		this._static._microReportIDMap[this._static._WORK_ITEMS_OPEN_BY_TYPE] = "workitems.OpenWorkItemsByType";
		this._static._microReportIDMap[this._static._WORK_ITEMS_OPEN_BY_PRIORITY] = "workitems.WorkItemsByPriority";
		this._static._microReportIDMap[this._static._WORK_ITEMS_BLOCKING] = "workitems.BlockingWorkItems";
		this._static._microReportIDMap[this._static._WORK_ITEMS_NEW_BY_SEVERITY] = "workitems.NewWorkItemsBySeverity";
		this._static._microReportIDMap[this._static._WORK_ITEMS_CLOSED_BY_PRIORITY] = "workitems.WorkItemsByPriority";
		this._static._microReportIDMap[this._static._WORK_ITEMS_BURNDOWN] = "workitems.Burndown";
		this._static._microReportIDMap[this._static._WORK_ITEMS_BURNUP] = "workitems.Burnup";
		this._static._microReportIDMap[this._static._WORK_ITEMS_RELEASE_BURNDOWN] = "workitems.ReleaseBurndown";
		this._static._microReportIDMap[this._static._WORK_ITEMS_STORY_POINTS] = "workitems.StoryPoints";
		this._static._microReportIDMap[this._static._WORK_ITEMS_STORY_POINTS_REMAINING] = "workitems.StoryPointsRemaining";
		this._static._microReportIDMap[this._static._WORK_ITEMS_ESTIMATED_VS_ACTUAL_WORK] = "workitems.EstimatedVsActualWork";
		this._static._microReportIDMap[this._static._WORK_ITEMS_TEAM_VELOCITY] = "workitems.TeamVelocity";
		this._static._microReportIDMap[this._static._WORK_ITEMS_CLOSED_DAILY] = "workitems.ClosedWorkItemsDaily";
		this._static._microReportIDMap[this._static._WORK_ITEMS_COMPARISON] = "workitems.WorkItemComparison";
		this._static._microReportIDMap[this._static._WORK_ITEMS_DEFERRED] = "workitems.DeferredWorkItems";
		
		// TODO: MP: POST 2.0: Use extension point post 2.0
		// Builds
		this._static._microReportIDMap[this._static._REPORT_BUILD_HEALTH] = "build.BuildHealth";
		this._static._microReportIDMap[this._static._REPORT_BUILD_DURATION] = "build.BuildHealth";
		this._static._microReportIDMap[this._static._REPORT_JUNIT_FAILURES] = "build.CodeHealth";
		// TODO: MP: POST 2.0: Check: No associated normal report:  this._static._REPORT_JUNIT_TESTS
		this._static._microReportIDMap[this._static._REPORT_COMPILATION_WARNINGS] = "build.CodeHealth";
		this._static._microReportIDMap[this._static._REPORT_FREQUENT_TEST_FAILURES] = "build.FrequentTestFailures";
		this._static._microReportIDMap[this._static._REPORT_FREQUENT_TEST_REGRESSIONS] = "build.FrequentTestRegressions";

		// TODO: MP: POST 2.0: Use extension point post 2.0
		// Source Control
		this._static._microReportIDMap[this._static._REPORT_FILE_CHANGES] = "scm.ProjectActivity";
		this._static._microReportIDMap[this._static._REPORT_CHANGE_SETS] = "scm.ProjectActivity";
		
		// URL
		this._static._URI_VIEWPAGE_ACTION = "#action=jazz.viewPage&id=com.ibm.team.reports";
		this._static._URI_VIEWREPORT_ACTION = "#action=com.ibm.team.reports.viewReport";
		this._static._URI_COMP_QUERY = "&queryUUID=";
		this._static._URI_COMP_REPORT = "&reportUUID=";
		this._static._URI_COMP_PROJECT = "&projectAreaUUID=";

		// Special parameter values
		this._static._SPECIAL_CURRENT_ITERATION = "{Current Iteration}";
		this._static._SPECIAL_MY_TEAM_AREAS = "{My Team Areas}";
		
		// Dashboard drill-in URL components
		this._static._DASHBOARD_DRILL_ACTION_VIEW_TEAM = "#action=com.ibm.team.dashboard.viewDashboard&team=";
		this._static._DASHBOARD_DRILL_ACTION_VIEW_ALL = "#action=com.ibm.team.dashboard.viewAllDashboards";
		
		// Memento data
		this._static._MEMENTO_PARAMETERS = "parameters";
		
		// Content area parameter section
		this._static._UNKNOWN_PARAMETER = "(?)";
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
		var deferred = new dojo.Deferred();
		deferred.addCallback(dojo.hitch(this, this._doRefresh));
		this._getProjectAreaWhitelist(deferred);
	},

	/*
 	 * @param Object result
 	 * 					. Boolean success
 	 * 
 	 * @return Object
 	 */	
	_doRefresh: function(result) {
		
		if (!this._reportUUID) {
			// No report ID
			var errorObject = new Error(this.messages.msgMissingReportID); 
			this._logError(errorObject);
			this._displayErrorMessage(this.messages.msgUnknownReportType, errorObject);
			this.getSite().doneRefresh();
			return result;
		} else if (!this._getProjectAreaUUID() ||
					!this._getProjectAreaName()) {
			// No project area
			// Remote viewlet
			if (this._canEditDashboard()) {
				this._displayMsgSelectPreferences();
			} else {
				this._displayMessage(this.messages.msgUnconfiguredReport);
			}			
			this.getSite().doneRefresh();
			return result;			
		} else if (this._isReportDeployed() === false) {
			// Report not deployed
			this._displayMsgReportNotDeployed();	
			this._projectAreaWhitelist.stale = true;
			this.getSite().doneRefresh();
			return result;
		}

		// TEST
		//this._preferenceProvider._debugDefaultValues();

		var queryUUID = this._getReportQueryUUID();
		
		if (this._isValidUUID(queryUUID)) {
			this._getReport(queryUUID);
		} else if (this._hasPrimedData()) {
			this._getPrimedValuesReport();
		} else {
			this._getDefaultValuesReport();
		}
		return result;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */		
	_isReportDeployed: function() {
		var projectAreaUUID = this._getProjectAreaUUID();
		if (!projectAreaUUID) {
			return false;
		} else if ((projectAreaUUID in this._projectAreaWhitelist.entries) === false) {
			return false;
		}
		return true;
	},

	/*
 	 * @param String preferenceId
 	 * 
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceProvider: function(preferenceId) {
		return this._preferenceProvider;		
	},

	/*
	 * @param void
	 * 
	 * @return Boolean
	 */
	_canEditDashboard: function() {
		if (this.getSite()._container.dashboardBundle.permissions.modify) {
			return true;
		}
		return false;		
	},		
	
	/*
	 * @param void
	 * 
	 * @return Boolean
	 */
	_hasPrimedData: function() {
		var data = this.getViewletState().get(this._static._MEMENTO_PARAMETERS);
		if (data) {
			return true;
		}
		return false;
	},	
	
	/*
	 * @param void
	 * 
	 * @return void
	 */
	_getPrimedValuesReport: function() {
		
		var data = this.getViewletState().get(this._static._MEMENTO_PARAMETERS);
		// TEST
		//console.debug(data);
			
		this._preferenceProvider.getPrimedValuesQuery(
			data,
			dojo.hitch(this, this._processTransientQuery));
	},		
	
	/*
	 * @param void
	 * 
	 * @return String
	 */
	_getReportQueryUUID: function() {
		if (this._savedQueryUUID) {
			return this._savedQueryUUID;
		} else if (this._transientQueryUUID) {
			return this._transientQueryUUID;
		}
		return "";
	},	
	
	/*
	 * @param String queryUUID
	 * 
	 * @return void
	 */
	_setSavedReportQueryUUID: function(queryUUID) {

		// BUG:
		// (1) Auto-save off
		// (2) Change query preference
		// (3) Save preferences
		// (4) Do not save Dashboard
		// (5) Reload page
		//     => New query not saved
		//     => Old query deleted
		//     => Error: Query descriptor not found  
		
		if (this._transientQueryUUID) {
			// Delete the transient query
			this._deleteReportQueryUUID(this._transientQueryUUID, true);
		} 
		//else if (this._savedQueryUUID) {
			// Delete the old saved query
			// BUG: Saved queries are leaking
			// Cannot delete the saved query because this viewlet may have been
			// duplicated from another viewlet still using that query.
			// See Work Item 85220 for details.
			//this._deleteReportQueryUUID(this._savedQueryUUID, false);
		//}

		// Set the new saved query
		// (Discard the transient or old saved query reference if applicable)
		this._savedQueryUUID = queryUUID;
		this._transientQueryUUID = null;
		
		// Remove primed data
		if (this._hasPrimedData()) {
			this.getViewletState().set(this._static._MEMENTO_PARAMETERS, null);
			this._logInfo(new Error("Parameters memento data deleted."));
		}
	},		
	
	/*
	 * @param String queryUUID
	 * 
	 * @return void
	 */
	_setTransientReportQueryUUID: function(queryUUID) {
		
		if (this._transientQueryUUID) {
			this._logError(new Error("Transient query UUID already exists"));
		} else if (this._savedQueryUUID) {
			this._logError(new Error("Saved query UUID already exists"));
		}
		
		this._transientQueryUUID = queryUUID;
	},		
	
	/*
	 * @param void
	 * 
	 * @return void
	 */
	_getDefaultValuesReport: function() {
		this._preferenceProvider.getDefaultValuesQuery(dojo.hitch(this, this._processTransientQuery));	
	},		

	/*
	 * @param String queryUUID
	 * 
	 * @return void
	 */
	_processTransientQuery: function(queryUUID) {
		
		if (this._isValidUUID(queryUUID)) {
			this._setTransientReportQueryUUID(queryUUID);
			this._getReport(queryUUID);
		} else if (this._canEditDashboard()) {
			this._displayMsgSelectPreferences();
			this.getSite().doneRefresh();
		} else {
			this._displayMessage(this.messages.msgUnconfiguredReport);
			this.getSite().doneRefresh();
		}
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_addParameters: function() {
		
		// TODO: MP: Add updateParameters method?
		// TODO: MP: Check if parameters / scope changed before re-adding parameters?
		// TODO: MP: POST 2.0: Add animation
		// TODO: MP: POST 2.0: Should add loading message
		
		// Note: Safe to get meta data since the report descriptor has already
		// been saved at this point
		var projectAreaUUID = this._getProjectAreaUUID();
		if (!projectAreaUUID) {
			this._logError(new Error("Project area is not defined"));
			return;
		}		
		
		var queryUUID = this._getReportQueryUUID();
		if (this._isValidUUID(queryUUID) === false) {
			this._logError(new Error(this.messages.msgInvalidQueryUUID));
			return;
		}

		this._preferenceProvider.getParametersMetaData(
			projectAreaUUID, 
			queryUUID, 
			dojo.hitch(this, this._addParameterRows));
			
	},

	/*
 	 * @param Object data
 	 * 
 	 * @return void
 	 */	
	_addParameterRows: function(data) {

		this._deleteParameterHoverPopups();

		var newTableBody = document.createElement("tbody");
		dojo.addClass(newTableBody, "parameter-rows");
		newTableBody.id = this._getParameterRowsID();

		// Project Area
		this._addProjectAreaRow(newTableBody);
		
		// TEST
		// console.debug(data);
		
		for (var name in data) {
			var parameter = data[name];
			
			// Row
			var row = document.createElement("tr");		

			// Column 1
			var col1 = document.createElement("td");
			dojo.addClass(col1, "label");
			col1.innerHTML = parameter.displayName + ":";

			// Column 2
			var col2 = document.createElement("td");
			dojo.addClass(col2, "value");
			var valueNode = this._getFormattedParameterValues(name, parameter.values);
			col2.appendChild(valueNode);

			// Add columns to row
			row.appendChild(col1);
			row.appendChild(col2);

			// Add row to table
			newTableBody.appendChild(row);
		}

		var oldTableBody = dojo.byId(this._getParameterRowsID());		
		
		try {
			if (oldTableBody) {
				this._parameterTable.replaceChild(newTableBody, oldTableBody);
			} else {
				this._parameterTable.appendChild(newTableBody);
			}
		} catch (errorObject) {
			this._logError(errorObject);
			this._removeAllChildren(this._parameterTable);
			this._parameterTable.appendChild(newTableBody);
		}

	},

	/*
 	 * @param Element tableBody
 	 * 
 	 * @return void
 	 */	
	_addProjectAreaRow: function(tableBody) {

		// Row
		var row = document.createElement("tr");		

		// Column 1
		var col1 = document.createElement("td");
		dojo.addClass(col1, "label");
		col1.innerHTML = this.messages.msgProjectArea + ":";
		// Column 2
		var col2 = document.createElement("td");
		dojo.addClass(col2, "value");
		col2.innerHTML = this._getProjectAreaName();

		// Add columns to row
		row.appendChild(col1);
		row.appendChild(col2);
		
		// Add row to table
		tableBody.appendChild(row);
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getServerRoot: function() {
		
		if (typeof(this._serverRoot) === "string") {
			return this._serverRoot;
		}
		
		// e.g. Data server root:
		// Local:   undefined
		// Remote:  https://127.0.0.1:9444/jazz
		var root = dojo.getObject("net.jazz.ajax._dataServerRoot");
		if (!root) {
			// e.g. Context root:
			// Local:   /jazz
			// Remote:  https://127.0.0.1:9444/jazz 
			root = dojo.getObject("net.jazz.ajax._contextRoot");
		}
		if (!root) {
			root = "";
		}
		this._serverRoot = root;
		
		return this._serverRoot;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */
	_getServerURL: function() {
		// e.g.
		// https://localhost:9443/jazz/web/projects/Rational%20Team%20Concert?debug=true
		
		if (this._serverURL) {
			return this._serverURL;
		}
		
		var url = this._getServerRoot() + 
					"/web/projects/" +
					encodeURIComponent(this._getProjectAreaName());
		
		if (djConfig.isDebug) {
			url = url + "?debug=true";
		}

		this._serverURL = url;
		
		return this._serverURL;
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 * 				String name
 	 * 				String itemId
 	 */	
	_getProjectArea: function() {
		// Report via saved query
		var scopeItem = this.getScopeItem();
		if (scopeItem) {
			return scopeItem;
		}
		// Report via primed values
		// Report via default values
		if (this._local) {
			// Local viewlet
			return com.ibm.team.process.web.projectArea;
		}
		// Remote viewlet
		return null;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getProjectAreaName: function() {
		var projectArea = this._getProjectArea();
		if (projectArea &&
			projectArea.name) {
			return projectArea.name;
		} 
		return null;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getProjectAreaUUID: function() {
		var projectArea = this._getProjectArea();
		if (projectArea &&
			projectArea.itemId) {
			return projectArea.itemId;
		} 
		return null;
	},

	/*
 	 * @param String name
 	 * @param Array values
 	 * 
 	 * @return Element
 	 */	
	_getFormattedParameterValues: function(name, values) {
		
		if (values.length <= this._static._MAX_VISIBLE_PARAMETERS) {
			return this._getCustomFormatParameterValues(name, values);
		}

		var subsetValues = values.slice(0, this._static._MAX_VISIBLE_PARAMETERS);
		var formattedValuesElement = this._getCustomFormatParameterValues(name, subsetValues);
		this._addParameterEllipses(formattedValuesElement, name, values);
		
		return formattedValuesElement;
	},

	/*
 	 * @param String name
 	 * @param Array values
 	 * 
 	 * @return Element
 	 */	
	_getCustomFormatParameterValues: function(name, values) {
		
		if (name === this._static._PARAMETER_NAME_TEAM) {
			// Team Area
			return this._getFormattedTeamAreaValues(values);
		} else if (name === this._static._PARAMETER_NAME_CATEGORY) {
			// Category
			return this._getFormattedCategoryValues(values);
		} else if (name === this._static._PARAMETER_NAME_DURATION) {
			// Duration
			return this._getFormattedDurationValues(values);
		} else if (name === this._static._PARAMETER_NAME_ITERATION) {
			// Iteration
			return this._getFormattedIterationValues(values);
		}

		return this._getFormattedOtherValues(values);
	},

	/*
 	 * @param Array values
 	 * 
 	 * @return Element
 	 */	
	_getFormattedTeamAreaValues: function(values) {
		
		var content = document.createElement("div");
		
		if ((values.length === 1) && !values[0]) {
			// All
			var allText = document.createTextNode(this.messages.msgAll);
			var allLink = document.createElement("a");
			// e.g.
			// https://localhost:9443/jazz/web/projects/Rational%20Team%20Concert?debug=true
			// #action=com.ibm.team.dashboard.viewAllDashboards
			allLink.href = this._getServerURL() + 
							this._static._DASHBOARD_DRILL_ACTION_VIEW_ALL;
			allLink.appendChild(allText);
			content.appendChild(allLink);			
			return content;
		}			

		// List		
		for (var index = 0; index < values.length; index++) {
			
			var rawValue = values[index];
			var node;
			
			if (!rawValue) {
				// Error
				node = document.createTextNode(this._static._UNKNOWN_PARAMETER);
			} else if (rawValue === this._static._SPECIAL_MY_TEAM_AREAS) {
				// My Team Areas
				node = document.createTextNode(this.messages.msgMyTeamAreas);
			} else {
				// Display last segment only
				// e.g. 
				// Before: /Jazz Development/Dashboard
				// After:  Dashboard
				var segments = rawValue.split(/\//);
				var displayValue = segments[segments.length - 1];
				var displayText = document.createTextNode(displayValue);
				
				node = document.createElement("a");
				node.title = rawValue;
				node.href = this._getTeamDashboardURL(rawValue);
				node.appendChild(displayText);
			}
			
			content.appendChild(node);

			// Comma separated
			if (index !== (values.length - 1)) {
				var comma = document.createTextNode(", ");
				content.appendChild(comma);
			}
		}	
		
		return content;			
	},

	/*
 	 * @param String rawTeamArea
 	 * 
 	 * @return String
  	 */	
	_getTeamDashboardURL: function(rawTeamArea) {
		
		// Replace the first forward slash (if any)
		// e.g.
		// Before: /RTC Development/Repository/Gateway
		// After:  RTC Development/Repository/Gateway
		var teamArea = rawTeamArea.replace(/^\//, "");
		
		// Encode component parts
		// e.g.
		// Before: RTC Development/Repository/Gateway
		// After:  RTC%20Development/Repository/Gateway
		var delimeter = "/";
		var segments = teamArea.split(delimeter);
		for (var index = 0; index < segments.length; index++) {
			segments[index] = encodeURIComponent(segments[index]);
		}		
		var teamDashboard = segments.join(delimeter);
		
		// e.g.
		// https://localhost:9443/jazz/web/projects/Rational%20Team%20Concert?debug=true
		// #action=com.ibm.team.dashboard.viewDashboard&team=RTC%20Development/Repository/Gateway
		var url = this._getServerURL() + 
					this._static._DASHBOARD_DRILL_ACTION_VIEW_TEAM + teamDashboard; 

		return url;
	},

	/*
 	 * @param Array values
 	 * 
 	 * @return Element
 	 */	
	_getFormattedCategoryValues: function(values) {
		
		var content = document.createElement("div");
		
		if ((values.length === 1) && !values[0]) {
			// All
			var allText = document.createTextNode(this.messages.msgAll);
			content.appendChild(allText);			
			return content;
		}			
		
		// List
		for (var index = 0; index < values.length; index++) {
			var rawValue = values[index];
			var displayValue;
			var span = document.createElement("span");

			if (rawValue) {
				// Display last segment only
				// e.g. 
				// Before: Work Items/Web UI
				// After:  Web UI
				var segments = rawValue.split(/\//);
				displayValue = segments[segments.length - 1];
				span.title = rawValue;
			} else {
				// Error
				displayValue = this._static._UNKNOWN_PARAMETER;
			}
			
			// Comma separated
			if (index !== (values.length - 1)) {
				displayValue = displayValue + ", ";
			}

			var displayText = document.createTextNode(displayValue);
			span.appendChild(displayText);
			content.appendChild(span);
		}	
		
		return content;		
	},

	/*
 	 * @param Array values
 	 * 
 	 * @return Element
 	 */	
	_getFormattedDurationValues: function(values) {
		
		var content = document.createElement("div");
		var rawValue = values[0];
		var displayValue; 
		
		if (rawValue) {
			// Remove initial digit
			// e.g. Before: 7 Last 50 Builds
			// e.g. After:  Last 50 Builds
			var matches = rawValue.match(/^[0-9]+ (.+)/);
			if ((matches instanceof Array) &&
				(matches.length > 1) &&
				matches[1]) {
				// [0]: Entire match
				// [1]: First match group
				displayValue = matches[1];
			} else {
				displayValue = rawValue;
			}
		} else {
			// Error
			displayValue = this._static._UNKNOWN_PARAMETER;
		}
		
		var text = document.createTextNode(displayValue);
		content.appendChild(text);
		
		return content;
	},

	/*
 	 * @param Array values
 	 * 
 	 * @return Element
 	 */	
	_getFormattedIterationValues: function(values) {

		var content = document.createElement("div");
		var result;		
		
		if ((values.length === 1) && !values[0]) {
			// All
			result = this.messages.msgAll;
		} else {
			// List
			result = "";
			
			for (var index = 0; index < values.length; index++) {
	
				var rawValue = values[index];
				var displayValue;
	
				if (!rawValue) {
					// Error
					displayValue = this._static._UNKNOWN_PARAMETER;
				} else if (rawValue === this._static._SPECIAL_CURRENT_ITERATION) {
					// Current Iteration
					displayValue = this.messages.msgCurrentIteration;
				} else {
					displayValue = rawValue;				
				}
				
				// Comma separated
				if (index !== (values.length - 1)) {
					displayValue = displayValue + ", ";
				}
				
				result = result + displayValue;
			}	
		}
		
		var text = document.createTextNode(result);
		content.appendChild(text);
		
		return content;			
	},

	/*
 	 * @param Array values
 	 * 
 	 * @return Element
 	 */	
	_getFormattedOtherValues: function(values) {
		
		var content = document.createElement("div");
		var displayValue;
		
		if (values.length === 1) {
			var rawValue = values[0]; 
			if (!rawValue) {
				// All
				displayValue = this.messages.msgAll;
			} else if (rawValue === 'true') {
				// Boolean
				displayValue = this.messages.msgYes;
			} else if (rawValue === 'false') {
				// Boolean
				displayValue = this.messages.msgNo;
			} else {
				displayValue = rawValue;
			}
		} else {
			// List
			displayValue = values.join(", ");
		}	
		
		var text = document.createTextNode(displayValue); 
		content.appendChild(text);
		
		return content;
	},

	/*
 	 * @param Element content
 	 * @param String name
 	 * @param Array values
 	 * 
 	 * @return void
 	 */	
	_addParameterEllipses: function(content, name, values) {

		var span = document.createElement("span");
		var comma = document.createTextNode(", ");
		var ellipses = document.createTextNode("...");
		
		span.appendChild(ellipses);
		content.appendChild(comma);
		content.appendChild(span);
		
		this._parameterHoverPopups.push(
					new HoverPopup({
						around: span,
						html: this._getParameterHoverContent(name, values),
						orientation: "R",
						openDelay: 300}));
	},

	/*
 	 * @param String name
 	 * @param Array values
 	 * 
 	 * @return String
 	 */	
	_getParameterHoverContent: function(name, values) {

		var content = "";

		if (name === this._static._PARAMETER_NAME_TEAM) {
			// Team Area
			content = this._getTeamHoverContent(values, true);
		} else if (name === this._static._PARAMETER_NAME_CATEGORY) {
			// Category
			content = this._getTeamHoverContent(values, false);
		} else {
			// Other
			content = values.join("<br>");
		}

		var html = "<span style=\"font-size:8pt;\">" + content + "</span>";
			
		return html;
	},

	/*
 	 * @param Array values
 	 * @param Boolean showLinks
 	 * 
 	 * @return String
 	 */	
	_getTeamHoverContent: function(values, showLinks) {

		// Split each value into path segments
		var sortedValues = this._sortRawTeamValues(values);
		var valueMatrix = [];
		var index;
		var maxColumnCount = 0;
		for (index = 0; index < sortedValues.length; index++) {
			valueMatrix[index] = sortedValues[index].split(/\//);
			if (valueMatrix[index].length > maxColumnCount) {
				maxColumnCount = valueMatrix[index].length;
			}
		}

		// Normalize values into a hierarchy by replacing matching values
		// with whitespace
		var targetValue = ""; 
		var tabCount = 0;
		var whitespace = "&nbsp;&nbsp;&nbsp;";
		for (var columnY = 0; columnY < maxColumnCount; columnY++) {
			for (var rowX = 0; rowX < sortedValues.length; rowX++) {

				if (columnY >= valueMatrix[rowX].length) {
					continue;
				}

				var currentValue = valueMatrix[rowX][columnY];

				if ((currentValue === targetValue) &&
					this._isSameParentValue(valueMatrix, rowX, columnY)) {
					// Replace value with whitespace padding to represent value's
					// proper level within the hierarchy
					valueMatrix[rowX][columnY] = this._getRepeatedString(whitespace, tabCount);
				} else {
					// Set as value to match
					targetValue = currentValue;
					// If this is not the last segment of the path, display it
					// as an unselected value on its own row in the hierarchy
					if (columnY !== (valueMatrix[rowX].length - 1)) {
						valueMatrix[rowX][columnY] = "<span style=\"color:gray;\">" + 
													valueMatrix[rowX][columnY] +
													"</span><br>" + 
													this._getRepeatedString(whitespace, (tabCount + 1));
						tabCount--;
					}
				}
			}
			tabCount++;
		}

		// Collapse value matrix into assembled content
		var content = "";

		if (showLinks) {
			for (index = 0; index < valueMatrix.length; index++) {
				var lastIndex = valueMatrix[index].length - 1;
				var indent;
				if (lastIndex > 0) {
					// Note: Does not include the last entry
					indent = valueMatrix[index].slice(0, lastIndex).join("");
				} else {
					indent = "";
				}
				// Turn the last entry into a link
				var link = "<a href=\"" + 
							this._getTeamDashboardURL(sortedValues[index]) + 
							"\">" +
							valueMatrix[index][lastIndex] +
							"</a>";
						
				content = content + indent + link + "<br>";
			}
		} else {
			for (index = 0; index < valueMatrix.length; index++) {
				content = content + valueMatrix[index].join("") + "<br>";
			}
		}		

		return content;
	},

	/*
 	 * @param Array valueMatrix
 	 * @param Number rowX
 	 * @param Number columnY
 	 *  
 	 * @return Boolean
 	 */	
	_isSameParentValue: function(valueMatrix, rowX, columnY) {
		// Pre: valueMatrix is sorted
		// Pre: rowX is a positive integer
		// Pre: rowY is a positive integer
		if (rowX === 0) {
			return true;
		} else if (columnY === 0) {
			return true;
		} 
		// e.g. rowX = 1; columnY = 1 
		// Not same parent
		//    0       1
		// 0: Admin / Web UI
		// 1: Build / Web UI
		var currentParent = valueMatrix[rowX][columnY - 1];
		var previousParent = valueMatrix[rowX - 1][columnY - 1];
		if (currentParent === previousParent) {
			return true;
		}
		return false;
	},

	/*
 	 * @param Array values
 	 * 
 	 * @return Array
 	 */	
	_sortRawTeamValues: function(values) {
		var sortedValues = [];
		// Remove leading '/' (if any)
		for (var index = 0; index < values.length; index++) {
			sortedValues.push(values[index].replace(/^\//, ""));
		}
		// Sort array
		sortedValues.sort();
		
		return sortedValues;
	},

	/*
 	 * @param String text
 	 * @param Number count
 	 * 
 	 * @return String
 	 */	
	_getRepeatedString: function(text, count) {
		var result = text;
		for (var index = 1; index < count; index++) {
			result = result + text;
		}
		return result;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_deleteParameterHoverPopups: function() {
		
		for (var index = 0; index < this._parameterHoverPopups.length; index++) {
			var widget = this._parameterHoverPopups[index];
			if (widget) {
				widget.destroy();
			}
		}
		this._parameterHoverPopups = [];
	},

	/*
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */
	_getReport: function(queryUUID) {

		var projectAreaUUID = this._getProjectAreaUUID();
		if (!projectAreaUUID) {
			this._logError(new Error("Project area is not defined"));
			this._displayErrorMessage(this.messages.msgReportLoadingFailed);
			return;
		}

		// Update parameters
		if (this._parametersAreaOpen) {
			this._addParameters();
		}
		
		// Show the loading message		
		this._updateLoadingMessage(this.messages.msgLoading);
		this._showLoadingArea();
		
		// Remember the zoom state at the time of this request
		var previousZoomState = this.isZoomed();
		
		var self = this;
		var serviceObject = {
			success: function(html) {
				self._processReportHTML(html, previousZoomState);
				self.getSite().doneRefresh();
			},
			failure: function(errorObject) {
				self._logError(errorObject);
				self._displayErrorMessage(self.messages.msgReportLoadingFailed, errorObject);
				self.getSite().doneRefresh();
			}
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._callRESTPostRenderQuery(
			handler,
			projectAreaUUID,
			queryUUID,
			['Zoom'],
			[previousZoomState]);

	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * @return void
 	 */	
	_addReportToViewlet: function(html) {
		// Update loading message
		this._updateLoadingMessage("");
		// Get the attach point
		var node = this._displayArea;
		// Add the report HTML to the node
		this._addHTMLToNode(html, node);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * @param Element node
 	 * @return void
 	 */	
	_addHTMLToNode: function(html, node) {
		// Create a wrapping div for the parameters and report nodes
		var container = document.createElement("div");
		container.setAttribute("id", this._getReportDomID());	
		
		// Show the parameters area
		this._updateVisibility(this._parametersArea, true);

		// Create the report node
		var reportNode = document.createElement("div");
		// Set the content to be the html
		reportNode.innerHTML = html;
		// Add the report node
		container.appendChild(reportNode);
		
		// Add the wrapping div to the node
		this._addReportNode(node, container);
		// Show the display area
		this._showDisplayArea();		
	},		

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getReportDomID: function() {
		return "report:" + this.id;	
	},	

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getParameterRowsID: function() {
		return "params:" + this.id;	
	},	

	/*
 	 * @param Element targetParent
 	 * @param Element newReportNode
 	 * 
 	 * @return void
 	 */		
	_addReportNode: function(targetParent, newReportNode) {
		// Get the old report node (if any)
		var oldReportNode = dojo.byId(this._getReportDomID());		
		
		try {
			if (oldReportNode) {
				targetParent.replaceChild(newReportNode, oldReportNode);
			} else {
				targetParent.appendChild(newReportNode);
			}
		} catch (errorObject) {
			this._logError(errorObject);
			this._removeAllChildren(targetParent);
			targetParent.appendChild(newReportNode);
		}
	},	

	/*
 	 * @param Node parent
 	 * 
 	 * @return void
 	 */	
	_removeAllChildren: function(parent) {
		while (parent.hasChildNodes()) {
			parent.removeChild(parent.firstChild);
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getParentReportUUID: function() {
		// Get the associated normal report UUID
		// From extension		
		var parentReportUUID = this.getPreference(this._static._PREFERENCE_ID_REPORT_PARENT);
		if (parentReportUUID) {
			return parentReportUUID;
		}
		// From hard-coded mapping
		parentReportUUID = this._static._microReportIDMap[this._reportUUID];
		if (parentReportUUID) {
			return parentReportUUID;
		}
		// Use micro report ID
		return this._reportUUID;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * @param String hashkey
 	 * @param boolean previousZoomState
 	 * @return boolean
 	 */	
	_processReportHTML: function(html, previousZoomState) {
		// Check if a report was found
		if (previousZoomState != this.isZoomed()) {
			// NO-OP
			// If the zoom state made at request time is different than 
			// the zoom state on request completion (now), do not add the 
			// report (but still cache it)
			this._logInfo(new Error("Report results discarded. Zoom state changed between request initiation and completion."));
		} else if (this._containsReport(html)) {
			// Report found
			if (!this._canViewDashboard) {
				// TODO: MP: Disable drill-in links via API
				// No Reports Web UI vertical
				// Disable chart drill-in
				// Note: Only applicable to Build Duration report (at the moment)
				html = this._removeHrefs(html);
			}				
			// Add the report to the display area
			this._addReportToViewlet(html);
		} else {
			// Report NOT found
			this._addReportToViewlet(this._getHTMLNoResultsMessage());
		}
	},	

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getHTMLNoResultsMessage: function() {
		var html = "<div class=\"no-results\">" + this._messageNoResults + "</div>";
		return html;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * 
 	 * @return String
 	 */	
	_removeHrefs: function(html) {
		// Before
		// <area title="4.838" target="_self" href="#action=com.ibm.team.reports..." coords="91,139,91,206,100,206,100,139" shape="poly"/>	
		// After
		// <area title="4.838" target="_self" coords="91,139,91,206,100,206,100,139" shape="poly"/>		

		var result = html.replace(/href=\".*?\"/g, "");

		return result;
	},	

	/*
 	 * @param String html
 	 * 
 	 * @return boolean
 	 */	
	_containsReport: function(html) {
		var matcher = new String(html);
		if (matcher.match(/\.png/)) {
			// Image results
			return true;
		} else if (this._containsReportTable(matcher)) {
			// Table results
			return true;
		}
		return false;
	},		

	/*
 	 * @param String htmlString
 	 * 
 	 * @return boolean
 	 */	
	_containsReportTable: function(htmlString) {
		// Look for table rows
		var matches = htmlString.match(/<tr/g);

		if ((matches instanceof Array) === false) {
			// No table rows
			return false;
		} else if (matches.length < 10) {
			// Expected at least 10 table rows corresponding to 10 unit tests
			return false;
		}
		return true;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getImageServerRoot: function() {
		var url;
		if (this._local) {
			// Local viewlet
			url = this._getServerRoot();
		} else {
			// Remote viewlet
			// Create a proxy url in order to embed images from another 
			// domain in the current domain.
			// Absolute links on their own work; however, security related
			// issues (e.g. firewall, invalid certificates, etc.) prevent the 
			// images from resolving in test environments.
			url = jazz.app.proxy.encodeProxiedURI(this._getServerRoot());
		}
		url = url + "/";
		
		return url;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getDrillInServerRoot: function() {
		if (this._local) {
			// Local viewlet
			return "";
		}
		// Remote viewlet
		return this._getServerURL();
	},	

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */	
	_callRESTPostSaveQueryDescriptor: function(handler, reportUUID, projectAreaUUID, queryUUID) {

		var arguments = {
			projectAreaUUID: projectAreaUUID,
			reportUUID: reportUUID,
			queryUUID: queryUUID,
			name: "",  					// Preserve existing query name
			keepInTempStorage: false,
			sharedIfNullFolder: true,
			ignoreParameters: true		// Preserve existing parameter names and values
		};
		this._callRESTReportsService(handler, "postSaveQueryDescriptor", arguments);
	},

	/*
 	 * @param ServiceResponseHandler handler
 	 * 
 	 * @return void
 	 */	
	_callRESTGetContainingProjectAreaItemIds: function(handler) {
		var arguments = {
			reportId: this._reportUUID
		};
		this._callRESTReportsService(handler, "getContainingProjectAreaItemIds", arguments);
	},	
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String queryUUID
 	 * @param Boolean onlyIfInTempStorage
 	 * 
 	 * @return void
 	 */	
	_callRESTPostDeleteQueryDescriptor: function(handler, queryUUID, onlyIfInTempStorage) {
		var arguments = {
			queryUUID: queryUUID,
			onlyIfInTempStorage: onlyIfInTempStorage
		};
		this._callRESTReportsService(handler, "postDeleteQueryDescriptor", arguments);
	},		
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String projectAreaUUID
 	 * @param String queryUUID
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * 
 	 * @return void
 	 */	
	_callRESTPostRenderQuery: function(handler, projectAreaUUID, queryUUID, parameterName, parameterValue) {

		var arguments = {
			projectAreaUUID: projectAreaUUID,
			queryUUID: queryUUID,
			parameterName: parameterName,
			parameterValue: parameterValue,
			showArchived: false,
			urlPrefix: this._getImageServerRoot(),
			serverURL: this._getDrillInServerRoot()
		};
		
		this._callRESTReportsService(handler, "postRenderQuery", arguments);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param String method
 	 * @param Object arguments
 	 * @return void
 	 */	
	_callRESTReportsService: function(handler, method, arguments) {
		// Create the service request
		var request = new ServiceRequest(
					this._static._REPORT_REST_SERVICE_URI,
					method,
					arguments);
		this.getServiceTracker().register(request, handler);
		// Invoke the service
		TeamServerClient.invokeService(request, handler);
	},

	// Utility Methods -------------------------------------------------------->

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventPrefLinkClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Load the settings area
		this.getSite().toggleSettings();
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String message
 	 * @return void
 	 */	
	_displayMessage: function(message) {
		// Get attach point
		var node = this._displayArea;
		
		// Create a wrapping div 
		var container = document.createElement("div");
		container.setAttribute("id", this._getReportDomID());
		// Create a text node for the message
		var textNode = document.createTextNode(message);
		// Assemble contents		
		container.appendChild(textNode);
		// Add the wrapping div to the node
		this._addReportNode(node, container);		
		
		// Show the display area
		this._showDisplayArea();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return void
 	 */	
	_updateLoadingMessage: function(text) {
		// Get the attach point
		var node = this._loadingArea;
		// Get the text node
		var textNode = node.firstChild;
		
		if (!textNode &&
			(text.length === 0)) {
			// Nothing to do
			return;
		} else if (text.length === 0) {
			// Remove the old text node
			node.removeChild(textNode);
		} else if (!textNode) {
			// Create a new text node
			textNode = document.createTextNode(text);
			node.appendChild(textNode);
		} else {
			// Modify the existing text node
			textNode.replaceData(0, textNode.length, text); 
		}
	},		

	/*
 	 * @param String message
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */	
	_alertErrorMessage: function(message, errorObject) {
		var text = this._getErrorStatusMessage(this.messages.msgError, message, errorObject);
		alert(text);		
	},	
	
	/*
 	 * @param String message
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */	
	_displayWarningMessage: function(message, errorObject) {
		var text = this._getErrorStatusMessage(this.messages.msgWarning, message, errorObject);
		this._displayMessage(text);		
	},		
	
	/*
 	 * @param String message
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */	
	_displayErrorMessage: function(message, errorObject) {
		var text = this._getErrorStatusMessage(this.messages.msgError, message, errorObject);
		this._displayMessage(text);		
	},	

	/*
 	 * @param String prefix
 	 * @param String message
 	 * @param Error errorObject
 	 * 
 	 * @return String
 	 */	
	_getErrorStatusMessage: function(prefix, message, errorObject) {
		var text = dojo.string.substitute(prefix, [message]);
		if (errorObject) {
			if (errorObject.message) {
				text = text + ": " + errorObject.message;
			} 
			if (errorObject.status === 403) {
				text = text + ": " + this.messages.msgReadPermissionError;
			}
		}
		return text;
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_displayMsgSelectPreferences: function() {
		var container = document.createElement("div");
		container.id = this._getReportDomID();
		
		// Create actionable status message
		var links = I18nUtil.addMessage(container, this.messages.msgActionConfigureReport);
		var link = links[0];
		link.href = "javascript:void(0);";
		this.connect(link, "onclick", this._handleEventPrefLinkClicked);		
		
		this._addReportNode(this._displayArea, container);
		this._showDisplayArea();
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_displayMsgReportNotDeployed: function() {
		if (this._canEditDashboard()) {
			this._displayActionReportNotDeployed();
		} else {
			var message = dojo.string.substitute(this.messages.msgReportNotDeployed, [this._reportUUID, this._getProjectAreaName()]);
			this._displayMessage(message);
		}			
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_displayActionReportNotDeployed: function() {

		// Hide the parameters area
		this._updateVisibility(this._parametersArea, false);		
		
		var container = document.createElement("div");
		container.id = this._getReportDomID();
		var message = dojo.string.substitute(this.messages.msgActionReportNotDeployed, [this._reportUUID, this._getProjectAreaName()]);
		
		// Create actionable status message
		var links = I18nUtil.addMessage(container, message);
		var link = links[0];
		link.href = "javascript:void(0);";
		this.connect(link, "onclick", this._handleEventPrefLinkClicked);		
		
		this._addReportNode(this._displayArea, container);
		this._showDisplayArea();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showDisplayArea: function() {
		// Show the loading area
		var loadingNode = this._loadingArea;
		this._updateVisibility(loadingNode, false);
		// Hide the loading image
		this.getSite().setLoading(false);
		// Hide the display area
		var displayNode = this._displayArea;
		this._updateVisibility(displayNode, true);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showLoadingArea: function() {
		// Show the loading area
		var loadingNode = this._loadingArea;
		this._updateVisibility(loadingNode, true);
		// Show the loading image
		this.getSite().setLoading(true);
		// Hide the display area
		var displayNode = this._displayArea;
		this._updateVisibility(displayNode, false);
	},		
	
	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */
	_logInfo: function(errorObject) {
		// Log info only in debug mode
		if (djConfig.isDebug) {
			console.log(this._getErrorMessage("INFO", errorObject));
		}
	},	

	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */
	_logError: function(errorObject) {
		console.error(this._getErrorMessage("ERROR", errorObject));
	},	
	
	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */
	_logWarning: function(errorObject) {
		console.warn(this._getErrorMessage("WARNING", errorObject));
	},		
	
	/*
 	 * @param String prefix
 	 * @param Error errorObject
 	 * 
 	 * @return String
 	 */
	_getErrorMessage: function(prefix, errorObject) {
		
		var classID = "";
		var lineNumber = "";
		var message = "";

		// Class name
		if (this.id) {
			classID = this.id + ": ";
		} 

		if (errorObject) {
			// Line number
			if (errorObject.lineNumber) {
				lineNumber = "(line " + errorObject.lineNumber + "): ";
			} 
			// Error message
			if (errorObject.message) {
				message = errorObject.message;
			} else {
				message = "Message unavailable";
			}
			if (errorObject.status === 403) {
				message = message + ": " + this.messages.msgReadPermissionError;
			}			
		} else {
			// Error message
			message = "Undefined exception";			
		}
		
		var errorMessage = "[" + prefix + "]: " + 
							classID +
							lineNumber +
							message;
							
		return errorMessage;
	},		
	
	/*
 	 * @param String value
 	 * 
 	 * @return Boolean
 	 */	
	_isValidUUID: function(value) {
		// See com.ibm.team.repository.common.UUID.validateUUID(char[])
		// e.g. _H9EB5esEEdycivs5PH0c-g
		// A valid UUID must:
		// * Have a fixed length of 23 characters
		// * Start with an underscore character
		// * Contain characters only in the range of the MIME64 encoding
		//   (Not checked for performance reasons)
        if (value &&
			(value.length === 23) &&
			(value.charAt(0) === "_")) {
            return true;
        }
		return false;
	},	
	
	/*
 	 * @param String text
 	 * 
 	 * @return String
 	 */		
	 _hashString: function(text) {
		var hash = text.length;
		for (var index = 0; index < text.length; index++) {
			hash = ((hash << 5) ^ (hash >> 27)) ^ text.charCodeAt(index);
		}
		var hashKeyString = "K" + new Number(hash).toString();
		
		return hashKeyString;
	}
	
});

/*
 * Static Methods
 */
dojo.mixin(com.ibm.team.reports.viewlets.web.ui.internal.TrendReportViewlet, {

	/*
 	 * @param String transientQueryUUID
 	 * @param String classID
 	 * 
 	 * @return void
 	 */	
	deleteTransientReportQueryUUID: function(transientQueryUUID, classID) {
		
		var serviceObject = {
			success: function() {
				if (djConfig.isDebug) {
					console.log("[INFO]: " + classID + ": Transient report query deleted");
				}
			},
			failure: function(errorObject) {
				console.error("[ERROR]: " + classID + ": Failed to delete the transient report query");
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		var arguments = {
			queryUUID: transientQueryUUID,
			onlyIfInTempStorage: true
		};
		var request = new ServiceRequest(
					"com.ibm.team.reports.common.internal.service.IReportRestService",
					"postDeleteQueryDescriptor",
					arguments);

		TeamServerClient.invokeService(request, handler);		
	}
	
});

})();

