/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletChooserBlock");
/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletChooserTree");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.util.internal.ErrorUtility");

dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

/*
 * Aliases 
 */
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DiscoveryRegistry = com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry;
var HTML = jazz.util.html;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ViewletChooserTree = com.ibm.team.dashboard.web.ui.internal.ViewletChooserTree;
var ViewletRegistry = com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ErrorUtility = com.ibm.team.dashboard.web.util.internal.ErrorUtility;

var SERVICE_IMG = "com.ibm.team.dashboard.common.internal.service.IDashboardImageService";
var LOCAL_TREE_ID = "local";

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletChooserBlock",
	[dijit._Widget, dijit._Templated], {

	/*
 	 * Template
 	 */
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ViewletChooserBlock.html"),
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	
	
	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return void 
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.loading = this.messages.loading;
		this.selectFromChooser = this.messages.selectFromChooser;
		this.appliesTo = this.messages.appliesTo;
		this.description = this.messages.description;
		this.preview = this.messages.preview;
		this.viewletPreviewImage = this.messages.viewletPreviewImage;
		this.addViewletText = this.messages.addViewletText;
		this.msgClose = this.messages.close;
		this.missingViewletPreviewImage = this.messages.missingViewletPreviewImage;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {

		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();
		
		// Instance Variables //----------------------------------------------->
		
		// Viewlet Tree
		this._currentViewletTree = null;
		this._currentServiceRequest = null;
		this._viewletTreeMap = new Object();

		this._detailPanes = new Object();
		this._currentDetailPaneKey = null;		

		this._introDetailPane = null;
		this._categoryDetailPane = null;
		this._itemDetailPane = null;

		// Servers
		this._servers = null;
		this._currentServerIndex = 0;

		// Initializers //----------------------------------------------------->

		this._initialize();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

		this._static._TYPE_INTRO = "i";
		this._static._TYPE_CATEGORY = "c";
		this._static._TYPE_VIEWLET = "v";

	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	startup: function() {
		if (!this._started) {
			this._started = true;
			this._callRESTGetRepositories();
		}
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_initialize: function() {
		// Register detail panes
		this._registerDetailPanes();
		// Initialize sites
		this._initCloseButtonSite();
		// Initialize panes
		this._initIntroDetailPane();
		this._initCategoryDetailPane();
		this._initItemDetailPane();
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_registerDetailPanes: function() {
		// Intro Details Pane
		this._detailPanes[this._static._TYPE_INTRO] = this._viewletIntroPane;
		// Category Details Pane
		this._detailPanes[this._static._TYPE_CATEGORY] = this._viewletCategoryPane;
		// Item Details Pane
		this._detailPanes[this._static._TYPE_VIEWLET] = this._viewletItemPane;			
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_initCloseButtonSite: function() {
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/images/close.gif");
		
		var self = this;
		
		this.connect(this._closeButton, "onclick", function() {
			// Close chooser
			self.dashboardContainer.closeViewletChooser();
		});		
		this.connect(this._closeButton, "onmouseover", function() {
			// Highlight on
			self._closeButton.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/images/close-hov.gif");
		});
		this.connect(this._closeButton, "onmouseout", function() {
			// Highlight off
			self._closeButton.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/images/close.gif");
		});		
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_initIntroDetailPane: function() {
		// Set the initial pane to be the intro
		this._currentDetailPaneKey = this._static._TYPE_INTRO;
		// Get the intro detail pane
		var pane = this._getDetailPane(this._currentDetailPaneKey);
		
		// Title
		this._headerTitle.innerHTML = this.messages.viewletChooser;
		// Icon
		this._headerIcon.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/icons/tree-category.gif");
		// Button
		this._updateVisibility(this._viewletButton, false);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_initCategoryDetailPane: function() {
		// NO-OP
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_initItemDetailPane: function() {
		this._createButtonAddViewlet();		
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String key
 	 * 
 	 * @return Element
 	 */
	_getDetailPane: function(key) {
		return this._detailPanes[key];
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_handleEventSelect: function() {
		// Get model object
		var modelObj = this._getSelectedModelObj();
		// Validate model object
		if (!modelObj) {
			// Adapt model object
			modelObj = new Object();
			modelObj.type = this._static._TYPE_INTRO;
		}		
		// Update
		var server = this._servers[this._currentServerIndex];
		this._updateDetailsSection(modelObj, server);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param boolean show
 	 * 
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			node.style.display = "block";
		} else {
			node.style.display = "none";
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String message
 	 * @param String details
 	 * 
 	 * @return void
 	 */	
	_showErrorMessage: function(message, details) {
		var buf = [message];
		if (details) {
			buf.push("<div class=\"error-details\">");
			buf.push(details);
			buf.push("</div>");
		}
		this._errorMessage.innerHTML = buf.join("");
		this._updateVisibility(this._errorMessage, true);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_hideErrorMessage: function() {
		this._updateVisibility(this._errorMessage, false);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_showLoadingMessage: function() {
		this._updateVisibility(this._loadingMessage, true);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_hideLoadingMessage: function() {
		this._updateVisibility(this._loadingMessage, false);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * @param Object server
 	 * 
 	 * @return boolean
 	 */	
	_updateDetailsSection: function(modelObj, server) {
		// Update the detail pane contents
		if (modelObj.type === this._static._TYPE_CATEGORY) {
			// Category	
			this._updateCategoryDetailPane(modelObj);
		} else if (modelObj.type === this._static._TYPE_VIEWLET) {
			// Viewlet
			this._updateViewletDetailPane(modelObj, server);
		} else {
			// Intro
			this._updateIntroDetailPane(modelObj);
		}

		// Get the current detail pane selected
		var oldPane = this._getDetailPane(this._currentDetailPaneKey);
		// Hide the old pane
		if (oldPane) {
			this._updateVisibility(oldPane, false);
		}
		
		// Get the new details pane selected
		var newPane = this._getDetailPane(modelObj.type);
		// Show the new pane
		if (newPane) {
			this._updateVisibility(newPane, true);		
		}
		
		// Set the new pane as the current detail pane
		this._currentDetailPaneKey = modelObj.type;

	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return void
 	 */
	_updateCategoryDetailPane: function(modelObj) {
		// Bread Crumbs
		this._categoryTitle.innerHTML = "";
		this._categoryTitle.style.marginTop = "0";
		this._categoryTitle.style.marginBottom = "0";
		// Title
		this._updateVisibility(this._headerTitle, true);
		this._headerTitle.innerHTML = modelObj.label;
		// Description
		var description = modelObj.description;
		if (!description) {
			description = this.messages.noDescriptionSpecified;
		}
		this._viewletCategoryDescription.innerHTML = HTML.removeScript(description);
		
		// Icon
		this._updateVisibility(this._headerIcon, true);
		this._headerIcon.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/icons/folder.gif");
		// Button
		this._updateVisibility(this._viewletButton, false);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return void
 	 */
	_updateIntroDetailPane: function(modelObj) {
		// Bread Crumbs
		this._categoryTitle.innerHTML = "";
		this._categoryTitle.style.marginTop = "0";
		this._categoryTitle.style.marginBottom = "0";
		// Title
		this._updateVisibility(this._headerTitle, true);
		this._headerTitle.innerHTML = this.messages.viewletChooser;
		// Icon
		this._updateVisibility(this._headerIcon, true);
		this._headerIcon.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/icons/tree-category.gif");
		// Button
		this._updateVisibility(this._viewletButton, false);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * @param Object server
 	 * 
 	 * @return void
 	 */
	_updateViewletDetailPane: function(modelObj, server) {
		// Update the icon field
		this._updateDetailsIconField(modelObj);
		// Update the name field
		this._updateDetailsNameField(modelObj);
		// Button
		this._updateVisibility(this._viewletButton, true);
		// Update the category field
		this._updateDetailsCategoryField(modelObj);
		// Update the applicability field
		this._updateDetailsAppliesToField(modelObj);
		// Update the description field
		this._updateDetailsDescriptionField(modelObj);
		// Update the preview field
		this._updateDetailsPreviewField(modelObj, server);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return void
 	 */	
	_updateDetailsIconField: function(modelObj) {
		this._updateVisibility(this._headerIcon, true);
		this._headerIcon.src = modelObj.iconSrc;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return void
 	 */	
	_updateDetailsNameField: function(modelObj) {
		this._updateVisibility(this._headerTitle, true);
		this._headerTitle.innerHTML = modelObj.label;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return void
 	 */	
	_updateDetailsCategoryField: function(modelObj) {
		// Bread Crumbs
		this._categoryTitle.style.marginTop = "7px";
		this._categoryTitle.style.marginBottom = "7px";
		// Set the category
		this._categoryTitle.innerHTML = modelObj.category;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return void
 	 */	
	_updateDetailsAppliesToField: function(modelObj) {
		// Get the applicability of the item selected
		var rawAppliesTo = modelObj.applicability;		
		// Normalize the value
		var normalizedAppliesTo = null;
		if (rawAppliesTo === "teams") {
			normalizedAppliesTo = this.messages.appliesToTeams;
		} else if (rawAppliesTo === "projects") {
			normalizedAppliesTo = this.messages.appliesToProjects;
		} else if (rawAppliesTo === "projects-and-teams") {
			normalizedAppliesTo = this.messages.appliesToProjectsTeams;
		} else if (rawAppliesTo === "contributors") {
			normalizedAppliesTo = this.messages.appliesToContributors;
		} else if (rawAppliesTo === "all") {
			normalizedAppliesTo = this.messages.appliesToAll;
		} else {
			normalizedAppliesTo = this.messages.appliesToUnknown;
		}
		// Get the applicability field node		
		var node = dojo.byId("_detailsFieldAppliesTo");
		// Set the applicability
		node.innerHTML = normalizedAppliesTo;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return void
 	 */	
	_updateDetailsDescriptionField: function(modelObj) {
		// Get the description of the item selected
		var description = modelObj.description;		
		// Ensure a description was provided
		if (!description) {
			description = this.messages.noDescriptionSpecified;
		}
		// Get the description field node		
		var node = dojo.byId("_detailsFieldDescription");
		// Set the description
		node.innerHTML = HTML.removeScript(description);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * @param Object server
 	 * 
 	 * @return void
 	 */	
	_updateDetailsPreviewField: function(modelObj, server) {

		if (!modelObj.preview) {
			// Missing image
			this._updateVisibility(this._detailsFieldPreviewImage, false);
			this._updateVisibility(this._detailsFieldPreviewMissing, true);
		} else {
			// Preview image
			this._updateVisibility(this._detailsFieldPreviewImage, true);
			this._updateVisibility(this._detailsFieldPreviewMissing, false);
			// Preview Format: com.ibm.team.dashboard.viewlets.web/ui/internal/images/icon-wiki.gif
			var previewImagePath;
			this._detailsFieldPreviewImage.src = Util.getServiceURL(SERVICE_IMG, null, {path: modelObj.preview}, server.viewletServiceRoot);
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */	
	_isLocalServer: function() {
		return this._currentServerIndex == 0;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */          
	_createButtonAddViewlet: function() {
		this.connect(this._viewletButton, "onclick", this._handleEventAddViewlet);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return Object
 	 */	
	_getSelectedModelObj: function() {
		// Get selection
		var treeSelection = this._currentViewletTree.getSelection();
		// Validate selection
		if ((treeSelection instanceof Array) === false) {
			return null;
		} else if (treeSelection.length === 0) {
			return null;
		}
		
		// Get ID
		var id = treeSelection[0].getId();
		// Get model object
		var modelObj = this._currentViewletTree.getModelObject(id);
		
		return modelObj;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_handleEventAddViewlet: function() {
		// Get model object
		var modelObj = this._getSelectedModelObj();
		// Validate model object
		if (!modelObj) {
			return;
		} else if (modelObj.type !== this._static._TYPE_VIEWLET) {
			return;
		} else if (!modelObj.label) {
			return;
		} else if (!modelObj.viewletDefId) {
			return;
		}
		
		var server = this._servers[this._currentServerIndex];
		var origin = server.rootServices;
		
		var serviceObj = {
			_self: this,
			_modelObj: modelObj,
			_origin: origin,
			success: function(/* object[] */ definitions) {
				// Add the viewlet
				this._self.dashboardContainer.currentPage.addViewlet(
					this._modelObj.label, 								// Title
					definitions[0], 										// Definition ID
					this._modelObj.icon, 								// Icon
					this._self._getModelObjPreferences(this._modelObj), // Preferences
					null, 												// Scope
					null, 												// Scope Item ID
					null, 												// Scope Subteams
					this._modelObj.showBackground, 						// Show Background
					this._modelObj.trim, 								// Trim
					null, 												// Memento
					0, 													// Destination Column Number
					0, 													// Destination Viewlet Index
					false,												// User Title
					this._origin);										// Viewlet origin
			},
			failure: function() {
				
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		ViewletRegistry.getDefinition(modelObj.viewletDefId, server.viewletServiceRoot, false, srh);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return Object
 	 */	
	_getModelObjPreferences: function(modelObj) {
		var result = new Object();
		for (var index = 0; index < modelObj.preferences.length; index++) {
			var preference = modelObj.preferences[index];
			var delimIndex = preference.indexOf("=");
			var key = preference.substring(0, delimIndex);
			var value = preference.substring(delimIndex + 1);
			result[key] = value;
		}
		return result;		
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_callRESTGetRepositories: function() {
		
		this._showLoadingMessage();

		var serviceObj = {
			_self: this,
			success: function(/* object[] */ servers) {
				this._self._servers = servers;
				this._self._processServers();		
			},
			failure: function(/* Error */ error) {
				this._self._logError(error);
				this._self._hideLoadingMessage();
				this._self._showErrorMessage(this._self.messages.msgRepositoryDiscoveryFailed);
			}
		};
		DiscoveryRegistry.getServers(dojo.hitch(serviceObj, serviceObj.success), dojo.hitch(serviceObj, serviceObj.failure));		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_processServers: function() {
		
		// Validate servers		
		if (!(this._servers instanceof Array) ||
			(this._servers.length === 0)) {
			ErrorUtility.logError(new Error("Server list is empty"), this.id);
			this._showErrorMessage(this.messages.msgRepositoryDiscoveryFailed);
			return;
		}		
		
		if (this._servers.length > 1) {
			// Many repositories
			// Create a repository selector 
			this._createRepositorySelector();
		}
		else {
			// show msg helping user configure friends
			var links = I18nUtil.addMessage(this._viewletIntroPaneHelp, this.messages[this._isAdmin() ? "viewletChooserHelpAdmin" : "viewletChooserHelp"]);
			links[0].href = Util.getContextPath() + "/admin" + (djConfig.isDebug ? "?debug=true" : "") + "#action=com.ibm.team.repository.admin.crossServer";
			links[1].href = Util.getContextPath() + "/admin" + (djConfig.isDebug ? "?debug=true" : "");
		}
		// Create the tree
		this._getViewletHierarchyTree(this._servers[this._currentServerIndex]);

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_createRepositorySelector: function() {
		
		// Create repository selector
		var select = document.createElement("select");
		
		// Process DTO		
		for (var index = 0; index < this._servers.length; index++) {

			// Validate server
			var server = this._servers[index];
			var title = server.title;
			var rootServices = server.rootServices;
			if (!server) {
				ErrorUtility.logWarning(new Error("Server not defined [" + index + "]"), this.id);
				continue;
			} else if (!title) {
				ErrorUtility.logWarning(new Error("Server title not defined [" + index + "]"), this.id);
				continue;
			} else if ((index > 0) && !rootServices) {
				// Note: The local repository is always stored at index 0 and
				// it always has a null root services url 
				ErrorUtility.logWarning(new Error("Server root services url not defined [" + index + "]"), this.id);
				continue;
			}
			
			// Create Option
			var option = document.createElement("option");
			option.value = index;
			
			if (index === this._currentServerIndex) {
				option.selected = "selected";
			}
			
			var optionText = document.createTextNode(title + 
													 "  [" +
													 this._getServerName(rootServices ? rootServices : location.href) +
													 "]");
			option.appendChild(optionText);
			select.appendChild(option);
		}	
		
		this.connect(select, "onchange", dojo.hitch(this, this._handleEventRepositorySelect));
		
		this._repoSelector.appendChild(select);
		this._updateVisibility(this._repoSelector, true);
		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String url
 	 * 
 	 * @return String
 	 */
	_getServerName: function(url) {
		// BEFORE:
		// https://rrc.xyz.net:9443/jazz
		// AFTER:
		// rrc.xyz.net
		
		var pattern = /:\/\/(.+?)[:|\/]/;
		var result = url.match(pattern);
		
		if (((result instanceof Array) === false) ||
			(result.length !== 2)) {
			this._logError(new Error("Unexpected url format encountered: " + url));
			return url;
		}
		
		return result[1];
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */
	_handleEventRepositorySelect: function(event) {
		
		if (!event) {
			this._logError(new Error("Undefined event encountered"));
			return;
		} else if (!event.currentTarget) {
			this._logError(new Error("Event does not have a current target"));
			return;
		} else if (!event.currentTarget.value) {
			this._logError(new Error("Event does not have a current target"));
			return;
		} else if ((event.currentTarget.value in this._servers) === false) {
			this._logError(new Error("Server not found"));
			return;
		}
		
		this._currentServerIndex = parseInt(event.currentTarget.value, 10);
		var server = this._servers[this._currentServerIndex];
		// Get the viewlet tree
		this._getViewletHierarchyTree(server);
		
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object server
 	 * 
 	 * @return void
 	 */
	_callRESTGetViewletHierarchy: function(server) {
		this._showLoadingMessage();
		
		if (this._isLocalServer()) {
			this._callRESTGetViewletHierarchy2(server);
		}
		else {
			var errorCallback = dojo.hitch(this, function(/* Error */ error) {
				this._logError(error);
				this._hideLoadingMessage();
				this._showErrorMessage(this.messages.msgRepositoryConnectionFailed, error.message);
			});
			DiscoveryRegistry.getServerData(server.rootServices, false, dojo.hitch(this, this._callRESTGetViewletHierarchy2, server), errorCallback);
		}
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object server
 	 * @param Object serverData
 	 * 
 	 * @return void
 	 */
	_callRESTGetViewletHierarchy2: function(server, serverData) {
		// add extra data into server obj
		if (serverData) {
			dojo.mixin(server, serverData);
		}
		
		// fetch and validate remote server version
		var serviceObj = {
			_self: this,
			_server: server,
			success: function(/* string */ version) {
				// version service method was only added in 1.0.0.1, so success
				// here means remote server is >= 1.0.0.1.
				this._self._currentServiceRequest = null;
				this._self._callRESTGetViewletHierarchy3(this._server);	
			},
			failure: function(/* Error */ error) {
				var errorMsg = (error.status == 400) ? this._self.messages.msgIncompatible : this._self.messages.msgRepositoryConnectionFailed;
				this._self._currentServiceRequest = null;
				this._self._hideLoadingMessage();
				this._self._showErrorMessage(errorMsg);
			}
		};
		
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		this._currentServiceRequest = DashboardClient.getVersion(server.viewletServiceRoot, srh);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object server
 	 * 
 	 * @return void
 	 */
	_callRESTGetViewletHierarchy3: function(server) {
		var serviceObj = {
			_self: this,
			_server: server,
			success: function(/* ViewletHierarchyDTO */ dto) {
				this._self._currentServiceRequest = null;
				this._self._createViewletHierarchyTree(dto, this._server);	
				this._self._hideLoadingMessage();
			},
			failure: function(/* Error */ error) {
				this._self._currentServiceRequest = null;
				this._self._logError(error);
				this._self._hideLoadingMessage();
				this._self._showErrorMessage(this._self.messages.msgRepositoryConnectionFailed);
			}
		};
		
		// Get the chooser info
		var projectAreaName;
		var scope;
		var scopeItemId;
		if (this._isLocalServer()) {
			projectAreaName = this._getCurrentProjectAreaName();
			scope = this.scope;
			scopeItemId = this.scopeItemId;
		}
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		
		this._currentServiceRequest = DashboardClient.getViewletHierarchy(projectAreaName, server.viewletServiceRoot ? true : false, scope, scopeItemId, server.viewletServiceRoot, srh);
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */
	_getCurrentProjectAreaName: function() {
		var name = dojo.getObject("com.ibm.team.process.web.projectArea.name");
		if (!name) {
			ErrorUtility.logInfo(new Error("Unable to retrieve the current project area name"), this.id);
		}
		return name;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object server
 	 * 
 	 * @return void
 	 */
	_getViewletHierarchyTree: function(server) {

		// Abort any prior request
		if (this._currentServiceRequest) {
			this._currentServiceRequest.abort();
			this._currentServiceRequest = null;
			this._hideLoadingMessage();
		}

		var treeID = server.rootServices;
		if (!treeID) {
			treeID = LOCAL_TREE_ID;
		}

		// Hide the current viewlet tree
		if (this._currentViewletTree) {
			this._updateVisibility(this._currentViewletTree.domNode, false);
		}
		this._hideErrorMessage();
		
		if (treeID in this._viewletTreeMap) {
			// Tree already exists
			// Get cached tree
			this._currentViewletTree = this._viewletTreeMap[treeID];
			this._updateVisibility(this._currentViewletTree.domNode, true);
			// Select the proper details pane
			this._handleEventSelect();
		} else {
			// Tree does not exist
			// Create new tree
			this._callRESTGetViewletHierarchy(server);
		}
		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param ViewletHierarchyDTO root
 	 * @param Object server
 	 * 
 	 * @return void
 	 */
	_createViewletHierarchyTree: function(root, server) {
		// Validate root
		if (!root) {
			return;
		}
		
		var treeID = server.rootServices;
		if (!treeID) {
			treeID = LOCAL_TREE_ID;
		}
		
		// Tree does not exist
		// Create the model
		var model = this._createTreeModel(root);
  		// Create the tree
		this._currentViewletTree = new ViewletChooserTree({
			model: model,
			onDblClickHandler: dojo.hitch(this, this._handleEventAddViewlet),
			onEnter: dojo.hitch(this, this._handleEventAddViewlet),
			onClick: dojo.hitch(this, this._handleEventSelect),
			dataServerRoot: server.viewletServiceRoot
		});
		this._viewletChooser.appendChild(this._currentViewletTree.domNode);
		// Store the tree
		this._viewletTreeMap[treeID] = this._currentViewletTree;
		
		// Select the proper details pane
		this._handleEventSelect();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param ViewletHierarchyDTO root
 	 * 
 	 * @return Array
 	 */
	_createTreeModel: function(root) {
		// Create the array for tree items
		var items = new Array();
		// Get the root categories
		var categories = root.viewletCategories;
		// Get the category count
		var categoryCount = 0;
		if (categories) {
			categoryCount = categories.length;
		}
		// Process categories
		for (var index = 0; index < categoryCount; index++) {
			// Get the current category
			var category = categories[index];
			// Create the tree item out of the category DTO
			items[index] = this._createViewletCategory(category, null);
		}
		// Sort child items
		items.sort(this._sortViewletEntries);		
		
		return items;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param ViewletCategoryDTO category
 	 * @param String categoryCrumbs
 	 * 
 	 * @return Object
 	 */
	_createViewletCategory: function(category, categoryCrumbs) {
		// Escape
		var categoryName = HTML.escape(category.name);
		// Create the array for tree items
		var children = new Array();
		// Get the child categories
		var categories = category.viewletCategories;
		// Get the category count
		var categoryCount = 0;
		if (categories) {
			categoryCount = categories.length;
		}
		// Update the category bread crumbs
		if (!categoryCrumbs) {
			categoryCrumbs = categoryName;
		} else {
			categoryCrumbs = categoryCrumbs + " &gt; " + categoryName;
		}
		// Process child categories
		for (var categoryIndex = 0; categoryIndex < categoryCount; categoryIndex++) {
			// Get the current category
			var childCategory = categories[categoryIndex];
			// Create the tree item out of the category DTO
			children[categoryIndex] = this._createViewletCategory(childCategory, categoryCrumbs);
		}
		// Sort child items
		children.sort(this._sortViewletEntries);				
		// Get the child items
		var items = category.viewletEntries;
		// Get the category count
		var itemCount = 0;
		if (items) {
			itemCount = items.length;
		}	
		// Process child items		
		var viewletChildren = new Array();
		for (var viewletIndex = 0; viewletIndex < itemCount; viewletIndex++) {
			// Get the current item
			var childItem = items[viewletIndex];
			// Create the tree item out of the item DTO
			viewletChildren.push(this._createViewletItem(childItem, categoryCrumbs));
		}	
		// Sort child items
		viewletChildren.sort(this._sortViewletEntries);
		// Merge viewlets with categories
		this._mergeArrays(children, viewletChildren);
		
		// Create the data store object
		var dataStoreCategory = new Object();
		// Set properties
		dataStoreCategory.label = categoryName;
		dataStoreCategory.categoryId = category.id;
		dataStoreCategory.type = this._static._TYPE_CATEGORY;

		// Description
		if (category.description) {
			dataStoreCategory.description = category.description;
		}
		
		if (children.length > 0) {
			dataStoreCategory.children = children;
		}
		
		return dataStoreCategory;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object entry1
 	 * @param Object entry2
 	 * 
 	 * @return Number
 	 */
	_sortViewletEntries: function(entry1, entry2) {
		if (entry1.label < entry2.label) {
			// Less than
			return -1;
		} else if (entry1.label > entry2.label) {
			// Greater than
			return 1;
		}
		// Equal
		return 0;
	},

	/*
 	 * Private Method
 	 * 
 	 * Merges array2 into array1
 	 * 
 	 * @param Array array1
 	 * @param Array array2
 	 * 
 	 * @return void
 	 */
	_mergeArrays: function(array1, array2) {
		for (var index = 0; index < array2.length; index++) {
			array1.push(array2[index]);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param ViewletEntryDTO item
 	 * @param String categoryCrumbs
 	 * 
 	 * @return Object
 	 */
	_createViewletItem: function(item, categoryCrumbs) {
		// Create the data store object
		var dataStoreItem = new Object();
		// Set properties
		dataStoreItem.label = HTML.escape(item.title);
		dataStoreItem.type = this._static._TYPE_VIEWLET;
		dataStoreItem.viewletDefId = item.viewletDefId;
		dataStoreItem.category = categoryCrumbs;
		dataStoreItem.description = item.description;
		dataStoreItem.icon = item.icon;
		dataStoreItem.preview = item.preview;
		dataStoreItem.applicability = item.appliesTo;
		dataStoreItem.showBackground = item.showBackground;
		dataStoreItem.trim = item.trim;
		
		// TODO: MP: Revisit: Clone the object to avoid changing data structure?
		/*
		 * Save preferences in data store as an array of "id=value"
		 * strings (saving as object with properties does not seem to
		 * work(?)).
		 */
		dataStoreItem.preferences = new Array();
		if (item.preferences) {
			for (var i = 0; i < item.preferences.length; i++) {
				var pref = item.preferences[i];
				dataStoreItem.preferences.push(pref.id + "=" + pref.value);
			}
		}
		
		return dataStoreItem;		
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return boolean
 	 */
	_isAdmin: function() {
		var data = InitializationData.get("com.ibm.team.repository.service.canGrantPermissionInitializer");
		return (data != null && data.canGrantPermission) ? true : false;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Error
 	 * 
 	 * @return void
 	 */
	_logError: function(errorObject) {
		
		var className = "";
		if (this.declaredClass) {
			className = this.declaredClass + ": ";
		} 
		
		var lineNumber = "";
		if (errorObject.lineNumber) {
			lineNumber = "(line " + errorObject.lineNumber + "): ";
		} 
		
		var errorMessage = "[ERROR]: " +
							className +
							lineNumber +
							errorObject.message;
							
		console.error(errorMessage);
	},		
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		// Viewlet Tree
		if (this._viewletTreeMap) {
			
			for (var treeID in this._viewletTreeMap) {
				if (this._viewletTreeMap[treeID]) {
					this._viewletTreeMap[treeID].destroy();
					delete this._viewletTreeMap[treeID];
				}
			}
			delete this._viewletTreeMap;
		}
		
		if (this._currentViewletTree) {
			this._currentViewletTree.destroy();
			delete this._currentViewletTree;
		}
		
		// Servers
		if (this._servers) {
			delete this._servers;
		}
		delete this._currentServerIndex;
		
		this.inherited(arguments);
	}
	
});

})();
