/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.data.dom"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.app.proxy"); //$NON-NLS-1$
dojo.require("jazz.util.error"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.DeferredLoadReadStore");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.DeferredModel");//$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
var AddProjectLinkDialog = com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog;

var CSSClass = {
	SERVICE_PROVIDER: 'serviceProviderIcon', //$NON-NLS-1$
	SERVICE_PROVIDER_CATALOG: 'serviceProviderCatalogIcon', //$NON-NLS-1$
	NO_SERVICE_PROVIDERS: 'noServiceProvidersIcon' //$NON-NLS-1$
};

(function(){

	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/AddProjectLinkDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		
		constructor: function(arg){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.projectAreaItemId = arg.projectAreaItemId; //item id of the project area adding the link to
			this.projectAreaName = arg.projectAreaName; //the name of the project area adding the link to
			this.originalProjectAreaName = arg.originalProjectAreaName; //the name of the project area before any edits
			this.firstServerChange = true;
			
			// This non-standard ugliness is necessary so that the message with the embeddeed link can be
			// freely translated.
			var anchorBegin = "<a class='serversLink' dojoAttachPoint='crossServerLink' href='#action=com.ibm.team.repository.admin.crossServer' title='" + this.messages.AddProjectLinkDialogHTML_NoServerDefinedTitle + "'>"; //$NON-NLS-1$ //$NON-NLS-2$
			this.noServerDefinedLink = dojo.string.substitute(this.messages.AddProjectLinkDialogHTML_NoServerDefined, [anchorBegin, "</a>"]); //$NON-NLS-1$
		},
		
		postCreate: function(){
			this._getServers();
			
			this.sourceProjectName.appendChild(document.createTextNode(this.projectAreaName));
			
			this.connect(this.serverSelector, "onchange", "_updateForServerChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.finishButton.disabled = true;
			this.originalHeaderHTML = this.dialogHeading.innerHTML;
		},
		
		startup: function(){
			com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.resizeI18NButtons(this.dialogFooter, [this.finishButton, this.cancelButton]);
			this.inherited(arguments);
		},
		
		_createServiceProvidersTree: function(serviceProviders){
			dojox.data.dom.removeChildren(this.serviceProvidersTree);
			if (this._serviceProvidersTree) {
				this._serviceProvidersTree.destroyRecursive();
				delete this._serviceProvidersTree;
			}
			var self = this;
			var store = new com.ibm.team.process.web.ui.internal.admin.projects.DeferredLoadReadStore({
				data: {
					identifier: "id", //$NON-NLS-1$
					label: "name", //$NON-NLS-1$
					items: serviceProviders
				}
			});
			var model = new com.ibm.team.process.web.ui.internal.admin.projects.DeferredModel({
				store: store
			});
			
			this._serviceProvidersTree = new dijit.Tree({
				persist: false,
				model: model,
				showRoot: false,
				
				getIconClass: function(item){
					if (item && !item.root) {
						var type = this.model.store.getValue(item, "type"); //$NON-NLS-1$
						if (type === ProcessConstants.SERVICE_PROVIDER_TYPE) {
							return CSSClass.SERVICE_PROVIDER;
						} else if (type === ProcessConstants.SERVICE_PROVIDER_CATALOG_TYPE) {
							return CSSClass.SERVICE_PROVIDER_CATALOG;
						} else if (type === ProcessConstants.SERVICE_PROVIDER_NOT_LOADED_TYPE || type === ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE) {
							return CSSClass.NO_SERVICE_PROVIDERS;
						}
					}
				},
				/*
				 * The methods below this point were taken from Tree.js so that we could
				 * intercept clicks on the icon and still allow the keyboard to function correctly
				 * in the tree.  As Tree.js changes we'll need to update these appropriately.
				 */
				focusNode: function(/* _tree.Node */node){ // from Tree.js
					// set focus so that the label will be voiced using screen readers
					node.labelNode.focus();
					// Process related additions
					this._focusChanged(node);
				},
				
				_onBlur: function(){
					if (this.lastFocused) {
						// Process related removals
						//var labelNode = this.lastFocused.labelNode;
						// we want to see what was last focused in the tree
						// dojo.removeClass(labelNode, "dijitTreeLabelFocused");	
					}
				},
				
				_onTreeFocus: function(/*Widget*/node){
					// summary:
					//		called from onFocus handler of treeitem labelNode to set styles, wai state and tabindex
					//		for currently focused treeitem.
					
					if (node) {
						if (node != this.lastFocused) {
							this.blurNode();
						}
						var labelNode = node.labelNode;
						// set tabIndex so that the tab key can find this node
						labelNode.setAttribute("tabIndex", "1003"); //$NON-NLS-1$ //$NON-NLS-2$
						dijit.setWaiState(labelNode, "selected", true); //$NON-NLS-1$
						dojo.addClass(labelNode, "dijitTreeLabelFocused"); //$NON-NLS-1$
						this.lastFocused = node;
						
						// process related addition
						// tree doesn't focus the node when tabbing in to the tree
						this.focusNode(node);
					}
				},
				
				_focusChanged: function(node){
					var type = this.model.store.getValue(node.item, "type"); //$NON-NLS-1$
					if (type === ProcessConstants.SERVICE_PROVIDER_CATALOG_TYPE) {//nested catalog
						self._selectedServiceProvider = undefined;
						self.finishButton.disabled = true;
					} else if (type === ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE || type === ProcessConstants.SERVICE_PROVIDER_NOT_LOADED_TYPE) { //$NON-NLS-1$
						self._selectedServiceProvider = undefined;
						self.finishButton.disabled = true;
					} else {
						self._selectedServiceProvider = node.item;
						self.finishButton.disabled = false;
					}
					
					self._updateLinkPreview();
				}
			});
			
			this.serviceProvidersTree.appendChild(this._serviceProvidersTree.domNode);
		},
		
		_getServerURL: function(){
			if (this.serverSelector.selectedIndex === -1 || this.serverSelector.options.length === 0) {
				return ProcessConstants.EMPTY;
			}
			return this.serverSelector.options[this.serverSelector.selectedIndex].resource;
		},
		
		_getServiceProviders: function(){
			delete this._selectedServiceProvider;
			dojox.data.dom.removeChildren(this.targetServiceProviderName);
			dojox.data.dom.removeChildren(this.serviceProvidersTree);
			var serverURL = this._getServerURL();
			if (serverURL.length === 0) {
				this.finishButton.disabled = true;
				return;
			}
			
			var deferred = jazz.client.xhrGet({
				url: serverURL,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, this._readServiceProvidersResponse),
				error: dojo.hitch(this, function(error){
					this._handleErrorGettingServiceProviders(deferred, error, this.messages.AddProjectLinkDialog_FailedServiceProviderCatalogResource);
					return error;
				})
			});
		},
		
		_readServiceProvidersResponse: function(serviceProviderXml){
			var children = serviceProviderXml.childNodes;
			var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var serviceProviderCatalog;
			var descriptionNodeChildren = descriptionNode.childNodes;
			if (this.linkTypes) {
				for (var i = 0; i < this.linkTypes.length; i++) {
					var linkType = this.linkTypes[i];
					var tagName = UIHelper.getTagNameForLinkType(linkType);
					var namespace = UIHelper.getNamespaceForLinkType(linkType);
					serviceProviderCatalog = UIHelper.getNode(descriptionNodeChildren, tagName, namespace);
					if (serviceProviderCatalog) {
						break;
					}
				}
			}
			if (!serviceProviderCatalog) {
				var serviceProviders = [];
				var serviceProviderTreeNode = {};
				serviceProviderTreeNode.name = this.messages.AddProjectLinkDialog_NotSupportedLinkType;
				serviceProviderTreeNode.id = this.messages.AddProjectLinkDialog_NotSupportedLinkType;
				serviceProviderTreeNode.type = ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE;
				serviceProviders.push(serviceProviderTreeNode);
				this._createServiceProvidersTree(serviceProviders);
				return serviceProviderXml;
			}
			this._currentLink = {};
			this._currentLink.linkType = linkType;
			this._currentLink.linkLabel = this._getLinkTypeLabel(linkType);
			var serviceProviderCatalogResourceNode = UIHelper.getAttributeNode(serviceProviderCatalog, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var matchingServiceProviderCatalogResource = serviceProviderCatalogResourceNode.value;
			var deferred = jazz.client.xhrGet({
				url: matchingServiceProviderCatalogResource,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, this._populateServiceProviders),
				error: dojo.hitch(this, function(error){
					this._handleErrorGettingServiceProviders(deferred, error, this.messages.AddProjectLinkDialog_FailedMatchingServiceProviderCatalogResource);
					return error;
				})
			});
			return serviceProviderXml;
		},
		
		_getServers: function(){
			var index = window.location.hash.toLowerCase().indexOf("testdiscovery=true"); //$NON-NLS-1$
			var rootServicesXml;
			if (index > -1) {
				rootServicesXml = "processtest/rootservices.xml"; //$NON-NLS-1$
			} else {
				rootServicesXml = ProcessConstants.ROOTSERVICESXML;
			}
			var deferred = jazz.client.xhrGet({
				url: rootServicesXml,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, this._readServers),
				error: dojo.hitch(this, function(error){
					this._handleErrorGettingServers(deferred, error, this.messages.AddProjectLinkDialog_FailedRootServicesResource);
					return error;
				})
			});
		},
		
		_handleErrorGettingServers: function(deferred, error, contextHeaderMessage){
			if (error.message) {
				if (error.dojoType === "popupBlocked") { //$NON-NLS-1$
					this._reportErrorFetchingRemote(error.message, contextHeaderMessage);
				} else {
					this._reportErrorFetchingRemote(decodeURIComponent(jazz.util.error.getDefaultXhrErrMsg(deferred)), contextHeaderMessage);
				}
			} else if (error.responseText) {
				this._reportErrorFetchingRemote(error.responseText, contextHeaderMessage);
			} else {
				this._reportErrorFetchingRemote(this.messages.AddProjectLinkDialog_NoErrorDetails, contextHeaderMessage);
			}
		},
		
		_reportErrorFetchingRemote: function(errorMessage, contextHeaderMessage){
			var iconPath = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/warning.gif"); //$NON-NLS-1$ //$NON-NLS-2$ 
			var imgHtml = "<img src=\"" + iconPath + "\"\/>"; //$NON-NLS-1$ //$NON-NLS-2$
			var brHtml = "<br />"; //$NON-NLS-1$
			if (!contextHeaderMessage) {
				contextHeaderMessage = this.messages.AddProjectLinkDialog_ErrorFetchingRemote;
			}
			
			this.dialogHeading.innerHTML = imgHtml + " " + contextHeaderMessage + brHtml + brHtml + errorMessage; //$NON-NLS-1$
			this.finishButton.disabled = true;
			dojox.data.dom.removeChildren(this.targetServiceProviderName);
		},
		
		_readServers: function(rootServicesXml){
			if (!rootServicesXml) {
				return;
			}
			var children = rootServicesXml.childNodes;
			var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var descriptionNodeChildren = descriptionNode.childNodes;
			
			this._determineLinkTypes(descriptionNodeChildren);
			
			var friendsNode = UIHelper.getNode(descriptionNodeChildren, "friends", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$ 
			var friendsResourceNode = UIHelper.getAttributeNode(friendsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var friendsResourceURL = friendsResourceNode.value;
			var deferred = jazz.client.xhrGet({
				url: friendsResourceURL,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, this._populateServers),
				error: dojo.hitch(this, function(error){
					this._handleErrorGettingServers(deferred, error, this.messages.AddProjectLinkDialog_FailedFriendsResource);
					return error;
				})
			});
			return rootServicesXml;
		},
		
		_determineLinkTypes: function(descriptionNodeChildren){
			//from root services walk the registered catalogs and find the matching project area
			//from this match you have the namespace and tag name
			//we know all the of the possible namespaces and tagnames so look for them out of the rootservices
			//TODO this will change when link types are extensible
			
			var tagNames = [ProcessConstants.ServiceProviderTypes.CM, ProcessConstants.ServiceProviderTypes.RM, ProcessConstants.ServiceProviderTypes.QM];
			var namesSpaces = [ProcessConstants.NAMESPACE_OSLC_CM, ProcessConstants.NAMESPACE_OSLC_RM, ProcessConstants.NAMESPACE_OSLC_QM];
			var catalogs = [];
			for (var i = 0; i < tagNames.length; i++) {
				var tagName = tagNames[i];
				var namespace = namesSpaces[i];
				var serviceProviderCatalog = UIHelper.getNode(descriptionNodeChildren, tagName, namespace);
				if (serviceProviderCatalog) {
					var catalog = {};
					var catalogResourceNode = UIHelper.getAttributeNode(serviceProviderCatalog, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
					catalog.resourceNode = catalogResourceNode.value;
					catalog.tagName = tagName;
					catalog.namespace = namespace;
					catalogs.push(catalog);
				}
			}
			
			if (catalogs.length === 0) { //look for a fronting server
				this._determineLinkTypesFromFrontingServer(descriptionNodeChildren);
				return;
			}
			if (this.originalProjectAreaName) {
				this._getDetermineLinkTypesDeferred(catalogs);
			} else { //new project area
				if (catalogs.length > 0) {
					this.linkTypes = this._getLinkTypesForTagName(catalogs.pop().tagName);
				}
			}
		},
		
		_determineLinkTypesFromFrontingServer: function(descriptionNodeChildren){
			var friendsNode = UIHelper.getNode(descriptionNodeChildren, "friends", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$ 
			var friendsResourceNode = UIHelper.getAttributeNode(friendsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var friendsResourceURL = friendsResourceNode.value;
			var deferred = jazz.client.xhrGet({
				url: friendsResourceURL,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, function(friendsListingXml){
					var children = friendsListingXml.childNodes;
					var rdfNode = UIHelper.getNode(children, "RDF", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
					var allFriends = UIHelper.getNodes(rdfNode.childNodes, "Friend", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
					dojo.forEach(allFriends, function(friend){
						var friendChildren = friend.childNodes;
						var specialNode = UIHelper.getNode(friendChildren, "internal", ProcessConstants.NAMESPACE_JFS); //$NON-NLS-1$
						if (specialNode && specialNode.firstChild && specialNode.firstChild.data === "true") { //$NON-NLS-1$
							var rootServicesNode = UIHelper.getNode(friendChildren, "rootServices", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
							var rootServicesResourceNode = UIHelper.getAttributeNode(rootServicesNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
							var deferred = jazz.client.xhrGet({
								url: rootServicesResourceNode.value,
								headers: {
									"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
								},
								handleAs: "xml", //$NON-NLS-1$
								load: dojo.hitch(this, function(rootServicesXml){
									if (!rootServicesXml) {
										return;
									}
									var children = rootServicesXml.childNodes;
									var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
									var descriptionNodeChildren = descriptionNode.childNodes;
									this._determineLinkTypes(descriptionNodeChildren);
									return rootServicesXml;
								}),
								error: dojo.hitch(this, function(error){
									this._handleErrorGettingServers(deferred, error, this.messages.AddProjectLinkDialog_FailedRootServicesResource);
									return error;
								})
							});
						}
						
					}, this);
					return friendsListingXml;
				}),
				error: dojo.hitch(this, function(error){
					this._handleErrorGettingServers(deferred, error, this.messages.AddProjectLinkDialog_FailedFriendsResource);
					return error;
				})
			});
		},
		
		/**
		 * Makes sequential xhr calls to the catalogs to find the catalog
		 * that contains the listing of the project area under edit.
		 */
		_getDetermineLinkTypesDeferred: function(catalogs){
			if (catalogs.length === 0) {
				return;
			}
			var catalog = catalogs.pop();
			return jazz.client.xhrGet({
				url: catalog.resourceNode,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, function(serviceProviderCatalogXML){
					var options = this._determineServiceProviders(serviceProviderCatalogXML);
					for (var j = 0; j < options.length; j++) {
						var serviceProviderCatalogOption = options[j];
						if (this.originalProjectAreaName == serviceProviderCatalogOption.name) {
							this.linkTypes = this._getLinkTypesForTagName(catalog.tagName);
							return serviceProviderCatalogXML;
						}
					}
					if (catalogs.length > 0) {
						return this._getDetermineLinkTypesDeferred(catalogs);
					}
					return serviceProviderCatalogXML;
				}),
				error: dojo.hitch(this, function(error){
					return error;
				})
			});
		},
		
		_getLinkTypesForTagName: function(serviceProviderTag){
			if (serviceProviderTag == ProcessConstants.ServiceProviderTypes.CM) {
				return [ProcessConstants.LinkTypes.TESTED_BY, ProcessConstants.LinkTypes.IMPLEMENTS];
			} else if (serviceProviderTag == ProcessConstants.ServiceProviderTypes.QM) {
				return [ProcessConstants.LinkTypes.TESTS, ProcessConstants.LinkTypes.VALIDATES];
			} else if (serviceProviderTag == ProcessConstants.ServiceProviderTypes.RM) {
				return [ProcessConstants.LinkTypes.IMPLEMENTED_BY, ProcessConstants.LinkTypes.VALIDATED_BY];
			}
			return [];
		},
		
		_populateServers: function(friendsListingXml){
			if (!friendsListingXml) {
				return friendsListingXml;
			}
			
			var children = friendsListingXml.childNodes;
			var rdfNode = UIHelper.getNode(children, "RDF", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var allFriends = UIHelper.getNodes(rdfNode.childNodes, "Friend", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
			if (allFriends.length > 0) {
				var servers = [];
				dojo.forEach(allFriends, function(friend){
					var friendChildren = friend.childNodes;
					var titleNode = UIHelper.getNode(friendChildren, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
					var friendTitle = titleNode.firstChild.data;
					var rootServicesNode = UIHelper.getNode(friendChildren, "rootServices", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
					var rootServicesResourceNode = UIHelper.getAttributeNode(rootServicesNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
					var anOption = new Option(friendTitle, friendTitle);
					anOption.resource = rootServicesResourceNode.value;
					servers.push(anOption);
				}, this);
				servers.sort(function(option1, option2){
					return com.ibm.team.process.web.client.internal.ProcessClient.stringCompare(option1.value, option2.value);
				});
				var selectOption = new Option(this.messages.AddProjectLinkDialog_SelectServer, this.messages.AddProjectLinkDialog_SelectServer);
				selectOption.resource = ProcessConstants.EMPTY;
				servers.unshift(selectOption);
				var optionArr = this.serverSelector.options;
				dojo.forEach(servers, function(serverOption){
					//appendChild(anOption) does not work on IE7, so modify array directly
					optionArr[optionArr.length] = serverOption;
				}, this);
			} else {
				this._handleNoServersDefined();
			}
			return friendsListingXml;
		},
		
		_handleNoServersDefined: function(){
			dojo.toggleClass(this._dialogBody, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this.dialogHeading, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this.finishButton, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this._dialogHeadingNoServers, "hidden", false); //$NON-NLS-1$
		},
		
		_populateServiceProviders: function(serviceProviderCatalogXml){
			if (!serviceProviderCatalogXml) {
				return serviceProviderCatalogXml;
			}
			this.dialogHeading.innerHTML = this.originalHeaderHTML;
			var serviceProviders = this._determineServiceProviders(serviceProviderCatalogXml);
			if (serviceProviders.length > 0) {
				serviceProviders.sort(function(provider1, provider2){
					return com.ibm.team.process.web.client.internal.ProcessClient.stringCompare(provider1.name, provider2.name);
				});
				this._createServiceProvidersTree(serviceProviders);
				this._updateLinkPreview();
			} else {
				var serviceProviderTreeNode = {};
				serviceProviderTreeNode.name = [this.messages.AddProjectLinkDialog_NoServiceProviders];
				serviceProviderTreeNode.id = [this.messages.AddProjectLinkDialog_NoServiceProviders + new Date().getTime()];
				serviceProviderTreeNode.type = [ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE];
				serviceProviders.push(serviceProviderTreeNode);
				this._createServiceProvidersTree(serviceProviders);
			}
			return serviceProviderCatalogXml;
		},
		
		_determineServiceProviders: function(serviceProviderCatalogXml){
			var serviceProviders = [];
			var children = serviceProviderCatalogXml.childNodes;
			var serviceProviderCatalog = UIHelper.getNode(children, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			children = serviceProviderCatalog.childNodes; //children of ServiceProviderCatalog
			var entries = UIHelper.getNodes(children, "entry", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			var providers = [];
			var providerCatalogs = [];
			dojo.forEach(entries, function(entry){
				var serviceProvider = UIHelper.getNode(entry.childNodes, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				if (serviceProvider) {
					providers.push(serviceProvider);
				}
				var serviceProviderCatalog = UIHelper.getNode(entry.childNodes, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				if (serviceProviderCatalog) {
					providerCatalogs.push(serviceProviderCatalog);
				}
			});
			
			dojo.forEach(providers, function(serviceProvider){
				var serviceProviderTreeNode = {};
				UIHelper.configureServiceProviderTreeNode(serviceProviderTreeNode, serviceProvider);
				serviceProviders.push(serviceProviderTreeNode);
			});
			
			dojo.forEach(providerCatalogs, function(serviceProviderCatalog){
				var serviceProviderCatalogNode = {};
				var stubNode = {};
				UIHelper.configureServiceProviderCatalogTreeNode(serviceProviderCatalogNode, stubNode, this.messages.AddProjectLinkDialog_NoServiceProvidersSpecified, serviceProviderCatalog);
				serviceProviders.push(serviceProviderCatalogNode);
			}, this);
			return serviceProviders;
		},
		
		_updateForServerChange: function(){
			if (this.firstServerChange) {
				var optionArr = this.serverSelector.options;
				optionArr[0] = null; //remove the Select Server... option
				this.firstServerChange = false;
			}
			this.finishButton.disabled = true;
			this._getServiceProviders();
			this._updateLinkPreview();
		},
		
		_updateLinkPreview: function(){
			dojox.data.dom.removeChildren(this.linkType);
			if (this._currentLink) {
				this.linkType.appendChild(document.createTextNode(" " + this._currentLink.linkLabel)); //$NON-NLS-1$
			}
			var serviceProvider = this._getSelectedServiceProvider();
			dojox.data.dom.removeChildren(this.targetServiceProviderName);
			if (serviceProvider && !serviceProvider.isError) {
				var targetServiceProvider = serviceProvider.name;
				this.targetServiceProviderName.appendChild(document.createTextNode(" " + targetServiceProvider)); //$NON-NLS-1$
			}
		},
		
		_handleErrorGettingServiceProviders: function(deferred, error, contextErrorHeading){
			if (error.message) {
				if (error.dojoType === "popupBlocked") { //$NON-NLS-1$
					this._reportErrorFetchingRemote(error.message, contextErrorHeading);
				} else {
					this._reportErrorFetchingRemote(decodeURIComponent(jazz.util.error.getDefaultXhrErrMsg(deferred)), contextErrorHeading);
				}
			} else if (error.responseText) {
				this._reportErrorFetchingRemote(error.responseText, contextErrorHeading);
			} else {
				this._reportErrorFetchingRemote(this.messages.AddProjectLinkDialog_NoErrorDetails, contextErrorHeading);
			}
		},
		
		getAssociatedProject: function(){
			var associatedProjectLink = {};
			var associatedServiceProviderInfo = {};
			var selectedServiceProvider = this._getSelectedServiceProvider();
			associatedServiceProviderInfo.targetUrl = selectedServiceProvider.url[0];
			associatedServiceProviderInfo.presentationUrl = selectedServiceProvider.presentationUrl[0];
			associatedServiceProviderInfo.consumerRegistryUrl = selectedServiceProvider.consumerRegistryUrl[0];
			associatedServiceProviderInfo.servicesUrl = selectedServiceProvider.servicesUrl[0];
			associatedServiceProviderInfo.label = selectedServiceProvider.name[0];
			associatedProjectLink.projectAreaInfo = associatedServiceProviderInfo;
			associatedProjectLink.linkType = this._currentLink.linkType;
			associatedProjectLink.linkLabel = this._currentLink.linkLabel;
			return associatedProjectLink;
		},
		
		_getSelectedServiceProvider: function(){
			return this._selectedServiceProvider;
		},
		
		_getLinkTypeLabel: function(linkType){
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return this.messages.LinkType_Implements;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return this.messages.LinkType_Implemented;
				case ProcessConstants.LinkTypes.VALIDATES:
					return this.messages.LinkType_Validates;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return this.messages.LinkType_Validated;
				case ProcessConstants.LinkTypes.TESTS:
					return this.messages.LinkType_Tests;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return this.messages.LinkType_Tested;
			}
		}
	});
	
})();
