/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TimelineModel"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TimelineModel", dijit.tree.ForestStoreModel, { //$NON-NLS-1$
		query: {
			enabledContext: ProcessConstants.PermissionContext.DEV_LINE
		},
		childrenAttrs: ["items"], //$NON-NLS-1$
		/*
		 * Constructs the Timeline model
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 *    store: the store the model should use for it's data
		 *  }
		 */
		constructor: function(args){
		},
		
		// From TreeStoreModel...needed to override so we could pass
		// in a queryOptions to do a deep query
		getRoot: function(onItem, onError){
			if (this.root) {
				onItem(this.root);
			} else {
				this.store.fetch({
					query: this.query,
					queryOptions: {
						deep: true
					},
					onComplete: dojo.hitch(this, function(items){
						if (items.length != 1) {
							// These are intentionally not translated...this entire
							// method was taken directly from TreeStoreModel so
							// that one parm to fetch could be overridden
							throw new Error(this.declaredClass + ": query " + dojo.toJson(this.query) + " returned " + items.length + //$NON-NLS-1$ //$NON-NLS-2$
							" items, but must return exactly one item"); //$NON-NLS-1$
						}
						this.root = items[0];
						onItem(this.root);
					}),
					onError: onError
				});
			}
		},
		// From ForestStoreModel...needed to override so we could pass
		// in a queryOptions to do a deep query
		_requeryTop: function(){
			// reruns the query for the children of the root node,
			// sending out an onSet notification if those children have changed
			var oldChildren = this.root.children || [];
			this.store.fetch({
				query: this.query,
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this, function(newChildren){
					this.root.children = newChildren;
					
					// If the list of children or the order of children has changed...	
					if (oldChildren.length != newChildren.length ||
					dojo.some(oldChildren, function(item, idx){
						return newChildren[idx] != item;
					})) {
						this.onChildrenChange(this.root, newChildren);
					}
				})
			});
		}
		
	});
	
})();
