﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

class BarShape extends ChartShape {
	
	private var _tooltipPositionX:Number;
	
	private var _tooltipPositionY:Number;

	public var scale:Number;
	
	public var scaleIncrement:Number;

	public static var SCALE_RATE:Number = 0.05;
	
	public static var MAX_BAR_THICKNESS:Number = 35; // Pixels
	
	
	public function BarShape(outlineColour:Number, fillColour:Number, name:String, value:Number, animate:Boolean) {
		super(outlineColour, fillColour, name, value, animate);
	}

	public function draw(parentMC:MovieClip, x1:Number, x2:Number, y1:Number, y2:Number) {
		this.mc = parentMC.createEmptyMovieClip(this.id, parentMC.getNextHighestDepth());
		
		var model:ChartModel = ChartEngine.getInstance().getModel();
		if (model.animation) {
			this.animateBar(x1, x2, y1, y2);
		} else {
			this.drawBar(x1, x2, y1, y2);
		}
		
		this._addEventHandlers();	
	}

	public function animateBar(x1:Number, x2:Number, y1:Number, y2:Number):Void {
		// NO-OP
		// Subclasses to override
	}
	
	public function drawBar(x1:Number, x2:Number, y1:Number, y2:Number):Void {
		
		this._tooltipPositionX = x2;
		this._tooltipPositionY = y1 - this.tooltip.getTooltipHeight() - 10;

		this.mc.beginFill(this.fillColour);
		this.mc.lineStyle(1, this.outlineColour);
		
		this.mc.moveTo(x1, y1);

		// Bottom edge
		this.mc.lineTo(x2, y1);
		// Right edge
		this.mc.lineTo(x2, y2);
		// Top edge
		this.mc.lineTo(x1, y2);

		this.mc.endFill();		
		
	}
	
	private function _addEventHandlers():Void {
		var context = this;
		
		this.mc.onRollOver = function() {
			this._alpha = 60;
			context.tooltip.show();
		};
		this.mc.onRollOut = function() {
			this._alpha = 100;
			context.tooltip.hide();
		};	

		var model:ChartModel = ChartEngine.getInstance().getModel();
		if (model.onClickHandler) {
			this.mc.onPress = function() {
				context.doDrillIn();
			};
		}
	}
	
	public function doRollOver():Void {
		// TODO: Generalize further
		this.tooltip.setPosition(this._tooltipPositionX, this._tooltipPositionY);
		super.doRollOver();
	}
	
	public function destroy():Void {
		
		delete this._tooltipPositionX;
		delete this._tooltipPositionY;
		delete this.scale;
		delete this.scaleIncrement;		
		
		super.destroy();
	}		
	
}