/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("jazz.ui.HoverPopup");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.form.TextBox");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var HoverPopup = jazz.ui.HoverPopup;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab",
	dijit.layout.ContentPane, {
		
	constructor: function(args) {
		this._messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this._settingsArea = args.settingsArea;
		// this is only used by PreferenceTab, but needs to be here
		// due to COMPUTED's reuse of LIST logic
		this._computedCombos = new Array();
		this._fields = new Object();
		this._rows = new Object();
	},
	
	getNumberOfFields: function() { return 1; },
	isRequired: function() { return true; },
	cancelSettings: function() {},
	setup: function() {},
	initWorkingCopy: function() {},
	getDefaultValue: function(data) { return null; },
	settingChanged: function(settingId) {},
	saveSettings: function(newSettings) {},
	
	createFormElements: function (data) {
		var table = document.createElement("table");
		table.className="prefs-table";
		var tbody = document.createElement("tbody");
		for(var i=0; i < data.length; i++){
			// There is no UI representation for the hidden preference type
			if (data[i].preferenceType == Constants.SettingsArea.FIELD_HIDDEN) {
				continue;
			}
			var tr = document.createElement("tr");
			var label = document.createElement("td");
			label.className = "label";
			var name = data[i].name;
			var field = document.createElement("td");
			field.className="field";
			this._rows[data[i].id] = tr;
			var content = name.substr(0,1).toUpperCase() + name.substr(1) + (data[i].required ? "<span class=\"required-asterisk\">*</span>" : "");
			var wrapper = document.createElement("label");
			wrapper.htmlFor = this._settingsArea.id + "-" + data[i].id;
			if (data[i].description) {
				new HoverPopup({around: wrapper, html: data[i].description, persist: false});
				dojo.addClass(tr,"showHover");
				wrapper.className = "hover-style";
			}
			wrapper.innerHTML = content; 
			label.appendChild(wrapper);
			this._fields[data[i].id] = this.createFormField(data[i], field, this.getDefaultValue(data[i]));
			tr.appendChild(label);
			tr.appendChild(field);
			tbody.appendChild(tr);
		}
		table.appendChild(tbody);
		return table;
	},
	createFormField: function(data, field, value) {
		switch(data.preferenceType) {
		case Constants.SettingsArea.FIELD_STRING:
			var inputDiv = document.createElement("div");
			var textBox = new dijit.form.TextBox({name: data.id, value: value, trim: true, id: this._settingsArea.id + "-" + data.id});
			textBox.startup();
			textBox.domNode.className += " string";
			if (data.startDisabled)
				textBox.domNode.disabled = true;
			this.connect(textBox.domNode, "onchange", data.eventHandler ? data.eventHandler : this._handleTextboxChange);
			// workaround for dojo bug: onchange not triggering on blur
			this.connect(textBox.domNode, "onblur", data.eventHandler ? data.eventHandler : this._handleTextboxChange);
			inputDiv.appendChild(textBox.domNode);
			field.appendChild(inputDiv);
			return textBox.domNode;
		case Constants.SettingsArea.FIELD_BOOLEAN:
			var inputDiv = document.createElement("div");
			var input = document.createElement("input");
			input.name = data.id;
			input.type = "checkbox";
			input.className = "checkbox";
			input.id = this._settingsArea.id + "-" + data.id;
			if (data.startDisabled)
				input.disabled = true;
			// use onclick since onchange with checkboxes is buggy in IE
			this.connect(input, "onclick", data.eventHandler ? data.eventHandler : this._handleCheckboxChange);
			inputDiv.appendChild(input);
			field.appendChild(inputDiv);
			if (value) {
				input.checked = true;
				input.defaultChecked = true;
			}
			return input;
		case Constants.SettingsArea.FIELD_COMPUTED:
			var result = this._settingsArea.viewlet.getPreferenceOptions(data.id, this._settingsArea.workingCopy);
			if (result) {
				delete data.computedDefault;
				dojo.mixin(data, result);
				delete data.optionsNotReady;
			}
			else {
				data.optionsNotReady = true;
			}
			//no return - fall through into next case block
		case Constants.SettingsArea.FIELD_LIST:
			var inputDiv = document.createElement("div");
			var select = document.createElement("select");
			select.name = data.id;
			select.id = this._settingsArea.id + "-" + data.id;
			if (data.optionsNotReady) {
				select.disabled = true;
				var option = document.createElement("option");
				option.appendChild(document.createTextNode(this._messages.settingsLoadingOpts));
				select.appendChild(option);
				setTimeout(dojo.hitch(this, function() {
					field.innerHTML = "";
					this.createFormField(data, field, value);
					}), 250);
			} else if (data.options)
				this._populateComboOptions(data.options, select, data.computedDefault ? data.computedDefault : value);
			else {
				for (var x in data.groups) {
					if (data.groups[x].name) {
						var group = document.createElement("optgroup");
						group.label = data.groups[x].name;
						this._populateComboOptions(data.groups[x].options, group, data.computedDefault ? data.computedDefault : value)
						select.appendChild(group);
					} else
						this._populateComboOptions(data.groups[x].options, select, data.computedDefault ? data.computedDefault : value)
				}
			}
			if (data.startDisabled)
				select.disabled = true;
			this.connect(select, "onchange", data.eventHandler ? data.eventHandler : this._handleComboboxChange);
			if (data.preferenceType == Constants.SettingsArea.FIELD_COMPUTED && !data.optionsNotReady) {
				this._computedCombos.push({
					select: select,
					data: data,
					field: field
				});
				this._settingsArea.workingCopy.preferences[data.id] = select.options[select.selectedIndex].value
			}
			inputDiv.appendChild(select);
			field.appendChild(inputDiv);
			return select;
		}
	},
	_populateComboOptions: function(options, parent, value) {
		for (var x in options) {
			var option = document.createElement("option");
			option.value = options[x].value;
			if (options[x].padding)
				option.style.paddingLeft = options[x].padding + "px";
			option.appendChild(document.createTextNode(options[x].label));
			if (value == options[x].value)
				option.selected = "selected";
			parent.appendChild(option);
		}
	},
	_handleTextboxChange: function(event) {},
	_handleComboboxChange: function(event) {},
	_handleCheckboxChange: function(event) {}

});

})();