/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItemActionBar"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.Menu"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.Actions"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.Separator"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.MenuAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.dialogs.SaveErrorDialog"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var Assert				= com.ibm.team.apt.web.ui.internal.common.Assert;

var Actions				= com.ibm.team.apt.web.ui.internal.view.plan.actions.Actions;
var Separator			= com.ibm.team.apt.web.ui.internal.view.plan.actions.Separator;
var MenuAction			= com.ibm.team.apt.web.ui.internal.view.plan.actions.MenuAction;
var SaveErrorDialog		= com.ibm.team.apt.web.ui.internal.view.plan.dialogs.SaveErrorDialog;

var doc= document;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var TransparentIconUrl	=dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/transparent.gif"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItemActionBar", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------

	_item: null,
	_entry: null,
	_viewModel: null,
	_itemMovePolicy: null,
	
	_actionDropDown: null,
	_selector: null,
	
	_checkbox: null,
	_selected: false,
	
	constructor: function() {
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getViewModel: function() {
		return this._viewModel;
	},
	
	setSelected: function(doSelect) {
		this._selected= doSelect;
		
		this.__updateSelection();
	},
	
	// ---- subclass API ---------------------------------------------------------------------------------------------------

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templateString: "<div dojoAttachPoint=\"_selector\"></div>", //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		var localDoc= doc;
		dojo.addClass(this._selector, "selector"); //$NON-NLS-1$
		
		var container= localDoc.createElement("div"); //$NON-NLS-1$
		dojo.addClass(container, "actionBarContainer"); //$NON-NLS-1$
		
		this._checkbox= localDoc.createElement("input"); //$NON-NLS-1$
		dojo.addClass(this._checkbox, "checkbox"); //$NON-NLS-1$
		this._checkbox.type= "checkbox"; //$NON-NLS-1$
		this.__updateSelection();
		
		this.connect(this._checkbox, "onclick", dojo.hitch(this, function(event) { //$NON-NLS-1$
			this.getViewModel().setEntrySelectionState(this._entry, this._checkbox.checked);
		}));
	
		container.appendChild(this._checkbox);
		
		this._actionDropDown=localDoc.createElement("a"); //$NON-NLS-1$
		dojo.addClass(this._actionDropDown, "actionDropDownMenu"); //$NON-NLS-1$
		this._actionDropDown.href= "#"; //$NON-NLS-1$
		this._actionDropDown.setAttribute("open", "false"); //$NON-NLS-1$ //$NON-NLS-2$
		this._actionDropDown.setAttribute("title", MSGS.ACTION_ACTIONS_DROP_DOWN_TITLE); //$NON-NLS-1$
		
		//Make the child of the link a real image so that we can pick up it's alt text when images are off
		var image = localDoc.createElement("img"); //$NON-NLS-1$
		image.src = TransparentIconUrl;
		image.alt = MSGS.ACTION_ACTIONS_DROP_DOWN_TITLE;
		this._actionDropDown.appendChild(image);
		
		container.appendChild(this._actionDropDown);
		this.connect(this._actionDropDown, "onclick", "__onActionDropDownOpen"); //$NON-NLS-1$ //$NON-NLS-2$
		
		this._selector.appendChild(container);
	},
	
	destroy: function() {
		this.inherited(arguments);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	

	__updateSelection: function() {
		if (!this._selected) {
			this._checkbox.setAttribute("checked", "false"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.toggleClass(this._item.domNode, "selected", false); //$NON-NLS-1$
			this._checkbox.checked= this._checkbox.defaultChecked= false;
		} else {
			this._checkbox.setAttribute("checked", "true"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.toggleClass(this._item.domNode, "selected", true); //$NON-NLS-1$
			this._checkbox.checked= this._checkbox.defaultChecked= true;
		}
	},
	
	__onActionDropDownOpen: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		if (this._menu) {
			this._closeMenu();
			return;
		}
		
		var menu = new dijit.Menu();
		dojo.addClass(menu.domNode, "com-ibm-team-apt-action-drop-down-menu"); //$NON-NLS-1$
		
		var selection= this.getViewModel().getSelectedEntries();
		
		var actions= Actions.createViewEntryActions(this.getViewModel(), this._itemMovePolicy, {
			showErrorMessage: function(message, description) {
				if (message != null) {
					new SaveErrorDialog({message: message, description: description});
				}
			}
		}, this._item);
		
		this._addActions(menu, actions, selection);
		
		var pos = dojo.coords(this._actionDropDown, true);
		
		this._actionDropDown.setAttribute("open", "true"); //$NON-NLS-1$ //$NON-NLS-2$
		dijit.popup.open({
			popup: menu,
			parent: this,
			x: pos.x + 5,
			y: pos.y + pos.h - 2,
			onExecute: dojo.hitch(this, this._closeMenu),
			orient: 'BL' //$NON-NLS-1$
		});
		
		//register event handler that causes menu to close
		this._menuCloseOnBody = this.connect(this, "_onBlur", this._closeMenu); //$NON-NLS-1$
		
		this._menu = menu;
		
		menu.focus();
	},
	
	_addActions: function(menu, actions, selection) {
		var hasAction= false;
		for ( var i = 0; i < actions.length; i++) {
			var action= actions[i];
			if (action instanceof Separator) {
				if (hasAction) {
					menu.addChild(new dijit.MenuSeparator());
				}
				hasAction= false;
			} else if (action instanceof MenuAction) {
				hasAction= true;
		
				var subMenu = new dijit.Menu();
				dojo.addClass(subMenu.domNode, "com-ibm-team-apt-action-drop-down-menu"); //$NON-NLS-1$
				
				menu.addChild(new dijit.PopupMenuItem({
					label: action.getName(this._entry, selection) + " >", //$NON-NLS-1$
					iconClass: null,
					popup: subMenu
				}));
				
				this._addActions(subMenu, action.createActions(), selection);
			} else if (action.canHandle(this._entry, selection)) {
				hasAction= true;
				menu.addChild(new com.ibm.team.apt.web.ui.internal.view.common.IconUrlMenuItem({
					label: action.getName(this._entry, selection),
					iconUrl: action.getIconUrl(),
					_action: action,
					_entry: this._entry,
					onClick: function() {this._action.handle(this._entry, selection);}
				}));		
			}
		}
	},
	
	_closeMenu: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		this.disconnect(this._menuCloseOnBody);
		dijit.popup.close(this._menu);
		this._menu = null;
		this._actionDropDown.focus();
		this._actionDropDown.setAttribute("open", "false"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItemActionBar.constructorArgs= function(viewModelViewerItem, viewModel, itemMovePolicy, entry) {
	return {
		_item: viewModelViewerItem,
		_viewModel: viewModel,
		_itemMovePolicy: itemMovePolicy,
		_entry: entry,
		_selected: viewModelViewerItem.isSelected()
	};
};

dojo.declare("com.ibm.team.apt.web.ui.internal.view.common.IconUrlMenuItem", //$NON-NLS-1$
		[dijit.MenuItem],
		{
		// summary: A line item in a Menu Widget

		// Make 3 columns
		//   icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString:
			 '<tr class="dijitReset dijitMenuItem" dojoAttachPoint="focusNode" waiRole="menuitem" tabIndex="-1"' //$NON-NLS-1$
			+'dojoAttachEvent="onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick">' //$NON-NLS-1$
			+'<td class="dijitReset" waiRole="presentation"><img class="dijitMenuItemIcon" src=${iconUrl} dojoAttachPoint="iconNode"></img></td>' //$NON-NLS-1$
			+'<td class="dijitReset dijitMenuItemLabel" dojoAttachPoint="containerNode"></td>' //$NON-NLS-1$
			+'<td class="dijitReset dijitMenuArrowCell" waiRole="presentation">' //$NON-NLS-1$
				+'<div dojoAttachPoint="arrowWrapper" style="display: none">' //$NON-NLS-1$
					+'<div class="dijitMenuExpand"></div>' //$NON-NLS-1$
					+'<span class="dijitMenuExpandA11y">+</span>' //$NON-NLS-1$
				+'</div>' //$NON-NLS-1$
			+'</td>' //$NON-NLS-1$
			+'</tr>', //$NON-NLS-1$

		__sentinel: null // terminates this class definition
});

})();
