/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorBanner");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull 		= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 		= net.jazz.ajax.toolbox.lang.checkNotEmpty;

var WorkItemUtil 		= com.ibm.team.workitem.web.client.util;
var ImagePool		= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var NEXT_PREV_MAX_LENGTH = 4;
var REGEX = new RegExp(/index=[0-9]+&?/);


/**
 * Provides a UI to iterate through the work items in a query result.
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorBanner",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemEditorBanner.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/WorkItemEditorBanner.css"),
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},	
		postCreate: function() {
			if (dojo.isIE && dojo.isIE < 7) {
	 			NEXT_PREV_MAX_LENGTH = 3;
			}
			this._menuDiv = document.createElement("span");
			this._menuDiv.className = "com-ibm-team-workitem-menu";
		},
		destroy: function() {
			this.inherited(arguments);
		},

		/*
		 *  This method is called to set the query result info when the work item editor is loaded
		 */
		setQueryInfo: function(workItemNumber, queryTitle, queryUrl, queryResults) {
			checkNotNull(workItemNumber, "WorkItemBanner.setQueryInfo.workItemNumber");
//			checkNotNull(queryUrl, "WorkItemBanner.setQueryInfo.workItemNumber");
//			checkNotNull(queryTitle, "WorkItemBanner.setQueryInfo.workItemNumber");
//			checkNotNull(queryResults, "WorkItemBanner.setQueryInfo.workItemNumber");
			
			dojox.data.dom.removeChildren(this.pageNumberContainer);
			dojox.data.dom.removeChildren(this.breadcrumbs);
			
			var currentIndex = this._getCurrentIndex(workItemNumber, queryResults);
			
			if(currentIndex === 0) {
				this.domNode.style.display = "none";
				return;
			} else {
				this.domNode.style.display = "block";
			}
			
			var startIndex;
			if (queryUrl) {
				try{
				var indexString = REGEX.exec(queryUrl + "&"); // index=41&
				if (indexString) {
					startIndex = parseInt(indexString[0].replace(/&/, "").replace("index=", ""));
				}
				} catch(e) { }; // ignore
			}
			startIndex = startIndex || 1;
			
			this.queryUrl = queryUrl;
			this.queryTitle = queryTitle;
			
			var anchor = document.createElement("A");
			anchor.href = queryUrl;
			anchor.appendChild(document.createTextNode(queryTitle));
			this.breadcrumbs.appendChild(anchor);
			this.breadcrumbs.appendChild(document.createTextNode(" > "));
			
			this.pageNumberContainer.style.display="block";
			// show previous page text or link
			this.pageNumberContainer.appendChild(this._getPreviousElement(workItemNumber, queryResults));
			
			// show count info
			var pageInfoSpan= document.createElement("span");
			
			pageInfoSpan.className= "pageInfoSpan";
			
			currentIndex = currentIndex + startIndex -1;
			bold = document.createElement("B");
			bold.appendChild(document.createTextNode(currentIndex + ""));
			pageInfoSpan.appendChild(bold);
			pageInfoSpan.appendChild(document.createTextNode(this.messages.labelIn));
			bold = document.createElement("B");
			bold.appendChild(document.createTextNode(startIndex + " - " + (startIndex + queryResults.length -1)));
			pageInfoSpan.appendChild(bold);
			
			this.pageNumberContainer.appendChild(pageInfoSpan);
			
			// show next page link or text
			this.pageNumberContainer.appendChild(this._getNextElement(workItemNumber, queryResults));
		},
		_showZeroResultHeader: function(workItemNumber) {
			var bold = document.createElement("B");
			bold.appendChild(document.createTextNode(this.messages.labelCurrentQuery));
			this.breadcrumbs.appendChild(bold);
			this.breadcrumbs.appendChild(document.createTextNode(this.messages.labelNone));
			this.pageNumberContainer.appendChild(this._getEmptyPageNavElement());
			this.pageNumberContainer.style.display="none";
		},
		_getPreviousElement: function(workItemNumber, queryResults) {
			var prevSummary, prevUrl, prevNumber, prevList=[], listTitle;
			var itemIndex =0;
			var listTitleSpan = document.createElement("SPAN");
			for(itemIndex; itemIndex<queryResults.length;itemIndex++) {
				if(queryResults[itemIndex].indices[0] == workItemNumber) {
// "<B>Current Work Item:</B> <SPAN style='color:#035D81'>" + queryResults[itemIndex].indices[0] + ": " + queryResults[itemIndex].indices[3] + "</SPAN>";
					var b = document.createElement("B");
					b.appendChild(document.createTextNode(this.messages.labelCurrentWorkItem));
					listTitleSpan.appendChild(b);
					listTitleSpan.appendChild(document.createTextNode(" "));
					var span = document.createElement("SPAN");
					span.style.color = "#035D81";
					span.appendChild(document.createTextNode(queryResults[itemIndex].indices[0] + ": " + queryResults[itemIndex].indices[3]));
					listTitleSpan.appendChild(span);
					break;
				}
				prevSummary = queryResults[itemIndex].indices[0] + ": " + queryResults[itemIndex].indices[3];
				prevNumber =queryResults[itemIndex].indices[0];
				prevUri = WorkItemUtil.getViewWorkItemUri(prevNumber)+ "&showLoadingPage=false";
				prevList.unshift({ href:prevUri,  itemId: prevNumber, summary: prevSummary});
				
			}

			// Show only a few items
			if(prevList.length > NEXT_PREV_MAX_LENGTH) {
				var shortList = [];
				for(var i=0;i<NEXT_PREV_MAX_LENGTH;i++) {
					shortList[i] = prevList[i];
				}
				var left = prevList.length - NEXT_PREV_MAX_LENGTH;
				prevList = shortList;
				prevList.footerText = this.messages.buttonBackToCurrentQuery;
			} else {
				prevList.footerText =  this.messages.buttonBackToCurrentQuery;
			}
			prevList.footerUrl = this.queryUrl;					
			if(prevSummary) {
				prevUri = WorkItemUtil.getViewWorkItemUri(prevNumber) + "&showLoadingPage=false";
				prevList.workItemId = workItemNumber;
				return this._getLinkElement(prevUri, this.messages.buttonPrevious, listTitleSpan, prevList, true);
			}

			return	this._getDisabledTextElement(this.messages.buttonPrevious);
		},
		_getNextElement: function(workItemNumber, queryResults) {
			var nextSummary, nextUrl, nextNumber, nextList=[], listTitle;
			var itemIndex =queryResults.length - 1;
			var listTitleSpan = document.createElement("SPAN");
			for(itemIndex; itemIndex>=0;itemIndex--) {
				if(queryResults[itemIndex].indices[0] == workItemNumber) {
//					listTitle =  "<B>Current Work Item:</B> <SPAN style='color:#035D81'>" + queryResults[itemIndex].indices[0] + ": " + queryResults[itemIndex].indices[3] + "</SPAN>";
					var b = document.createElement("B");
					b.appendChild(document.createTextNode(this.messages.labelCurrentWorkItem));
					listTitleSpan.appendChild(b);
					listTitleSpan.appendChild(document.createTextNode(" "));
					var span = document.createElement("SPAN");
					span.style.color = "#035D81";
					span.appendChild(document.createTextNode(queryResults[itemIndex].indices[0] + ": " + queryResults[itemIndex].indices[3]));
					listTitleSpan.appendChild(span);
					break;
				}
				nextSummary = queryResults[itemIndex].indices[0] + ": " + queryResults[itemIndex].indices[3];
				nextNumber =queryResults[itemIndex].indices[0];
				nextUri = WorkItemUtil.getViewWorkItemUri(nextNumber)+ "&showLoadingPage=false";
				nextList.unshift({ href:nextUri,  itemId: nextNumber, summary: nextSummary});
				
			}

			// Show only a few items
			if(nextList.length > NEXT_PREV_MAX_LENGTH) {
				var shortList = [];
				for(var i=0;i<NEXT_PREV_MAX_LENGTH;i++) {
					shortList[i] = nextList[i];
				}
				var left = nextList.length - NEXT_PREV_MAX_LENGTH;
				nextList = shortList;
				nextList.footerText = this.messages.buttonBackToCurrentQuery;
			} else {
				nextList.footerText =  this.messages.buttonBackToCurrentQuery;
			}
			nextList.footerUrl = this.queryUrl;					
			if(nextSummary) {
				nextUri = WorkItemUtil.getViewWorkItemUri(nextNumber)+ "&showLoadingPage=false";
				nextList.workItemId = workItemNumber;
				return this._getLinkElement(nextUri, this.messages.buttonNext, listTitleSpan, nextList, false);
			}

			return	this._getDisabledTextElement(this.messages.buttonNext);
		},

		_getCurrentIndex: function(workItemNumber,queryResults) {
			if(!queryResults) {
				return 0;
			}
			for(var i=0; i<queryResults.length;i++) {
				if(queryResults[i].indices[0] == workItemNumber) {
					return i+1;
				}
			}
			return 0;
		},
		_getEmptyPageNavElement:function () {
			var span= document.createElement("span");
			span.className= "disabledText";
			span.innerHTML = "&nbsp;" /* InnerHTML OK */
			return span;
		} ,
		_getDisabledTextElement: function(str) {
			var span= document.createElement("span");
			span.className= "disabledText";
			span.appendChild(document.createTextNode(str));
			return span;
		},
		_addPopupElement: function(itemListTitleSpan, itemList, isPrevious) {
			var popupDiv = document.createElement("SPAN");
			var linkElement = document.createElement("A");
			var imgElement = document.createElement("IMG");
			imgElement.className = "dropdownImage"
			imgElement.src = ImagePool.POPUP_DOWN_ARROW;

			popupDiv.className = "dropdownImagePopupDiv";
			popupDiv.itemListTitleSpan = itemListTitleSpan;
			popupDiv.itemList = itemList;
			popupDiv.itemListRendererObject  = this;
			popupDiv.itemListRendererSelector = "discloseResults";
			
			var disclosed= this.discloseResults(itemList);
			if (disclosed) {
				new jazz.ui.HoverPopup({
					around: [popupDiv, imgElement],
					html: disclosed.innerHTML,
					orientation: isPrevious ? "L" : "R"
				});
			}
			this.connect(imgElement, "onmouseover",
					function(e) {
						imgElement.className = "dropdownImage-mouseover"
					});
			this.connect(imgElement, "onmouseout",
					function(e) {
						imgElement.className = "dropdownImage"
					});
			linkElement.appendChild(imgElement);
			popupDiv.appendChild(linkElement);
			return popupDiv;
		},
		_getLinkElement: function(href, text, listTitleSpan, itemList, isPreviousElement) {
			checkNotEmpty(text, "text");
			checkNotEmpty(href, "href");
			var topDiv= document.createElement("SPAN");

			var linkElement = document.createElement("a");
			linkElement.href = href;
			if(itemList.length > 0) {
				 if(isPreviousElement) {
				 	linkElement.title = dojo.string.substitute(this.messages.buttonBackToItem, [ itemList[0].summary ]);
				 } else {
				 	linkElement.title = dojo.string.substitute(this.messages.buttonForwardToItem, [ itemList[0].summary ]);
				 }
			}
			linkElement.appendChild(document.createTextNode(text));
			
				var self = this;
				this.connect(linkElement, "onclick",
					function(e) {
						self._handleNextClick(e);
					});
			linkElement.style.marginRight = "2px";
			topDiv.appendChild(linkElement);
			topDiv.appendChild(this._addPopupElement(listTitleSpan, itemList, isPreviousElement));				
			
			return topDiv;
		},
		discloseResults: function(itemList){
			if(!itemList || !this.parentView) // should never happen
				return null;
			
			var ul = document.createElement("UL");
			ul.className="disclosureList";
			
			
			try {
			for(var i=0;i<itemList.length;i++) {
//					result += "<li><a href=\"" + itemList[i].href + "\">" + WorkItemUtil._breakupAllLongWords(itemList[i].summary, 55) + "</a>";
					var li = document.createElement("LI");
					var a = document.createElement("A");
					a.href= itemList[i].href;
					a.appendChild(document.createTextNode(WorkItemUtil._breakupAllLongWords(itemList[i].summary, 55)));
					
					li.appendChild(a);
					ul.appendChild(li);
			}	
			} catch(e) {
				console.debug("error in WorkItemEditorBanner.discloseResults");
				return null;
			}
			return ul;
		},
		_handleNextClick: function(e) {
			if(!e) return; // IE6 problem
			e.target.isMouseOver = false;
		}
	}
);

})();