/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.OwnerAttributePart");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.ContributorWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
/*
 * Handle presentation operations pertaining to the owner attribute
 */
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var ContributorWidget = com.ibm.team.workitem.web.ui.internal.view.layout.ContributorWidget;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.OwnerAttributePart",
	ContributorWidget,
	{
        constructor: function() {
			this.ownerMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
		},
		postCreate: function()  {
			this._warning.title = this.ownerMessages.titleOwnerDoesNotBelongToTeamArea;
  			this.moreLabel = this.ownerMessages.buttonMorePrompt;
			this.moreTitle= this.ownerMessages.titleOwnerMore;


			this.workingCopy.addListener({
				path: ["attributes", "category"],
				event: "onchange",
				listener: this,
				functionName: "_fetchOwners"
			});

			this.workingCopy.addListener({
				path: ["attributes", "owner"],
				event: "onchange",
				listener: this,
				functionName: "_handleOwnerChange"
			});
			this._fetchOwners();
			this.inherited(arguments);
		},
		_addCurrentOwnerToOwnerList: function(values, e) {
			return this._addExistingOwnerToOwnerList(values, this.workItem.attributes.owner); // use workItem, not workingCopy.
		},
		_addExistingOwnerToOwnerList: function(ownerList, otherOwner) {
			if(!otherOwner) return ownerList;
			var found = false;
			var otherId = otherOwner.id || otherOwner.itemId;
			for(var i=0;i<ownerList.length;i++) {
				var ownerId = ownerList[i].id || ownerList[i].itemId;
				if(ownerId == otherId) {
					found = true;
					break;
				}
			}
			if(!found) {
				var item = otherOwner;
				item.isOutOfProjectArea = true;
				item.id = item.id || item.itemId;
				item.label = item.label || item.name;
				ownerList.push(item);
			}
			return ownerList;
		},
		onSelectCreate: function(value){
			this._setAttribute(value, true);
			},
		_handleOwnerChange: function(event) {
			var ownerList = this._addCurrentOwnerToOwnerList(this.values);		
			var currentSelection = this.workingCopy.getValue( {
					path: ["attributes", "owner"]
			});
			ownerList = this._addExistingOwnerToOwnerList(ownerList, currentSelection);
			this.updateOptions(ownerList, currentSelection.id || currentSelection.itemId);
			if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent({target: this._select});
			}
		},
		_fetchOwners: function() {
			var self= this;
			var handlerObj= {
				success: function(values) {
					try {
						if(self.parentWidget && self.parentWidget.isDestroyed) return; // returned after destroyed
						self.values = values;
						var ownerList = self._addCurrentOwnerToOwnerList(values);
						var currentSelection = self.workingCopy.getValue({
							path: ["attributes", "owner"]
						});
						ownerList = self._addExistingOwnerToOwnerList(ownerList, currentSelection);
						
						self.updateOptions(ownerList, currentSelection.id || currentSelection.itemId);
					} catch(e) {
						// ignore. this path can fail if the async op returns after the widget is destroyed
					}
			},
				error: function(errorObj) {
					// ignore if the parentController is no longer set (called after destroy)
					if (self.parentController && self.parentController.isDestroyed != true) {
						if (errorObj.message) {
							var msg = dojo.string.substitute(self.ownerMessages.errorFetchingOwners, [errorObj.message]);
						}
						else {
							var msg = dojo.string.substitute(self.ownerMessages.errorFetchingOwners, [" "]);
						}
						if(dojo.isFunction(self.parentController.handleError)) {
							self.parentController.handleError(errorObj, msg);							
						} else {
							alert(msg);
						}
					}
				}
			};
			var srh= new ServiceResponseHandler(handlerObj, "success", "error");
			var categoryId = this.workingCopy.getValue({path: ["attributes", "category", "id"]});
			if(!categoryId) return; // not yet initialized, will be picked up when the category value is set
			var appArgs= {
				categoryItemId: categoryId
			};
			if(this.workingCopy.getValue({
				path: ["attributes", "target", "id"]
			})) {
				appArgs.intervalItemId = this.workingCopy.getValue({
					path: ["attributes", "target", "id"]
				});
			}
			appArgs.projectAreaItemId = this.workingCopy.getProjectAreaItemId();
			WorkItemClient.getOwners(srh, appArgs);
		},
		needsWarning: function(selectedValueId, defaultValue) {
			var needsWarning = false;
			for(var i=0;i<this._possibleValues.length;i++) { 
				if(this._possibleValues[i].id == selectedValueId) {
					if (this._possibleValues[i].isOutOfProjectArea) {
						needsWarning = i > 0; // Skip the "Unassigned" value
					}
					break;		
				}
			}
			return needsWarning;
		}
 	}
);

})(); 