/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ValidatedAttributePart");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
/*
 * This is a wrapper widget that provides the presentation and value
 * processing for Duration, presented in 
 */
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ValidatedAttributePart",
	[dijit._Widget, dijit._Templated],
	{
		
		templateString: '<div class="ValidatedAttributePart">' +
						'<div dojoAttachPoint="_content" style="position:relative;left:10px;"></div>' +
						'</div>',
      constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},	
		postCreate: function() {
			/*
			 * value[0]
			 *    label
			 *    validationRegex
			 *    id
			 */
			var value = this._getAttributeValue();
			if(value) {
 				if (value.validationRegex) {
					this.validationRegex = value.validationRegex;
					this.validationMessage = value.validationMessage ? value.validationMessage : this.messages.msgValueNotValid;
				}
			} else {
				value = { id: "" };
			}

			if (!this.validationRegex && this.creationSpec.validationRegex) {
				this.validationRegex = this.creationSpec.validationRegex;
				this.validationMessage = this.creationSpec.validationMessage ? this.creationSpec.validationMessage : this.messages.msgValueNotValid;
			}

			this.isNumber = this.creationSpec.isNumber == true;
			
			var creationArgs = {
						parentWidget: this,
						onValueChanged: "onValueChanged",
						parentController: this.parentController,
						presentation: {
							attribute: {
								attributeTypeId: "ValidatedAttribute"
							}
						},
						selection:  [ {
							id: value.id
							} 
						 ] ,
						 converter: this._TextWidget_ValidatedAttributeConverter,
						 converterParms: {
						 		toolTip: this.validationMessage,
								value: value
						 }
					};
					
			this.inputWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget(creationArgs);
			this.inputWidget.domNode.style.maxWidth = "19em";
			this.inputWidget._textInput.style.width = "85%";
			
			if(this.workingCopy.isGuest()) {
				dojo.addClass(this.inputWidget._textInput, "GuestField");
			}
			
			this._content.appendChild(this.inputWidget.domNode);
			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});			
			
		},
		_handleAttributeChange: function(parms) {
			var value = parms.attributeValue; /* id */
			this._setTextValue(value);
		},
		_getAttributeValue: function() {
			return this.workingCopy.getValue({ path: ["attributes", this.attributeId] });			
		},
		_setAttributeValue: function(value, isInit){
			if(this.isNumber) {
				value = value.replace(/,/g, "");
			}
			if(this.isBoolean) {
				value = value=="t" || value =="true";
			}
		    value = value || "";			

			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", this.attributeId, "id"],
				savedAttributePath: ["attributes", this.attributeId, "id"],
				attributeId: this.attributeId,
				isInitialization: isInit,
				value: value
			});
		},
		_setTextValue: function(value){
			value = value || "";
			if(value) {
				this.inputWidget._textInput.value = value;
			}
		},
		onValueChanged: function(e){
			this._setAttributeValue(this.getValue());
		},
		setReadOnly: function(isReadOnly) {
			this.inputWidget.setReadOnly(isReadOnly);
		},
	     getValue: function() {
			return this.inputWidget._textInput.value;
		 },
	//* This runs in the context of TextWidget
		//
		_TextWidget_ValidatedAttributeConverter: function (value, parms, isGettingValidValue) {
			return this.parentWidget._validationConverter(value, parms, isGettingValidValue);
		},
		_validationConverter: function (value, parms, isGettingValidValue) {
			var valid = false;
			if(!this.validationRegex || !value || value.length == 0) {
				valid = true;
			} 

			if(value && this.validationRegex && value.length > 0) {
				try{
					var regex = new RegExp("^" + this.validationRegex + "$");
 					valid = value.search(regex) != -1;
				} catch(e) {
					console.debug("Invalid regular expression for ValidateAttributePart: " + this.validationRegex);
					valid = true;
				}

			} 
			
			if(isGettingValidValue) {
				if(!value || value.length == 0) {
					return "";
				}				
				return value;
			}

			return {isCustom: true, isValid: valid };
		}
	}
)

})();
