/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.actions.AddAttachedPageAction"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.ProgressMonitoredServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.dialogs.InputDialog"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.TabAttachedPage"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.Action"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var ResponseHandler= com.ibm.team.apt.web.client.internal.ProgressMonitoredServiceResponseHandler;	
var PlanningClient= com.ibm.team.apt.web.client.internal.PlanningClient;	

var InputDialog= com.ibm.team.apt.web.ui.internal.view.plan.dialogs.InputDialog;
var TabAttachedPage= com.ibm.team.apt.web.ui.internal.view.plan.TabAttachedPage;

var Assert= com.ibm.team.apt.web.ui.internal.common.Assert;

var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var IconUrl= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/create_page-hov.gif"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.AddAttachedPageAction", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.view.plan.actions.Action], {

	_callCtrl: null,
	_planPage: null,
		
	constructor: function(callCtrl, planPage) {
		this._callCtrl= callCtrl;
		this._planPage= planPage;
	},
	
	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getId: function() {
		return "attachedPage.add"; //$NON-NLS-1$
	},
	
	getName: function() {
		return MSGS.ACTION_NEW_ATTACHED_PAGE;
	},
	
	getIconUrl: function() {
		return IconUrl;
	},
	
	canHandle: function() {
		return !this._planPage.isEditMode();
	},
	
	handle: function() {
		var fnCreatePage= dojo.hitch(this, function(name){
			var self= this;
			var serviceCallback= {
				_success: function(result) {
					self._callCtrl.refreshPlan({id: self._planPage.getPlanId(), page: "wikiPage_" + name}); //$NON-NLS-1$
				},
				_error: function(error) {
					alert(error);
				}
			};
			var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$ 
			responseHandler.setLabel(MSGS.MONITOR_NEW_ATTACHED_PLAN, this._callCtrl);	
			PlanningClient.postAttachedPage(responseHandler, this._planPage.getPlanId(), name);
		});
	
		var fnValidate= dojo.hitch(this, function(name){
			if(name.length < 1)
				return MSGS.VALIDATION_ATTACHED_PAGE_EMPTY_NAME;
			if(name.length > 250)
				return MSGS.VALIDATION_ATTACHED_PAGE_LONG_NAME;
			if(name.indexOf('$') != -1 || name.indexOf(':') != -1 || name.indexOf('/') != -1) //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ 
				return MSGS.VALIDATION_ATTACHED_PAGE_INVALID_NAME;
			var attachedPage= this._planPage._getAttachedPages();
			for(var i = 0; i < attachedPage.length; i++) {
				if(attachedPage[i].label.toLowerCase() == name.toLowerCase() || "overview" == name.toLowerCase()) //$NON-NLS-1$
					return MSGS.VALIDATION_ATTACHED_PAGE_DUP_NAME;
			}
			return null;				
		});
	
		var dialog= new InputDialog({
			message: MSGS.DIALOGS_NEW_PAGE_TITLE,
			description: MSGS.DIALOGS_NEW_PAGE_DESCRIPTION,
			onOk: fnCreatePage,
			fnValidate: fnValidate
		});
	},

	__sentinel: null // terminates this class definition
});

})();
