/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemAction"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.ImageClient"); //$NON-NLS-1$

(function() {
	
var ImageClient				= com.ibm.team.apt.web.client.internal.ImageClient;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemAction", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.view.plan.actions.Action], {
	
	_viewModel: null,
	_workItemType: null,
	_movePolicy: null,
	
	constructor: function(viewModel, workItemType, itemMovePolicy) {
		this._viewModel= viewModel;
		this._workItemType= workItemType;
		this._movePolicy= itemMovePolicy;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getId: function() {
		return "new.planItem." + this._workItemType.itemId; //$NON-NLS-1$
	},
	
	getName: function() {
		return this._workItemType.label;
	},
	
	getIconUrl: function() {
		return this._workItemType.iconURL ? ImageClient.getImageUrl(this._workItemType.iconURL) : null;
	},
	
	canHandle: function(targetEntry, selection) {
		var target;
		var location;
		if (arguments.length != 2) {
			target= this._getTarget(); 
			location= this._getLocation();
		} else {
			target= targetEntry;
			location= com.ibm.team.apt.ui.Create.After;
		}
		
		if (target.getElement() instanceof com.ibm.team.apt.ui.structure.GroupElement)
			location= com.ibm.team.apt.ui.Create.Child;
		
		return this._movePolicy.canCreate(target, this._workItemType, location);
	},
	
	handle: function(targetEntry, selection) {
		var target;
		var location;
		if (arguments.length != 2) {
			target= this._getTarget(); 
			location= this._getLocation();
		} else {
			target= targetEntry;
			location= com.ibm.team.apt.ui.Create.After;
		}
		
		if (target.getElement() instanceof com.ibm.team.apt.ui.structure.GroupElement)
			location= com.ibm.team.apt.ui.Create.Child;
		
		return this._movePolicy.create(target, this._workItemType, location);
	},
	
	// --- spi ---------------------------------
	
	_getTarget: function() {
		return this.__findFirstLeaf();
	},
	
	_getLocation: function() {
		return com.ibm.team.apt.ui.Create.Before;
	},
	
	// --- implementation ------------------------------------
	
	__findFirstLeaf: function() {
		return this._viewModel.readModel(function(readAccessor) {
			return this.__findFirstLeafRec(readAccessor.getRootEntry(), readAccessor.getEntryNavigator(true));
		}, this);
	},
	
	__findFirstLeafRec: function(entry, n) {
		var children= n.childEntries(entry);
		if (children.length == 0)
			return entry;
		
		var child= children[0];
		if (child.getElement() instanceof com.ibm.team.apt.client.PlanItem)
			return entry;
		
		return this.__findFirstLeafRec(child, n);
	},
		
	__sentinel: null // terminates this class definition
});

})();
