/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.actions.ToolBarManager"); //$NON-NLS-1$

dojo.require("dijit.Menu"); //$NON-NLS-1$
//dojo.require("dijit.MenuItem"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.Action"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var Action= com.ibm.team.apt.web.ui.internal.view.plan.actions.Action;
var removeChildren= dojox.data.dom.removeChildren;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var ACTION_ID= "actionId"; //$NON-NLS-1$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.ToolBarManager", //$NON-NLS-1$
			 null, {

	_actions: null,
	_styles: null,
	
	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	
	constructor: function(styles){
		this._actions= [];
		
		if(!styles)
			styles= {};
		styles.action= styles.action || "action"; //$NON-NLS-1$
		styles.disabled= styles.disabled || "disabled"; //$NON-NLS-1$
		styles.toggle= styles.toggle || "toggle"; //$NON-NLS-1$
		styles.dropDown= styles.dropDown || "dropDown"; //$NON-NLS-1$
		styles.separator= styles.separator || "separator" // $NON-NLS-1$
		this._styles= styles;
	},
	
	// ---- api ------------------------------------------------------------------------------------------------------------
	
	add: function(action) {
		this._actions.push(action);
	},
	
	create: function(parent) {
		dojo.addClass(parent, "toolBar"); //$NON-NLS-1$
		dojo.forEach(this._actions, function(action) {
			this.__widgetForAction(parent, action);
		}, this);
	},
	
	update: function() {
		this.__update(this._actions);
	},
	
	fillToolBar: function(/*node*/ toolbar) {
		this.create(toolbar);
	},
	
	fillMenu: function(/*dijit.Menu*/ menu) {
		this.__fillMenu(menu, this._actions);
	},
	
	// -- implementation ---------------------------------------------------------------------------------------------------
		
	__widgetForAction: function(parent, action) {
		
		var result= null;
		var isSeparator= action instanceof com.ibm.team.apt.web.ui.internal.view.plan.actions.Separator;
		var style= action.getStyle();
		
		if(isSeparator) {
			result= document.createElement("div"); //$NON-NLS-1$ 
			dojo.addClass(result, this._styles.separator);
			
		} else if((style & Action.STYLE_FLAT) != 0 || (style & Action.STYLE_DROP_DOWN) != 0){
			var link= document.createElement("a"); //$NON-NLS-1$ 
			link.href= "#"; //$NON-NLS-1$
			link.title= action.getName();
			if(!action.getIconUrl()){
				link.appendChild(document.createTextNode(action.getName()));
			} else {
				var img= document.createElement("img"); //$NON-NLS-1$ 
				img.src= action.getIconUrl();
				img.alt= action.getName();
				link.appendChild(img);
			}
			result= document.createElement("div"); //$NON-NLS-1$
			result.appendChild(link);
			dojo.addClass(result, this._styles.action);
			
		} else if((style & Action.STYLE_PUSH) != 0){
			result= document.createElement("button"); //$NON-NLS-1$
			result.appendChild(document.createTextNode(action.getName()));
		}
		
		result.setAttribute(ACTION_ID, action.getId());
		
		if(!isSeparator){
			
			var self= this;
			var handler= function(evt){
				if (evt) {
					dojo.stopEvent(evt);
				}
				
				if ((style & Action.STYLE_DROP_DOWN) != 0) {
					self._openMenu(evt, action);
				} else {
					action.handle(evt);
				}
				
				self.update();
			};
			selectHandle= dojo.connect(result, "onclick", handler); //$NON-NLS-1$
		}
		
		parent.appendChild(result);
		action.domNode= result;
		return result;
	},
	
	_openMenu: function(evt, action) {
		if (evt) {
			dojo.stopEvent(evt);
		}
		
		if (this._menu) {
			this._closeMenu();
			return;
		}
		
		var subActions= action.createActions(dojo.hitch(this, function(actions) {
			this._createDropDownMenu(actions, evt);
		}));
		
		if (subActions.length == 0)
			return;
		
		this._createDropDownMenu(subActions, evt);
	},
	
	_createDropDownMenu: function(subActions, evt) {
		var menu = new dijit.Menu();
		dojo.addClass(menu.domNode, "com-ibm-team-apt-action-drop-down-menu"); //$NON-NLS-1$
		
		this.__fillMenu(menu, subActions);
		this.__update(subActions);
		
		var pos = dojo.coords(evt.target, true);
		
		dijit.popup.open({
			popup: menu,
			parent: evt.target,
			x: pos.x + 5,
			y: pos.y + pos.h - 2,
			onExecute: dojo.hitch(this, this._closeMenu),
			orient: 'BL' //$NON-NLS-1$
		});
		
		this._menu = menu;
		
		var documentBody= dojo.body();
		this._menuCloseOnBody= dojo.connect(window.document, "onclick", this, this.__menuCloseListener); //$NON-NLS-1$
		
		menu.focus();
	},
	
	__menuCloseListener: function(evt){ //$NON-NLS-1$
		try {
			var documentBody= dojo.body();
			var node= evt.target || evt.srcElement;
			var found= false;
			while (node) {
				if (node === documentBody) {
					break;
				}

				if (node === this._menu.domNode || node === this._node) {
					found= true;
					break;
				}
				node= node.parentNode;
			}
			if (!found) {
				this._closeMenu();
			}
		} catch (e) {
		}
	},
	
	_closeMenu: function(evt) {
		if (evt) {
			dojo.stopEvent(evt);
		}
		dojo.disconnect(this._menuCloseOnBody);
		dijit.popup.close(this._menu);
		this._menu = null;
	},
	
	__update: function(actions) {
		dojo.forEach(actions, function(action) {
			var style= action.getStyle();
			var node= action.domNode;	
			
			// disable push buttons
			if((style & Action.STYLE_PUSH) != 0){
				node.disabled= !action.canHandle();
			}
			
			if(!action.canHandle()){
				dojo.addClass(node, this._styles.disabled);			
			} else {
				dojo.removeClass(node, this._styles.disabled);
			}
			
			if((style & Action.STYLE_TOGGLE) != 0) {
				var add= action.isChecked();
				if(add) {
					dojo.addClass(node, this._styles.toggle);
				} else {
					dojo.removeClass(node, this._styles.toggle);
				}
			}
			
		}, this);
	},
	
	__fillMenu: function(menu, actions) {
		dojo.forEach(actions, function(action) {
			var menuItem= null;
			var style= action.getStyle();
			var handler= function(){
				action.handle();
			}; 
			
			if((style & Action.STYLE_TOGGLE) != 0){
				menuItem= new dijit.CheckedMenuItem({
					label: action.getName(),
					_action: action,
					onClick: handler
				});
				menuItem.attr("checked", action.isChecked()); //$NON-NLS-1$
				
			} else {
				menuItem= new com.ibm.team.apt.web.ui.internal.view.plan.actions.IconUrlMenuItem({
					label: action.getName(),
					iconUrl: action.getIconUrl(),
					_action: action,
					onClick: handler
				});
			}			
			menuItem.setDisabled(!action.canHandle());
			menu.addChild(menuItem);
			action.domNode= menuItem.domNode;
		}, this);
	},
	
	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.IconUrlMenuItem", //$NON-NLS-1$
		[dijit.MenuItem],
		{
		// summary: A line item in a Menu Widget

		// Make 3 columns
		//   icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString:
			 '<tr class="dijitReset dijitMenuItem" dojoAttachPoint="focusNode" waiRole="menuitem" tabIndex="-1"' //$NON-NLS-1$
			+'dojoAttachEvent="onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick">' //$NON-NLS-1$
			+'<td class="dijitReset" waiRole="presentation"><div class="dijitMenuItemIcon" style="background-image: url(${iconUrl})" dojoAttachPoint="iconNode"></div></td>' //$NON-NLS-1$
			+'<td class="dijitReset dijitMenuItemLabel" dojoAttachPoint="containerNode"></td>' //$NON-NLS-1$
			+'<td class="dijitReset dijitMenuArrowCell" waiRole="presentation">' //$NON-NLS-1$
				+'<div dojoAttachPoint="arrowWrapper" style="display: none">' //$NON-NLS-1$
					+'<div class="dijitMenuExpand"></div>' //$NON-NLS-1$
					+'<span class="dijitMenuExpandA11y">+</span>' //$NON-NLS-1$
				+'</div>' //$NON-NLS-1$
			+'</td>' //$NON-NLS-1$
			+'</tr>', //$NON-NLS-1$

		__sentinel: null // terminates this class definition
});

})();
