/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPage"); //$NON-NLS-1$

dojo.require("dojo.date.locale"); //$NON-NLS-1$
dojo.require("dojo.date.stamp"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.Tooltip"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.page.PlanningPageController"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPageControl"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.ImageClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.ProgressMonitoredServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.DeletePlanAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.EditPlanAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.ToolBarManager"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var PlanningUtil		= com.ibm.team.apt.web.client.util;
var PlanningClient		= com.ibm.team.apt.web.client.internal.PlanningClient;
var ImageClient			= com.ibm.team.apt.web.client.internal.ImageClient;
var SearchResultsPageControl	= com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPageControl;
var ResponseHandler		= com.ibm.team.apt.web.client.internal.ProgressMonitoredServiceResponseHandler;
var ProgressBar			= com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar;

var ToolBarManager		= com.ibm.team.apt.web.ui.internal.view.plan.actions.ToolBarManager;
var DeletePlanAction	= com.ibm.team.apt.web.ui.internal.view.plan.actions.DeletePlanAction;
var EditPlanAction		= com.ibm.team.apt.web.ui.internal.view.plan.actions.EditPlanAction;

replaceChildren			= dojox.data.dom.replaceChildren;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPage", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_searchResults: null,
	_startIndex: 0,
	_endIndex: 0,
	_serviceCallController: null,
	
	_headerPageNavigation: null,
	_footerPageNavigation: null,
	_resultsTable: null,
	_tableBody: null,
	_noResultsMsg: null,
	
	_labelNoResults: MSGS.SEARCHRESULTS_LABEL_NO_RESULTS,

	constructor: function() {
		this._isVisible= false;
		this._nextTeamLoadFetchIndex= -1;
		this._fetchInProgress= false;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------	

	getTitle: function() {
		return bind(MSGS.SEARCHRESULTS_PAGETITLE, [this._searchResults.getQueryDescriptor().getLabel()]);
	},

	onPageShow: function() {
		this._isVisible= true;
		this._ensureTeamLoadFetched();
	},

	onPageHide: function() {
		this._isVisible= false;
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/search/templates/SearchResultsPage.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		this._nextTeamLoadFetchIndex= this._startIndex;
		
		var constructorArgs= SearchResultsPageControl.constructorArgs(this._searchResults, this._startIndex, "header"); //$NON-NLS-1$
		new SearchResultsPageControl(constructorArgs, this._headerPageNavigation);

		constructorArgs= SearchResultsPageControl.constructorArgs(this._searchResults, this._startIndex, "footer"); //$NON-NLS-1$
		new SearchResultsPageControl(constructorArgs, this._footerPageNavigation);
		
		this._populateTableBody();
	},

	destroy: function() {
		this._searchResults.release();
		this.inherited(arguments);
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	_ensureTeamLoadFetched: function() {
		if (this._isVisible && !this._fetchInProgress) {
			var requestIndex= this._nextTeamLoadFetchIndex;
			if (requestIndex < this._endIndex) {
				this._fetchInProgress= true;
				var requestSize= Math.min(5, this._endIndex - requestIndex);

				var serviceCallback= {
					_success: dojo.hitch(this, function(result) {
						this._nextTeamLoadFetchIndex= requestIndex + requestSize;
						this._fetchInProgress= false;
						this._ensureTeamLoadFetched();
						this._displayPlanProgress(result);
					}),
					_error: dojo.hitch(this, function(error) {
						console.log("Failed fetching teamload starting at index " + requestIndex + ". This batch will be skipped.") //$NON-NLS-1$ //$NON-NLS-2$
						this._fetchInProgress= false;
						this._nextTeamLoadFetchIndex= requestIndex + requestSize;
						this._serviceCallController.remoteError(error);
					})
				}

				var planIds = dojo.map(this._searchResults.getSearchResults().slice(requestIndex, requestIndex + requestSize), function(iterationPlan){
					return iterationPlan.itemId;
				});
				
				var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$
				responseHandler.setLabel(MSGS.MONITOR_FETCH_TEAMLOAD, this._serviceCallController);		
				PlanningClient.fetchTeamLoad(responseHandler, planIds);
			}
		}
	},
	
	_displayPlanProgress: function(resultDTO) {
		dojo.forEach(resultDTO.loadInfos, dojo.hitch(this, function(progressInfoDTO) {
			
			PlanningUtil.runWithCondition(dojo.hitch(this, function() {
				var progressNode= this._findElement("progressBar", progressInfoDTO.planId); //$NON-NLS-1$
				if(!progressNode)
					return;
				
				var options= {};
				options.parameters= {};
				options.parameters["showLabels"]= false; //$NON-NLS-1$
				if(progressInfoDTO.complexityAttribute) {
					options.parameters["complexityLabel"]= progressInfoDTO.complexityAttribute.label; //$NON-NLS-1$
					options.parameters["complexityUnit"]= progressInfoDTO.complexityAttribute.unit; //$NON-NLS-1$
				}
				
				var element= new com.ibm.team.apt.client.DTOBasedProgressInformation(-1, -1, progressInfoDTO);
				var labelProviderCtor= dojo.getObject(progressInfoDTO.labelProviderImplementation);
				var labelProvider= new labelProviderCtor(options);
				constructorArgs= ProgressBar.constructorArgs(element, labelProvider, "25em", "1.5em"); //$NON-NLS-1$ //$NON-NLS-2$
				var progress= new ProgressBar(constructorArgs, progressNode);

				var lblProgress= document.createElement("span"); //$NON-NLS-1$
				lblProgress.innerHTML= labelProvider.getProgressText(element);
				var lblQuality= document.createElement("span"); //$NON-NLS-1$
				lblQuality.innerHTML= labelProvider.getQualityText(element);
				replaceChildren(this._findElement("progressNumbers", progressInfoDTO.planId), lblProgress); //$NON-NLS-1$
				replaceChildren(this._findElement("estimate", progressInfoDTO.planId), lblQuality); //$NON-NLS-1$
				
			}), PlanningClient.getScriptsLoadedCondition());
		}));
	},

	_findElement: function(prefix, planId) {
		var result= dojo.query("span[planCellId=\"" + prefix + "_" + planId+ "\"]", this._tableBody); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		return result.length > 0 ? result[0] : 0; 
	},

	_populateTableBody: function() {
		if (this._searchResults.getResultCount() > 0) {

			var searchResults= this._searchResults.getSearchResults();
			var currentIteration= null;
			
			var lastRow= null;
			for (var i= this._startIndex; i < this._endIndex; i++) {
				var iterationPlan= searchResults[i];
				var iteration= iterationPlan.getIteration();
				if (iteration !== currentIteration) {
					if (lastRow != null) {
						dojo.addClass(lastRow, "iterationGroupEnd"); //$NON-NLS-1$
					}

					this._createHeader(iteration, lastRow == null);
					currentIteration= iteration;
				}

				lastRow= document.createElement("tr"); //$NON-NLS-1$
				this._populatePlanRow(lastRow, iterationPlan);
				this._tableBody.appendChild(lastRow);
			}
		} else {
			dojo.removeClass(this._noResultsMsg, "hidden"); //$NON-NLS-1$
		}
	},

	_createHeader: function(iteration, isFirst) {
		var totalColumns= 6;

		if (!isFirst) {
		this._tableBody.appendChild(this._createSeparatorRow("separatorRow", totalColumns)); //$NON-NLS-1$
		}

		this._tableBody.appendChild(this._createSeparatorRow("headerColumnLine", totalColumns)); //$NON-NLS-1$

		var iterationRow= document.createElement("tr"); //$NON-NLS-1$
		dojo.addClass(iterationRow, "iteration"); //$NON-NLS-1$


		var icon= document.createElement("img"); //$NON-NLS-1$
		icon.className= "icon"; //$NON-NLS-1$
		icon.src= ImageClient.getImageUrl(iteration.iconURL);

		var textContainer= document.createElement("span"); //$NON-NLS-1$
		
		var iterationLabel= document.createElement("b"); //$NON-NLS-1$
		iterationLabel.appendChild(document.createTextNode(iteration.label));

		textContainer.appendChild(iterationLabel);
		
		var timeSpanLabel=  bind(MSGS.SEARCHRESULTS_ITERATION_LABEL, [PlanningUtil.formatDate(iteration.startDate, MSGS.SEARCHRESULTS_ITERATION_NOSTART), PlanningUtil.formatDate(iteration.endDate, MSGS.SEARCHRESULTS_ITERATION_NOEND)]);
		textContainer.appendChild(document.createTextNode(timeSpanLabel));

		var resultCell= document.createElement("td"); //$NON-NLS-1$
		resultCell.colSpan= 6;
		
		resultCell.appendChild(icon);
		resultCell.appendChild(textContainer);
		iterationRow.appendChild(resultCell);
		this._tableBody.appendChild(iterationRow);

		var headerRow= document.createElement("tr"); //$NON-NLS-1$
		dojo.addClass(headerRow, "header"); //$NON-NLS-1$
		headerRow.appendChild(this._createHeaderColumnElement(MSGS.SEARCHRESULTS_COLUMN_NAME, "name", 1)); //$NON-NLS-1$
		headerRow.appendChild(this._createHeaderColumnElement("", "teamArea", 1)); //$NON-NLS-1$ //$NON-NLS-2$
		headerRow.appendChild(this._createHeaderColumnElement(MSGS.SEARCHRESULTS_COLUMN_TEAMAREA, "teamArea", 1)); //$NON-NLS-1$
		headerRow.appendChild(this._createHeaderColumnElement(MSGS.SEARCHRESULTS_COLUMN_PROGRESS, "progress", 3)); //$NON-NLS-1$
		this._tableBody.appendChild(headerRow);
	},
	
	_populatePlanRow: function(tableRow, iterationPlan) {
		tableRow.className= "plan"; //$NON-NLS-1$
		
		var targetURI= PlanningUtil.getPlanViewURI(iterationPlan.itemId);

		var columnElements= [];
		
		columnElements.push(this._createNameColumnElement(iterationPlan, targetURI));
		columnElements.push(this._createActionBarColumnElement(iterationPlan, tableRow));
		columnElements.push(this._createTeamAreaColumnElement(iterationPlan, targetURI));
		columnElements.push(this._createPlaceholderColumnElement(iterationPlan, targetURI, "progressNumbers")); //$NON-NLS-1$
		columnElements.push(this._createPlaceholderColumnElement(iterationPlan, targetURI, "progressBar")); //$NON-NLS-1$
		columnElements.push(this._createPlaceholderColumnElement(iterationPlan, targetURI, "estimate")); //$NON-NLS-1$

		for (var i= 0; i < columnElements.length; i++) {
			var element= columnElements[i];
			var cell= document.createElement("td"); //$NON-NLS-1$
			dojo.addClass(cell, element.className);
			cell.appendChild(element.element);
			tableRow.appendChild(cell);
		}
	},
	
	_createNameColumnElement: function(iterationPlan, targetURI) {
		var result= document.createElement("a"); //$NON-NLS-1$
		
		var icon= document.createElement("img"); //$NON-NLS-1$
		icon.className= "icon"; //$NON-NLS-1$
		icon.src= ImageClient.getImageUrl(iterationPlan.iconURL);

		result.appendChild(icon);
		result.appendChild(document.createTextNode(iterationPlan.label));
		result.href= targetURI;
		
		return {
			element: result,
			className: "name" //$NON-NLS-1$
		};
	},

	_createActionBarColumnElement: function(iterationPlan, row) {
		var result= document.createElement("div"); //$NON-NLS-1$
			
		var toolbar= new ToolBarManager();
		toolbar.add(new DeletePlanAction(iterationPlan, this._serviceCallController, row));
		toolbar.add(new EditPlanAction(iterationPlan, this._serviceCallController));
		toolbar.create(result); //$NON-NLS-1$
		
		return {
			element: result,
			className: "actions" //$NON-NLS-1$
		};
	},

	_createTeamAreaColumnElement: function(iterationPlan) {
		var teamArea= iterationPlan.getTeamArea();
		
		var result= document.createElement("span"); //$NON-NLS-1$
		result.appendChild(document.createTextNode(teamArea.label));
		return {
			element: result,
			className: "teamArea" //$NON-NLS-1$
		};
	},
	
	_createPlaceholderColumnElement: function(iterationPlan, targetURI, columnName) {
		var result= document.createElement("span"); //$NON-NLS-1$
		result.innerHTML= "&nbsp;"; //$NON-NLS-1$
		result.setAttribute("planCellId", columnName + "_" + iterationPlan.itemId); //$NON-NLS-1$ //$NON-NLS-2$
		return {
			element: result,
			className: columnName
		};
	},

	_createHeaderColumnElement: function(label, className, colSpan) {
		var result= document.createElement("td"); //$NON-NLS-1$
		result.appendChild(document.createTextNode(label));
		dojo.addClass(result, "headerColumn"); //$NON-NLS-1$
		dojo.addClass(result, className);
		
		if (parseInt(colSpan) > 1) {
			result.colSpan= colSpan;
		}
		return result;
	},

	_createSeparatorRow: function(className, colSpan) {
		var result= document.createElement("tr"); //$NON-NLS-1$
		var cell= document.createElement("td"); //$NON-NLS-1$
		dojo.addClass(cell, className);
		cell.colSpan= colSpan;
		cell.appendChild(document.createElement("div")); //$NON-NLS-1$
		result.appendChild(cell);
		return result;
	},

	__sentinel: null // terminates this class definition
});

/**
 * This helper method creates the arguments object for the widget's constructor.
 * @param {SearchResults} searchResults The search results displayed 
 * @return {Object} a widget private object to be passed in as a constructor argument
 */
com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPage.constructorArgs= function(searchResults, startIndex, serviceCallController) {
	searchResults.acquire();

	return {
		_searchResults: searchResults,
		_startIndex: startIndex,
		_endIndex: Math.min(startIndex + searchResults.getResultPageSize(), searchResults.getResultCount()),
		_serviceCallController: serviceCallController
	};
};

})();
