/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.viewlets.ui.utils.QueryPreferenceProvider");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.rational.cm.web._base");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.ui.utils.NavigatorChooser");

dojo.requireLocalization("com.ibm.rational.cm.web.viewlets.ui", "ViewletMessages");

(function(){

var messages;
var cq = com.ibm.rational.cm.web;
var _buildLabel = function(text){
	if(!text){
		text = "";
	}
	return "<input type='text' readonly='readonly' value='" + text.replace("'","&#39;") + "'/>";
}

dojo.declare(
	"com.ibm.rational.cm.web.viewlets.ui.utils.QueryPreferenceProvider",
	com.ibm.team.dashboard.web.ui.CustomPreferenceProvider,
{
	constructor: function(args){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.viewlets.ui", "ViewletMessages");
		this._prefId = args.prefId;
		this._fieldsPrefId = args.fieldsPrefId;
		this._fetchingMap = {};
		this._labelMap = {};
		this._initValue = null;
	},
	
	initDefault: function(stableLocation){
		if(stableLocation && stableLocation.length > 0 && stableLocation != "-1"){
			this._initValue = stableLocation;
		}
	},
	
	// override CustomPreferenceProvider
	getLabel: function(value){
		var label = this.getTextLabel(value);
		if(label != ""){
			return _buildLabel(label);
		}
		this._fetchLabel(value);
		return null; // return null so that it shows loading label
	},
	
	// override CustomPreferenceProvider
	getLoadingLabel: function(){
		return _buildLabel(messages.Loading);
	},
	
	getTextLabel: function(value){
		if(!value || value.length === 0 || value == "-1"){ // not selected
			if(this._initValue){
				value = this._initValue;
			}else{
				return messages.ProviderNoQuery;
			}
		}
		if(value && value.length > 0 && value != "-1"){
			var label = this._labelMap[value];
			if(label && label.length > 0){ // use cached label
				return label;
			}
		}
		return "";
	},
	
	_fetchLabel: function(/* String */ stableLocation){
		// check if fetching is already being processed
		if(this._fetchingMap[stableLocation]){ return; }
		
		var self = this;
		dojo["require"]("com.ibm.rational.cm.web.ui.page.PropertyManager");
		com.ibm.rational.cm.web.ui.page.propManager.getProperties({
			resourceId: stableLocation,
			onBegin: function(){
				self._fetchingMap[stableLocation] = true;
			},
			onLoad: function(props){
				self._labelMap[stableLocation] = props.displayName
			}
		});
	},

	// override CustomPreferenceProvider
	createChooser: function(onOk, settings, viewlet){
//		dojo["require"]("com.ibm.rational.cm.web.ui.utils.NavigatorChooser");
		var self = this;
		return com.ibm.rational.cm.openNavigatorChooser({
			filter: { type: "LIST" },
			onOk: function(arr){
				if(arr.length === 0){ return; }
				var item = arr[0];
				var stableLocation = item.stableLocation[0];
				self._labelMap[stableLocation] = item.title[0];
				
				// check if this is a prompt query, and show message that query will be executed without dynamic filters
				cq.xhrGet({
					url: "cqquerywizard.cq",
					content: {
						action: "GetDynamicFilters",
						reLoad: true,
						resourceId: stableLocation
					},
					load: function(response, ioArgs){
						if(response && response.data && response.data.length > 0){
							alert(messages.PromptWarning);
						}
						onOk(stableLocation, self._prefId);
					},
					handleError: true,
					preventCache: true
				});
				
				
				// notify presentation fields preference provider
				var fieldsPrefId = self._fieldsPrefId;
				if(fieldsPrefId){
					viewlet.getPreferenceProvider(fieldsPrefId).getColumnData(stableLocation, function(obj){
						onOk(dojo.toJson(obj), fieldsPrefId);
					});
				}
			}
		});
	}
});

})();
 