/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
dojo.provide("com.ibm.rational.cm.web.ui.view.common.ButtonDialog");

dojo.require("dojo.i18n");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.view.common.Dialog"); // needed for DialogUnderlay & _checkIfSingleChild

dojo.requireLocalization("com.ibm.rational.cm.web.ui","common");

(function(){

var cq = com.ibm.rational.cm.web;
var cqNS = "com.ibm.rational.cm.web.ui";

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.ButtonPane",
	[ dijit.layout.BorderContainer, dijit._Templated ], // RATLC0123997
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/common/templates/ButtonPane.html"),

	okLabel: "",
	hasOkBtn: true, // If true (default) it will have OK/Cancel button. If false, it will have Close button only.
	cshId: null,

	gutters: false, // dijit.layout.BorderContainer property

	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization(cqNS, "common", this.lang);
		this.inherited(arguments);
	},

	postCreate: function(){
		if(this.hasOkBtn){
			// only add OK button if hasOkBtn=true
			var okBtn = this.okBtn = dojo.doc.createElement("BUTTON");
			this.connect(okBtn, "onclick", "onOk");
			okBtn.innerHTML = (this.okLabel && this.okLabel.length > 0) ? this.okLabel : this.messages.lblOk;
			var parentNode = this.cancelBtn.parentNode;
			parentNode.insertBefore(this.okBtn, parentNode.firstChild);
		}else{ // otherwise, set the cancel label to "Close"
			this.cancelBtn.innerHTML = this.messages.lblClose;
		}
		
		this.inherited(arguments);

		// only add CSH for customize dialog in result set pane
		var cshId = this.cshId;
		if(cshId && cshId.length > 0){
			var help = dojo.doc.createElement("div");
			dojo.addClass(help, "helpIcon");
			help.innerHTML = "<a href='javascript:void(0)' onclick='dojo.publish(\"GET_CSH\"," +
				"[{id:\""+cshId+"\"}]);' id='"+cshId+"'>" +
				 "<img style='display:inline;' " +
				 "src='"+cq.convertUrl("images/help.gif")+"'/></a>";
			var btnNode = this.btnCell.domNode;
			btnNode.insertBefore(help, btnNode.firstChild);
		}		
	},
	
	setContent: function(data){
		this._content = data;
		var content = (data instanceof dijit._Widget) ? data.domNode : data;
		this.mainContent.attr("content", content);
	},
	
	getContent: function(){
		return this._content;
	},
	
	setOkEnabled: function(/* boolean */ enable){
		this.okBtn.disabled = !enable;
	},
	
	onOk: function(e){},
	
	// Child objects connect to the dialog onCancel handler to perform necessary cleanup
	onCancel: function(e){
		this.hide();
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.ButtonPaneMixin",
	null,
{
	okLabel: "",
	hasOkBtn: true,
	cshId: null,
	
	postCreate: function(){
		// add button pane
		this.containerNode.innerHTML = "";
		var self = this;
		this._btnPane = new com.ibm.rational.cm.web.ui.view.common.ButtonPane({
			okLabel: this.okLabel,
			hasOkBtn: this.hasOkBtn,
			cshId: this.cshId,
			onOk: function(e){
				if(self.validate()){
					self._onSubmit(e);
					self.hide();
				}
			},
			onCancel: function(e){
			// Invoke the onCancel for the dialog object
				self.onCancel(e);
			}
		});
		this.containerNode.appendChild(this._btnPane.domNode);

		this.inherited(arguments);
	},
	
	/*
	 * User-overridable function that performs validation when OK button is clicked
	 * return true to execute onOk function and hide the dialog
	 * return false to do nothing
	 */
	validate: function(){
		return true;
	},
	
	setContent: function(data){
		this._btnPane.setContent(data);
	},
	
	getContent: function(){
		return this._btnPane.getContent();
	},

	setOkEnabled: function(/* boolean */ enable){
		this._btnPane.setOkEnabled(enable);
	},

	destroy: function(){
		if(this._btnPane){
			this._btnPane.destroy();
		}
		this.inherited(arguments);
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.ButtonDialog",
	[ com.ibm.rational.cm.web.ui.view.common.Dialog, com.ibm.rational.cm.web.ui.view.common.ButtonPaneMixin ],
{
	startup: function(){
		this.inherited(arguments);
		this._btnPane.startup();
	},
	
	resize: function(size){
		this.inherited(arguments);
			
		// explicitly call resize since it is not treated as _singleChild
		var btnPane = this._btnPane;
		if(btnPane && btnPane.resize){
			var cb = this._contentBox;
			// adjust with magic number in order to void unnecessary scrollbar (pecause of padding?)
			btnPane.resize({ w: (cb.w-15), h: (cb.h-22) });
		}
	}
});

})();