/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

/*
 OP values		
		0	"",
		1	"~Equals~",
		*2	"~Does Not Equal~",
		3	"~Less Than~",
		4	"~Less Than Or Equal To~",
		5	"~Greater Than~",
		6	"~Greater Than Or Equal To~",
		7	"~Contains~",
		*8	"~Does Not Contain~",
		9	"~Between~",
		*10	"~Not Between~",
		11	"~Is Null~",
		*12	"~Is Not Null~",
		13	"~In~",
		*14	"~Not In~"	
*/
 
dojo.provide("com.ibm.rational.cm.web.ui.view.filter.CqFilterOPListBox");

dojo.require("dojo.i18n");
dojo.require("dojo.number");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.data.ItemFileReadStore");
dojo.requireLocalization("com.ibm.rational.cm.web.ui","cqQueryFilters");

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.filter.CqFilterOPListBox",
	dijit._Widget,
	{	
		/*
			could be used if we use Dojo Markup to classify this widget
		*/
		widgetType: "CqFilterOPListBox",
		
		ops: [],
		
		i18nStrings: null,
		
		inpWidget: null,
		
		listwdgt: null,
		
		isSetValue: false,
		
		_processData: function(){
			if((this.ops == null)||(this.ops == [])){ return; }

			var tmpdata = { identifier: "op", items: [] };
			var op;
			var step = 0;								
			for(op in this.ops){
				tmpdata.items[step] = {						
					name: this.filterStrings.operator[this.ops[op]],
					op: this.ops[op] 
				};	
				step++;	
			}
			this.listwdgt.store = new dojo.data.ItemFileReadStore({data: tmpdata});							
		},
		
		_opChanged: function(e){
			if(!this.isSetValue){
				this.onOpChanged(this.getValue());
			}
		},
		
		onOpChanged: function(op, /* boolean */ isAuto){},
		
		getValue: function(){
			return dojo.number.parse(this.listwdgt.getValue());	
		},
		
		setValue: function(v){
			this.isSetValue = true;
			this.listwdgt.setValue(v);
			this.isSetValue = false;	
		},
		
		setStore: function(ops) {
			this.ops = ops;
			this._processData();
		},
		
		connectInputWidget: function(inpWidget){
			// disconnect previous connect first
			if(this._inpConn){
				this.disconnect(this._inpConn);
			}
			if(inpWidget && inpWidget.declaredClass == "com.ibm.rational.cm.web.ui.view.filter.CqFilterInput"){
				this._inpConn = this.connect(inpWidget, "onValueChanged", dojo.hitch(this, function(values, isAuto){
					if(isAuto){ return; }
		
					// RATLC01061586 : Cannot add more than one value for a contains field
					var newOp = null;
					if(values && values.length > 1){
						switch(this.getValue()){
							case 1: // Is Equal
							case 3: // Less Than
							case 4: // Less Than Or Equal To
							case 5: // Greater Than
							case 6: // Greater Than Or Equal To
							case 7: // Contains
								newOp = 13; // In
								break;
							case 2: // Is Not Equal
							case 8: // Does Not Contain
								newOp = 14; // Not In
								break;
						}
						if(newOp && dojo.indexOf(this.ops, newOp) != -1){
							this.setValue(newOp);
							this.onOpChanged(newOp, true);
						}
					}

				}));
			}
		},
		
		setDisabled: function(/*Boolean*/ disabled){
			this.listwdgt.setDisabled(disabled);
		},
		
		destroy: function(){
			// RATLC01040572
			// The popup list is not displayed after several creations.
			// This might be fixed in a later dojo version.
			var popup = this.listwdgt._popupWidget;
			if(popup && popup.domNode){				
				popup.destroyRecursive();
			}
			this.inherited('destroy', arguments);
		},
		
		postMixInProperties: function(args, frag){
			this.filterStrings = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "cqQueryFilters", this.lang);
			this.inherited("postMixInProperties", arguments);
		},
		
		/*
			Plugin the construction of the CqRecord Widget
		*/		
		postCreate: function() {
			if(this.inpWidget){
				this.connectInputWidget(this.inpWidget);
			}
			this.listwdgt = new dijit.form.FilteringSelect({				
			}, dojo.doc.createElement('select'));			
			this.domNode.appendChild(this.listwdgt.domNode);			
			this.connect(this.listwdgt, "onChange", this._opChanged);
			this.inherited('postCreate', arguments);
			this._processData();				
		}
	}
);

com.ibm.rational.cm.web.ui.view.filter.getDefaultOperator = function(fieldType, compops){
	// RATLC01063931 : Default operator on dynamic field (SHORT STRING) does not match other clients
	if(fieldType == "SHORT_STRING"){
		var index = dojo.indexOf(compops, 7); // check if valid Operaters have "Contains"?
		if(index != -1){
			return compops[index];
		}
	}
	return compops[0];
}