/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2008. All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
 
dojo.provide("com.ibm.rational.cm.web.ui.view.record.CqTable");
dojo.provide("com.ibm.rational.cm.web.ui.view.record.CqRow");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._Container");

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqTable",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer, dijit._Contained],
{
	templateString:"<table class=\"cqTable\" cellspacing=\"1\"><tbody dojoAttachPoint=\"containerNode\"></tbody></table>\n",		

	// store: String||dojo.data.Store
	//	The store to get data to display in the table
	store: null,
	
	// query: String
	//	query to get top level node(s) of table (ex: {type:'continent'})
	query: null,

	COLUMN_KEY: 'field',
	columns: null, // defines column to display and their labels
				// e.g. [{ field:'column1', label:'label1' }, { field:'column1', label:'label1' }, ... ]
	showHeader: true, // boolean
	header: null, // a reference to CqRow widget of header
	rows: [], // an array storing references to CqRow widget
	
	start: 0, // start index of records to retrieve (for pagination)
	count: null, // number of records to be retrieved (for pagination)
	isSelectable: false,
	_selector: null,
	
	fetchData: function(){
		if(!this.store){ return; }
		var props = {
			query:this.query,
			onComplete:dojo.hitch(this, this.onItemsLoaded),
			onError:dojo.hitch(this, this.gotError)
		};
		if(this.start){ props.start = this.start; }
		if(this.count){ props.count = this.count; }
		this.store.fetch(props);
	},
		
	postCreate: function(){
		if(this.isSelectable){
			dojo["require"]("dojo.dnd.Selector");
			var selector = this._selector = new dojo.dnd.Selector(this.domNode);
			var self = this;

			this.connect(this.domNode, "onclick", function(e){
				// find row index of clicked row
				var inIndex = -1;
				var nodes = self.containerNode.childNodes;
				for(var i = 0, len = nodes.length; i < len; i++){
					if(nodes[i] == selector.anchor){
						inIndex = i;
						break;
					}
				}
				
				if(inIndex != -1){
					self.handleRowClick(inIndex, e.ctrlKey, e.shiftKey);
					var rowWidget = dijit.byNode(selector.anchor);
					self.focusChild(rowWidget);
				}
			});
			this.connectKeyNavHandlers([dojo.keys.UP_ARROW], [dojo.keys.DOWN_ARROW]);
		}
		this.fetchData();
	},
	
	uninitialize: function() {
		if(this._selector){
			this._selector.destroy();
		}
		this.inherited(arguments);
	},
	
	_updateSelector: function(){
		this._selector.selectNone();
		var node = this.focusedChild.domNode;
		this._selector.selection = [node];
		this._selector.anchor = node;
		this._selector._addItemClass(node, "Anchor");
	},

	selectChild: function(childWidget){
		this.focusChild(childWidget);
		this._updateSelector();
	},
	
	focusNext: function(){
		this.inherited("focusNext", arguments);
		this._updateSelector();
	},
	
	focusPrev: function(){
		this.inherited("focusPrev", arguments);
		this._updateSelector();
	},

	startup: function(){
		dojo.forEach(this.getChildren(), function(child){ child.startup(); });
		if(this.isSelectable){
			this.startupKeyNavChildren();
		}
	},
	
	handleRowClick: function(inIndex, inCtrlKey, inShiftKey){
//console.debug("shima handleRowClick: "+inIndex+", "+inCtrlKey+", "+inShiftKey);
	},
	
	onItemsLoaded: function(/*Object*/ results, /*Object*/ dataObject){
		var isFirstChild = true;
		this.clear();

		// set column data if defined
		if(dataObject.store && dataObject.store.colData){
			this.setColumns(dataObject.store.colData);
		}

		dojo.forEach(results, dojo.hitch(this, function(item){
			if(isFirstChild){
				this.addHeader(item);
				isFirstChild = false;
			}
			var row = this.addRow(item, false);
			this.rows.push(row);
		}));
	},
	
	gotError: function(error, request){
		console.error("An error occurred while retrieving data (com.ibm.rational.cm.web.ui.view.record.CqTable): "+error+", "+request);
	},
	
	clear: function(){
		dojo.forEach(this.getChildren(), dojo.hitch(this, function(child){
			this.removeChild(child);
		}));
		this.columns = null;
		this.header = null;
		this.rows = [];
	},

	addHeader: function(/*dojo.data.Item*/ item){
		// if columns is not specified, use attribute names as labels
		if(!this.columns){
			this.columns = [];
			dojo.forEach(this.store.getAttributes(item), dojo.hitch(this, function(attr){
				this.columns.push({ field:attr, label:attr });
			}));
		}
	
		// add CqRow widget
		if(this.showHeader){
			this.header = this.addRow(item, true);
		}
	},
	
	addRow: function(/*dojo.data.Item*/ item, /*boolean*/ isHeader){
		var row = new com.ibm.rational.cm.web.ui.view.record.CqRow({
			columns: this.columns,
			data: item,
			isHeader: isHeader,
			isSelectable: this.isSelectable
		});
		this.addChild(row);
		return row;
	},
	
	setStore: function(store){
		this.store = store;
	},
	
	setColumns: function(columns){
		dojo.forEach(columns, dojo.hitch(this, function(columnObj){
			if(!columnObj.label){
				columnObj.label = columnObj.field;
			}
		}));
		this.columns = columns;
	},
	
	setStart: function(start){
		if(start){
			this.start = start;
		}else{
			this.start = 0;
		}
	},
	
	setCount: function(count){
		this.count = count;
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqRow",
	[dijit._Widget, dijit._Templated, dijit._Container, dijit._Contained],
{
	templateString:"<tr dojoAttachPoint=\"containerNode\"></tr>\n",
	
	columns: null, // see com.ibm.rational.cm.web.ui.view.record.CqTable.columns
	data: null,
	isHeader: false,
	isSelectable: false,
	cells: null,
	_isHidden: false,
	
	postCreate: function(){
		if(this.isSelectable){
			dojo.addClass(this.domNode, "dojoDndItem");
		}
	
		this.cells = {};
		var i = 0;
		dojo.forEach(this.columns, dojo.hitch(this, function(columnObj){
			var colName = columnObj.field;
			var cell = null;
			if(this.isHeader){
				cell = new com.ibm.rational.cm.web.ui.view.record.CqHeaderCell({ value:columnObj.label, index:i });
			}else{
				var val = this.data[colName] || "";
				cell = new com.ibm.rational.cm.web.ui.view.record.CqCell({ value:val });
			}
			this.addChild(cell);
			this.cells[colName] = cell;
			i++;
		}));
	},
	
	focus: function(){
		dijit.focus(this.domNode);
	},
	
	setData: function(data){
		dojo.forEach(this.columns, dojo.hitch(this, function(columnObj){
			var colName = columnObj.field;
			var cell = this.cells[colName];
			if(cell){
				cell.setValue(data[colName]);
			}
		}));
		this.data = data;
	},
	
	show: function(){
		this.domNode.style.display = '';
		this._isHidden = false;
	},
	
	hide: function(){
		this.domNode.style.display = 'none';
		this._isHidden = true;
	},
	
	isHidden: function(){
		return this._isHidden;
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqCell",
	[dijit._Widget, dijit._Templated, dijit._Container, dijit._Contained],
{
	value: '',
	
	templateString:"<td>${value}</td>\n",
	
	getValue: function(){
		return this.value;
	},
	
	setValue: function(val){
		this.value = val;
		this.domNode.innerHTML = val;
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqHeaderCell",
	com.ibm.rational.cm.web.ui.view.record.CqCell,
{
	index: null,
	templateString:'<th class=\"cqHeaderCell_${index}\">${value}</th>\n'
});