/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
dojo.provide("com.ibm.rational.cm.web.ui.view.report.CqReportPane");
var cq = com.ibm.rational.cm.web;

(function(){

    dojo.require("dijit._Widget");
    dojo.require("com.ibm.rational.cm.web._base.xhr");
    dojo.requireLocalization("com.ibm.rational.cm.web.ui", "cqStrings");
    
    dojo.declare("com.ibm.rational.cm.web.ui.view.report.CqReportPane", dijit._Widget, {
        title: '',
        url: null,
        
		/* Create and initialize IFRAME that will render the report.
		 * Firefox and IE handle frame sizing differently.  IE works well with height=100%.
		 * Firefox requires specific height to be set.  See postCreate() function for IE overrides.
		 * RATLC01160174 : [Falcon FVT] Reporting: Reports do not fill available space in IE
		 */
        buildRendering: function(){
            var ifrm = this.domNode = dojo.doc.createElement("IFRAME");
            ifrm.width = "100%";
            ifrm.height = this.getFrameHeight();
            ifrm.frameBorder = "0";
			this.domNode.style.height = ifrm.height;           
        },
        
		/* 
		 * Determine the appropriate height for the frame.
		 */
        getFrameHeight: function(){
            var doc = dojo.doc;
            var oBody = doc.body;
            var docHt = 0, sh, oh;
            if (doc.height) 
                docHt = doc.height;
            else 
                if (oBody) {
                    if (oBody.scrollHeight) 
                        docHt = sh = oBody.scrollHeight;
                    if (oBody.offsetHeight) 
                        docHt = oh = oBody.offsetHeight;
                    if (sh && oh) 
                        docHt = Math.max(sh, oh);
                }
            return docHt;
        },
        
        
        // Update the frame size to better suit the current browser size
        resizeFrame: function(){
            var height = this.getFrameHeight();
            this.domNode.height = height;
			this.domNode.style.height = height;
        },
        
        
        
        postCreate: function(){
            this.inherited("postCreate", arguments);
            var ifrm = this.domNode;
            ifrm.src = this.url;
            
            var self = this;            
            if (dojo.isIE) {
				// RATLC01160174 - IE and FF different behaviors for IFRAME height
				ifrm.height = "100%";
				this.domNode.style.overflow = "hidden";
            	this.domNode.style.height = ifrm.height;
				
                self.connect(ifrm, "onreadystatechange", function(){
                    if (ifrm.readyState == "complete") {
                        self.onReturn();
                    }
                });
            }
            else {
				// Firefox requires height to be set.  IE works well with height=100%
				self.connect(window, "onresize", "resizeFrame");
                self.connect(ifrm, "onload", function(){
                    self.onReturn();
                });
            }
        },
        
        // a stub function that can be connected
        onReturn: function(){
        },
        
        update: function(url){
            this.url = url;
            this.domNode.src = url;
        }
    });
})();

com.ibm.rational.cm.web.ui.view.report.util = {

    // If this is a report, get the query stable location associated
    // with the report to test for dynamic filters.
    getQuery: function(resourceId, page, /* Function */ func){
        var self = this;
        cq.xhrGet({
            url: "cqreport.cq",
            content: {
                action: "GetQueryForReport",
                report: resourceId
            },
            load: function(response, ioArgs){
                func(response.query.LocationId);
            },
			handleError: true,
            preventCache: true
        });
    },
    
    
    executeReport: function(contentPane /* CqReportPane */, message){
    
        if (!contentPane) {
            return;
        }
        var resourceId = message.id;
        
        // RATLC01160174 : [Falcon FVT] Reporting: Reports do not fill available space in IE
        // childTab.containerNode.style.height = "100%";
        // childTab.containerNode.style.overflow = "hidden";
        
        var queryParams = {
            action: "ExecuteReport",
            report: resourceId
        };
        
        // run with data parameter
        if (message.filtervalues) {
            queryParams.filters = dojo.toJson(message.filtervalues.filters);
        }
        
        var self = this;
        cq.xhrGet({
            url: "cqreport.cq",
            content: queryParams,
            load: function(response, ioArgs){
                contentPane.update(response.runReportCommand);
            },
			customChecker: function(response){
                if(!response.runReportCommand || response.runReportCommand.length == 0){
                    alert(dojo.i18n.getLocalization("cq", "cqRecordStrings").noCrystalReportServer);
                    return false;
                }
				return true;
			},
			handleError: true,
            preventCache: false
        });
    }
}
